@@ -23,7 +23,7 @@
  *   If not, see <http://www.gnu.org/licenses/>.
  */
 
-# define AVG_VERSION      "avg : V1.00 Peter Boesecke 2015-05-19"
+# define AVG_VERSION      "avg : V1.04 Peter Boesecke 2017-06-16"
 
 /*+++------------------------------------------------------------------------
 NAME
@@ -40,6 +40,13 @@ AUTHOR
 
 HISTORY
   2016-05-19 V1.00 PB extracted from saxs_average.c V4.07
+  2016-08-11 V1.01 PB string2avgmode, fill_empty_avgpoints:
+                      unused variables removed and format corrected
+  2016-11-29 V1.02 PB fill_empty_avgpoints: initialized j_1 and j_2 with 0
+                      for avoiding compilation warnings.
+  2017-05-19 V1.03 PB ioalloc
+  2017-06-16 V1.04 PB new_avg_data: allocates and resizes AvgData buffers.
+
 --------------------------------------------------------------------------*/
 
 /****************************************************************************
@@ -48,8 +55,6 @@ HISTORY
 
 # include "avg.h"
 
-# define FREE( a) { if ( a) { free( a); a = NULL; } }
-
 /*--------------------------------------------------------------------------
 NAME
 
@@ -91,8 +96,7 @@ RETURN VALUE
 
   -------------------------------------------------------------------------*/
 int string2avgmode( const char *string )
-{ int  NE=1; // TRUE
-  int i, value=InValidAvgMode;
+{ int i, value=InValidAvgMode;
   size_t n;
 
   n = strlen(string);
@@ -194,7 +198,7 @@ AvgPoint * append_avgpoint( AvgListOfPoints * listofpoints, long i_1, long i_2 )
     size_t size_point;
 
     size_point = sizeof ( AvgPoint );
-    new = malloc ( size_point ); 
+    new = MALLOC ( size_point ); 
     if (!new)
       goto append_avgpoint_error;
 
@@ -296,7 +300,7 @@ AvgListOfPoints * new_avglistofpoints( void ) {
   AvgListOfPoints * new=NULL;
 
   size_listofpoints = sizeof(AvgListOfPoints);
-  new = malloc(size_listofpoints);
+  new = MALLOC(size_listofpoints);
   if (!new) 
     goto new_avglistofpoints_error;
 
@@ -479,8 +483,7 @@ long fill_empty_avgpoints( float *data, long dim_1, long dim_2,
                       int debug, int *pstatus )
 {
 
-  long int i, j_1, j_2, i_1, i_2;
-  long int i_1min;
+  long int j_1=0, j_2=0;
   float *pdata;
 
   float *copy=NULL, *pcopy;
@@ -488,8 +491,6 @@ long fill_empty_avgpoints( float *data, long dim_1, long dim_2,
 
   int status;
 
-  float value;
-
   AvgPoint * emptypoint=NULL;
   float f1_1, f1_2, f3_1, f3_2; // index coordinates
   float sum, weight;
@@ -506,7 +507,7 @@ long fill_empty_avgpoints( float *data, long dim_1, long dim_2,
 
     // allocate memory for copy
     copy_size = sizeof(float)*dim_1*dim_2;
-    copy = (float *) malloc (copy_size);
+    copy = (float *) MALLOC (copy_size);
     if (!copy) {
       status=-2; // memory allocation error
       goto fill_empty_avgpoints_error;
@@ -538,7 +539,7 @@ long fill_empty_avgpoints( float *data, long dim_1, long dim_2,
     } // else already defined as dummy
 
     if (debug) {
-      printf("fill_empty_avgpoints: internal dummy is %g\n",dummy,ddummy);
+      printf("fill_empty_avgpoints: internal dummy is %g+-%g\n",dummy,ddummy);
     }
 
     // replace empty points in data
@@ -591,27 +592,149 @@ fill_empty_avgpoints_error:
 /*---------------------------------------------------------------------------
 NAME
 
-  new_avg_data, free_avg_data --- Creates and releases common variables 
+  new_avg_data --- Allocates all avg_data buffers frame data, dummy and ddummy
+
+SYNOPSIS
+
+  AvgData *new_avg_data(AvgData *avg_data, long dim_1, long dim_2, long dim_3);
+
+DESCRIPTION
+
+  If avg_data is NULL the AvgData structure is allocated, otherwise it is 
+  reused. The data buffer sizes of avg_data are resized according to dim_1, 
+  dim_2 and dim_3. If dim_3 is zero all buffer memories are released.
+
+    avg_data sizeof(AvgData);
+    avg_data->data sizeof(float)*dim_1*dim_2*dim_3
+    avg_data->dummy sizeof(float)*dim_1
+    avg_data->ddummy sizeof(float)*dim_1
+  
+  In case of success the pointer to avg_data is returned. In case of an
+  error avg_data is released and NULL is returned.
 
 PURPOSE
-  Creates and releases variables used by all functions.
-  Must be declared in Command Block with InitOptions.
+
+  If the input value of avg_data is NULL a new AvgData structure is allocated 
+  and used, otherwise the input avg_data structure is used. In case of success,
+  the pointer to the used avg_data structure is returned and must be released with
+  free_avg_data for avoiding memory leaks. In case of an error, the AvgData
+  structure is released and the return values is NULL. If dim_1, dim_2 and dim_3 are 
+  positive or zero, the buffers of data, dummy and ddummy are increased in size
+  if necessary, otherwise they are kept.
+
+  If dim_1, dim_2 and dim_3 are negative, the buffers of data, dummy and ddummy 
+  are released and the frame counter is reset to 0.
+
+RETURN VALUE
+
+  In case of success: pointer to avg_data
+  in case of error: NULL
 
 ---------------------------------------------------------------------------*/
-AvgData * new_avg_data( void )
-{ return( (AvgData *) calloc( 1, sizeof ( AvgData ) ) );
+AvgData *new_avg_data(AvgData *avg_data, long dim_1, long dim_2, long dim_3)
+{
+  size_t size;
+
+  if (avg_data==NULL) {
+    if (!(avg_data =  (AvgData *) CALLOC( 1, sizeof ( AvgData ) ) ) ) {
+      goto new_avg_data_error;
+    }
+  }
+
+  if ( (dim_1>=0) && (dim_2>=0) && (dim_3>=0) ) {
+
+    /* allocate frame averaging buffers with dim_1*dim_2*dim_3 pixels */
+    size = sizeof(float)*dim_1*dim_2*dim_3;
+    //++++++++ if (avg_data->data_size != size) {
+    //++++++++   FREE(avg_data->data); 
+    //++++++++   avg_data->data_size = 0;
+    //++++++++ }
+    if (avg_data->data_size < size) {
+      if (!(avg_data->data= (float*) REALLOC( avg_data->data, size ))) {
+        fprintf(stderr,"ERROR: Cannot allocate %zu bytes for data (%ldx%ldx%ld floats)\n",\
+          size, dim_1,dim_2,dim_3);
+        avg_data->data_size = 0;
+        goto new_avg_data_error;
+      }
+      avg_data->data_size = size;
+    }
+
+    size = sizeof(float)*dim_3;
+    //++++++++ if (avg_data->dummy_size != size) {
+    //++++++++   FREE(avg_data->dummy);
+    //++++++++   avg_data->dummy_size = 0;
+    //++++++++ }
+    if (avg_data->dummy_size < size) {
+      if (!(avg_data->dummy= (float*) REALLOC( avg_data->dummy, size ))) {
+        fprintf(stderr,"ERROR: Cannot allocate %zu bytes for dummies (%ld floats)\n",
+          size, dim_3);
+        avg_data->dummy_size = 0;
+        goto new_avg_data_error;
+      }
+      avg_data->dummy_size = size;
+    }
+
+    size = sizeof(float)*dim_3;
+    //++++++++ if (avg_data->ddummy_size != size) {
+    //++++++++   FREE(avg_data->ddummy);
+    //++++++++   avg_data->ddummy_size = 0;
+    //++++++++ }
+    if (avg_data->ddummy_size < size) {
+      if (!(avg_data->ddummy= (float*) REALLOC( avg_data->ddummy, size ))) {
+        fprintf(stderr,"ERROR: Cannot allocate %zu bytes for ddummies (%ld floats)\n",
+          size, dim_3);
+        avg_data->ddummy_size = 0;
+        goto new_avg_data_error;
+      }
+      avg_data->ddummy_size = size;
+    }
+
+  } else {
+    // Reiinitialize buffers and counters
+    FREE(avg_data->data);
+    avg_data->data_size=0;
+    FREE(avg_data->dummy);
+    avg_data->dummy_size=0;
+    FREE(avg_data->ddummy);
+    avg_data->ddummy_size=0;
+
+    avg_data->counter=0;
+
+  }
+
+  return(avg_data);
+
+new_avg_data_error:
+
+  free_avg_data(avg_data);
+
+  return(NULL);
+
 } // new_avg_data
 
-AvgData * free_avg_data ( AvgData * avg_data )
+/*---------------------------------------------------------------------------
+NAME
+
+  free_avg_data --- Release AvgData structure
+
+PURPOSE
+
+  Release AvgData structure
+
+RETURN VALUE
+
+  NULL
+
+---------------------------------------------------------------------------*/
+AvgData *free_avg_data ( AvgData *avg_data )
 {
   if (avg_data) {
-    if (avg_data->data) free( avg_data->data );
-    if (avg_data->dummy) free (avg_data->dummy);
-    if (avg_data->ddummy) free (avg_data->ddummy);
-    free (avg_data);
-    avg_data = NULL;
+    FREE (avg_data->data);
+    FREE (avg_data->dummy);
+    FREE (avg_data->ddummy);
+    FREE (avg_data);
   }
-  return( avg_data );
+  return( NULL );
 } // free_avg_data
 
 /*---------------------------------------------------------------------------
@@ -690,7 +813,7 @@ AvgValue average( const long dim[], const float *frames,
   ave.n = 0;
 
   // 1) extract pixel values (i_1, i_2) from frames and copy them to values
-  if (!( values = (double *) malloc( sizeof(double)*dim[3] ) )) {
+  if (!( values = (double *) MALLOC( sizeof(double)*dim[3] ) )) {
     status = -2; // memory allocation error
     goto average_error;
   }
