@@ -23,7 +23,7 @@
  *   If not, see <http://www.gnu.org/licenses/>.
  */
 
-# define BSLIO_VERSION      "bslio : V0.67 Peter Boesecke 2015-12-08"
+# define BSLIO_VERSION      "bslio : V0.72 Peter Boesecke 2017-05-21"
 /*+++***********************************************************************
 NAME
 
@@ -93,6 +93,14 @@ HISTORY
     2015-12-06  PB V0.66 isspace() -> ISSPACE()
     2015-12-08  PB V0.67 indicator2table: 
                          index (unused-but-set-variable) commented
+    2016-06-02  PB V0.68 indicator2table: the variable index is not used,
+                         but strtol must be called, otherwise error:
+                         CannotOpenAsBslFile
+    2016-11-06  PB V0.69 open_bsl_file: It is now possible reading
+                         compressed bsl header files (BslRead only).
+    2016-12-19  PB V0.70 num_ncasecmp -> strlib_ncasecmp
+    2017-02-13  PB V0.71 bslio_version: char * => const char *
+    2017-05-21  PB V0.72 ioalloc
 
 ***************************************************************************/
 
@@ -119,6 +127,7 @@ HISTORY
 
 # include "bslio.h"
 # include "numio.h"
+# include "strlib.h"
 
 # define ISGRAPH(c) isgraph((int)(c))
 # define ISSPACE(c) isspace((int)(c))
@@ -198,27 +207,27 @@ NAME
 
 SYNOPSIS
 
-   char *bslio_version ( void );
+   const char *bslio_version ( void );
 
 RETURN VALUE
 
-char * version string
+const char * version string
 ---------------------------------------------------------------------------*/
-char *bslio_version ( void )
+const char *bslio_version ( void )
 { return ( BSLIO_VERSION );
 } /* bslio_version */
 
 /*---------------------------------------------------------------------------
 NAME
 
-   newstr_bsl --- allocate memory and copy a character string into it
+   newstr_bsl --- allocate memory and copy a character string to it
 
 SYNOPSIS
 
    char * newstr_bsl( const char * string );
 
 DESCRIPTION
-  Allocates strlen(string)+1 bytes of memory and copies string into it.
+  Allocates strlen(string)+1 bytes of memory and copies string to it.
   In case of success the pointer to the allocated memory is returned. The
   null pointer is returned in case of an error.
 
@@ -229,7 +238,7 @@ RETURN VALUE
 char * newstr_bsl( const char * string )
 { char * newstr_bsling;
 
-  if (!(newstr_bsling = (char *) malloc(strlen(string)+1))) return((char *) NULL);
+  if (!(newstr_bsling = (char *) MALLOC(strlen(string)+1))) return((char *) NULL);
   (void) strcpy(newstr_bsling,string);
 
   return( newstr_bsling );
@@ -280,7 +289,7 @@ SYNOPSIS
        char *getpath_bsl ( char *buffer, size_t buflen, const char * filename );
 
 DESCRIPTION
-       Extract path from filename and copies it into buffer. If the buffer is
+       Extract path from filename and copies it to buffer. If the buffer is
        too short or another error has occurred NULL is returned. If filename 
        has no path the current working directory is determined with 
        getcwd and returned.
@@ -367,7 +376,7 @@ int compare_bsl_keys ( const char * s1, const char * s2, int smode )
                comp_result = strncmp ( s1, s2, MaxBslLineLen ); 
                break;
   case      Bsl_UpperCase: 
-               comp_result = num_strncasecmp ( s1, s2, MaxBslLineLen );
+               comp_result = strlib_ncasecmp ( s1, s2, MaxBslLineLen );
                break;
   case      Bsl_Number:
                l1 = strtol(s1, &err1 ,10);
@@ -378,7 +387,7 @@ int compare_bsl_keys ( const char * s1, const char * s2, int smode )
                  if (l1<l2) comp_result = -1;
                  else if (l1==l2) comp_result = 0;
                  else comp_result = 1;
-               } else comp_result = num_strncasecmp ( s1, s2, MaxBslLineLen );
+               } else comp_result = strlib_ncasecmp ( s1, s2, MaxBslLineLen );
                break;
   default:  printf("compare: Invalid smode\n"); exit(-1);
             break;
@@ -500,7 +509,7 @@ error:    NULL pointer
 BslDataSpec * new_bsl_data_spec ( void )
 { BslDataSpec * newdata_spec;
 
-  if ( (newdata_spec = (BslDataSpec *) malloc( sizeof(BslDataSpec) )) )
+  if ( (newdata_spec = (BslDataSpec *) MALLOC( sizeof(BslDataSpec) )) )
      init_bsl_data_spec ( newdata_spec );
   return(newdata_spec);
 
@@ -527,11 +536,11 @@ BslDataSpec * free_bsl_data_spec ( BslDataSpec * data_spec )
 
   if (!data_spec) return ( data_spec );
 
-  if (data_spec->BinaryFileName) free ( data_spec->BinaryFileName );
-  if (data_spec->Data)           free ( data_spec->Data );
-  if (data_spec->Dim)            free ( data_spec->Dim );
+  FREE ( data_spec->BinaryFileName );
+  FREE ( data_spec->Data );
+  FREE ( data_spec->Dim );
  
-  free(data_spec);
+  FREE(data_spec);
  
   return( (BslDataSpec *) NULL );
  
@@ -563,7 +572,7 @@ error:    NULL pointer
 BslFrame * new_bsl_frame ( const char * FrameKey )
 { BslFrame * newframe;
  
-  if ( (newframe = (BslFrame *) malloc( sizeof(BslFrame) )) ) 
+  if ( (newframe = (BslFrame *) MALLOC( sizeof(BslFrame) )) ) 
      init_bsl_frame ( newframe );
   newframe->FrameKey = newstr_bsl(FrameKey);
   newframe->DataSpec = new_bsl_data_spec();
@@ -606,7 +615,7 @@ BslFrame * search_bsl_frame ( BslMemory * memory, const char * FrameKey )
 /*---------------------------------------------------------------------------
 NAME
 
-  insert_bsl_frame --- insert a bsl frame into the frame list of a bsl memory
+  insert_bsl_frame --- insert a bsl frame to the frame list of a bsl memory
 
 SYNOPSIS
 
@@ -690,10 +699,10 @@ void remove_bsl_frame ( BslFrame * frame )
   if (*proot==frame) { *proot = ((BslFrame*) NULL); }
 
 
-  if (frame->FrameKey)       free(frame->FrameKey); 
+  FREE(frame->FrameKey); 
   free_bsl_data_spec( frame->DataSpec );
 
-  free(frame); 
+  FREE(frame); 
 
   return;
 
@@ -842,7 +851,7 @@ error:    NULL pointer
 BslMemory * new_bsl_memory ( const char * MemoryKey )
 { BslMemory * newmemory;
 
-  if ( (newmemory = (BslMemory*) malloc( sizeof(BslMemory) )) )
+  if ( (newmemory = (BslMemory*) MALLOC( sizeof(BslMemory) )) )
      init_bsl_memory ( newmemory );
   newmemory->MemoryKey = newstr_bsl(MemoryKey);
   return(newmemory);
@@ -883,7 +892,7 @@ BslMemory * search_bsl_memory ( BslFile * file, const char * MemoryKey )
 /*---------------------------------------------------------------------------
 NAME
 
-  insert_bsl_memory --- insert a bsl memory into the memory list of a bsl file 
+  insert_bsl_memory --- insert a bsl memory to the memory list of a bsl file 
 
 SYNOPSIS
 
@@ -967,10 +976,10 @@ void remove_bsl_memory ( BslMemory * memory )
   if (previous!=(BslMemory*) NULL)   previous->Next = next;
   if (*proot==memory) { *proot = ((BslMemory*) NULL); }
 
-  if (memory->MemoryKey)  free(memory->MemoryKey);
-  if (memory->FileName)   free(memory->FileName);
-  if (memory->Indicator)  free(memory->Indicator);
-  free(memory);
+  FREE(memory->MemoryKey);
+  FREE(memory->FileName);
+  FREE(memory->Indicator);
+  FREE(memory);
 
   return;
 
@@ -1162,12 +1171,13 @@ long * indicator2table ( const char * indicator )
   do { table_len++;
        previous = rest;
        // index = strtol(previous, &rest, 10); // unused-but-set-variable
+       strtol(previous, &rest, 10);
    } while (( *rest ) && ( previous<rest ));
 
   if (strisspace(previous)) table_len--;  
 
   /* allocate memory for indic_table */
-  indic_table = (long int *) malloc ( sizeof( long int) * (table_len+1) );
+  indic_table = (long int *) MALLOC ( sizeof( long int) * (table_len+1) );
   if (!indic_table) return(indic_table); 
   indic_table[0] = table_len;
 
@@ -1209,16 +1219,16 @@ long * get_bsl_dim ( const BslMemory * memory )
   if (!memory) return( dim );
   indic = indicator2table ( memory->Indicator );
   if (!indic) return( dim );
-  if (indic[0]<BslIndicNumber) { free(indic); return( dim ); }
+  if (indic[0]<BslIndicNumber) { FREE(indic); return( dim ); }
 
-  dim = (long *) malloc ( sizeof (long int) * 4 );
-  if (!dim) { free(indic); return ( dim ); }
+  dim = (long *) MALLOC ( sizeof (long int) * 4 );
+  if (!dim) { FREE(indic); return ( dim ); }
 
   dim[0] = 2;
   dim[1] = indic[1];
   dim[2] = indic[2];
 
-  free ( indic );
+  FREE ( indic );
 
   return ( dim );
 
@@ -1250,10 +1260,10 @@ long get_bsl_number ( const BslMemory * memory )
   if (!memory) return( number );
   indic = indicator2table ( memory->Indicator );
   if (!indic) return( number );
-  if (indic[0]<BslIndicNumber) { free(indic); return( number ); }
+  if (indic[0]<BslIndicNumber) { FREE(indic); return( number ); }
 
   number = indic[3];
-  free ( indic );
+  FREE ( indic );
 
   return ( number ); 
 
@@ -1413,8 +1423,7 @@ int read_bsl_header ( BslFile * file )
 
         if (indicator2frame_list ( memory )) return(-1);
       }
-//++++++++{+  } while (( memory_key ) && ( indicator )); // always true
-   } while ( indicator );
+  } while ( indicator );
   if (io_error) return(-1);
 
   return(0);
@@ -1439,30 +1448,32 @@ error:     int -1
 ---------------------------------------------------------------------------*/
 int free_bsl_file ( BslFile * file )
 {
-  /* remove memory list */
-  remove_bsl_memory_list( file );
+  if (file) {
+    /* remove memory list */
+    remove_bsl_memory_list( file );
 
-  /* close file */
-  if (fclose(file->Channel)) return(-1);
-  file->Channel = (FILE *) NULL;
+    /* close file */
+    if (fclose(file->Channel)) return(-1);
+    file->Channel = (FILE *) NULL;
 
-  /* release IO-buffer */
-  free(file->Buffer);
-  file->Buffer = (char *) NULL;
+    /* release IO-buffer */
+    FREE(file->Buffer);
+    file->Buffer = (char *) NULL;
 
-  /* release file name */
-  free(file->Name);
-  file->Name = (char *) NULL;
+    /* release file name */
+    FREE(file->Name);
+    file->Name = (char *) NULL;
 
-  /* release path */
-  free(file->Path);
-  file->Path = (char *) NULL;
+    /* release path */
+    FREE(file->Path);
+    file->Path = (char *) NULL;
 
-  /* free first and second header */
-  if (file->FirstHeader) free ( file->FirstHeader );
-  if (file->SecondHeader) free ( file->SecondHeader );
+    /* free first and second header */
+    FREE ( file->FirstHeader );
+    FREE ( file->SecondHeader );
 
-  init_bsl_file ( file );
+    init_bsl_file ( file );
+  }
 
   return(0);
 
@@ -1493,46 +1504,78 @@ error:    int      stream < 0
 ---------------------------------------------------------------------------*/
 int open_bsl_file ( const char * FileName , const char * mode )
 { int       stream;
-  BslFile   *file;
+  BslFile   *file=NULL;
   int       buflen = BslBufferSize; 
   char      buffer[BslBufferSize];
 
   if (!InitBslTable) init_bsl_file_table ( BslTable );
   stream =  search_free_bsl_stream ( BslTable );
-  if (stream<0) return( stream ); 
+  if (stream<0) 
+    goto open_bsl_file_error;
 
   file = &BslTable[stream];
 
   init_bsl_file ( file );
 
   if (strcmp(mode,BslOld)==0) { /* open old file */
-    file->Channel = fopen( FileName,"rb+"); } else
-  if (strcmp(mode,BslNew)==0) { /* open new file */
-    file->Channel = fopen( FileName,"wb+"); } else
-  if (strcmp(mode,BslAny)==0) { /* open old file or create new file */
-    if ( (file->Channel = fopen( FileName,"rb+")) ) {
-      } else {
-      file->Channel = fopen( FileName,"wb+"); } } else
-  if (strcmp(mode,BslRead)==0) { /* open old file read-only */
-    file->Channel = fopen( FileName,"rb"); } 
+    file->Channel = fopen( FileName,"rb+"); 
+  } else if (strcmp(mode,BslNew)==0) { /* open new file */
+    file->Channel = fopen( FileName,"wb+"); 
+  } else if (strcmp(mode,BslAny)==0) { /* open old file or create new file */
+    if ( !(file->Channel = fopen( FileName,"rb+")) ) {
+      file->Channel = fopen( FileName,"wb+"); 
+    }
+  } else if (strcmp(mode,BslRead)==0) { /* open old file read-only */
+    int cmpr, errval;
+    if ((cmpr=cmpr_checkfiletype(FileName))>UnCompressed) {
+      FILE *inp;
+      // uncompress file and copy to temporary file
+      if ((inp=fopen(FileName,"rb"))) {
+        file->Channel = tmpfile();
+        if (cmpr_inflatefile ( file->Channel, inp,
+                         cmpr, NULL, &errval )) {
+          fprintf(stderr,"%s cmpr_inflatefile errval=%d (%s)\n",
+             "open_bsl_file",errval,cmpr_errval2string(errval));
+          fclose(file->Channel);
+          file->Channel=NULL;
+        }
+        fclose(inp);
+        // rewind
+        fseek( file->Channel, 0, SEEK_SET);
+      } else file->Channel = NULL;
+    } else {
+      // open uncompressed file
+      file->Channel = fopen( FileName,"rb");
+    }
+  }
 
-  if (file->Channel == (FILE *) NULL) { free_bsl_file( file ); return(-1); }
+  if (file->Channel == (FILE *) NULL)
+    goto open_bsl_file_error;
 
   /* allocate IO-buffer */
-  file->Buffer = (char *) malloc(sizeof(char)*BslBufferSize);
-  if ( file->Buffer == (char *) NULL ) { free_bsl_file( file ); return(-1); }
-  if ( setvbuf(file->Channel, file->Buffer, _IOFBF, BslBufferSize) ) {
-    free_bsl_file( file ); return(-1); }
+  file->Buffer = (char *) MALLOC(sizeof(char)*BslBufferSize);
+  if ( file->Buffer == (char *) NULL ) 
+    goto open_bsl_file_error;
+
+  if ( setvbuf(file->Channel, file->Buffer, _IOFBF, BslBufferSize) )
+    goto open_bsl_file_error;
 
   /* file structure parameters */
   file->Name = newstr_bsl( FileName );
   file->Path = newstr_bsl( getpath_bsl( buffer, buflen, FileName ));
 
   /* read header(s) and locate data */
-  if (read_bsl_header ( file )) { free_bsl_file( file ); return(-1); } 
+  if (read_bsl_header ( file ))
+    goto open_bsl_file_error;
 
   return( stream );
 
+open_bsl_file_error:
+
+  free_bsl_file( file ); 
+
+  return(-1);
+
 } /* open_bsl_file */
 
 /*---------------------------------------------------------------------------
