@@ -61,9 +61,15 @@ HISTORY
                      before printing as %lu. %lz is not known by all
                      compilers.
   2016-01-12 V1.10 PB format converter SIZE_T renamed to SIZE_F
+  2016-11-06 V1.11 PB cmpr_frinflate, cmpr_inflate: a warning for
+                      Z_STREAM_END is only written when the parameter
+                      perrval is set.
+  2017-02-07 V1.12 PB added: cmpr_checkmagic
+  2017-02-12 V1.13 PB added: iodbg(IODBG_COMPRESSION), removed: CMPR_debug
+  2017-03-03 V1.14 PB cmpr_debug acts only on the IODBG_COMPRESSION flag.
 
 --------------------------------------------------------------------------*/
-# define CMPR_VERSION "cmpr : V1.10 Peter Boesecke 2016-01-12 -- zlib : " ZLIB_VERSION
+# define CMPR_VERSION "cmpr : V1.14 Peter Boesecke 2017-03-03 -- zlib : " ZLIB_VERSION
 /****************************************************************************
 *  Include                                                                  *
 ****************************************************************************/
@@ -97,27 +103,26 @@ HISTORY
 ***************************************************************************/
 # define SIZE_F (unsigned long)
 
-/****************************************************************************
-* Static Variables                                                          *
-****************************************************************************/
-static int CMPR_debug = 0;
-
 /*--------------------------------------------------------------------------
 NAME
 
-  cmpr_debug --- set / reset module cmpr into debug mode
+  cmpr_debug --- set / reset module cmpr to debug mode
 
 SYNOPSIS
 
-  void cmpr_debug ( int debug );
+  void cmpr_debug ( long debug );
 
 DESCRPTION
 
-  Writes ´debug´ into CMPR_debug.
+  Writes ´debug´ to iodbg.
 
 --------------------------------------------------------------------------*/
-void cmpr_debug ( int debug )
-{ CMPR_debug = debug;
+void cmpr_debug ( long debug )
+{ if (debug) {
+    iodbg_debug_or( IODBG_COMPRESSION );
+  } else {
+    iodbg_debug_and( ~IODBG_COMPRESSION );
+  }
 } /* cmpr_debug */
 
 /*--------------------------------------------------------------------------
@@ -177,7 +182,7 @@ int cmpr_deflate ( void * out, size_t outlen,
   size_t incr_inp=(size_t) 0, incr_out=(size_t) 0;
   size_t converted=(size_t) 0;
 
-  if (CMPR_debug) printf("\n cmpr_deflate BEGIN\n");
+  if (iodbg(IODBG_COMPRESSION)) printf("\n cmpr_deflate BEGIN\n");
 
   if (!out) outlen=(size_t) 0;
   if (!inp) inplen=(size_t) 0;
@@ -203,7 +208,7 @@ int cmpr_deflate ( void * out, size_t outlen,
     switch ( cmpr_method ) {
       case GzipCompression:
         /* initialize gzip compression */
-        if (CMPR_debug) {
+        if (iodbg(IODBG_COMPRESSION)) {
           printf("   gzip: deflateInit2(strm,level,method,windowBits,memLevel,strategy)\n");
           printf("         deflateInit2(...,%d,%d,%d,%d,%d)\n",
             Z_DEFAULT_COMPRESSION, Z_DEFLATED, (MAX_WBITS+16), 
@@ -222,7 +227,7 @@ int cmpr_deflate ( void * out, size_t outlen,
         }
         break;
       case ZCompression:
-        if (CMPR_debug) {
+        if (iodbg(IODBG_COMPRESSION)) {
           printf("   Z: deflateInit(strm,level)\n");
           printf("      deflateInit(...,%d)\n",
             Z_DEFAULT_COMPRESSION);
@@ -289,7 +294,7 @@ int cmpr_deflate ( void * out, size_t outlen,
   } // if (inplen>0)
 
   if ( (errval!=Z_STREAM_END) && (rest_inp>0) && (rest_out==0) ) {
-    if (CMPR_debug) {
+    if (iodbg(IODBG_COMPRESSION)) {
       printf(" cmpr_deflate: output buffer too short (%lu of %lu bytes used)\n",
         SIZE_F rest_out,SIZE_F outlen);
       printf("              %lu of %lu input bytes unprocessed.\n",
@@ -305,7 +310,7 @@ int cmpr_deflate ( void * out, size_t outlen,
     goto cmpr_deflate_error;
   } else errval=Z_OK;
 
-  if (CMPR_debug) 
+  if (iodbg(IODBG_COMPRESSION)) 
     printf(" cmpr_deflate %lu bytes converted END\n",SIZE_F converted);
 
   if (pconverted) *pconverted=converted;
@@ -322,7 +327,7 @@ cmpr_deflate_error1:
 
   fprintf(stderr,"cmpr_deflate: %s (%s)\n",cmpr_errval2string(errval),
     cmpr_version());
-  if (CMPR_debug) 
+  if (iodbg(IODBG_COMPRESSION)) 
     printf(" cmpr_deflate END (errval=%d)\n",errval);
 
   if (pconverted) *pconverted=converted;
@@ -368,7 +373,9 @@ ARGUMENTS
   size_t inplen        length of input buffer in bytes
   int cmpr_method      compression method
   size_t * pconverted  number of valid bytes in output buffer
-  int * perrval        zlib error value
+  int * perrval        zlib error value, 
+                       only if this parameter is set a warning is written
+                       to the error output if Z_STREAM_END is missing
 
 AUTHOR
   Peter Boesecke 2010-12-12
@@ -389,7 +396,7 @@ int cmpr_inflate ( void * out, size_t outlen,
   size_t incr_inp=(size_t) 0, incr_out=(size_t) 0;
   size_t converted=(size_t) 0;
 
-  if (CMPR_debug) printf("\n cmpr_inflate BEGIN\n");
+  if (iodbg(IODBG_COMPRESSION)) printf("\n cmpr_inflate BEGIN\n");
 
   if (!out) outlen=(size_t) 0;
   if (!inp) inplen=(size_t) 0;
@@ -415,7 +422,7 @@ int cmpr_inflate ( void * out, size_t outlen,
     switch ( cmpr_method ) {
       case GzipCompression:
         /* initialize gzip decompression */
-        if (CMPR_debug) {
+        if (iodbg(IODBG_COMPRESSION)) {
           printf("   gzip: inflateInit2(strm,windowBits)\n");
           printf("         inflateInit2(...,%d)\n",
             (MAX_WBITS+16));
@@ -432,7 +439,7 @@ int cmpr_inflate ( void * out, size_t outlen,
         break;
       case ZCompression:
         /* initialize Z decompression */
-        if (CMPR_debug) {
+        if (iodbg(IODBG_COMPRESSION)) {
           printf("   Z: inflateInit(strm)\n");
           printf("      inflateInit(...)\n");
         }
@@ -486,7 +493,7 @@ int cmpr_inflate ( void * out, size_t outlen,
   }
 
   if ( (errval!=Z_STREAM_END) && (rest_inp>0) && (rest_out==0) ) {
-    if (CMPR_debug) {
+    if (iodbg(IODBG_COMPRESSION)) {
       printf(" cmpr_inflate: output buffer too short (%lu of %lu bytes used)\n",
         SIZE_F rest_out,SIZE_F outlen);
       printf("              %lu of %lu input bytes unprocessed.\n",
@@ -497,12 +504,13 @@ int cmpr_inflate ( void * out, size_t outlen,
   }
 
   if (errval!=Z_STREAM_END) {
-    fprintf(stderr, "inflate should report Z_STREAM_END\n");
+    if ((perrval)||(iodbg(IODBG_COMPRESSION))) // write this message only if perrval is set
+      fprintf(stderr, "inflate should report Z_STREAM_END\n");
     errval=Z_DATA_ERROR;
     goto cmpr_inflate_error;
   } else errval=Z_OK;
 
-  if (CMPR_debug)
+  if (iodbg(IODBG_COMPRESSION))
     printf(" cmpr_inflate %lu bytes converted END\n",SIZE_F converted);
 
   if (pconverted) *pconverted=converted;
@@ -517,9 +525,10 @@ cmpr_inflate_error:
 
 cmpr_inflate_error1:
 
-  fprintf(stderr,"cmpr_inflate: %s (%s)\n",cmpr_errval2string(errval),
-    cmpr_version());
-  if (CMPR_debug) printf(" cmpr_inflate END (errval=%d)\n",errval);
+  if ((perrval)||(iodbg(IODBG_COMPRESSION)))
+    fprintf(stderr,"cmpr_inflate: %s (%s)\n",cmpr_errval2string(errval),
+      cmpr_version());
+  if (iodbg(IODBG_COMPRESSION)) printf(" cmpr_inflate END (errval=%d)\n",errval);
 
   if (pconverted) *pconverted=converted;
   if (perrval) *perrval=errval;
@@ -565,7 +574,9 @@ ARGUMENTS
   size_t inplen        maximum number of bytes to read
   int cmpr_method      compression method
   size_t * pconverted  number of valid bytes in output buffer
-  int * perrval        zlib error value
+  int * perrval        zlib error value,
+                       only if this parameter is set a warning is written
+                       to the error output if Z_STREAM_END is missing
 
 AUTHOR
   Peter Boesecke 2010-12-13
@@ -587,7 +598,7 @@ int cmpr_frinflate ( void * out, size_t outlen,
   size_t incr_inp=(size_t) 0, incr_out=(size_t) 0;
   size_t converted=(size_t) 0;
 
-  if (CMPR_debug) printf("\n cmpr_frinflate BEGIN\n");
+  if (iodbg(IODBG_COMPRESSION)) printf("\n cmpr_frinflate BEGIN\n");
 
   if (!out) outlen=(size_t) 0;
   if (!inp) inplen=(size_t) 0;
@@ -612,7 +623,7 @@ int cmpr_frinflate ( void * out, size_t outlen,
     switch ( cmpr_method ) {
       case GzipCompression:
         /* initialize gzip decompression */
-        if (CMPR_debug) {
+        if (iodbg(IODBG_COMPRESSION)) {
           printf("   gzip: inflateInit2(strm,windowBits)\n");
           printf("         inflateInit2(...,%d)\n",
             (MAX_WBITS+16));
@@ -629,7 +640,7 @@ int cmpr_frinflate ( void * out, size_t outlen,
         break;
       case ZCompression:
         /* initialize Z decompression */
-        if (CMPR_debug) {
+        if (iodbg(IODBG_COMPRESSION)) {
           printf("   Z: inflateInit(strm)\n");
           printf("      inflateInit(...)\n");
         }
@@ -688,7 +699,7 @@ int cmpr_frinflate ( void * out, size_t outlen,
   }
 
   if ( (errval!=Z_STREAM_END) && (rest_inp>0) && (rest_out==0) ) {
-    if (CMPR_debug) {
+    if (iodbg(IODBG_COMPRESSION)) {
       printf(" cmpr_inflate: output buffer too short (%lu of %lu bytes used)\n",
         SIZE_F rest_out,SIZE_F outlen);
       printf("              %lu of %lu input bytes unprocessed.\n",
@@ -699,12 +710,13 @@ int cmpr_frinflate ( void * out, size_t outlen,
   }
 
   if (errval!=Z_STREAM_END) {
-    fprintf(stderr, "inflate should report Z_STREAM_END\n");
+    if ((perrval)||(iodbg(IODBG_COMPRESSION))) // write this message only if perrval is set
+      fprintf(stderr, "inflate should report Z_STREAM_END\n");
     errval=Z_DATA_ERROR;
     goto cmpr_frinflate_error;
   } else errval=Z_OK;
 
-  if (CMPR_debug)
+  if (iodbg(IODBG_COMPRESSION))
     printf(" cmpr_frinflate %lu bytes converted END\n",SIZE_F converted);
 
   if (pconverted) *pconverted=converted;
@@ -719,9 +731,10 @@ cmpr_frinflate_error:
 
 cmpr_frinflate_error1:
 
-  fprintf(stderr,"cmpr_frinflate: %s (%s)\n",cmpr_errval2string(errval),
-    cmpr_version());
-  if (CMPR_debug) printf(" cmpr_frinflate END (errval=%d)\n",errval);
+  if ((perrval)||(iodbg(IODBG_COMPRESSION))) // write this message only if perrval is set
+    fprintf(stderr,"cmpr_frinflate: %s (%s)\n",cmpr_errval2string(errval),
+      cmpr_version());
+  if (iodbg(IODBG_COMPRESSION)) printf(" cmpr_frinflate END (errval=%d)\n",errval);
 
   if (pconverted) *pconverted=converted;
   if (perrval) *perrval=errval;
@@ -783,7 +796,7 @@ int cmpr_inflatefile ( FILE * outp, FILE * inp,
   size_t incr_out=(size_t) 0;
   size_t converted=(size_t) 0;
 
-  if (CMPR_debug) printf("\n cmpr_inflatefile BEGIN\n");
+  if (iodbg(IODBG_COMPRESSION)) printf("\n cmpr_inflatefile BEGIN\n");
 
   /* decompress all input */
 
@@ -801,7 +814,7 @@ int cmpr_inflatefile ( FILE * outp, FILE * inp,
       switch ( cmpr_method ) {
         case GzipCompression:
           /* initialize gzip decompression */
-          if (CMPR_debug) {
+          if (iodbg(IODBG_COMPRESSION)) {
             printf("   gzip: inflateInit2(strm,windowBits)\n");
             printf("         inflateInit2(...,%d)\n",
               (MAX_WBITS+16));
@@ -818,7 +831,7 @@ int cmpr_inflatefile ( FILE * outp, FILE * inp,
           break;
         case ZCompression:
           /* initialize Z decompression */
-          if (CMPR_debug) {
+          if (iodbg(IODBG_COMPRESSION)) {
             printf("   Z: inflateInit(strm)\n");
             printf("      inflateInit(...)\n");
           }
@@ -881,7 +894,7 @@ int cmpr_inflatefile ( FILE * outp, FILE * inp,
 
     } while ((strm.avail_in>0)||(!feof(inp))); // continue until buffer empty
 
-  if (CMPR_debug)
+  if (iodbg(IODBG_COMPRESSION))
     printf(" cmpr_inflatefile %lu bytes converted END\n",SIZE_F converted);
 
   if (pconverted) *pconverted=converted;
@@ -896,9 +909,10 @@ cmpr_inflatefile_error:
 
 cmpr_inflatefile_error1:
 
-  fprintf(stderr,"cmpr_inflatefile: %s (%s)\n",cmpr_errval2string(errval),
-    cmpr_version());
-  if (CMPR_debug) printf(" cmpr_inflatefile END (errval=%d)\n",errval);
+  if ((perrval)||(iodbg(IODBG_COMPRESSION))) // write this message only if perrval is set
+    fprintf(stderr,"cmpr_inflatefile: %s (%s)\n",cmpr_errval2string(errval),
+      cmpr_version());
+  if (iodbg(IODBG_COMPRESSION)) printf(" cmpr_inflatefile END (errval=%d)\n",errval);
 
   if (pconverted) *pconverted=converted;
   if (perrval) *perrval=errval;
@@ -958,7 +972,7 @@ int cmpr_deflatefile ( FILE * outp, FILE * inp,
   size_t incr_out=(size_t) 0;
   size_t converted=(size_t) 0;
 
-  if (CMPR_debug) printf("\n cmpr_deflatefile BEGIN\n");
+  if (iodbg(IODBG_COMPRESSION)) printf("\n cmpr_deflatefile BEGIN\n");
 
   /* Compresssion Levels:
      Z_NO_COMPRESSION
@@ -976,7 +990,7 @@ int cmpr_deflatefile ( FILE * outp, FILE * inp,
   switch ( cmpr_method ) {
     case GzipCompression:
       /* initialize gzip compression */
-      if (CMPR_debug) {
+      if (iodbg(IODBG_COMPRESSION)) {
         printf("   gzip: deflateInit2(strm,level,method,windowBits,memLevel,strategy)\n");
         printf("         deflateInit2(...,%d,%d,%d,%d,%d)\n",
           Z_DEFAULT_COMPRESSION, Z_DEFLATED, (MAX_WBITS+16),
@@ -995,7 +1009,7 @@ int cmpr_deflatefile ( FILE * outp, FILE * inp,
       }
       break;
     case ZCompression:
-      if (CMPR_debug) {
+      if (iodbg(IODBG_COMPRESSION)) {
         printf("   Z: deflateInit(strm,level)\n");
         printf("      deflateInit(...,%d)\n",
           Z_DEFAULT_COMPRESSION);
@@ -1040,7 +1054,7 @@ int cmpr_deflatefile ( FILE * outp, FILE * inp,
 
       if (strm.avail_in!=0) {
         /* all input must be used */
-        if (CMPR_debug) {
+        if (iodbg(IODBG_COMPRESSION)) {
           printf(" cmpr_deflatefile: not all input deflated (%u bytes unprocessed)\n",
             strm.avail_in);
         }
@@ -1060,7 +1074,7 @@ int cmpr_deflatefile ( FILE * outp, FILE * inp,
     goto cmpr_deflatefile_error;
   } else errval=Z_OK;
 
-  if (CMPR_debug)
+  if (iodbg(IODBG_COMPRESSION))
     printf(" cmpr_deflatefile %lu bytes converted END\n",SIZE_F converted);
 
   if (pconverted) *pconverted=converted;
@@ -1077,7 +1091,7 @@ cmpr_deflatefile_error1:
 
   fprintf(stderr,"cmpr_deflatefile: %s (%s)\n",cmpr_errval2string(errval),
     cmpr_version());
-  if (CMPR_debug) printf(" cmpr_deflatefile END (errval=%d)\n",errval);
+  if (iodbg(IODBG_COMPRESSION)) printf(" cmpr_deflatefile END (errval=%d)\n",errval);
 
   if (pconverted) *pconverted=converted;
   if (perrval) *perrval=errval;
@@ -1174,7 +1188,7 @@ int cmpr_checkfiletype(const char *filename)
 char * cmpr_filename(char * buffer, size_t buflen, const char *filename, int cmpr)
 { char * pb;
   size_t blen;
-  if ( cmpr_checkfiletype(filename) > 1 ) {
+  if ( cmpr_checkfiletype(filename) > UnCompressed ) {
     // copy filename to buffer without extension 
     filename_body ( buffer, buflen, filename );
   } else {
@@ -1200,3 +1214,144 @@ char * cmpr_filename(char * buffer, size_t buflen, const char *filename, int cmp
   return( buffer );
 
 } // cmpr_filename
+
+/***************************************************************************
+* Magic Patterns                                                           *
+***************************************************************************/
+enum CmprMagicType { CmprInValidMagic=0,
+                     CmprUnCompressed,
+                     CmprGzipMagic,
+                     CmprZMagic,
+                     CmprEndMagic };
+
+# define CMPRMAGICHEADLEN 3  // 2+1
+
+// xxCMPRMAGIC[0] = number of bytes to compare, terminate with 0
+const unsigned char CMPRMAGICGZIP[]   =
+  { 0x02, 0x1F, 0x8B, 0x00 }; // Gzip
+const unsigned char CMPRMAGICZ[]   =
+  { 0x02, 0x78, 0x9C, 0x00 }; // Zlib compression
+
+// Last element must be NULL (InValid,UnCompressed,Gzip,Z,End)
+const unsigned char *CMPRMAGIC[] = { 0x00, 0x00, CMPRMAGICGZIP, CMPRMAGICZ, NULL };
+
+/*
+ * Compares head with with magic pattern and returns the number of
+ * different bytes or -1 in case of an error.
+ */
+int cmpr_compare_magic( const unsigned char *head, size_t headlen, const unsigned char *magic )
+{
+  int ndiff=-1; // number of different characters
+
+  if (magic) {
+    size_t bytes_to_compare=0;
+    size_t i;
+
+    bytes_to_compare = magic[0];
+    if (bytes_to_compare>0) ndiff = 0;
+
+    if (headlen>=bytes_to_compare) {
+      const unsigned char *ph=head, *pm=magic+1;
+
+      for (i=1;i<=bytes_to_compare;i++) {
+        if ( *pm != *ph ) ndiff++;
+        ph++; pm++;
+      }
+    }
+  }
+
+  return(ndiff);
+
+} /* cmpr_compare_magic */
+
+/*---------------------------------------------------------------------------
+NAME
+
+   cmpr_checkmagic --- reads a magic pattern and returns the DCompression
+
+SYNOPSIS
+
+   // int cmpr_checkmagic ( const char *fname );
+   int cmpr_checkmagic ( FILE *channel )
+
+DESCRIPTION
+  Reads a pattern after the current file positions and compares it
+  with a table of known magic patterns. Using tis function is transparent. 
+  The file pointer is repositioned where it was found and
+  the DCompression type corresponding to the magic pattern is returned.
+
+PATTERNS
+
+dd count=8 ibs=1  if=../hdf/multi.edf.h5 > testpattern
+
+GZIP  1F 8B                              (gzip compressed file)
+Z     78 9C                              (Zlib compressed file)
+
+ARGUMENT
+const char * fname      filename
+
+RETURN VALUE
+success:   DCompression
+error:     InValidDCompression
+
+HISTORY
+06-Feb-2017 PB
+  --------------------------------------------------------------------------+*/
+int cmpr_checkmagic ( FILE *channel )
+{
+  int cmpr_method=InValidDCompression;
+  size_t bytes_read=0; // number of read bytes
+  unsigned char filehead[CMPRMAGICHEADLEN]="";
+  int magictype=CmprInValidMagic;
+  size_t i;
+  long cur_pos;
+
+  if (!channel) {
+    goto cmpr_checkmagic_error;
+  }
+
+  if ( !feof( channel ) ) {
+    /*
+     * Read start pattern of the file
+     */
+
+    cur_pos = ftell( channel );
+
+    bytes_read = fread(filehead,1,CMPRMAGICHEADLEN,channel);
+    filehead[CMPRMAGICHEADLEN-1] = '\0'; // terminate with 0
+
+    /*
+     * repositioning at start and clearing eof, if it has occured
+     */
+    if (fseek( channel, cur_pos, SEEK_SET )) {
+      goto cmpr_checkmagic_error;
+    }
+
+    magictype = CmprUnCompressed;
+    if (bytes_read>0) {
+      /* check pattern in testbuf */
+      for (i=CmprGzipMagic;i<CmprEndMagic;i++) {
+        if (!cmpr_compare_magic( filehead, bytes_read, CMPRMAGIC[i] )) {
+          magictype = i;
+          break;
+        }
+      }
+    }
+  }
+
+  // translate CmprMagicType to DCompression
+  switch(magictype) {
+    case CmprUnCompressed: cmpr_method = UnCompressed; break;
+    case CmprGzipMagic:    cmpr_method = GzipCompression; break;
+    case CmprZMagic:       cmpr_method = ZCompression; break;
+    default: // CmprInValidMagic
+      cmpr_method = InValidDCompression;
+  }
+
+  return(cmpr_method);
+
+cmpr_checkmagic_error:
+
+  return(InValidDCompression);
+
+} /* cmpr_checkmagic */
