@@ -97,9 +97,12 @@ HISTORY
     01-Jun-2014 PB V2.47 edf_block_numbers
     15-Jun-2014 PB V2.48 DBClass and DBInstance removed,
                          edf_chain_number
+    06-Feb-2017 PB V2.61 enum DFType becomes public.
+
 ***************************************************************************/
 
 #ifndef EDFIO
+#  define EDFIO 
 
 /***************************************************************************
 * General Definitions                                                      *
@@ -119,6 +122,8 @@ HISTORY
 # include <math.h>
 
 # include "cmpr.h"
+# include "iodbg.h"
+# include "ioalloc.h"
 
 /***************************************************************************
 * Functions                                                                *
@@ -211,8 +216,11 @@ PUBLIC extern const long
                               int *pErrorValue, int *pstatus),
 *edf_chain_numbers          ( int stream,
                               int *pErrorValue, int *pstatus);
+PUBLIC extern const char
+*edf_errval2string          ( int ErrorValue ),
+*edf_report_data_error      ( int ErrorValue );
+
 PUBLIC extern char
-*edf_report_data_error      ( int ErrorValue ),
 *edf_dataformat_version     ( void );
 
 /***************************************************************************
@@ -246,7 +254,9 @@ edf_raster_order2number     ( const long order[] );
 PUBLIC extern const char
 * edf_datatype2string       ( int data_type ),
 * edf_byteorder2string      ( int byte_order ),
-* edf_compression2string    ( int data_compression );
+* edf_compression2string    ( int data_compression ),
+* edf_dftype2string         ( int dftype ),
+* edf_magictype2string      ( int magictype );
 
 PUBLIC extern void
 edf_bswap                   ( void * dest, const void * src, size_t item,
@@ -254,21 +264,50 @@ edf_bswap                   ( void * dest, const void * src, size_t item,
 edf_showdatatypes           ( int full ),
 edf_showmachinetypes        ( int full );
 
-/***************************************************************************
-* Debug routines                                                           *
-***************************************************************************/
-PUBLIC extern void
-edfio_debug                 ( int debug );
-
 PUBLIC extern int
-edf_print_filetable         ( FILE * out, int level, int verbose ),
-edf_dump                    ( FILE * out, int stream, int format, 
-                              char * chainkey[], char * blockkey[],
-                              char * keyword[] ); 
+edf_magicfiletype           ( FILE *channel );
+
+/***************************************************************************
+* Debug                                                                    *
+***************************************************************************/
+
+# define EDFIO_VERBOSE          0x1
+# define EDFIO_SHOWATT          0x2
+# define EDFIO_SHOWATTVAL       0x4
+# define EDFIO_SHOWDATA         0x8
+# define EDFIO_SHOWDATAVAL      0x10
+# define EDFIO_SHOWHEADER       0x20
+# define EDFIO_DEBUG1           0x40
+# define EDFIO_DEBUG2           0x80
+# define EDFIO_SHOWHIERARCHY    0x100
+# define EDFIO_COMPRESSION      0x200
+# define EDFIO_RASTER           0x400
+# define EDFIO_HISTORY          0x800
+# define EDFIO_HEADER           0x1000
+# define EDFIO_NUMUNITS         0x2000
+# define EDFIO_NUMCONSTANTS     0x4000
+# define EDFIO_NUMPROGRAMS      0x8000
 
 PUBLIC extern const char
 *edf_libversion             ( void ),
-*edfio_version              ( void );
+*edfio_version              ( void ),
+*edfio_usage2str            ( void );
+
+PUBLIC extern long
+edfio_debug_set             ( long debug ),
+edfio_debug                 ( void ),
+edfio_level_set             ( long level ),
+edfio_level                 ( void );
+
+PUBLIC extern int
+edfio_fprint_debug          ( FILE *out );
+
+PUBLIC extern int
+edf_print_filetable         ( FILE *out, long level, int verbose ),
+edf_dump                    ( FILE *out, int stream, int format,
+                              char *chainkey[], char *blockkey[],
+                              char *keyword[], char *str[],
+                              int domark, int dolabel );
 
 /***************************************************************************
 * History routines                                                         *
@@ -298,7 +337,7 @@ edf_history_copy            ( const char * history_key_copy,
                               const char * history_key ),
 edf_history_free            ( const char * history_key ),
 edf_history_print           ( FILE * out, const char * history_key,
-                              int level, int verbose );
+                              long level, int verbose );
 
 /***************************************************************************
 * Key Order routines                                                       *
@@ -318,6 +357,9 @@ edf_read_header             ( int stream, long DataNumber,
 edf_write_header            ( int stream, long DataNumber,
                               int DataChain, const char * header_key,
                               int * pErrorValue, int * pstatus ),
+edf_copy_header             ( const char *header_key_out, 
+                              const char *header_key_in,
+                              int * pErrorValue, int * pstatus ),
 edf_first_header_element    ( const char * header_key,
                               const char ** pkey, const char ** pvalue,
                               int * pErrorValue, int * pstatus ),
@@ -335,15 +377,49 @@ edf_delete_header_element   ( const char * header_key,
                               int * pErrorValue, int * pstatus ), 
 edf_free_header             ( const char * header_key ),
 edf_print_header            ( FILE * out, const char * header_key,
-                              int level, int verbose ),
+                              long level, int verbose ),
 edf_print_header_stat       ( FILE * out, const char * header_key,
-                              int level, int verbose );
+                              long level, int verbose );
 
 PUBLIC extern unsigned long 
 edf_stat_header_element     ( const char *header_key,
                               const char *key, const char *statkey,
                               int *pErrorValue, int *pstatus );
 
+PUBLIC extern char 
+
+*edf_keyvalue               ( char *buffer, size_t buflen, 
+                              const char *header_key,
+                              const char *string, long npar, int *pstatus );
+
+/***************************************************************************
+* H5 search list                                                           *
+***************************************************************************/
+PUBLIC extern const char 
+*edf_searchlists( void ),
+*edf_searchlistname( const char *searchlistname );
+
+/***************************************************************************
+* Data Format Types                                                        *
+***************************************************************************/
+PUBLIC enum DFType { InValidDFType, 
+                     EdfType, 
+                     BslType, 
+                     H5Type, 
+                     EndDFType };
+
+/***************************************************************************
+* Types of Magic Patterns                                                  *
+***************************************************************************/
+PUBLIC enum EdfMagicType { EdfInValidMagic,
+                           EdfEmptyMagic,
+                           EdfBslMagic,
+                           EdfH5Magic,
+                           Edf3Magic, Edf2Magic, Edf1Magic, Edf0Magic, EdfUMagic,
+                           EdfGzipMagic,
+                           EdfZMagic,
+                           EdfEndMagic };
+
 /***************************************************************************
 * Internal Parameter Definitions                                           *
 ***************************************************************************/
@@ -384,7 +460,7 @@ edf_byteorder            ( void );           /* return internal byteorder */
 * Error Values                                                             *
 ***************************************************************************/
 
-enum ErrorValues {
+PUBLIC enum ErrorValues {
 RoutineFailed = -1,
 RoutineSucceeded,
 CouldNotMallocMemory,
@@ -395,8 +471,11 @@ InvalidStream,
 CouldNotOpenFile,
 EndOfFileDetected,
 CompressionError,
+CouldNotFindChain,
+CouldNotFindBlock,
 CouldNotFindHeader,
 CouldNotFindSymbol,
+MissingKeyDefinition,
 BadSizeDefinition,
 BadDataBlock,
 CouldNotFindKeyword,
@@ -415,7 +494,6 @@ CouldNotCloseBlock,
 CouldNotInsertChain,
 CouldNotInsertBlock,
 CouldNotInsertSymbol,
-MissingKeyDefinition,
 GeneralBlockNotFirst,
 ErrorCreatingGeneralBlock,
 ErrorReadingGeneralBlock,
@@ -441,6 +519,7 @@ CouldNotReadHeader,
 CouldNotFindStatParamKey,
 CouldNotUpdateChainInfo,
 CouldNotUpdateFileInfo,
+CompiledWithoutH5Files,
 NoKeyGiven
 };
 
@@ -451,13 +530,13 @@ NoKeyGiven
 * Data Type Translation Tables                                             *
 ***************************************************************************/
 
-enum DType { InValidDType,
+PUBLIC enum DType { InValidDType,
              Unsigned8=1, Signed8,      Unsigned16,    Signed16,
              Unsigned32,  Signed32,     Unsigned64,    Signed64,
              FloatIEEE32, FloatIEEE64,  FloatIEEE128,  Unused12,
              EndDType };
 
-enum MType { InValidMType,
+PUBLIC enum MType { InValidMType,
              MUnsignedChar=1,    MChar,       MUnsignedShort, MShort,
              MUnsignedInteger,   MInteger,    MUnsignedLong,  MLong,
              MFloat,             MDouble,     MLongDouble,
@@ -467,7 +546,7 @@ enum MType { InValidMType,
 * Byte Order Translation Tables                                            *
 ***************************************************************************/
 
-enum BOrder { InValidBOrder, 
+PUBLIC enum BOrder { InValidBOrder, 
               LowByteFirst=1, HighByteFirst, 
               EndBOrder };
 
