@@ -226,7 +226,7 @@ char * filename_parameter ( char * buffer, size_t buflen,
 
   if (parno<-1) return ( (char *) NULL ); // parameter not available
 
-  // copy pattern into temporary buffer
+  // copy pattern to temporary buffer
   len = MIN(BUFLEN,strlen(pattern)+1)-1;
   (void) strncpy (Temp, pattern, len+1);
   Temp[len] = '\0'; // terminating zero
@@ -579,7 +579,7 @@ char * filename_name ( char * buffer, size_t buflen, const char * pattern )
 /*+++------------------------------------------------------------------------
 NAME
 
-   ext_split --- split string into body and extension. 
+   ext_split --- split string in body and extension. 
 
 SYNOPSIS
 
@@ -587,7 +587,7 @@ SYNOPSIS
 
 DESCRIPTION
    This function searches the last character c in pattern and copies 
-   the string before (mode=0) or from c (mode=1) into buffer.
+   the string before (mode=0) or from c (mode=1) to buffer.
    If pattern does not contain c mode 0 returns pattern and mode 1
    returns an empty string.
    <pattern> = <buffer mode 0><buffer mode 1>
@@ -864,7 +864,7 @@ char * filename_pattern ( char * buffer, size_t buflen,
 
   if (pattern == (char *) NULL) return ( (char *) NULL );
 
-  // copy pattern into temporary buffer
+  // copy pattern to temporary buffer
   len = MIN(BUFLEN,strlen(pattern)+1)-1;
   (void) strncpy (Temp, pattern, len+1);
   Temp[len] = '\0';
