@@ -23,7 +23,7 @@
  *   If not, see <http://www.gnu.org/licenses/>.
  */
 
-# define ISOTIME_VERSION      "isotime : V1.8 Peter Boesecke 2015-12-08"
+# define ISOTIME_VERSION      "isotime : V1.12 Peter Boesecke 2018-02-23"
 /*+++------------------------------------------------------------------------
 NAME
    isotime --- routines for isotime conversion 
@@ -33,32 +33,38 @@ SYNOPSIS
    # include isotime.h
 
 HISTORY
-  2006-05-30 V1.0 Peter Boesecke
-  2006-06-05 V1.1 PB
-  2007-04-19 V1.2 PB code corrected to avoid compiler warnings with -Wall
-  2010-05-27 V1.3 PB trim: unsigned long -> long, otherwise the condition
-                           i>=0 in for loop would always be TRUE
-  2010-12-12 V1.4 PB _convert2epoch: all epoch value preset
-  2012-10-27 V1.5 PB cpo: skip leading white spaces,
-                     shiftepoch added
-  2012-10-28 V1.6 PB isotime_mode, isotime_str2mode, isotime_mode2str
-  2015-12-07 V1.7 PB toupper, isdigit and isspace replaced with the macros
-                     TOUPPER, ISDIGIT and ISSPACE.
-  2015-12-08 V1.8 PB epoch2isotime: 
-                     fract (unused-but-set-variable) commented
+  2006-05-30 V1.0  Peter Boesecke
+  2006-06-05 V1.1  PB
+  2007-04-19 V1.2  PB code corrected to avoid compiler warnings with -Wall
+  2010-05-27 V1.3  PB trim: unsigned long -> long, otherwise the condition
+                            i>=0 in for loop would always be TRUE
+  2010-12-12 V1.4  PB _convert2epoch: all epoch value preset
+  2012-10-27 V1.5  PB cpo: skip leading white spaces,
+                      epoch_add added
+  2012-10-28 V1.6  PB isotime_mode, isotime_str2mode, isotime_mode2str
+  2015-12-07 V1.7  PB toupper, isdigit and isspace replaced with the macros
+                      TOUPPER, ISDIGIT and ISSPACE.
+  2015-12-08 V1.8  PB epoch2isotime: 
+                      fract (unused-but-set-variable) commented
+  2017-05-21 V1.9  PB ioalloc
+  2017-10-08 V1.10 PB added function: epoch_status, delta2epoch
+  2018-02-09 V1.11 PB epoch2string: epoch_s must not start with a space
+                      => " " replaced with "+"
+  2018-02-23 V1.12 PB _string2epoch: reading sign (otherwise error if -1<epoch<0),
+                      _string2epoch: missing 'else' corrected (error reading
+                       fractions, now obsolete with sign)
 
 PUBLIC extern
        IsotimeEpoch isotime2epoch( const char * isotime_s ),
-       const char * epoch2isotime( char buffer[], size_t buflen,
+       const char  *epoch2isotime( char buffer[], size_t buflen,
                                    IsotimeEpoch epoch );
 
 LIMITATION
-  earliest date: 1901-12-13T20:45:52.000000+0000
-    latest date: 2038-01-19T03:14:07.999999+0000
+  Considering only Gregorian leap years.
+  earliest date: 1901-12-13T20:45:52.000000+0000 (for 32 bit long integers)
+    latest date: 2038-01-19T03:14:07.999999+0000 (for 32 bit long integers)
 
-  To extend this range the structure IsotimeEpoch needs to be extended. It
-  would be possible to add the number of days since 1970-01-01. This would
-  increase the time to +-~5e6 years.
+  The time range is extended when using 64 bit long integers.
 
   isotime modes: 
     IsotimeModeNoSpace: write isotime string without spaces (default), e.g.
@@ -453,6 +459,35 @@ cpoerr : buffer[0]='\0'; return( s );
 
 } // cpo
 
+/*+++------------------------------------------------------------------------
+NAME
+
+  _epoch_init --- initializes the IsotimeEpoch structure.
+
+SYNOPSIS
+
+  IsotimeEpoch _epoch_init( void );
+
+DESCRIPTION
+  Initializes the IsotimeEpoch structure.
+
+RETURN VALUE
+
+  Initializyed IsotimeEpoch structure.
+
+----------------------------------------------------------------------------*/
+IsotimeEpoch _epoch_init( void )
+{ IsotimeEpoch epoch;
+
+  epoch.status=0;
+  epoch.sec=(long) 0;
+  epoch.fract=(double) 0.0;
+  epoch.offset=(long) 0;
+
+  return(epoch);
+
+} // _epoch_init
+
 /*--------------------------------------------------------------------------*/
 /* _convert2epoch: Convert to IsotimeEpoch,                                 */
 /* returns epoch.status!=0 in case of an error                              */
@@ -461,14 +496,10 @@ IsotimeEpoch _convert2epoch(long year, long month, long day,
                             long hh, long mm, long ss, double uuuuuu, 
                             long osign, long Hh, long Mm, long Ss) 
 { long days, base;
-  IsotimeEpoch epoch;
+  IsotimeEpoch epoch=_epoch_init();
 
   if (ISOTIME_debug>0) fprintf(stderr,"_convert2epoch BEGIN\n");
 
-  epoch.offset = (long int) 0;
-  epoch.fract  = (double) 0.0;
-  epoch.sec    = (long int) 0;
-
   epoch.status = -1;
 
   days = year*365+leap_days(year) - DAYS_19700101;
@@ -536,7 +567,7 @@ IsotimeEpoch _isotime2epoch(const char * isotime_s)
   char timbuf[TLEN];
   char frabuf[FLEN];
   char offbuf[OLEN];
-  IsotimeEpoch epoch;
+  IsotimeEpoch epoch=_epoch_init();
 
   if (ISOTIME_debug>0) fprintf(stderr,"_isotime2epoch >>%s<< BEGIN\n",
     isotime_s);
@@ -599,30 +630,40 @@ IsotimeEpoch _isotime2epoch(const char * isotime_s)
 /*--------------------------------------------------------------------------*/
 IsotimeEpoch _string2epoch(const char *epoch_s)
 { long base;
-  IsotimeEpoch epoch;
+  IsotimeEpoch epoch=_epoch_init();
   char epobuf[EPOLEN], *pr;
   char secbuf[SLEN], *ps;
   char frabuf[SLEN], *pf;
 
+  int sign=1;
   long sec=0;
   double fract=0.0;
 
-  if (ISOTIME_debug>0) fprintf(stderr,"_string2epoch >>%s<<\n",epoch_s);
+  if (ISOTIME_debug>0) fprintf(stderr,"_string2epoch BEGIN >>%s<<\n",epoch_s);
 
   epoch.status = -1;
-  epoch.sec    = 0;
-  epoch.fract  = 0.0;
-  epoch.offset = 0;
+  //epoch.sec    = 0;
+  //epoch.fract  = 0.0;
+  //epoch.offset = 0;
 
   // trim isotime_s, convert to uppercase and copy to trimmed
   ps = trim( epobuf, EPOLEN, epoch_s );
   if (ISOTIME_debug>1) fprintf(stderr," trim returns >>%s<<\n",ps);
 
-  // copy seconds
-  pf = cps( secbuf, SLEN, ps );
-  if (ISOTIME_debug>1) fprintf(stderr," section >>%s<<\n",secbuf);
+  // get sign and skip it
+  if (ps) {
+    if (*ps=='-') {
+      sign=-1;
+      ps++;
+    }
+  }
+  if (ISOTIME_debug>1) fprintf(stderr," sign >>%d<<\n",sign);
 
-  // copy fraction
+  // copy seconds (no sign)
+  pf = cps( secbuf, SLEN, ps );
+  if (ISOTIME_debug>1) fprintf(stderr," seconds >>%s<<\n",secbuf);
+
+  // copy fraction (no sign)
   pr = cpf( frabuf, SLEN, pf );
   if (ISOTIME_debug>1) fprintf(stderr," fraction >>%s<<\n",frabuf);
 
@@ -637,15 +678,22 @@ IsotimeEpoch _string2epoch(const char *epoch_s)
   sscanf(secbuf,"%ld",&sec);
   sscanf(frabuf,"%lf",&fract);
 
-  epoch.sec = sec;
-  if (sec>=0) epoch.fract = fract;
-  epoch.fract = -fract;
+  epoch.sec = sec*sign;
+  epoch.fract = fract*sign;
+
+//++++  if (sec>=0) epoch.fract = fract;
+//++++  else epoch.fract = -fract;  // else corrected!!!
 
   // normalize fract and sec
   base = floor(epoch.fract);
   epoch.fract -= (double) base;
   epoch.sec   += base;
 
+  if (ISOTIME_debug>1) {
+    fprintf(stderr," epoch.sec >>%ld<<\n",epoch.sec);
+    fprintf(stderr," epoch.fract >>%lg<<\n",epoch.fract);
+  }
+
   epoch.status = 0;
 
   if (ISOTIME_debug>0) fprintf(stderr,"_string2epoch END\n");
@@ -653,7 +701,7 @@ IsotimeEpoch _string2epoch(const char *epoch_s)
   return ( epoch );
 
 _string2epocherr:
-  if (ISOTIME_debug>0) fprintf(stderr,"_string2epoch END\n");
+  if (ISOTIME_debug>0) fprintf(stderr,"_string2epoch ERROR\n");
   return ( epoch );
 
 } // _string2epoch
@@ -802,33 +850,69 @@ offerr: if (ISOTIME_debug>0) fprintf(stderr,"setoffset2epoch END\n");
 
 /*+++------------------------------------------------------------------------
 NAME
-  shiftepoch --- shifts epoch by shift 
+  delta2epoch --- converts a delta time in seconds to epoch
 
 SYNOPSIS
 
-  IsotimeEpoch shiftepoch(IsotimeEpoch epoch, IsotimeEpoch shift);
+  IsotimeEpoch delta2epoch ( double delta );
 
 DESCRIPTION
 
-  Adds shift to epoch. shift.offset is ignored.
+  Returns a relative epoch with delta seconds.
+
+RETURN VALUE
+
+  epoch
+
+----------------------------------------------------------------------------*/
+IsotimeEpoch delta2epoch ( double delta )
+{ IsotimeEpoch epoch=_epoch_init();
+
+  if (ISOTIME_debug>0) fprintf(stderr,"delta2epoch BEGIN\n");
+
+  // sec=1.234 =>  sec= 1.0; fract=(1.234-1.0) = 0.234
+  // sec=-1.234 => sec=-2.0; fract=(-1.234+2.0)= 0.766
+  // normalize delta
+  epoch.sec   = (long) floor(delta);
+  epoch.fract = delta-(double) epoch.sec;
+
+  epoch.status = 0;
+
+  if (ISOTIME_debug>0) fprintf(stderr,"delta2epoch END\n");
+
+  return(epoch);
+
+} // delta2epoch
+
+/*+++------------------------------------------------------------------------
+NAME
+  epoch_add --- shifts epoch by shift 
+
+SYNOPSIS
+
+  IsotimeEpoch epoch_add(IsotimeEpoch epoch, IsotimeEpoch shift);
+
+DESCRIPTION
+
+  Adds shift to epoch. shift.offset is not used. 
 
 RETURN VALUE
 
   updated epoch
 
 ----------------------------------------------------------------------------*/
-IsotimeEpoch shiftepoch(IsotimeEpoch epoch, IsotimeEpoch shift)
+IsotimeEpoch epoch_add(IsotimeEpoch epoch, IsotimeEpoch shift)
 { long base;
   char epobuf[EPOLEN];
 
-  if (ISOTIME_debug>0) fprintf(stderr,"shiftepoch BEGIN\n");
+  if (ISOTIME_debug>0) fprintf(stderr,"epoch_add BEGIN\n");
 
   if ( epoch.status ) goto shifterr;
   if ( shift.status ) goto shifterr;
 
   if (ISOTIME_debug>1) {
-    fprintf( stderr,"epoch >>%s<<, ", epoch2string(epobuf,EPOLEN,epoch));
-    fprintf( stderr,"shift >>%s<<\n", epoch2string(epobuf,EPOLEN,shift));
+    fprintf( stderr," epoch=%s,", epoch2string(epobuf,EPOLEN,epoch));
+    fprintf( stderr," shift=%s\n", epoch2string(epobuf,EPOLEN,shift));
   }
 
   epoch.sec   += shift.sec;
@@ -840,18 +924,21 @@ IsotimeEpoch shiftepoch(IsotimeEpoch epoch, IsotimeEpoch shift)
   epoch.sec   += base;
 
   if (ISOTIME_debug>1) {
-    fprintf( stderr,"result >>%s<<, ", epoch2string(epobuf,EPOLEN,epoch));
+    fprintf( stderr," result=%s\n", epoch2string(epobuf,EPOLEN,epoch));
   }
 
-  if (ISOTIME_debug>0) fprintf(stderr,"shiftepoch END\n");
+  if (ISOTIME_debug>0) fprintf(stderr,"epoch_add END\n");
 
   return ( epoch );
 
-shifterr: if (ISOTIME_debug>0) fprintf(stderr,"shiftepoch END\n");
+shifterr: 
+
+  if (ISOTIME_debug>0) fprintf(stderr,"epoch_add ERROR\n");
   epoch.status = -1;
+
   return ( epoch );
 
-} // shiftepoch
+} // epoch_add
 
 /*+++------------------------------------------------------------------------
 NAME
@@ -915,18 +1002,18 @@ NAME
 
 SYNOPSIS
 
-  const char * epoch2isotime( char buffer[], size_t buflen, IsotimeEpoch epoch )
+  const char *epoch2isotime( char buffer[], size_t buflen, IsotimeEpoch epoch )
 
 DESCRIPTION
 
-  (not implemented)
+  convert IsotimeEpoch to isotime string
 
 RETURN VALUE
 
   pointer to isotime string in buffer
 
 ----------------------------------------------------------------------------*/
-const char * epoch2isotime( char buffer[], size_t buflen, IsotimeEpoch epoch )
+const char *epoch2isotime( char buffer[], size_t buflen, IsotimeEpoch epoch )
 { char epobuf[EPOLEN]; 
   char osign_c;
   long year=0, month=0, day=0;
@@ -1053,7 +1140,7 @@ RETURN VALUE
    pointer to output string in buffer
 
 ----------------------------------------------------------------------------*/
-const char * epoch2string ( char buffer[], size_t buflen, IsotimeEpoch epoch )
+const char *epoch2string ( char buffer[], size_t buflen, IsotimeEpoch epoch )
 { 
   long base;
 
@@ -1067,7 +1154,7 @@ const char * epoch2string ( char buffer[], size_t buflen, IsotimeEpoch epoch )
     epoch.fract -= (double) base;
     epoch.sec   += base;
     if (epoch.sec>=0)
-      sprintf( buffer," %ld.%06ld",epoch.sec,(long) floor(epoch.fract*1e6+0.5));
+      sprintf( buffer,"+%ld.%06ld",epoch.sec,(long) floor(epoch.fract*1e6+0.5));
     else { // invert sign
       epoch.fract = -epoch.fract;
       epoch.sec   = -epoch.sec;
@@ -1137,13 +1224,33 @@ int isotime_str2mode ( const char *mode_s )
     else if (!strcmp(str,"space")) mode = IsotimeModeSpace;
     else mode = IsotimeModeInvalid;
 
-    free(str);
+    FREE(str);
   }
 
   return(mode);
 
 } // isotime_str2mode
 
+/*+++------------------------------------------------------------------------
+NAME
+  epoch_status --- return status of epoch
+
+SYNOPSIS
+
+  int epoch_status (IsotimeEpoch epoch);
+
+DESCRIPTION
+
+
+RETURN VALUE
+
+   status
+
+----------------------------------------------------------------------------*/
+int epoch_status (IsotimeEpoch epoch)
+{ return(epoch.status);
+} // epoch_status
+
 /*+++------------------------------------------------------------------------
 NAME
   isotime_version --- return version
