@@ -63,6 +63,14 @@ HISTORY
                       exist.
   2016-04-06 V1.04 PB project_1u added and included in project_1,
                       cnt checked, no updates if cnt==0
+  2016-06-18 V1.05 PB project_1u, project_2: In some case, e.g. if 
+                      ipolmin!=0, weight or varweight can be different 
+                      from zero, while line or varline are dummies. line 
+                      and varline are now checked for dummy values before 
+                      calculating an average.
+                      project_1u, project_2: a value of ipolmin 0 is used
+                      while these functions are running. It guarantees that 
+                      the full integration range contributes to the result.
 
 ***********************************************************************---*/
 
@@ -152,6 +160,11 @@ void project_1u        ( float *line, float *varline,
   float f_1;
   long  i, i0, i1;    // loop
   int   cnt; 
+  float ipolmin_cur;
+
+  // keep original Ipolmin in ipolmin_cur
+  ipolmin_cur = Ipolmin(-1);
+  Ipolmin(0); // do not use Ipolmin for Projection_1U
 
   i0 = MAX2(0,imin);
   i1 = MIN2(dim-1,imax);
@@ -185,23 +198,28 @@ void project_1u        ( float *line, float *varline,
         }
       }
 
-      if (mode==1) {
-        if (lineweight) {
-          if (fabs(lineweight[i])>eps) line[i] /= lineweight[i];
-          else line[i] = initvalue;
-        } else {
-          if (fabs(weight)>eps) line[i] /= weight;
-          else line[i] = initvalue;
-        }
+      if (mode==1) { // average and terminate
+        // if line[i] has not been updated it is still dummy
+        if ( NODUMMY( line[i], initvalue, dinitvalue) ) {
+          if (lineweight) {
+            if (fabs(lineweight[i])>eps) line[i] /= lineweight[i];
+            else line[i] = initvalue;
+          } else {
+            if (fabs(weight)>eps) line[i] /= weight;
+            else line[i] = initvalue;
+          }
 
-        if (varlineweight) {
-          if ( (varline[i]>=0)&&(fabs(varlineweight[i])>eps) )
-            varline[i] /= varlineweight[i]*varlineweight[i];
-          else varline[i] = VarDummy;
+          if (varlineweight) {
+            if ( (varline[i]>=0)&&(fabs(varlineweight[i])>eps) )
+              varline[i] /= varlineweight[i]*varlineweight[i];
+            else varline[i] = VarDummy;
+          } else {
+            if ( (varline[i]>=0)&&(fabs(varweight)>eps) )
+              varline[i] /= varweight*varweight;
+            else varline[i] = VarDummy;
+          }
         } else {
-          if ( (varline[i]>=0)&&(fabs(varweight)>eps) )
-            varline[i] /= varweight*varweight;
-          else varline[i] = VarDummy;
+          varline[i] = VarDummy;
         }
       }
 
@@ -215,13 +233,16 @@ void project_1u        ( float *line, float *varline,
         if (lineweight) lineweight[i] += weight;
       }
 
-      if (mode==1) {
-        if (lineweight) {
-          if (fabs(lineweight[i])>eps) line[i] /= lineweight[i];
-          else line[i] = initvalue;
-        } else {
-          if (fabs(weight)>eps) line[i] /= weight;
-          else line[i] = initvalue;
+      if (mode==1) { // average and terminate
+        // if line[i] has not been updated it is still dummy
+        if ( NODUMMY( line[i], initvalue, dinitvalue) ) {
+          if (lineweight) {
+            if (fabs(lineweight[i])>eps) line[i] /= lineweight[i];
+            else line[i] = initvalue;
+          } else {
+            if (fabs(weight)>eps) line[i] /= weight;
+            else line[i] = initvalue;
+          }
         }
       }
     }
@@ -230,6 +251,9 @@ void project_1u        ( float *line, float *varline,
 
   } // for
 
+  // reset Ipolmin
+  Ipolmin(ipolmin_cur);
+
 } /* project_1u */
 
 /*---------------------------------------------------------------------------
@@ -353,7 +377,13 @@ void project_2          ( float *line, float *varline, int dim,
   float f_2;
   long  i, i0, i1;    // loop
   int   cnt;
- 
+
+  float ipolmin_cur;
+
+  // keep original Ipolmin in ipolmin_cur
+  ipolmin_cur = Ipolmin(-1);
+  Ipolmin(0); // do not use Ipolmin for Projection_1U
+
   i0 = MAX2(0,imin);
   i1 = MIN2(dim-1,imax);
  
@@ -376,15 +406,16 @@ void project_2          ( float *line, float *varline, int dim,
       }
 
       if (ave) {
-        if (fabs(weight)>eps) line[i] /= weight;
-        else line[i] = initvalue;
-        if ( (varline[i]>=0)&&(fabs(varweight)>eps))
-          varline[i] /= varweight*varweight;
-        else varline[i] = VarDummy;
+        if (NODUMMY( line[i], dummy, ddummy) ) {
+          if (fabs(weight)>eps) line[i] /= weight;
+          else line[i] = initvalue;
+          if ( (varline[i]>=0)&&(fabs(varweight)>eps))
+            varline[i] /= varweight*varweight;
+          else varline[i] = VarDummy;
+        } else varline[i] = VarDummy;
       } 
 
     } else { 
-      // cnt = // unused-but-set-variable
       cnt = Isum2ldw ( data, dim_1, dim_2, dummy, ddummy,
                        f1_1, f_2,  f3_1, f_2+Df_2,
                        &sum, &weight);
@@ -393,14 +424,19 @@ void project_2          ( float *line, float *varline, int dim,
       }
 
       if (ave) {
-        if (fabs(weight)>eps) line[i] /= weight;
-        else line[i] = initvalue;
+        if (NODUMMY( line[i], dummy, ddummy) ) {
+          if (fabs(weight)>eps) line[i] /= weight;
+          else line[i] = initvalue;
+        }
       }
     }
  
     f_2+=Df_2;
  
   } // for
+
+  // reset Ipolmin
+  Ipolmin(ipolmin_cur);
  
 } /* project_2 */
 
