@@ -53,9 +53,12 @@ HISTORY
   2015-12-07 V1.7 PB isgraph -> ISGRAPH
   2015-12-08 V1.8 PB raster_order2str:
                      size_t psiz (unused-but-set-variable) commented
+  2017-02-12 V1.9 PB RN_debug => iodbg(IODBG_RASTER)
+  2017-03-03 V1.10 PB raster_debug changes only IODBG_RASTER flag.
+  2017-05-21 V1.11 PB ioalloc
 
 --------------------------------------------------------------------------*/
-# define RASTER_VERSION "raster : V1.8 Peter Boesecke 2015-12-08"
+# define RASTER_VERSION "raster : V1.11 Peter Boesecke 2017-05-21"
 /****************************************************************************
 *  Include                                                                  *
 ****************************************************************************/
@@ -83,15 +86,10 @@ HISTORY
 # define TOLOWER(s) (char)tolower((int)(s))
 # define TOUPPER(s) (char)toupper((int)(s))
 
-/****************************************************************************
-* Static Variables                                                          *
-****************************************************************************/
-static int RN_debug = 0;
-
 /*--------------------------------------------------------------------------
 NAME
 
-  raster_debug --- set / reset module raster_conversion into debug mode
+  raster_debug --- set / reset module raster_conversion to debug mode
 
 SYNOPSIS
 
@@ -99,11 +97,15 @@ SYNOPSIS
 
 DESCRPTION
 
-  Writes ´debug´ into RN_debug.
+  Writes ´debug´ to iodbg.
 
 --------------------------------------------------------------------------*/
-void raster_debug ( int debug )
-{ RN_debug = debug;
+void raster_debug ( long debug )
+{ if (debug) {
+    iodbg_debug_or( IODBG_RASTER );
+  } else {
+    iodbg_debug_and( ~IODBG_RASTER );
+  }
 } /* raster_debug */
 
 /*--------------------------------------------------------------------------
@@ -349,14 +351,14 @@ long * raster_number2order ( long * buffer, size_t nelem,
     output = buffer;
   } else {
     /* allocate output buffer and clear it */
-    if ( !(output_buffer = (long*) calloc( (N+1), sizeof(long) )) ) 
+    if ( !(output_buffer = (long*) CALLOC( (N+1), sizeof(long) )) ) 
       goto raster_number2order_error;
     output = output_buffer;
   }
 
   if (inverse) {
     /* allocate temporary buffer and clear it */
-    if ( !(internal_buffer = (long*) calloc( (N+1), sizeof(long) )) )
+    if ( !(internal_buffer = (long*) CALLOC( (N+1), sizeof(long) )) )
       goto raster_number2order_error;
     order = internal_buffer;
   } else order = output;
@@ -393,15 +395,15 @@ long * raster_number2order ( long * buffer, size_t nelem,
 
   if (inverse) order = raster_order_inversion ( output, nelem, order );
 
-  if (internal_buffer) free( internal_buffer );
+  FREE( internal_buffer );
 
   return( order );
 
 raster_number2order_error:
 
-  if (internal_buffer) free( internal_buffer );
+  FREE( internal_buffer );
 
-  if (output_buffer) free( output_buffer );
+  FREE( output_buffer );
 
   return( NULL );
 
@@ -513,7 +515,7 @@ long * raster_order_dimension ( long * buffer, size_t nelem,
   long * dest_dim=NULL;
   long i, I, N = 0l;
 
-  if (RN_debug) printf(" raster_order_dimension BEGIN\n");
+  if (iodbg(IODBG_RASTER)) printf(" raster_order_dimension BEGIN\n");
 
   if (!(src_dim&&src_order)) {
     fprintf(stderr,"%s src_dim or src_order is NULL\n",ROD_Error);
@@ -532,7 +534,7 @@ long * raster_order_dimension ( long * buffer, size_t nelem,
 
   if (!(buffer)) {
     /* allocate array and clear it */
-    if ( !(dest_dim = (long*) calloc( (N+1), sizeof(long) )) ) {
+    if ( !(dest_dim = (long*) CALLOC( (N+1), sizeof(long) )) ) {
       fprintf(stderr,"%s buffer allocation error\n",ROD_Error);
       errval=3;
       goto raster_order_dimension_error;
@@ -571,15 +573,15 @@ long * raster_order_dimension ( long * buffer, size_t nelem,
 
   if (perrval) *perrval = errval;
 
-  if (RN_debug) printf(" raster_order_dimension END\n");
+  if (iodbg(IODBG_RASTER)) printf(" raster_order_dimension END\n");
 
   return( buffer );
 
 raster_order_dimension_error:
 
-  if (dest_dim) free(dest_dim);
+  FREE(dest_dim);
   if (perrval) *perrval = errval;
-  if (RN_debug) printf(" raster_order_dimension END (errval=%d)\n",errval);
+  if (iodbg(IODBG_RASTER)) printf(" raster_order_dimension END (errval=%d)\n",errval);
   return( NULL );
 
 } /* raster_order_dimension */
@@ -988,7 +990,7 @@ int raster_normalization ( void * dest, const void * src,
   int status=0;
   int errval=0;
 
-  if (RN_debug) printf(" raster_normalization BEGIN\n");
+  if (iodbg(IODBG_RASTER)) printf(" raster_normalization BEGIN\n");
 
   if (!data_dim) {
     fprintf(stderr,"SEVERE %s Data dimension pointer missing!",RN_Error);
@@ -1013,21 +1015,21 @@ int raster_normalization ( void * dest, const void * src,
   if ( raster_order_normalization ( dest, src, data_dim, order, item, &errval ) )
     goto raster_normalization_error;
 
-  if (order) free ( order );
+  FREE ( order );
 
   if (perrval) *perrval=errval;
 
-  if (RN_debug) printf(" raster_normalization END\n");
+  if (iodbg(IODBG_RASTER)) printf(" raster_normalization END\n");
 
   return ( status );
 
 raster_normalization_error:
 
-  if (order) free ( order );
+  FREE ( order );
 
   if (perrval) *perrval=errval;
 
-  if (RN_debug) printf(" raster_normalization END (errval=%d)\n",errval);
+  if (iodbg(IODBG_RASTER)) printf(" raster_normalization END (errval=%d)\n",errval);
 
   return ( -1 );
 
@@ -1092,7 +1094,7 @@ int raster_order_normalization ( void * dest, const void * src,
 
   int errval=0;
 
-  if (RN_debug) printf(" raster_order_normalization BEGIN\n");
+  if (iodbg(IODBG_RASTER)) printf(" raster_order_normalization BEGIN\n");
 
   if (!data_dim) {
     fprintf(stderr,"SEVERE %s Data dimension pointer missing!",RON_Error);
@@ -1124,7 +1126,7 @@ int raster_order_normalization ( void * dest, const void * src,
 
   // raster_number 1 is calculated without dimension limit 
   if ( ( src==dest ) && ( raster_order2number( order )==1l ) ) {
-    if (RN_debug) {
+    if (iodbg(IODBG_RASTER)) {
       printf("         raster_number = %ld\n", 1l);
       printf("         src and dest arrays are identical\n");
       printf("         normalization not required\n");
@@ -1137,10 +1139,10 @@ int raster_order_normalization ( void * dest, const void * src,
     used_size = item * (size_t) n_element;
 
     if ( ovlpchk( dest, used_size, src, used_size ) ) {
-      if (RN_debug) 
+      if (iodbg(IODBG_RASTER)) 
         printf(" raster_order_normalization: creating temporory copy \n");
 
-      if ( !(tmp_src = malloc( used_size )) ) {
+      if ( !(tmp_src = MALLOC( used_size )) ) {
         fprintf(stderr,"%s IS %s\n",RON_Error, RON_NoMem);
         goto raster_order_normalization_error;
       }
@@ -1152,27 +1154,27 @@ int raster_order_normalization ( void * dest, const void * src,
     }
 
     /* calculate loop parameters */
-    if (!(IS = (long *) malloc ( sizeof(long) * (n+2) )) ) {
+    if (!(IS = (long *) MALLOC ( sizeof(long) * (n+2) )) ) {
       errval=3;
       fprintf(stderr,"%s IS %s\n",RON_Error, RON_NoMem); 
       goto raster_order_normalization_error;
     }
-    if (!(IInc = (long *) malloc ( sizeof(long) * (n+2) )) ) {
+    if (!(IInc = (long *) MALLOC ( sizeof(long) * (n+2) )) ) {
       errval=3;
       fprintf(stderr,"%s IInc %s\n",RON_Error, RON_NoMem); 
       goto raster_order_normalization_error;
     }
-    if (!(ILoop = (long *) malloc ( sizeof(long) * (n+2) )) ) {
+    if (!(ILoop = (long *) MALLOC ( sizeof(long) * (n+2) )) ) {
       errval=3;
       fprintf(stderr,"%s ILoop %s\n",RON_Error, RON_NoMem); 
       goto raster_order_normalization_error;
     }
-    if (!(IWrap = (long *) malloc ( sizeof(long) * (n+2) )) ) {
+    if (!(IWrap = (long *) MALLOC ( sizeof(long) * (n+2) )) ) {
       errval=3;
       fprintf(stderr,"%s IWrap %s\n",RON_Error, RON_NoMem); 
       goto raster_order_normalization_error;
     }
-    if (!(I = (long *) malloc ( sizeof(long) * (n+2) )) ) {
+    if (!(I = (long *) MALLOC ( sizeof(long) * (n+2) )) ) {
       errval=3;
       fprintf(stderr,"%s I %s\n",RON_Error, RON_NoMem); 
       goto raster_order_normalization_error;
@@ -1195,7 +1197,7 @@ int raster_order_normalization ( void * dest, const void * src,
       return(-1);
     }
 
-    if (RN_debug) {
+    if (iodbg(IODBG_RASTER)) {
       // raster_number is only used as debug info for n<=MAX_RASTER_NUMBER_DIMENSION
       raster_number = raster_order2number ( order );
 
@@ -1213,26 +1215,25 @@ int raster_order_normalization ( void * dest, const void * src,
     ps = src;
     reorder_raster ( n, dest, &ps, item, I, IS, ILoop, IInc, IWrap );
 
-    free ( I ); free ( IWrap ); free ( ILoop ); free ( IInc ); free ( IS );
-    if (tmp_src) free (tmp_src);
+    FREE ( I ); FREE ( IWrap ); FREE ( ILoop ); FREE ( IInc ); FREE ( IS );
+    FREE (tmp_src);
 
   } // if ( src==dest ) ...
 
   if (perrval) *perrval=errval;
 
-  if (RN_debug) printf(" raster_order_normalization END\n");
+  if (iodbg(IODBG_RASTER)) printf(" raster_order_normalization END\n");
 
   return(0);
 
 raster_order_normalization_error:
 
-  if (I) free ( I ); if (IWrap) free ( IWrap ); if (ILoop) free ( ILoop ); 
-  if (IInc) free ( IInc ); if (IS) free ( IS );
-  if (tmp_src) free (tmp_src);
+  FREE ( I ); FREE ( IWrap ); FREE ( ILoop ); FREE ( IInc ); FREE ( IS );
+  FREE (tmp_src);
 
   if (perrval) *perrval=errval;
 
-  if (RN_debug) printf(" raster_order_normalization END (errval=%d)\n",errval);
+  if (iodbg(IODBG_RASTER)) printf(" raster_order_normalization END (errval=%d)\n",errval);
 
   return(-1);
 
@@ -1298,7 +1299,7 @@ long * raster_order_multiplication (long *buffer, size_t nelem,
     for ( i=0;i<=N;i++ ) y_order[i]=0l;
   } else {
     /* allocate array and clear it */
-    if ( !(y_order = (long*) calloc( (N+1), sizeof(long) )) )
+    if ( !(y_order = (long*) CALLOC( (N+1), sizeof(long) )) )
       goto raster_order_multiplication_error;
   }
   y_order[0] = N;
@@ -1315,7 +1316,7 @@ long * raster_order_multiplication (long *buffer, size_t nelem,
 
 raster_order_multiplication_error:
 
-  if ( (y_order)&&(!buffer) ) free( y_order );
+  if ( (y_order)&&(!buffer) ) FREE( y_order );
 
   return( NULL );
 
@@ -1363,9 +1364,9 @@ long raster_multiplication ( long a, long x )
   
   value = raster_order2number ( y_order );
 
-  if (a_order) free ( a_order );
-  if (x_order) free ( x_order );
-  if (y_order) free ( y_order );
+  FREE ( a_order );
+  FREE ( x_order );
+  FREE ( y_order );
 
   return ( value );
 
@@ -1426,7 +1427,7 @@ long * raster_order_inversion ( long *buffer, size_t nelem,
     for ( i=0;i<=N;i++ ) a_order[i]=0l;
   } else {
     /* allocate array and clear it */
-    if ( !(a_order = (long*) calloc( (N+1), sizeof(long) )) )
+    if ( !(a_order = (long*) CALLOC( (N+1), sizeof(long) )) )
       goto raster_order_inversion_error;
   }
   a_order[0] = N;
@@ -1443,7 +1444,7 @@ long * raster_order_inversion ( long *buffer, size_t nelem,
 
 raster_order_inversion_error:
 
-  if ( (a_order)&&(!buffer) ) free( a_order );
+  if ( (a_order)&&(!buffer) ) FREE( a_order );
 
   return( NULL );
 
@@ -1485,7 +1486,7 @@ long raster_order_determinante ( const long order[] )
   N=order[0];
 
   // duplicate order array
-  if ( !(order_copy=malloc(sizeof(long)*(N+1))) )
+  if ( !(order_copy=MALLOC(sizeof(long)*(N+1))) )
     goto order_determinante_error;
   for (l=0;l<=N;l++) order_copy[l]=order[l];
 
@@ -1512,13 +1513,13 @@ long raster_order_determinante ( const long order[] )
     }
   }
 
-  free(order_copy);
+  FREE(order_copy);
 
   return(determinante);
 
 order_determinante_error:
 
-  if (order_copy) free(order_copy);
+  FREE(order_copy);
 
   return(0l);
 
@@ -1560,8 +1561,8 @@ long raster_inversion ( long x )
 
   value = raster_order2number ( a_order );
 
-  if (a_order) free ( a_order );
-  if (x_order) free ( x_order );
+  FREE ( a_order );
+  FREE ( x_order );
 
   return ( value );
 
@@ -1664,13 +1665,13 @@ long * raster_str2order( long * buffer, size_t nelem,
     }
     order=buffer;
   } else {
-    if ( !(order = (long*)  calloc( n+1l, sizeof(long) )) ) {
+    if ( !(order = (long*)  CALLOC( n+1l, sizeof(long) )) ) {
       errval=1;  // allocation error
       goto raster_string2order_error;
     }
   }
 
-  if ( !(index = (long*)  calloc( n+1l, sizeof(long) )) ) {
+  if ( !(index = (long*)  CALLOC( n+1l, sizeof(long) )) ) {
     errval=2;  // allocation error
     goto raster_string2order_error;
   }
@@ -1753,7 +1754,7 @@ long * raster_str2order( long * buffer, size_t nelem,
     goto raster_string2order_error;
   }
 
-  free( index );
+  FREE( index );
 
   if (tail) *tail = ps;
 
@@ -1765,9 +1766,9 @@ raster_string2order_error:
 
   // String could not be read
 
-  if ( (order)&&(!buffer) ) free( order );
+  if ( (order)&&(!buffer) ) FREE( order );
 
-  if (index) free( index );
+  FREE( index );
 
   if (tail) *tail = str; // the whole string could not be read
 
@@ -1852,7 +1853,7 @@ long raster_str2number( long n, const char *str, const char **tail, int *perrval
 
   raster_number = raster_order2number ( order );
 
-  free(order);
+  FREE(order);
 
   if (perrval) *perrval=errval;
 
@@ -1862,7 +1863,7 @@ raster_str2number_error:
 
   if (perrval) *perrval=errval;
 
-  if (order) free(order);
+  FREE(order);
 
   return(0l);
 
@@ -1871,7 +1872,7 @@ raster_str2number_error:
 /*--------------------------------------------------------------------------
 NAME
 
-  raster_order2str --- writes order into a comma separated string 
+  raster_order2str --- writes order to a comma separated string 
 
 SYNOPSIS
 
@@ -1931,7 +1932,7 @@ char * raster_order2str( char * buffer, size_t nelem,
     value=buffer;
     valsiz=nelem;
   } else {
-    if ( !(value = (char*)  malloc((siz+1l)*sizeof(char)) ) ) {
+    if ( !(value = (char*)  MALLOC((siz+1l)*sizeof(char)) ) ) {
       errval=1;  // allocation error
       goto raster_order2str_error;
     }
@@ -1961,8 +1962,7 @@ raster_order2str_error:
 
   // String could not be created
 
-//  if ( (order)&&(!buffer) ) free( order ); ++++++++??????????????
-  if ( (value)&&(!buffer) ) free( value );
+  if ( (value)&&(!buffer) ) FREE( value );
 
   if (perrval) *perrval=errval;
 
@@ -1982,7 +1982,7 @@ SYNOPSIS
 
 DESCRIPTION
 
-  Transforms a configuration order array for N dimensions into a NxN matrix
+  Transforms a configuration order array for N dimensions to a NxN matrix
   that can be used for coordinate transformations.
 
 ARGUMENTS
@@ -2014,7 +2014,7 @@ double * raster_order2matrix ( double * buffer, size_t nelem,
   double *Out = (double *) NULL, *pOut;
   double sign;
 
-  if (RN_debug) printf(" raster_order2matrix BEGIN\n");
+  if (iodbg(IODBG_RASTER)) printf(" raster_order2matrix BEGIN\n");
 
   if (!order) {
     fprintf(stderr,"%s %s\n",R2M_Error,R2M_NoOrder);
@@ -2031,7 +2031,7 @@ double * raster_order2matrix ( double * buffer, size_t nelem,
     Out=buffer;
   } else {
     /* allocate array */
-    if ( !(Out = (double*) malloc( sizeof(double) * (N*N) )) )
+    if ( !(Out = (double*) MALLOC( sizeof(double) * (N*N) )) )
       goto raster_order2matrix_error;
   }
 
@@ -2054,15 +2054,15 @@ double * raster_order2matrix ( double * buffer, size_t nelem,
     Out[l*N+n] = sign; // Out[l][n]
   }
 
-  if (RN_debug) printf(" raster_order2matrix END\n");
+  if (iodbg(IODBG_RASTER)) printf(" raster_order2matrix END\n");
 
   return( Out );
 
 raster_order2matrix_error:
 
-  if (Out) free( Out );
+  FREE( Out );
 
-  if (RN_debug) printf(" raster_order2matrix END (error)\n");
+  if (iodbg(IODBG_RASTER)) printf(" raster_order2matrix END (error)\n");
 
   return( NULL );
 
@@ -2103,7 +2103,7 @@ double * raster_matrix_product ( double * buffer, size_t nelem,
   double *Out = (double *) NULL;
   int l, m, n;
 
-  if (RN_debug) printf(" raster_matrix_product BEGIN\n");
+  if (iodbg(IODBG_RASTER)) printf(" raster_matrix_product BEGIN\n");
 
   if (buffer) {
     if (nelem<L*N) {
@@ -2113,7 +2113,7 @@ double * raster_matrix_product ( double * buffer, size_t nelem,
     Out=buffer;
   } else {
     /* allocate array */
-    if ( !(Out = (double*) malloc( sizeof(double) * (L*N) )) )
+    if ( !(Out = (double*) MALLOC( sizeof(double) * (L*N) )) )
       goto raster_matrix_product_error;
   }
 
@@ -2124,15 +2124,15 @@ double * raster_matrix_product ( double * buffer, size_t nelem,
         Out[l*N+n]+=A[m*N+n]*B[l*M+m]; // Out[l][n] += A[m][n]*B[l][m]
     }
 
-  if (RN_debug) printf(" raster_matrix_product END\n");
+  if (iodbg(IODBG_RASTER)) printf(" raster_matrix_product END\n");
 
   return( Out );
 
 raster_matrix_product_error:
 
-  if ( Out ) free ( Out );
+  FREE ( Out );
 
-  if (RN_debug) printf(" raster_matrix_product END (error)\n");
+  if (iodbg(IODBG_RASTER)) printf(" raster_matrix_product END (error)\n");
 
   return( NULL );
 
@@ -2166,7 +2166,7 @@ RETURN VALUE
 int raster_fprint_matrix( FILE *out, size_t nrows, size_t ncols,
                           double M[], const char * label )
 { size_t row, col;
-  if (RN_debug) printf(" raster_fprint_matrix BEGIN\n");
+  if (iodbg(IODBG_RASTER)) printf(" raster_fprint_matrix BEGIN\n");
   // show data
   fprintf(out,"%14s \n",label);
   for (row=0;row<nrows;row++) {
@@ -2176,7 +2176,7 @@ int raster_fprint_matrix( FILE *out, size_t nrows, size_t ncols,
   }
   printf("\n");
 
-  if (RN_debug) printf(" raster_fprint_matrix END\n");
+  if (iodbg(IODBG_RASTER)) printf(" raster_fprint_matrix END\n");
   return(0);
 
 } // raster_fprint_matrix
