@@ -23,7 +23,7 @@
  *   If not, see <http://www.gnu.org/licenses/>.
  */
 
-# define READASCII_VERSION "readascii : V1.3 2016-01-12 Peter Boesecke"
+# define READASCII_VERSION "readascii : V1.4 2017-05-21 Peter Boesecke"
 /*+++***********************************************************************
 NAME
 
@@ -85,6 +85,7 @@ HISTORY
   2015-12-07 V1.2 PB RA_rm_descr:
                      verbose (unused-but-set-variable) commented
   2016-01-12 V1.3 PB format converter SIZE_T renamed to SIZE_F
+  2017-05-21 V1.4 PB ioalloc
 
 ***************************************************************************/
 
@@ -141,7 +142,7 @@ PUBLIC const char * RA_version ( void )
 char * RA_newstr( const char * string )
 { char * newstring;
 
-  if (!(newstring = (char *) malloc(strlen(string)+1))) return((char *) NULL);
+  if (!(newstring = (char *) MALLOC(strlen(string)+1))) return((char *) NULL);
   (void) strcpy(newstring,string);
 
   return( newstring );
@@ -208,7 +209,7 @@ int RA_descr_new  ( RADescr ** proot, const char * key, RADescr ** pdescr )
 
   if (notfound) {
     /* create new hlist */
-    if (!(newhlist = (RADescr *) malloc(sizeof(RADescr)))) return(-1);
+    if (!(newhlist = (RADescr *) MALLOC(sizeof(RADescr)))) return(-1);
     newhlist->Key = RA_newstr(key); if (!newhlist->Key) return(-1);
 
     /* insert newhlist before next */
@@ -264,14 +265,14 @@ int RA_descr_remove  ( RADescr ** proot, RADescr * descr )
       else *proot = next;
 
     // release memory of descr
-    if (current->filename) free(current->filename);
-    if (current->Key) free(current->Key);
+    FREE(current->filename);
+    FREE(current->Key);
 
-    if (current->buffer) free( current->buffer );
-    if (current->delimiterset) free( current->delimiterset );
-    if (current->commentset) free( current->commentset );
+    FREE( current->buffer );
+    FREE( current->delimiterset );
+    FREE( current->commentset );
 
-    free ( current );
+    FREE ( current );
 
   }
 
@@ -346,7 +347,7 @@ int RA_printdescr( FILE * out, RADescr * descr )
 
 /*+++------------------------------------------------------------------------
 NAME
-  RA_ReadElement --- copies the next element into buffer
+  RA_ReadElement --- copies the next element to buffer
 
 SYNOPSIS
 
@@ -357,7 +358,7 @@ SYNOPSIS
 DESCRIPTION
 
   Leading and trailing delimiters are skipped. It reads the next element from 
-  'in', copies it into buffer and positions the file pointer at the start of 
+  'in', copies it to buffer and positions the file pointer at the start of 
   the next element or at the first character after a trailing new line. If 
   buflen is 0, the next element is just skipped and buffer is not filled. 
   In this case buffer is not accessed. In both cases the pointer to buffer 
@@ -710,7 +711,7 @@ PUBLIC long RA_readfile( RADescr * descr, float ** pdata, int * pstatus )
   
   bufsiz = sizeof(float)*maxcnt;
   if (bufsiz) {
-    descr->buffer = (float *) malloc( bufsiz );
+    descr->buffer = (float *) MALLOC( bufsiz );
     if (descr->buffer) {
       descr->bufsiz = bufsiz;
       if (pdata) *pdata=descr->buffer;
@@ -823,7 +824,7 @@ PUBLIC int RA_setdummy( RADescr * descr, float dummy )
 
 PUBLIC int RA_setdelimiterset( RADescr * descr, const char *delimiterset )
 { if (delimiterset) {
-    if (descr->delimiterset) free(descr->delimiterset);
+    FREE(descr->delimiterset);
     descr->delimiterset = RA_newstr(delimiterset);
     if (!(descr->delimiterset)) return(1);
   }
@@ -832,7 +833,7 @@ PUBLIC int RA_setdelimiterset( RADescr * descr, const char *delimiterset )
 
 PUBLIC int RA_setcommentset( RADescr * descr, const char *commentset )
 { if (commentset) {
-    if (descr->commentset) free(descr->commentset);
+    FREE(descr->commentset);
     descr->commentset = RA_newstr(commentset);
     if (!(descr->commentset)) return(1);
   }
