@@ -59,6 +59,8 @@ DESCRIPTION
 # include <fcntl.h>
 # include <math.h>
 # include <float.h>
+# include "ioalloc.h"
+# include "numio.h"
 
 /***************************************************************************
 * Functions                                                                *
@@ -78,16 +80,41 @@ PUBLIC extern int
   // returns 1 if s either NULL, has length 0 or contains only white spaces
   strlib_is_empty( const char *s ),
 
+  // returns 1 if str is a comma separated list encapsulated in brackets
+  strlib_is_list( const char *str ),
+
   // splits s at each white space and write pointers to sv[]
   strlib_split ( char *sv[], int sc, char * s ),
 
   // non case sensitive comparison (compatible to strncasecmp)
-  strlib_ncasecmp(const char *s1, const char *s2, size_t n);
+  strlib_ncasecmp(const char *s1, const char *s2, size_t n),
+
+  // non case sensitive comparison (compatible to strcasecmp)
+  strlib_casecmp(const char *s1, const char *s2),
+
+  // case sensitive comparison (compatible to strncmp)
+  strlib_ncmp(const char *s1, const char *s2, size_t n),
+
+  // case sensitive comparison (compatible to strcmp)
+  strlib_cmp(const char *s1, const char *s2),
+
+  // case sensitive comparison with wildcard
+  strlib_wildcmp( const char *key1, const char *wild ),
+
+  // "natural" string comparison
+  strlib_numcmp(const char *s1, const char *s2 ),
+
+  // "natural" non case sensitve string comparison
+  strlib_numcasecmp(const char *s1, const char *s2 );
+
 
 PUBLIC size_t 
 
   // search a string for a set of characters
-  strlib_spn(const char *s, const char *accept);
+  strlib_spn(const char *s, const char *accept),
+
+  // get the number of occurences of c in str
+  strlib_c_count(const char *str, int c );
 
 PUBLIC extern char
 
@@ -121,20 +148,40 @@ PUBLIC extern char
   *strlib_param ( char * buffer, size_t buflen,
                   const char *s, char separator, int parno ),
 
-  // split a string into tokens
+  // split a string in tokens
   *strlib_tok(char *s, const char *sep),
 
-  // split a string into tokens (thread safe)
+  // split a string in tokens (thread safe)
   *strlib_tok_r(char *s, const char *sep, char **lasts),
 
-  // copy input string into new allocated memory
-  *strlib_newstr( const char *string );
+  // copy input string to a new allocated memory
+  *strlib_newstr( const char *string ),
+
+  // copy n bytes (at most) of the input string to 
+  // a new allocated memory of n+1 bytes.
+  *strlib_newstrn( const char *string, size_t n ),
+
+  // replace substrings of type num((<expression>)) with the 
+  // calculated expression values
+  *strlib_numberstring( char *buffer, size_t buflen, 
+                        const char *string, int *pstatus );
 
 PUBLIC extern const char
 
+  // convert str to array
+  **strlib_list2stringarray( const char *str, size_t *nelements ),
+
+  // free all memory allocated by stringarray and its elements
+  **strlib_freestringarray( const char **stringarray, size_t nelements ),
+
   // return version string
   *strlib_version ( void );
 
+PUBLIC extern void 
+
+  // free memory
+  *strlib_free( void *string );
+
 #endif /* _STRLIB_ */
 
 /****************************************************************************
