@@ -23,7 +23,7 @@
  *   If not, see <http://www.gnu.org/licenses/>.
  */
 
-# define SX_VERSION      "sx : V1.12 Peter Boesecke 2014-07-06"
+# define SX_VERSION      "sx : V1.13 Peter Boesecke 2015-05-21"
 /*+++------------------------------------------------------------------------
 NAME
   sx.c --- 3d orientation parameter transformation
@@ -163,7 +163,7 @@ int fprint_debug( FILE *out )
 /*--------------------------------------------------------------------------
 NAME
 
-  sx_debug_set --- set / reset module sx into debug mode
+  sx_debug_set --- set / reset module sx to debug mode
 
 SYNOPSIS
 
@@ -171,7 +171,7 @@ SYNOPSIS
 
 DESCRPTION
 
-  Set / reset module sx into debug mode.
+  Set / reset module sx to debug mode.
 
 --------------------------------------------------------------------------*/
 PUBLIC int sx_debug_set( int debug )
@@ -338,7 +338,7 @@ PUBLIC SXParams * sx_new    ( SXParams * params )
   if (SxDebug&SX_DEBUG) printf( "sx_new\n" );
 
   if (!params) {
-    if (!(new=malloc ( sizeof(SXParams) ))) 
+    if (!(new=MALLOC ( sizeof(SXParams) ))) 
       goto sx_new_error;
     params=new;
   }
@@ -353,7 +353,7 @@ PUBLIC SXParams * sx_new    ( SXParams * params )
 
 sx_new_error:
 
-  if (new) free(new);
+  FREE(new);
 
   if (SxDebug&SX_DEBUG) printf( "sx_new END (error)\n" );
 
@@ -416,7 +416,7 @@ DESCRPTION
 PUBLIC SXParams * sx_free   ( SXParams * params )
 {
   if (SxDebug&SX_DEBUG) printf( "sx_free\n" );
-  if ( params ) free( params );
+  FREE( params );
   if (SxDebug&SX_DEBUG) printf( "sx_free END\n" );
 
   return( NULL );
@@ -1480,10 +1480,10 @@ PUBLIC SXParams *sx_tf_params ( SXParams * params_out,
       }
     } // if ( fabs( TRT[2][2] ) > SxEps )
 
-    if (inv_T) free(inv_T);
-    if (T) free(T);
-    if (inv_t_order) free(inv_t_order);
-    if (t_order) free(t_order);
+    FREE(inv_T);
+    FREE(T);
+    FREE(inv_t_order);
+    FREE(t_order);
 
   }
 
@@ -1502,11 +1502,11 @@ PUBLIC SXParams *sx_tf_params ( SXParams * params_out,
 
 sx_tf_params_error:
 
-  if (inv_T) free(inv_T);
-  if (T) free(T);
-  if (inv_t_order) free(inv_t_order);
-  if (t_order) free(t_order);
-  if (new) free(new);
+  FREE(inv_T);
+  FREE(T);
+  FREE(inv_t_order);
+  FREE(t_order);
+  FREE(new);
 
   if (perrval) *perrval=errval;
 
@@ -1678,7 +1678,7 @@ sx_tf_img_error:
 /*--------------------------------------------------------------------------
 NAME
 
-  sx_errval2str --- writes error message into a buffer
+  sx_errval2str --- writes error message to a buffer
 
 SYNOPSIS
 
@@ -1693,7 +1693,7 @@ ARGUMENTS
 
 DESCRPTION
 
-  Writes an error message into the buffer corresponding to errval.
+  Writes an error message to the buffer corresponding to errval.
 
   SX_INVALID_PROJECTION, SX_INVALID_ORIENTATION, 
   SX_MEMORY_ALLOCATION_ERROR, SX_ORDER_CALCULATION_ERROR, 
