@@ -54,6 +54,7 @@
 *               size_t variables with SIZE_T to unsigned long               *
 *               before printing as %lu. %lz is not known by all             *
 *               compilers.                                                  *
+* 2017-05-21 PB update for ioalloc                                          *
 ****************************************************************************/
 
 /***************************************************************************
@@ -70,8 +71,6 @@
 ***************************************************************************/
 # define SIZE_T (unsigned long)
 
-# define FREE(s) if (s) { free(s); s=NULL; }
-
 static char ROCA_Usage[ROCABUFLEN];
 
 /****************************************************************************
@@ -253,9 +252,9 @@ char * sprint_beam_parameters( char * buffer, size_t bufsiz, double Par[] )
   char *pb;
   size_t bs, bd;
 
-  if ( !(ParBeam=(double *) malloc (sizeof(double)*NParams)) )
+  if ( !(ParBeam=(double *) MALLOC (sizeof(double)*NParams)) )
     goto sprint_beam_parameters_error;
-  if ( !(IParBeam=(int *) malloc (sizeof(int)*NParams)) )
+  if ( !(IParBeam=(int *) MALLOC (sizeof(int)*NParams)) )
     goto sprint_beam_parameters_error;
 
   for (num=0;num<NParams;num++) IParBeam[num]=0;
@@ -1655,11 +1654,11 @@ int CholeskySolve( int n, double A[], double B[], double X[] )
 
   if (ROCA_debug&ROCA_DEBUG) fprintf( stdout, "CholeskySolve\n" );
 
-  G = (double *) malloc( n * n * sizeof(double) );
-  Y = (double *) malloc( n * sizeof(double) );
+  G = (double *) MALLOC( n * n * sizeof(double) );
+  Y = (double *) MALLOC( n * sizeof(double) );
 
   if (ROCA_debug&ROCA_SHOWTEMP) {
-    tmp = (double *) malloc( n * n * sizeof(double) ); // used for check
+    tmp = (double *) MALLOC( n * n * sizeof(double) ); // used for check
     for (i=0;i<n*n;i++) { G[i]=0.0; tmp[i]=0.0; }
     for (i=0;i<n;i++) Y[i]=0.0;
   }
@@ -1758,11 +1757,11 @@ int roca_save_step( int step, size_t ndata, RO *Ordered,
   if (ROCA_debug&ROCA_DEBUG) fprintf( stdout, "roca_save_step\n" );
 
   // allocate matrices
-  if (!( angle_step=(double *) malloc(ndata*sizeof(double)) ) )
+  if (!( angle_step=(double *) MALLOC(ndata*sizeof(double)) ) )
     goto roca_save_step_error;
-  if (!( azimuth_step=(double *) malloc(ndata*sizeof(double)) ) )
+  if (!( azimuth_step=(double *) MALLOC(ndata*sizeof(double)) ) )
     goto roca_save_step_error;
-  if (!( ParCur=(double *) malloc(NParams*sizeof(double)) ) )
+  if (!( ParCur=(double *) MALLOC(NParams*sizeof(double)) ) )
     goto roca_save_step_error;
 
   // calculate iteration step
@@ -1870,9 +1869,9 @@ int get_RocaParDev( size_t ndata, RO *Ordered,
 
   nopt=Ordered->nopt;
 
-  if (!( A=(double *) malloc(nopt*ndata*sizeof(double)) ) )
+  if (!( A=(double *) MALLOC(nopt*ndata*sizeof(double)) ) )
     goto get_RocaParDevError;
-  if (!( B=(double *) malloc(ndata*sizeof(double)) ) )
+  if (!( B=(double *) MALLOC(ndata*sizeof(double)) ) )
     goto get_RocaParDevError;
 
   // get final matrix B
@@ -1984,15 +1983,15 @@ int RocaOptimize( double ParOpt[NParams], double ParDev[NParams],
 
   // allocate matrices
 
-  if (!( A=(double *) malloc(nopt*ndata*sizeof(double)) ) ) 
+  if (!( A=(double *) MALLOC(nopt*ndata*sizeof(double)) ) ) 
     goto RocaOptimizeError; 
-  if (!( B=(double *) malloc(ndata*sizeof(double)) ) ) 
+  if (!( B=(double *) MALLOC(ndata*sizeof(double)) ) ) 
     goto RocaOptimizeError; 
-  if (!( ATA=(double *) malloc(nopt*nopt*sizeof(double)) ) ) 
+  if (!( ATA=(double *) MALLOC(nopt*nopt*sizeof(double)) ) ) 
     goto RocaOptimizeError;  // ATA[nopt][nopt] -> L=nopt, N=nopt, M=ndata
-  if (!( ATB=(double *) malloc(nopt*sizeof(double)) ) ) 
+  if (!( ATB=(double *) MALLOC(nopt*sizeof(double)) ) ) 
     goto RocaOptimizeError; // ATB[1][nopt] -> L=1, N=nopt, M=ndata
-  if (!( DOpt=(double *) malloc(nopt*sizeof(double)) ) ) 
+  if (!( DOpt=(double *) MALLOC(nopt*sizeof(double)) ) ) 
     goto RocaOptimizeError;
 
   // Optimization loop BEGIN
