@@ -33,6 +33,7 @@
 *               compilers.                                                  *
 *               tolower and isgraph replaced with the macros                *
 *               TOLOWER and ISGRAPH.                                        *
+* 2017-05-21 PB updated for ioalloc                                         *
 ****************************************************************************/
 
 /****************************************************************************
@@ -54,8 +55,6 @@
 # define STRCMP(s1,s2) strcmp ( (char *) s1, (char *) s2 )
 # define STRCPY(s1,s2) strcpy ( (char *) s1, (char *) s2 )
 
-# define FREE(s) if (s) { free(s); s=NULL; }
-
 # define MAXLEN 1024
 
 /***************************************************************************
@@ -124,7 +123,7 @@ char * newstr( const char * string )
   if (ListDebug) printf("newstr:\n");
 
   if (!string) return( (char *) NULL );
-  if (!(value = (char *) malloc(STRLEN(string)+1))) return((char *) NULL);
+  if (!(value = (char *) MALLOC(STRLEN(string)+1))) return((char *) NULL);
   (void) STRCPY(value,string);
 
   return( value );
@@ -323,8 +322,8 @@ int insert_list( const char *Key, LList **plist )
   /* create list, if not found, otherwise keep existing */
   if ( notfound ) {
     /* create new list */
-    if (!(list = (LList *) malloc( sizeof(LList) ) )) return(-1);
-    if (!(list->Key = newstr( Key ))) { free(list); return(-1); }
+    if (!(list = (LList *) MALLOC( sizeof(LList) ) )) return(-1);
+    if (!(list->Key = newstr( Key ))) { FREE(list); return(-1); }
     list->Elements = NULL;
     list->ElementsLen = (size_t) 0;
 
@@ -402,9 +401,9 @@ int delete_list( LList *list,  LList **pnext )
     else ListRoot = next;
 
   /* free list */
-  if (list->Key) free(list->Key);
+  if (list->Key) FREE(list->Key);
   free_elements( list );
-  free(list);
+  FREE(list);
   
   return(0);
 } // delete_list
@@ -497,8 +496,8 @@ int insert_element( LList *list, const char *Key, LElement **pelement )
   /* create element, if not found */
   if ( notfound ) {
     /* create new element */
-    if (!(element = (LElement *) malloc( sizeof(LElement) ) )) return(-1);
-    if (!(element->Key = newstr( Key ))) { free(element); return(-1); }
+    if (!(element = (LElement *) MALLOC( sizeof(LElement) ) )) return(-1);
+    if (!(element->Key = newstr( Key ))) { FREE(element); return(-1); }
     element->Value = NULL;
     element->List = list;
 
@@ -629,7 +628,7 @@ int put_element_value( LElement *element, const char *Value)
   if (!ListInit) list_init( );
 
   if (!element) return(-1);
-  if (element->Value) free(element->Value); element->Value=newstr(Value);
+  if (element->Value) FREE(element->Value); element->Value=newstr(Value);
   return(0);
 } // put_element_value
 
@@ -673,9 +672,9 @@ int delete_element( LElement *element,  LElement **pnext )
   element->List->ElementsLen--;
 
   /* free element */
-  if (element->Key) free(element->Key);
-  if (element->Value) free(element->Value);
-  free(element);
+  if (element->Key) FREE(element->Key);
+  if (element->Value) FREE(element->Value);
+  FREE(element);
 
   return(0);
   
@@ -912,7 +911,7 @@ int cnt_elements( LList *list, const char *Keys )
 
   // copy Keys 
   Keyslen = STRLEN(Keys);
-  if ( !(KeyBuf = (char *) malloc(Keyslen+1)) ) goto cnt_elements_error;
+  if ( !(KeyBuf = (char *) MALLOC(Keyslen+1)) ) goto cnt_elements_error;
   strncpy( KeyBuf, Keys, Keyslen ); 
   KeyBuf[Keyslen]='\0';
 
@@ -926,13 +925,13 @@ int cnt_elements( LList *list, const char *Keys )
     cnt+=is_element( list, Key );
   }
  
-  if (KeyBuf) free(KeyBuf);
+  if (KeyBuf) FREE(KeyBuf);
 
   return( cnt );
 
 cnt_elements_error:
 
-  if (KeyBuf) free(KeyBuf);
+  if (KeyBuf) FREE(KeyBuf);
 
   return(-1);
 
