@@ -79,13 +79,15 @@ HISTORY
                       ReferenceNotSupported
   2011-06-08 PB V1.54 InvalidOrientation
   2015-10-03 PB V1.55 AlreadyOpen, FileMustBeOpen
-
+  2017-02-28 PB V1.56 SaxsErrBlk: char * => const char * and depending
+                      functions: SetSaxsErrorExternal, SetSaxsErrorMessage
+  2017-10-16 PB V1.57 HeaderNotRead
 ---*/
 
 /***************************************************************************/
 /* SaxsError.c                                                             */
 /***************************************************************************/
-# define SE_Version "SaxsError V1.55 Peter Boesecke 2015-10-03"
+# define SE_Version "SaxsError V1.56 Peter Boesecke 2017-02-28"
 
 # include <errno.h>
 # include <stdio.h>
@@ -102,7 +104,7 @@ typedef struct SE_Block {
   char * RoutineName; /* pointer to routine name */
   char * Message;     /* pointer to an info string */
   int  ExternalError; /* external error */
-  char * (*ReportError) (int ExternalError );
+  const char *(*ReportError) (int ExternalError );
                       /* pointer to external error report function */
 } SaxsErrBlk;
 
@@ -148,14 +150,14 @@ int SaxsError_init ( void )
 /*---------------------------------------------------------------------------
 NAME
 
-   SaxsError_newstr --- allocate memory and copy a character string into it
+   SaxsError_newstr --- allocate memory and copy a character string to it
 
 SYNOPSIS
 
    char * SaxsError_newstr( const char * string );
 
 DESCRIPTION
-  Allocates strlen(string)+1 bytes of memory and copies string into it.
+  Allocates strlen(string)+1 bytes of memory and copies string to it.
   In case of success the pointer to the allocated memory is returned. The
   null pointer is returned in case of an error.
   If string is the NULL pointer the NULL pointer is returned.
@@ -313,6 +315,8 @@ int ReportInputStatus(int Status, int ExitFlag)
              cout = "NoInstruction"; break;
         case NoAccumulator:
              cout = "NoAccumulator"; break;
+        case HeaderNotRead:
+             cout = "HeaderNotRead"; break;
 	case ExternalErrorReport:
              cout = "ExternalErrorReport"; break;
 
@@ -340,7 +344,7 @@ int ReportInputStatus(int Status, int ExitFlag)
    int status        (o) : status variable
    int ExternalError (i) : external error number, only valid and used when
                            Status==ExternalErrorReport.
-   (char *) (*ReportError) ( int ExternalError ) : error function
+   (const char *) (*ReportError) ( int ExternalError ) : error function
                            which returns an error message according to
                            ExternalError, only used when
                            Status==ExternalErrorReport.
@@ -399,7 +403,7 @@ Synopsis
                               char *  (*ReportError) (int ) ) 
 
   int ExternalError        (i) : external error number
-  (char *) (*ReportError) ( int ExternalError ) :
+  (const char *) (*ReportError) ( int ExternalError ) :
                             pointer to error report function
 Description
   The routine sets the parameters ExternalError and ReportError in
@@ -413,7 +417,7 @@ Description
 
 ****************************************************************************/
 void SetSaxsErrorExternal ( int ExternalError, 
-                            char *  (*ReportError) (int ) ) 
+                            const char *(*ReportError) (int ) ) 
 {
   SaxsErrorBlock.ExternalError = ExternalError;
   SaxsErrorBlock.ReportError   = ReportError;
@@ -426,7 +430,7 @@ void SetSaxsErrorExternal ( int ExternalError,
 2 DESCRIPTION 
   void SetSaxsErrorMessage ( const char * Message );
 
-  Allocates memory, copies Message into it and links it with SaxsErrorBlock.Message.
+  Allocates memory, copies Message to it and links it with SaxsErrorBlock.Message.
   If the pointer SaxsErrorBlock.Memory is not NULL at entry, the specified memory is 
   freed before new memory is allocated.
 
