@@ -18,7 +18,7 @@
 /*    You should have received a copy of the GNU General Public License    */
 /*    along with this program.  If not, see <http://www.gnu.org/licenses/>.*/
 /***************************************************************************/
-/* SaxsError.h V1.55 by P. BOESECKE 2015-10-03                             */
+/* SaxsError.h V1.56 by P. BOESECKE 2017-10-16                             */
 /***************************************************************************/
 
 #ifndef _SAXS_ERROR
@@ -34,8 +34,8 @@
                   const char * RoutineName, 
                   const char * Message ),
     SetSaxsErrorExternal ( int ExternalError,
-                           char *  (*ReportError) (int ) ),
-    SetSaxsErrorMessage ( const char * Message );
+                           const char *(*ReportError) (int ) ),
+    SetSaxsErrorMessage ( const char *Message );
 
    extern int
     ReportSaxsStatus(int Status, int ExitFlag);
@@ -62,6 +62,7 @@ enum Status { Failed  = -1,  Success = 0, Abort = 1,    SkipInput,
               NoVariable,
               VariableError,              ProgramError,
               NoInstruction,              NoAccumulator,
+              HeaderNotRead,
               ExternalErrorReport = 100 };
 
 #endif /* _SAXS_ERROR */
