@@ -39,7 +39,7 @@
   The output value of 'GetString' is a pointer to a char string.
   It must be declared as char ** value; .
   The output value of 'GetLine' is a char string with fixed length.
-  It must be declared as char value[InputLineLength]; .
+  It must be declared as char value[IO_size]; .
 
 2 CALL
 char *SaxsInputVersion ( void );
@@ -52,7 +52,7 @@ int GetReal (FILE *Fptr, float *Value, int Flag, char *Prompt, float DefVal);
 int GetDouble (FILE *Fptr, double *Value, int Flag, \
                                       char *Prompt, double DefVal):
 int GetString (FILE *Fptr, char **Value, int Flag, char *Prompt, char *DefVal);
-int GetLine (FILE *Fptr  , char  Value[InputLineLength], int Flag, \
+int GetLine (FILE *Fptr  , char  Value[IO_size], int Flag, \
 						char *Prompt, char * DefVal);
 void CopyLine ( const char * s1, char * sout, int len , int conversion );
 char *RightFR ( char * s1, char * s2, char * sout, int len ),
@@ -64,7 +64,7 @@ char *LeftFL  ( char * s1, char * s2, char * sout, int len );
 	26-Dez-1994 PB routines from P. Daly
 	29-Dez-1994 PB
 	01-Jan-1995 PB new CopyLine function (SCopyLine from Separation.c)
-	10-Jan-1995 PB installed on RED, separation into input.c, input.h
+	10-Jan-1995 PB installed on RED, separation in input.c, input.h
 	12-Jan-1995 PB 2.1     GetLong
         09-Sep-1995 PB 2.2     WildCard defined
         11-Sep-1995 PB 3.0     fpexpr() and lexpr()
@@ -88,7 +88,7 @@ char *LeftFL  ( char * s1, char * s2, char * sout, int len );
         14-Oct-1996 PB 3.7     SaxsInputVersion
         26-Jun-1999 PB 3.8     DevSeparator, DirSeparator, ExtSeparator in
                                SaxsInput.h defined
-        17-Jul-1999 PB 4.0     InputLineLength increased from 128 to 256
+        17-Jul-1999 PB 4.0     IO_size increased from 128 to 256
         02-Aug-2000 PB 4.1     GetDouble added
         2000-08-04  PB 4.2     %-e -> %-g
         2000-12-04  PB 4.3     -> SaxsExpression.c, .h
@@ -109,13 +109,15 @@ char *LeftFL  ( char * s1, char * s2, char * sout, int len );
                                changed to '#'
         2007-04-19  PB 4.40    -Wall compiler warnings resolved
         2015-07-12  PB 4.41    ISSPACE, TOLOWER
+        2017-10-17  PB 4.42    DefaultCheck, NullCheck, SkipCheck, AbortCheck:
+                               checking NULL pointers, CopyLine
 
 ---*/
 
 /***************************************************************************/
 /* SaxsInput.c                                                                 */
 /***************************************************************************/
-# define SI_Version "SaxsInput V4.41 Peter Boesecke 2015-12-07"
+# define SI_Version "SaxsInput V4.42 Peter Boesecke 2017-10-16"
 
 # define ISGRAPH(c) isgraph((int)(c))
 # define ISSPACE(c) isspace((int)(c))
@@ -176,20 +178,21 @@ char * SaxsInputVersion ( void )
 *	      conversion = 0  : no conversion
 *	      conversion = 1  : converion to big letters
 ****************************************************************************/
-void CopyLine( const char * s1, char * sout, int len , int conversion )
+void CopyLine( const char *s1, char *sout, int len , int conversion )
 {
-  register int i;
+  if (sout) {
+    register int i;
+    if (s1) switch (conversion) {  	
+        case -1 : for (i=0;(i<len-1) && (*s1);i++) *sout++ = TOLOWER(*s1++);
+                  break;
+        case 1  : for (i=0;(i<len-1) && (*s1);i++) *sout++ = TOUPPER(*s1++);
+                  break;
+        default : for (i=0;(i<len-1) && (*s1);i++) *sout++ = *s1++;
+                  break;
+      }
+    *sout++ = '\0';  /* terminate string with NULL */
+  }
 
-  if (s1)
-  switch (conversion)
-   {  	case -1 : for (i=0;(i<len-1) && (*s1);i++) *sout++ = TOLOWER(*s1++);
-			break;
-  	case 1  : for (i=0;(i<len-1) && (*s1);i++) *sout++ = TOUPPER(*s1++);
-			break;
-  	default : for (i=0;(i<len-1) && (*s1);i++) *sout++ = *s1++;
-                        break;
-   }
-  *sout++ = '\0';  /* terminate string with NULL */
 } /* CopyLine */
 
 /****************************************************************************
@@ -197,9 +200,10 @@ void CopyLine( const char * s1, char * sout, int len , int conversion )
 ****************************************************************************/
 
 int AbortCheck (char *s1)
-{
-  register int i;
+{ register int i;
   char buf[5];
+
+  if (!s1) s1="";
   if (strchr (s1,EOD) != NULL) return (-1); CopyLine(s1,buf,5,0);
  /*check first four characters for special token*/
   for (i=0;buf[i];i++) buf[i]=TOLOWER(buf[i]);
@@ -215,6 +219,7 @@ int AbortCheck (char *s1)
 int SkipCheck (char *s1)
 { int slen;
 
+  if (!s1) s1="";
   slen=strlen(s1); 
   /* do not include terminating control character */
   if (slen>1) 
@@ -230,14 +235,16 @@ int SkipCheck (char *s1)
 *  NullCheck : Checks s1 on appearance of NullToken                         *
 ****************************************************************************/
 int NullCheck (char *s1)
-{
-register int i;
-char buf[5];
-CopyLine(s1,buf,5,0); /*check first four characters for special token*/
-	for (i=0;buf[i];i++) buf[i]=TOLOWER(buf[i]);
-	if (strstr (buf, NullToken1) != NULL) return (-1);
-	if (strstr (buf, NullToken2) != NULL) return (-1);
-	return(0);
+{ register int i;
+  char buf[5];
+
+  if (!s1) s1="";
+  CopyLine(s1,buf,5,0); /*check first four characters for special token*/
+  for (i=0;buf[i];i++) buf[i]=TOLOWER(buf[i]);
+  if (strstr (buf, NullToken1) != NULL) return (-1);
+  if (strstr (buf, NullToken2) != NULL) return (-1);
+  return(0);
+
 } /* NullCheck */
 
 /****************************************************************************
@@ -245,8 +252,10 @@ CopyLine(s1,buf,5,0); /*check first four characters for special token*/
 ***********************************************************************o**/
 int DefaultCheck (char *s1)
 {
-	if (*s1 == '\0') return(-1);
-	return(0);
+  if (!s1) s1="";
+  if (*s1 == '\0') return(-1);
+  return(0);
+
 } /* DefaultCheck */
 
 /****************************************************************************
@@ -464,7 +473,7 @@ int GetString (FILE *Fptr, char **Value, int Flag, char *Prompt, char *DefVal)
 *	 char		*Prompt prompt string
 *	 char		*DefVal default string
 ****************************************************************************/
-int GetLine (FILE *Fptr  , char  Value[InputLineLength], int Flag, \
+int GetLine (FILE *Fptr  , char  Value[IO_size], int Flag, \
              char *Prompt, char * DefVal)
 {
   buffer[0] = '\0';
@@ -482,10 +491,10 @@ int GetLine (FILE *Fptr  , char  Value[InputLineLength], int Flag, \
       if (Flag) {
         /* Skip input if default value is SkipToken */
         if (SkipCheck(DefVal)) return(SkipInput);
-        (void) CopyLine (DefVal, Value, InputLineLength, 0);
+        (void) CopyLine (DefVal, Value, IO_size, 0);
 	} else return(NoDefault);
       } else {
-      (void) CopyLine (buffer, Value, InputLineLength, 0);
+      (void) CopyLine (buffer, Value, IO_size, 0);
       }
     }
   return (Success);
