@@ -18,7 +18,7 @@
 /*    You should have received a copy of the GNU General Public License    */
 /*    along with this program.  If not, see <http://www.gnu.org/licenses/>.*/
 /***************************************************************************/
-/* SaxsOption.h V6.78 by Peter Boesecke 2013-11-17                         */
+/* SaxsOption.h V6.80 by Peter Boesecke 2017-02-11                         */
 /**************************************************************************/
 
 #ifndef _SAXS_OPTION
@@ -34,7 +34,6 @@
 # define SaxsHistoryKey "i0" /* key to history */
 
 /* Restrictions */
-# define IO_len InputLineLength
 # define IO_CodeLen      5 /* maximum length of option strings */
 # define IO_MaxFloatOpt 30 /* maximum number of definable float options */	
 # define IO_MaxFlexpOpt 30 /* maximum number of definable flexp options */
@@ -73,10 +72,10 @@ IO_Dummy         : similar to IO_DontReadWrite, but reading routines
 */
 
 /* Block structures */
-typedef struct { char     V[IO_len]; int I; } IO_line;
-typedef struct { char     V[IO_len]; int I; } IO_lvexp;
+typedef struct { char     V[IO_size]; int I; } IO_line;
+typedef struct { char     V[IO_size]; int I; } IO_lvexp;
 typedef struct { long int V;         int I; } IO_long;
-typedef struct { char     V[IO_len]; int I; } IO_flexp;
+typedef struct { char     V[IO_size]; int I; } IO_flexp;
 typedef struct { NumProg *V;         int I; } IO_prog;
 typedef struct { float    V;         int I; } IO_float;
 
@@ -92,13 +91,13 @@ typedef struct OptionDescriptor {
 	} OptDsc;
 
 enum CmdOptions
-{       NEmpty=0,     NTestBit,        NHelp,           NPrompt,
-        NShift,       NRSys,           NUSys,           NGeneralBlock,
-        NPassHeader,  NCreateVariance, NAntiAliased,    NVarianceWeight,
-        NOutputDataType, NOutputDataValueOffset,        NOutputCompression,
-        NBslInputByteOrder,
-        NAdd,         NIPMin,          NMaxLineWidth,   NMinHeaderSize,
-        NCommonOrientation,            NBeam,
+{       NEmpty=0,        NIODebug,        NIOLevel,        NTestBit,        
+        NHelp,           NPrompt,         NShift,          NRSys,
+        NUSys,           NGeneralBlock,   NPassHeader,     NCleanHeader,
+        NCreateVariance, NAntiAliased,    NVarianceWeight, NOutputDataType, 
+        NOutputDataValueOffset, NOutputCompression,        NBslInputByteOrder,
+        NAdd,            NIPMin,          NMaxLineWidth,   NMinHeaderSize,
+        NCommonOrientation,               NBeam,
         NArgFst,          NArgLst=NArgFst+IO_MaxFloatOpt-1,
         NFlexpFst,        NFlexpLst=NFlexpFst+IO_MaxFlexpOpt-1,
         NNumFst,          NNumLst=NNumFst+IO_MaxIntOpt-1,
@@ -116,8 +115,8 @@ enum ImgOptions
         NDDummy,        NOffset,        NBinSiz,        NPixSiz,
         NWaveLength,    NOrientation,   NIndexOrder,    NProjection,    
         NAxisType,      NCenter,        NSampleDistance,NDetRot,
-        NBeamCenter,    NBeamDistance,  NDetTilt,       NImgLast,
-        IO_ImgOptLen };
+        NBeamCenter,    NBeamDistance,  NDetTilt,       NSearchList,
+        NImgLast,       IO_ImgOptLen };
 
 typedef struct ImageBlock {
         int  blkno;                 /* block number */
@@ -126,7 +125,7 @@ typedef struct ImageBlock {
 	long int  StreamOpenMode;   /* actual open mode used internally */
         long int  FileNumber;       /* file number of actually open file */
         long int  LoopNumber;       /* loop number of image */
-        char      FileName[IO_len]; /* file name of actually open file */
+        char      FileName[IO_size]; /* file name of actually open file */
         /* --- image option variables */
         IO_line   Name;          /* filename with pattern */
         IO_long   FFirst;        /* first file number extracted from Name */ 
@@ -170,6 +169,9 @@ typedef struct ImageBlock {
         IO_flexp  BeamDistance;
         IO_flexp  DetTilt[4];
 
+        /* h5 open search list */
+        IO_line   SearchList;
+
 }       ImgBlk;
 
 typedef struct CommandBlock {
@@ -178,7 +180,7 @@ typedef struct CommandBlock {
 	const char * MainVersion; /* version string of main program */
 	const char * MainHelp;    /* help string of main program */
         void *  MainData;    /* pointer to data needed by the main program */
-        char    SaxsVersion[IO_len]; /* version string appended to history line */ 
+        char    SaxsVersion[IO_size]; /* version string appended to history line */ 
 	int     argc;        /* length of argument list */
 	char ** argv;        /* pointer to argument list */
 	int  *  parg_no;     /* pointer to argument number variable */
@@ -193,6 +195,8 @@ typedef struct CommandBlock {
         long int  Cycles;    /* contains number of cycles before output image is written */
         /* --- command option variables */
 	int     TestBit;     /* for program tests only: on/off */
+	IO_long IODebug;     /* Debug settings of file i/o routines */
+	IO_long IOLevel;     /* Debug level of file i/o routines */
 	IO_long Help;        /* show help and exit: on */
 	IO_long Prompt;      /* prompt mode: on/off */
 	IO_long RSys;        /* reference system for calculation */
@@ -200,6 +204,7 @@ typedef struct CommandBlock {
 	IO_long GeneralBlock;         /* write general block */ 
         IO_long PassHeader;           /* write the full input header into
                                          the output header */ 
+        IO_long CleanHeader;          /* remove inconsistencies when reading headers */
         IO_long CreateVariance;       /* create variance arrays for error
                                          propagation calculations */
         IO_long AntiAliased;          /* interpolation mode */ 
