@@ -18,7 +18,7 @@
 /*    You should have received a copy of the GNU General Public License    */
 /*    along with this program.  If not, see <http://www.gnu.org/licenses/>.*/
 /***************************************************************************/
-# define SAXSRPK_VERSION "SaxsReplacekey V1.23 Peter Boesecke 2015-12-08"
+# define SAXSRPK_VERSION "SaxsReplacekey V1.26 Peter Boesecke 2017-12-10"
 /*+++------------------------------------------------------------------------
 NAME
    SaxsReplacekey --- Replaces keys in strings with values
@@ -81,6 +81,14 @@ HISTORY
 2015-10-12 V1.21 definition of FBASE and FBODY adapted
 2015-10-13 V1.22 FBODY is deprecated, but still supported
 2015-12-08 V1.23 spkey: pkend (unused-but-set-variable) commented
+2017-10-08 V1.24 strlib_numberstring added to all keyvalue functions
+2017-10-09 V1.25 doublekeyvalue,floatkeyvalue,longkeyvalue,linekeyvalue:
+                 accepting NULL pointer for pstatus./
+2017-10-20 V1.26 spkey: wrong debug info "... cannot read ..." corrected
+                 spstring, spkey, spnum, replacekey:
+                 doublekeyvalue, floatkeyvalue, longkeyvalue, linekeyvalue:
+                 ImgBlk ib[] -> ImgHeadBlk ihb[],
+                 edf_read_header_line(...) replaced with ReadHeaderLine(...)
 
 --------------------------------------------------------------------------*/
 
@@ -153,15 +161,15 @@ PRIVATE char white_spaces[7] = { ' ', '\t', '\r', '\n', '\f', '\v', '\0' };
 /******************************************************************************
 * Routines                                                                    *
 ******************************************************************************/
-char * spstring( char ** pb, const char **ps, long npar,
-                 CmdBlk *pcb, ImgBlk ib[], long Num[],
-                 long blkno, int level, int * pstatus );
-char * spkey( char ** pb, const char **ps, long npar,
-              CmdBlk *pcb, ImgBlk ib[], long Num[],
-              long blkno, int level, int * pstatus );
+char *spstring( char **pb, const char **ps, long npar,
+                CmdBlk *pcb, ImgHeadBlk ihb[], long Num[],
+                long blkno, int level, int *pstatus );
+char *spkey( char **pb, const char **ps, long npar,
+             CmdBlk *pcb, ImgHeadBlk ihb[], long Num[],
+             long blkno, int level, int *pstatus );
 void spnum( long *value, long defval, 
-            char ** pb, const char **ps, long npar,
-            CmdBlk *pcb, ImgBlk ib[], long Num[],
+            char **pb, const char **ps, long npar,
+            CmdBlk *pcb, ImgHeadBlk ihb[], long Num[],
             long blkno, int level );
 
 /*+++------------------------------------------------------------------------
@@ -171,7 +179,7 @@ NAME
 
 SYNOPSIS
 
-   const char * replacekey_version ( void );
+   const char *replacekey_version ( void );
 
 DESCRIPTION
 
@@ -180,7 +188,7 @@ RETURN VALUE
    this module.
 
 ----------------------------------------------------------------------------*/
-const char * replacekey_version ( void )
+const char *replacekey_version ( void )
 { return(SAXSRPK_VERSION);
 } // replacekey_version
 
@@ -191,7 +199,7 @@ NAME
 
 SYNOPSIS
 
-   const char * replacekey_show ( void );
+   const char *replacekey_show ( void );
 
 DESCRIPTION
 
@@ -200,7 +208,7 @@ RETURN VALUE
    this module.
 
 ----------------------------------------------------------------------------*/
-const char * replacekey_show ( void )
+const char *replacekey_show ( void )
 { return(SAXSRPK_SHOW);
 } // replacekey_show
 
@@ -211,7 +219,7 @@ NAME
 
 SYNOPSIS
 
-    int hasreplacekey( const char * string );
+    int hasreplacekey( const char *string );
 
 DESCRIPTION
 
@@ -220,7 +228,7 @@ Returns 1 string has a key between [ and ], otherwise 0.
 HISTORY
 04-Oct-2015 Peter Boesecke
 ---------------------------------------------------------------------------*/
-int hasreplacekey( const char * string )
+int hasreplacekey( const char *string )
 { int found=0;
   char *ps=NULL, *pe=NULL;
 
@@ -265,14 +273,14 @@ NAME
 
 SYNOPSIS
 
-   char * replacekey ( char * buffer, size_t buflen,const char *string, long npar,
-                       CmdBlk *pcb, ImgBlk ib[], long Num[],
-                       long blkno, int * pstatus );
+   char *replacekey ( char *buffer, size_t buflen,const char *string, long npar,
+                      CmdBlk *pcb, ImgHeadBlk ihb[], long Num[],
+                      long blkno, int *pstatus );
 
 DESCRIPTION
     Copies the input string to the output buffer. If possible, all keywords 
     descriptors are replaced by the corresponding header values. The header 
-    values are read with edf_read_header_line(...) from the data file headers. 
+    values are read with ReadHeaderLine(...) from the data file headers. 
     If the keyword cannot be found the keyword descriptor is copied from the
     input string to the output string without a specific error message.
 
@@ -295,7 +303,7 @@ SYNTAX
 RESTRICTIONS
     All image blocks must be open. Only values from open image blocks can 
     be read. Otherwise the keys are not replaced.
-    If any of the pointers pcb or ib are NULL the keys are not replaced. The 
+    If any of the pointers pcb or ihb are NULL the keys are not replaced. The 
     input is copied to the output. 
 
 EXAMPLE
@@ -331,41 +339,46 @@ RETURN VALUE
    Pointer to the output buffer or NULL in case of an error. 
 
 ----------------------------------------------------------------------------*/
-char * replacekey ( char * buffer, size_t buflen, const char *string, long npar,
-                    CmdBlk *pcb, ImgBlk ib[], long Num[],
-                    long blkno, int * pstatus )
+char *replacekey ( char *buffer, size_t buflen, const char *string, long npar,
+                   CmdBlk *pcb, ImgHeadBlk ihb[], long Num[],
+                   long blkno, int *pstatus )
 
-{ char *pb, *value;
-  const char *ps;
+{ char *pb=NULL, *value=NULL;
+  const char *ps=NULL;
   size_t j;
-
-  *pstatus = Success;
+  int status=Success;
 
   if (pcb->TestBit>1) printf(" -- replacekey\n");
 
-  if (!buflen) return ( (char*) NULL );
+  if (!buflen) goto replacekey_error;
 
-  if ((!pcb)||(!ib)) {
-    // Do not replace keys, copy string to buffer 
+  if ((pcb)&&(ihb)) {
+    // prepare output string 
+    pb = buffer;
+    for (j=1;j<buflen;j++)
+      *pb++ = ' '; // fill with spaces
+    *pb = '\0'; // terminate with NUL
+
+    // copy
+    pb = buffer;
+    ps = string;
+    value = spstring( &pb, &ps, npar, pcb, ihb, Num, blkno, 0, &status);
+    if (status!=Success) goto replacekey_error;
+  } else {
+    // Do not replace keys, copy string to buffer
     strncpy(buffer,string,buflen-1);
     buffer[buflen-1]='\0'; // terminate with 0
-    return ( buffer ); 
+    value = buffer;
   }
 
-  // prepare output string 
-  pb = buffer;
-  for (j=1;j<buflen;j++)
-    *pb++ = ' '; // fill with spaces
-  *pb = '\0'; // terminate with NUL
-
-  // copy
-  pb = buffer;
-  ps = string;
-  value = spstring( &pb, &ps, npar, pcb, ib, Num, blkno, 0, pstatus);
-  if (*pstatus!=Success) return ( value );
-
   if (pcb->TestBit>2) printf(" -- replacekey: value = %s\n",value);
 
+  if (pstatus) *pstatus=status;
+  return( value );
+
+replacekey_error:
+
+  if (pstatus) *pstatus=status;
   return( value );
 
 } // replacekey 
@@ -391,9 +404,9 @@ SYNOPSIS
     If (Num == NULL) Num[i]==1 is used
  
 ----------------------------------------------------------------------------*/
-char * spstring( char ** pb, const char **ps, long npar,
-                 CmdBlk *pcb, ImgBlk ib[], long Num[],
-                 long blkno, int level, int * pstatus)
+char *spstring( char **pb, const char **ps, long npar,
+                CmdBlk *pcb, ImgHeadBlk ihb[], long Num[],
+                long blkno, int level, int *pstatus)
 {  char *value; 
 
    if (pcb->TestBit>1) printf(" -- spstring (level = %d)\n",level);
@@ -405,7 +418,7 @@ char * spstring( char ** pb, const char **ps, long npar,
    while ((**pb)&&(**ps))
      switch (**ps) {
        case SAXSRPK_START:
-         (*ps)++; spkey(pb,ps,npar,pcb,ib,Num,blkno,level+1,pstatus); break;
+         (*ps)++; spkey(pb,ps,npar,pcb,ihb,Num,blkno,level+1,pstatus); break;
        default :
          **pb = **ps; (*pb)++; (*ps)++; break;
      } /* switch */
@@ -423,7 +436,7 @@ char * spstring( char ** pb, const char **ps, long npar,
 /*+++------------------------------------------------------------------------
 NAME
 
-   spkey --- extract key from input string and insert value into output string
+   spkey --- extract key from input string and insert value in output string
 
 SYNOPSIS
    *ps points to the current position in the input string (updated)
@@ -446,9 +459,9 @@ USAGE
          <npar>: parameter number (long integer), default npar
 
 ----------------------------------------------------------------------------*/
-char * spkey( char ** pb, const char **ps, long npar, 
-              CmdBlk *pcb, ImgBlk ib[], long Num[],
-              long blkno, int level, int * pstatus)
+char *spkey( char **pb, const char **ps, long npar, 
+             CmdBlk *pcb, ImgHeadBlk ihb[], long Num[],
+             long blkno, int level, int *pstatus)
 {  const char *psstart, *psstop;
    char *value;
    char key[EdfMaxKeyLen+1], *pk; //, *pkend; unused-but-set-variable
@@ -468,6 +481,9 @@ char * spkey( char ** pb, const char **ps, long npar,
    *pstatus = Success;
    keystatus = Success;
 
+   // reset ReportSaxsImageError
+   SetSaxsErrorExternal ( ErrorValue, ReportSaxsImageError );
+
    // remember start position
    psstart = *ps;
  
@@ -496,12 +512,12 @@ char * spkey( char ** pb, const char **ps, long npar,
      switch (**ps) {
        case SAXSRPK_START:
          // in levels > 1 copy only the first parameter
-         (*ps)++; spkey(&pk,ps,1,pcb,ib,Num,blkno,level+1,pstatus); break;
+         (*ps)++; spkey(&pk,ps,1,pcb,ihb,Num,blkno,level+1,pstatus); break;
        case SAXSRPK_SEPARATOR: 
          (*ps)++; 
          if (pcb->TestBit>1) printf(" -- spnum\n");
          if (pcb->TestBit>2) printf("    in = \"%s\"\n",*ps);
-         spnum(&isblock,isblock,&pk,ps,1,pcb,ib,Num,blkno,level+1);
+         spnum(&isblock,isblock,&pk,ps,1,pcb,ihb,Num,blkno,level+1);
          // restrict range
          isblock = MAX2(0,isblock);
          isblock = MIN2(isblock,imax-1);
@@ -516,7 +532,7 @@ char * spkey( char ** pb, const char **ps, long npar,
            (*ps)++; 
            if (pcb->TestBit>1) printf(" -- spnum\n");
            if (pcb->TestBit>2) printf("    in = \"%s\"\n",*ps);
-           spnum(&ipar,ipar,&pk,ps,1,pcb,ib,Num,blkno,level+1);
+           spnum(&ipar,ipar,&pk,ps,1,pcb,ihb,Num,blkno,level+1);
 
            if (pcb->TestBit>2) 
              printf("    ipar = %ld, next = \"%s\"\n",ipar,*ps);
@@ -524,7 +540,7 @@ char * spkey( char ** pb, const char **ps, long npar,
          // search SAXSRPK_END:
          while (**ps == SAXSRPK_SEPARATOR ) {
            (*ps)++;
-           spnum(NULL,0,&pk,ps,1,pcb,ib,Num,blkno,level+1);
+           spnum(NULL,0,&pk,ps,1,pcb,ihb,Num,blkno,level+1);
 
          }
          break;
@@ -545,6 +561,8 @@ char * spkey( char ** pb, const char **ps, long npar,
    }
    if (pcb->TestBit>2) printf("    next = \"%s\"\n",*ps);
 
+   ImgBlk *ib=pcb->ib;
+
    // copy keys to *pb
    // first check for special keys
    stream = ib[isblock].Stream;
@@ -599,14 +617,15 @@ char * spkey( char ** pb, const char **ps, long npar,
 
      if ( !(( IO_DontReadWrite | IO_Dummy ) & StreamOpenMode ) ) {
        // now search header keys
-       edf_read_header_line ( stream, DataNumber, DataChain,
-                              key, Line, &ErrorValue, &keystatus );
-       SetSaxsErrorExternal ( ErrorValue, ReportSaxsImageError );
+       ReadHeaderLine  ( pcb, ihblock, Num, ihb, key, Line, &keystatus );
      } else keystatus = KeywordMissing;
-     if (pcb->TestBit>1) {
-       printf("    in spkey: key cannot be read from file (block=%ld)\n",ihblock);
-       printf("              stream=%d, DataNumber=%ld, DataChain=%d, key=%s\n",
-                             stream, DataNumber, DataChain, key);
+
+     if (keystatus) {
+       if (pcb->TestBit>1) {
+         printf("    in spkey: key cannot be read from file (block=%ld)\n",ihblock);
+         printf("              stream=%d, DataNumber=%ld, DataChain=%d, key=%s\n",
+                               stream, DataNumber, DataChain, key);
+       }
      }
    }
 
@@ -651,8 +670,8 @@ SYNOPSIS
    If value is NULL, only ps is updated.
 ----------------------------------------------------------------------------*/
 void spnum( long *value, long defval,
-            char ** pb, const char **ps, long npar,
-            CmdBlk *pcb, ImgBlk ib[], long Num[],
+            char **pb, const char **ps, long npar,
+            CmdBlk *pcb, ImgHeadBlk ihb[], long Num[],
             long blkno, int level )
 {  char num[EdfMaxValLen+1], *pn, *pnend;
    register int j;
@@ -672,7 +691,7 @@ void spnum( long *value, long defval,
      switch (**ps) {
        case SAXSRPK_START:
          // in levels > 1 copy only the first parameter (default)
-         (*ps)++; spkey(&pn,ps,1,pcb,ib,Num,blkno,level+1,&status); break;
+         (*ps)++; spkey(&pn,ps,1,pcb,ihb,Num,blkno,level+1,&status); break;
        case SAXSRPK_SEPARATOR:
        case SAXSRPK_END:
          *pn = '\0'; break;
@@ -699,26 +718,37 @@ SYNOPSIS
    the expression is returned.
 ----------------------------------------------------------------------------*/
 double doublekeyvalue( const char *string,
-                       CmdBlk *pcb, ImgBlk ib[], long Num[],
-                       long blkno, int * pstatus )
-{ static char line[IO_len];
+                       CmdBlk *pcb, ImgHeadBlk ihb[], long Num[],
+                       long blkno, int *pstatus )
+{ int status=Success;
+  static char line1[IO_size], line2[IO_size];
   double value=0.0;
  
-  *pstatus = Success;
+  if (!string) string="";
+  replacekey ( line1, IO_size, string, 1,
+               pcb, ihb, Num, blkno, &status );
+  if (status!=Success) goto doublekeyvalue_error;
+
+  strlib_numberstring( line2, IO_size, line1, NULL );
  
-  replacekey ( line, IO_len, string, 1,
-               pcb, ib, Num, blkno, pstatus );
-  if (*pstatus!=Success) return( value );
- 
-  value = doubleexpr(line,pstatus);
-  if (*pstatus!=Success) {
-    SetSaxsErrorMessage( line );
-    return( value );
+  value = doubleexpr(line2,&status);
+  if (status!=Success) {
+    SetSaxsErrorMessage( line1 );
+    goto doublekeyvalue_error;
   }
  
   if (pcb->TestBit)
-    printf("doublekeyvalue: string=%s, line=%s, value=%g\n",string,line,value);
+    printf("doublekeyvalue: string=%s, line=%s, value=%g\n",string,line1,value);
  
+  if (pstatus) *pstatus=status;
+  return( value );
+
+doublekeyvalue_error:
+
+  if (pcb->TestBit)
+    printf("doublekeyvalue: string=%s, line=%s, value=%g (%d) ERROR\n",string,line1,value,status);
+
+  if (pstatus) *pstatus=status;
   return( value );
  
 } // doublekeyvalue
@@ -726,33 +756,45 @@ double doublekeyvalue( const char *string,
 /*+++------------------------------------------------------------------------
 NAME
  
-   floatkeyvalue --- returns a float value, either from ib 
+   floatkeyvalue --- returns a float value
  
 SYNOPSIS
    All keys in string are replaced by their values. The float value of
    the expression is returned.
 ----------------------------------------------------------------------------*/
 float floatkeyvalue( const char *string,
-                     CmdBlk *pcb, ImgBlk ib[], long Num[],
-                     long blkno, int * pstatus ) 
-{ static char line[IO_len];
+                     CmdBlk *pcb, ImgHeadBlk ihb[], long Num[],
+                     long blkno, int *pstatus ) 
+{ int status=Success;
+  static char line1[IO_size], line2[IO_size];
   float value=0.0;
 
-  *pstatus = Success;
+  if (!string) string="";
 
-  replacekey ( line, IO_len, string, 1,
-               pcb, ib, Num, blkno, pstatus );
-  if (*pstatus!=Success) return( value );
+  replacekey ( line1, IO_size, string, 1,
+               pcb, ihb, Num, blkno, &status );
+  if (status!=Success) goto floatkeyvalue_error;
 
-  value = floatexpr(line,pstatus);
-  if (*pstatus!=Success) {
-    SetSaxsErrorMessage( line );
-    return( value );
+  strlib_numberstring( line2, IO_size, line1, NULL );
+
+  value = floatexpr(line2,&status);
+  if (status!=Success) {
+    SetSaxsErrorMessage( line1 );
+    goto floatkeyvalue_error;
   }
 
   if (pcb->TestBit)
-    printf("floatkeyvalue: string=%s, line=%s, value=%g\n",string,line,value);
+    printf("floatkeyvalue: string=%s, line=%s, value=%g\n",string,line1,value);
 
+  if (pstatus) *pstatus=status;
+  return( value );
+
+floatkeyvalue_error:
+
+  if (pcb->TestBit)
+    printf("floatkeyvalue: string=%s, line=%s, value=%g (%d) ERROR\n",string,line1,value,status);
+
+  if (pstatus) *pstatus=status;
   return( value );
 
 } // floatkeyvalue
@@ -767,27 +809,38 @@ SYNOPSIS
    the expression is returned.
 ----------------------------------------------------------------------------*/
 long longkeyvalue( const char *string,
-                   CmdBlk *pcb, ImgBlk ib[], long Num[],
-                   long blkno, int * pstatus )
-{ static char line[IO_len];
-  long value;
+                   CmdBlk *pcb, ImgHeadBlk ihb[], long Num[],
+                   long blkno, int *pstatus )
+{ int status=Success;
+  static char line1[IO_size], line2[IO_size];
+  long value=0l;
  
-  *pstatus = Success;
-  value = 0l;
+  if (!string) string="";
+
+  replacekey ( line1, IO_size, string, 1,
+               pcb, ihb, Num, blkno, &status );
+  if (status!=Success) goto longkeyvalue_error;
+
+  strlib_numberstring( line2, IO_size, line1, NULL );
  
-  replacekey ( line, IO_len, string, 1,
-               pcb, ib, Num, blkno, pstatus );
-  if (*pstatus!=Success) return( value );
- 
-  value = longexpr(line,pstatus);
-  if (*pstatus!=Success) {
-    SetSaxsErrorMessage( line );
-    return( value );
+  value = longexpr(line2,&status);
+  if (status!=Success) {
+    SetSaxsErrorMessage( line1 );
+    goto longkeyvalue_error;
   }
  
   if (pcb->TestBit)
-    printf("longkeyvalue: string=%s, line=%s, value=%ld\n",string,line,value);
+    printf("longkeyvalue: string=%s, line=%s, value=%ld\n",string,line1,value);
 
+  if (pstatus) *pstatus=status;
+  return( value );
+
+longkeyvalue_error:
+
+  if (pcb->TestBit)
+    printf("longkeyvalue: string=%s, line=%s, value=%ld (%d) ERROR\n",string,line1,value,status);
+
+  if (pstatus) *pstatus=status;
   return( value );
  
 } // longkeyvalue
@@ -802,23 +855,42 @@ SYNOPSIS
    line is returned. If npar is 0, the keys are replaced by by the full value,
    if npar is 1, the keys are replaced with the first parameter of value
    until the first white space.
+   The input string and the output buffer can be identical. The output buffer
+   is filled from a temporary buffer using strlib_numberstring.
 
    If (Num == NULL) Num[i]==1 is used
 ----------------------------------------------------------------------------*/
-char * linekeyvalue (char * buffer, size_t buflen, const char *string, long npar,
-                     CmdBlk *pcb, ImgBlk ib[], long Num[],
-                     long blkno, int * pstatus )
-{ char * value;
+char *linekeyvalue (char *buffer, size_t buflen, const char *string, long npar,
+                    CmdBlk *pcb, ImgHeadBlk ihb[], long Num[],
+                    long blkno, int *pstatus )
+{ int status=Success;
+  static char line[IO_size];
+  char *value;
+
+  if (!string) string="";
  
-  *pstatus = Success;
+  if (pcb->TestBit)
+    printf("linekeyvalue: string=%s, npar=%ld BEGIN\n",string,npar);
  
-  value = replacekey ( buffer, buflen, string, npar,
-                       pcb, ib, Num, blkno, pstatus );
-  if (*pstatus!=Success) return( value );
+  value = replacekey ( line, IO_size, string, npar,
+                       pcb, ihb, Num, blkno, &status );
+  if (status!=Success) goto linekeyvalue_error;
+
+  value = strlib_numberstring( buffer, buflen, value, NULL );
 
   if (pcb->TestBit)
-    printf("linekeyvalue: string=%s, npar=%ld, value=%s\n",string,npar,value);
+    printf("linekeyvalue: value=%s END\n",value);
  
+  if (pstatus) *pstatus=status;
+
+  return( value );
+
+linekeyvalue_error:
+
+  if (pcb->TestBit)
+    printf("linekeyvalue: value=%s status=%d ERROR\n",value,status);
+
+  if (pstatus) *pstatus=status;
   return( value );
  
 } // linekeyvalue
