@@ -38,6 +38,7 @@ HISTORY
 2006-08-08 V1.10 Peter Boesecke recursive use of spkey for all parameters
 2013-05-28 V1.18 Peter Boesecke replacekey_show added
 2015-10-04 V1.19 Peter Boesecke hasreplacekey added
+2017-10-19 V1.26 PB argument list of *keyvalue function changed
 ***************************************************************************/
 
 #ifndef _SAXSRPK_ 
@@ -52,6 +53,7 @@ HISTORY
 #  endif
 
 # include "SaxsOption.h"
+# include "SaxsImage.h"
 
 PUBLIC extern const char
    // return version
@@ -65,24 +67,24 @@ PUBLIC extern int
 PUBLIC extern double 
    // return double value 
    doublekeyvalue( const char *string,
-                   CmdBlk *pcb, ImgBlk ib[], long Num[],
+                   CmdBlk *pcb, ImgHeadBlk ihb[], long Num[],
                    long blkno, int * pstatus );
 
 PUBLIC extern float 
    // return float value
    floatkeyvalue( const char *string,
-                  CmdBlk *pcb, ImgBlk ib[], long Num[],
+                  CmdBlk *pcb, ImgHeadBlk ihb[], long Num[],
                   long blkno, int * pstatus );
 
 PUBLIC extern long 
    // return long value
    longkeyvalue( const char *string,
-                 CmdBlk *pcb, ImgBlk ib[], long Num[], 
+                 CmdBlk *pcb, ImgHeadBlk ihb[], long Num[], 
                  long blkno, int * pstatus );
 
 PUBLIC extern char 
  * linekeyvalue ( char * buffer, size_t buflen, const char *string, long npar,
-                  CmdBlk *pcb, ImgBlk ib[], long Num[], 
+                  CmdBlk *pcb, ImgHeadBlk ihb[], long Num[], 
                   long blkno, int * pstatus );
 
 #endif
