@@ -150,9 +150,12 @@
 2016-02-17  PB V1.52 ArgvBlocks: Create default filename without .gz suffix
 2016-03-09  PB V1.53 UpdateLinearParameters => UpdateImageParametersC and
                      parameter RSys added
-2016-04-02  PB V1.54 V1.54 Projection_1U added,
+2016-04-02  PB V1.54 Projection_1U added,
                      floor -> floorf (float), ceil -> ceilf (float), 
                      no double values are used here
+2016-11-06  PB V1.55 ArgvBlocks: calculation of number of blocks
+                     corrected. 
+2017-05-22  PB V1.56 FREE -> _FREE, malloc -> _MALLOC
 
 ---*/
 
@@ -173,6 +176,20 @@
 # define ROUND3( x ) CEILF( ( x ) - 0.5 )
 #endif
 
+//++++++++ _FREE and _MALLOC should be replaced with FREE and MALLOC from ioalloc 
+#ifndef _FREE
+# define _FREE( s) { if ( s) { free( s); s = NULL; } }
+#endif
+
+#ifndef _MALLOC
+# define _MALLOC( a) malloc( a)
+#endif
+
+#ifndef _CALLOC
+# define _CALLOC( a,b) calloc( ( a),( b))
+#endif
+
+
 /***************************************************************************
 * Portables Print Format for size_t variable (to be printed with %lu)      *
 ***************************************************************************/
@@ -200,7 +217,7 @@ char * SaxsRoutineVersion ( void )
 
 /*----------------------------------------------------------------------------
 RebinFloat2d
-  Two dimensional rebinning of a float array. The result is written into the
+  Two dimensional rebinning of a float array. The result is written to the
   input array and the new dimensions are returned.
 Parameters
   float * dataIn  (i)     pointer to input array
@@ -321,7 +338,7 @@ void TransformFloat2d( float *data, long int dim_1, long int dim_2,
 /*----------------------------------------------------------------------------
 GnawFloat2d
   Sets all pixels inside an ellipse around a dummy to dummy.
-  The result is written into the input array. The routine
+  The result is written to the input array. The routine
   allocates a short integer array with the same dimensions as the
   data array for intermediate results. Its memory is released after
   successful termination of GnawFloat2d.
@@ -360,7 +377,7 @@ void GnawFloat2d ( float * data, float * vardat,
 
     /* allocate memory for tmp array*/
     t_tmp = sizeof(short int)*dim_1*dim_2;
-    tmp = (short int *) malloc (t_tmp);
+    tmp = (short int *) _MALLOC(t_tmp);
     if (tmp == (short int *) NULL) {
       *pstatus=NotEnoughMemoryAvailable; return; }
 
@@ -412,7 +429,7 @@ void GnawFloat2d ( float * data, float * vardat,
     }
 
     /* free tmp array */
-    free(tmp);
+    _FREE(tmp);
 
   }
 
@@ -1344,7 +1361,7 @@ void GetLinearRange      ( CmdBlk *pcb,      ImgHeadBlk ihb[],
                              are now passed.
   27-Apr-1995 Peter Boesecke Checked that the output image is always
                              taken into account.
-  25-Dec-1999 Peter Boesecke splitted into two parts with GetLinearOverlap
+  25-Dec-1999 Peter Boesecke splitted in two parts with GetLinearOverlap
   2002-05-20  Peter Boesecke f1, f3, W1 and W3
   2002-05-25  Peter Boesecke
 ---------------------------------------------------------------------------*/                                                              
@@ -1917,17 +1934,17 @@ void Affine (  CmdBlk * pcb, ImgHeadBlk ihb[],
   double pp[3][2];
 
   float *Off_1, *Off_2, *Ps_1, *Ps_2;
-  Off_1 = (float *) malloc(sizeof(float)*imax);
+  Off_1 = (float *) _MALLOC(sizeof(float)*imax);
   if (!Off_1) { *pstatus = NotEnoughMemoryAvailable; return;}
-  Off_2 = (float *) malloc(sizeof(float)*imax);
+  Off_2 = (float *) _MALLOC(sizeof(float)*imax);
   if (!Off_2) { *pstatus = NotEnoughMemoryAvailable; 
-                 free(Off_1); return;}
-  Ps_1 = (float *) malloc(sizeof(float)*imax);
+                 _FREE(Off_1); return;}
+  Ps_1 = (float *) _MALLOC(sizeof(float)*imax);
   if (!Ps_1) { *pstatus = NotEnoughMemoryAvailable; 
-               free(Off_1); free(Off_2); return;}
-  Ps_2 = (float *) malloc(sizeof(float)*imax);
+               _FREE(Off_1); _FREE(Off_2); return;}
+  Ps_2 = (float *) _MALLOC(sizeof(float)*imax);
   if (!Ps_2) { *pstatus = NotEnoughMemoryAvailable; 
-               free(Off_1); free(Off_2); free(Ps_1); return;}
+               _FREE(Off_1); _FREE(Off_2); _FREE(Ps_1); return;}
 
   *pstatus = Success;
 
@@ -1935,13 +1952,13 @@ void Affine (  CmdBlk * pcb, ImgHeadBlk ihb[],
     fprintf(ERROUT,
       " ERROR in Affine: Not enough image blocks (%d / 2) found\n",
       imax);
-    free(Off_1); free(Off_2); free(Ps_1); free(Ps_2);
+    _FREE(Off_1); _FREE(Off_2); _FREE(Ps_1); _FREE(Ps_2);
     *pstatus=Failed; return; }
 
   GetReferenceParameters( pcb, ihb, 0, imax-1,
                           Off_1, Off_2, Ps_1,  Ps_2, pstatus );
   if (*pstatus!=Success) {
-     free(Off_1); free(Off_2); free(Ps_1); free(Ps_2); 
+     _FREE(Off_1); _FREE(Off_2); _FREE(Ps_1); _FREE(Ps_2); 
      return; }
 
   p0  = (double) Ps_1[InputBlock];   p1 = (double) Ps_2[InputBlock];
@@ -1949,7 +1966,7 @@ void Affine (  CmdBlk * pcb, ImgHeadBlk ihb[],
   pp0 = (double) Ps_1[OutputBlock];  pp1 = (double) Ps_2[OutputBlock];
   op0 = (double) Off_1[OutputBlock]; op1 = (double) Off_2[OutputBlock];
 
-  free(Off_1); free(Off_2); free(Ps_1); free(Ps_2);
+  _FREE(Off_1); _FREE(Off_2); _FREE(Ps_1); _FREE(Ps_2);
 
   /* include world coordinates in transformation matrix tp */
   tp[0][0] = t[0][0] * (p0/pp0);
@@ -2056,7 +2073,7 @@ Ask for all file names in the following way:
 The output image block is initialized with InitFunction before it is used.
 
 If block_lst >= 1, all input images are open for read and closed 
-afterwards.  The header block of the first image is read into ihb[1].
+afterwards.  The header block of the first image is read in ihb[1].
 The routine must be used for all images at once.
 
 PARAMETERS
@@ -2085,21 +2102,28 @@ void ArgvBlocks ( CmdBlk * pcb, ImgBlk ib[], ImgHeadBlk ihb[],
 { int blkno, first_input, last_input;
   size_t blocks; // number of blocks to allocate; 
   long image_1st=1, image_lst=1;
-  char linebuffer[IO_len];
+  char linebuffer[IO_size];
   const char *RoutineName = "ArgvBlocks";
-  long *num;
+  long *num=NULL;
 
-  char fname[IO_len];
+  char fname[IO_size];
 
   SetSaxsError( SR_Version, RoutineName, NULL );
 
   *pstatus = Success;
 
-  first_input = MAX2(1,block_1st);
-  last_input  = block_lst;
-  blocks      = MAX2(first_input,block_lst); // at least 1 block
+  if ((block_lst<block_1st)&&(block_1st<=0)) {
+    // number of used blocks is zero as described
+    blocks = 0;
+  } else {
+    blocks = MAX2(block_1st, block_lst);
+    if (block_1st<=0) blocks++; 
+    if (!blocks) blocks=1; // at least 1 block
+
+//++++++++++num = (long *) _MALLOC(sizeof(long)*blocks);
+    num = (long *) _CALLOC(blocks,sizeof(long));
+  }
 
-  num = (long *) malloc(sizeof(long)*blocks);
   if (!num) { 
     printf("ERROR: Failed to allocate %lu bytes\n",SIZE_T (sizeof(long)*blocks));
     exit(-1);
@@ -2107,6 +2131,9 @@ void ArgvBlocks ( CmdBlk * pcb, ImgBlk ib[], ImgHeadBlk ihb[],
 
   /* ---  File names and open modes --- */
 
+  first_input = MAX2(1,block_1st);
+  last_input  = block_lst;
+
   for (blkno = first_input;blkno <= last_input;blkno++) {
 
     /*--- Open mode ib[blkno].OpenMode of input files */
@@ -2122,15 +2149,16 @@ void ArgvBlocks ( CmdBlk * pcb, ImgBlk ib[], ImgHeadBlk ihb[],
     else argv_filename( pcb,linebuffer,ib,blkno,ib[blkno-1].Name.V,pstatus );
 
     /* Exit in all cases after an abort from the first argument */
-    if (*pstatus!=Success) { free(num); return; }
+    if (*pstatus!=Success) { _FREE(num); return; }
 
-    OpenImageFile(pcb,ib,blkno,ib[blkno].Name.V,ib[blkno].FFirst.V,
+    OpenImageFile(pcb,ib,blkno,ihb,
+                  ib[blkno].Name.V,ib[blkno].FFirst.V,
                   IO_Old|IO_FileProtect,pstatus);
-    if (*pstatus!=Success) { free(num); return; } 
+    if (*pstatus!=Success) { _FREE(num); return; } 
 
     /* Search for minimum and maximum image number */
     (void) SearchMinMaxImage (pcb,ib,blkno,&image_1st,&image_lst,pstatus);
-    if (*pstatus!=Success) { free(num); return; } 
+    if (*pstatus!=Success) { _FREE(num); return; } 
     if (!ib[blkno].First.I) ib[blkno].First.V = image_1st;
     if (!ib[blkno].Last.I)  ib[blkno].Last.V  = image_lst;
     num[blkno] = ib[blkno].First.V;
@@ -2146,21 +2174,21 @@ void ArgvBlocks ( CmdBlk * pcb, ImgBlk ib[], ImgHeadBlk ihb[],
 
     if (!ib[0].Name.I) { 
       if (Extension) { // create output file name from input file name
-        char Suffix[IO_len];
-        filename_name ( fname, IO_len, ib[1].Name.V ); // remove path
+        char Suffix[IO_size];
+        filename_name ( fname, IO_size, ib[1].Name.V ); // remove path
         // remove compression extensions first
-        filename_extension( Suffix, IO_len, fname);
+        filename_extension( Suffix, IO_size, fname);
         if (!strcmp(Suffix,".gz")) {
-          filename_body ( fname, IO_len, fname ); // remove .gz
+          filename_body ( fname, IO_size, fname ); // remove .gz
         }
-        filename_body ( ib[0].Name.V, IO_len, fname ); // remove extension
+        filename_body ( ib[0].Name.V, IO_size, fname ); // remove extension
         sprintf(ib[0].Name.V,"%s%s",ib[0].Name.V,Extension);
       } else sprintf(ib[0].Name.V,"%s",DefaultOutput);
     }
 
     /*--- File name ib[0].Name of output */
     argv_filename( pcb, "Output sequence", ib, 0, ib[0].Name.V, pstatus);
-    if (*pstatus!=Success) { free(num); return; } 
+    if (*pstatus!=Success) { _FREE(num); return; } 
 
   } /* output file */
 
@@ -2171,22 +2199,22 @@ void ArgvBlocks ( CmdBlk * pcb, ImgBlk ib[], ImgHeadBlk ihb[],
     /*--- Argument : number of first image */
     argv_long(pcb,"First image of input sequence 1",&ib[1].First,ib[1].First.V,
                pstatus);
-    if (*pstatus!=Success) { free(num); return; } 
+    if (*pstatus!=Success) { _FREE(num); return; } 
 
     /*--- Argument : number of last image */
     if (pcb->argc==*pcb->parg_no+1) image_lst = ib[1].First.V;
       else image_lst = ib[1].Last.V;
     argv_long( pcb,"Last image of input sequence 1", &ib[1].Last,image_lst,
                pstatus);
-    if (*pstatus!=Success) { free(num); return; } 
+    if (*pstatus!=Success) { _FREE(num); return; } 
 
     /*--- Argument : increment (default = 1) */
     argv_long( pcb,"Increment of input sequence 1", &ib[1].Inc, 1 , pstatus);
-    if (*pstatus!=Success) { free(num); return; } 
+    if (*pstatus!=Success) { _FREE(num); return; } 
 
     num[1]=ib[1].First.V;
     (void) ReadImageHeader( pcb, ib, 1, num, ihb, pstatus);
-    if (*pstatus!=Success) { free(num); return; } 
+    if (*pstatus!=Success) { _FREE(num); return; } 
     if (pcb->TestBit) PrintImageHeaderBlock(&ihb[1]);
 
   } /* first input file */
@@ -2195,7 +2223,7 @@ void ArgvBlocks ( CmdBlk * pcb, ImgBlk ib[], ImgHeadBlk ihb[],
     /*--- image numbers of all other images */
     (void) sprintf(linebuffer,"First image of input sequence %d",blkno);
     argv_long( pcb,linebuffer, &ib[blkno].First, ib[blkno].First.V, pstatus);
-    if (*pstatus!=Success) { free(num); return; } 
+    if (*pstatus!=Success) { _FREE(num); return; } 
 
   } /* all other input files */
 
@@ -2209,7 +2237,7 @@ void ArgvBlocks ( CmdBlk * pcb, ImgBlk ib[], ImgHeadBlk ihb[],
     } else {
       NewImageHeader(pcb,ib,0,num,ihb,NULL,InitFunction,pstatus);
     }
-    if (*pstatus!=Success) { free(num); return; } 
+    if (*pstatus!=Success) { _FREE(num); return; } 
 
     /*--- Argument : output dummy */
     sprintf(linebuffer,"Output dummy, e.g. %g",ihb[0].Dummy.V);
@@ -2232,10 +2260,10 @@ void ArgvBlocks ( CmdBlk * pcb, ImgBlk ib[], ImgHeadBlk ihb[],
 
   for (blkno=first_input;blkno<=last_input;blkno++) {
     CloseImageFile( pcb, ib, blkno, pstatus) ;
-    if (*pstatus!=Success) { free(num); return; }
+    if (*pstatus!=Success) { _FREE(num); return; }
   } /* close all input files */
 
-  free(num);
+  _FREE(num);
 
 } /* ArgvBlocks */
 
