@@ -910,7 +910,7 @@ ARGUMENTS
                          in the input image
   float vmax          maximum value of the mapping range in the input image
   short bps           bits per sample in the output image
-  char comment[]      output buffer with length InputLineLength.
+  char comment[]      output buffer with size IO_size.
                       It describes the mapping range
   int * pstatus       returned status (0: OK, -1: error, array is released)
 
