@@ -20,10 +20,10 @@
 /***************************************************************************/
 /*+++
 1 binary2saxs
-  Conversion of a binary file into ESRF data format.
+  Conversion of a binary file to ESRF data format.
 
 2 PURPOSE
-  Conversion of a binary file into ESRF data format. 
+  Conversion of a binary file to ESRF data format. 
 
   Arguments:
   binary2saxs [options] <fnam> <onam> <i1dim1> <i1dim2> <fskp> <ftyp> <odum>"
@@ -95,8 +95,10 @@
                        only effect on the header, not on the data
   2012-01-31  PB V4.68 ImageLoop parameters updated
   2015-11-16  PB V4.69 condition_2d, sx_tf_img
+  2017-10-20  PB V4.70 OpenImageFile,
+                       ReadImageDimension arguments updated
 ---*/
-# define Version "binary2saxs V4.69 2015-11-16 Peter Boesecke"
+# define Version "binary2saxs V4.70 2017-10-20 Peter Boesecke"
 # include <errno.h>
 # include <stdio.h>
 # include <unistd.h>
@@ -145,7 +147,7 @@ const char * STypeULong    = "unsigned long integer";
 /*----------------------------------------------------------------------------
  Static Messages
 ----------------------------------------------------------------------------*/
-static char Binary2SaxsMessage[IO_len];
+static char Binary2SaxsMessage[IO_size];
 
 /*---------------------------------------------------------------------------
 1 string2mtype 
@@ -231,7 +233,7 @@ void binary2saxs (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
   float Wmin_1, Wmax_1, W_1, DW_1;
   float Wmin_2, Wmax_2, W_2, DW_2;
 
-  char fname_full[IO_len];
+  char fname_full[IO_size];
 
   IO_line Fname;
   IO_line Ftype;
@@ -261,15 +263,15 @@ void binary2saxs (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
 
  /* --- read binary file and convert to float */
 
-     Fname = option_line ( SFname, pcb, pcb->ib, num, 1, pstatus );
+     Fname = option_line ( SFname, pcb, ihb, num, 1, pstatus );
      if (*pstatus!=Success) return;
-     Ftype = option_line ( SFtype, pcb, pcb->ib, num, 1, pstatus );
+     Ftype = option_line ( SFtype, pcb, ihb, num, 1, pstatus );
      if (*pstatus!=Success) return;
-     Fskip = option_long ( SFskip, pcb, pcb->ib, num, 1, pstatus );
+     Fskip = option_long ( SFskip, pcb, ihb, num, 1, pstatus );
      if (*pstatus!=Success) return;
-     Fras = option_long ( SFras, pcb, pcb->ib, num, 1, pstatus );
+     Fras = option_long ( SFras, pcb, ihb, num, 1, pstatus );
      if (*pstatus!=Success) return;
-     Fdvo = option_long ( SFdvo, pcb, pcb->ib, num, 1, pstatus );
+     Fdvo = option_long ( SFdvo, pcb, ihb, num, 1, pstatus );
      if (*pstatus!=Success) return;
      Fend = option_flag ( SFend, pcb, pstatus );
      if (*pstatus!=Success) return;
@@ -289,7 +291,7 @@ void binary2saxs (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
      }
 
      data_dim[0] = 2l;
-     (void) ReadImageDimension ( pcb, 1, ihb, 
+     (void) ReadImageDimension ( pcb, 1, num, ihb, 
                                  &data_dim[1], &data_dim[2], pstatus );
      if (*pstatus!=Success) return;
 
@@ -310,9 +312,9 @@ void binary2saxs (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
        }
 
      /* Generate filename of binary file */
-     filename_parameter (fname_full,IO_len,Fname.V,0);
-     filename_pattern (fname_full,IO_len,fname_full,(pcb->ib)[1].FileNumber);
-     filename_full (fname_full,IO_len,fname_full);
+     filename_parameter (fname_full,IO_size,Fname.V,0);
+     filename_pattern (fname_full,IO_size,fname_full,(pcb->ib)[1].FileNumber);
+     filename_full (fname_full,IO_size,fname_full);
 
      /* --- read binary data */
      printf("Reading %s\n",fname_full);
@@ -414,7 +416,7 @@ void binary2saxs (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
             ihb[1].Data, ihb[1].VarDat, item_number, &params,
             ihb[1].Data, ihb[1].VarDat, item_size, pcb->CommonOrientation.V, 1, &errval );
      if (errval) {
-       sx_errval2str( Binary2SaxsMessage, IO_len, errval );
+       sx_errval2str( Binary2SaxsMessage, IO_size, errval );
        SetSaxsErrorMessage( Binary2SaxsMessage );
        *pstatus=Failed;
        return;
@@ -501,11 +503,11 @@ Do all the keyboard io and return cb, and ib
 
 void user_io(CmdBlk * pcb, ImgBlk * ib, int * pstatus)
 {
-  char  progfn[InputLineLength];
+  char  progfn[IO_size];
   ImgHeadBlk ihb[BlockNum];
   long num[BlockNum];
-  char fname_pattern[IO_len];
-  char tmp_line[IO_len];
+  char fname_pattern[IO_size];
+  char tmp_line[IO_size];
 
   long  image_1st, image_lst;
   long  i1dim_1, i1dim_2;
@@ -521,7 +523,7 @@ void user_io(CmdBlk * pcb, ImgBlk * ib, int * pstatus)
   IO_lvexp *pFskip, *pFras, *pFdvo;
 
  /* Determine program name without directory */
-   (void) RightFR((char *) pcb->argv[0],DirSeparator,progfn,InputLineLength);
+   (void) RightFR((char *) pcb->argv[0],DirSeparator,progfn,IO_size);
 
   /* show usage if no arguments are given */
   if (pcb->argc<=1) printf("Usage: %s %s\n",progfn,pcb->MainHelp);
@@ -555,8 +557,8 @@ void user_io(CmdBlk * pcb, ImgBlk * ib, int * pstatus)
   /*--- Argument  : binary file name */
   if (!pFname->I) {
     /* Determine input file name without directory and without extension */
-    filename_name(pFname->V,IO_len,ib[1].Name.V);
-    filename_body(pFname->V,IO_len,pFname->V);
+    filename_name(pFname->V,IO_size,ib[1].Name.V);
+    filename_body(pFname->V,IO_size,pFname->V);
     }
 
   argv_line(pcb,"Binary file name",pFname,pFname->V, pstatus);
@@ -566,7 +568,7 @@ void user_io(CmdBlk * pcb, ImgBlk * ib, int * pstatus)
 //++++++++++++++ added for file name patterns 2004-04-10
   if (!ib[1].Name.I) {
     // copy binary name to filename
-    CopyLine( pFname->V, ib[1].Name.V, IO_len , 0 );
+    CopyLine( pFname->V, ib[1].Name.V, IO_size , 0 );
   }
 
   /* extract filenumbers from binary file name */
@@ -574,14 +576,16 @@ void user_io(CmdBlk * pcb, ImgBlk * ib, int * pstatus)
   if (*pstatus!=Success) return;
 
   /* Generate filename of first binary file and test, whether it exists */
-  filename_parameter (fname_pattern,IO_len,pFname->V,0);
-  filename_pattern (fname_pattern,IO_len,fname_pattern,ib[1].FFirst.V);
+  filename_parameter (fname_pattern,IO_size,pFname->V,0);
+  filename_pattern (fname_pattern,IO_size,fname_pattern,ib[1].FFirst.V);
   if (!filename_exists(fname_pattern)) { 
     SetSaxsErrorMessage(fname_pattern);
     *pstatus = FileNotFound; return; 
   }
 
-  OpenImageFile( pcb,ib,1,ib[1].Name.V, ib[1].FFirst.V,
+  /* No key value replacement -> ihb=NULL */
+  OpenImageFile( pcb,ib,1,NULL,
+      ib[1].Name.V, ib[1].FFirst.V,
       ib[1].OpenMode.V | IO_Old | IO_FileProtect | IO_DontReadData, pstatus);
   if (*pstatus!=Success) {
     SetSaxsErrorMessage(ib[1].Name.V); return;
@@ -616,7 +620,7 @@ void user_io(CmdBlk * pcb, ImgBlk * ib, int * pstatus)
   if (*pstatus!=Success) return;
   if (pcb->TestBit) PrintImageHeaderBlock(&ihb[1]);
 
-  (void) ReadImageDimension ( pcb, 1, ihb, &i1dim_1, &i1dim_2, pstatus );
+  (void) ReadImageDimension ( pcb, 1, num, ihb, &i1dim_1, &i1dim_2, pstatus );
   if (*pstatus!=Success) return; 
 
   if (i1dim_1<0) i1dim_1 = Default_dim_1; 
@@ -684,11 +688,11 @@ void user_io(CmdBlk * pcb, ImgBlk * ib, int * pstatus)
 
   /*--- output dimension 1 */
   sprintf(tmp_line,"%d",ihb[1].Dim[1]);
-  if (!(ib[0].Dim[1].I)) CopyLine(tmp_line,ib[0].Dim[1].V,IO_len,0);
+  if (!(ib[0].Dim[1].I)) CopyLine(tmp_line,ib[0].Dim[1].V,IO_size,0);
 
   /*--- output dimension 2 */
   sprintf(tmp_line,"%d",ihb[1].Dim[2]);
-  if (!(ib[0].Dim[2].I)) CopyLine(tmp_line,ib[0].Dim[2].V,IO_len,0);
+  if (!(ib[0].Dim[2].I)) CopyLine(tmp_line,ib[0].Dim[2].V,IO_size,0);
 
   GetReference(pcb->RSys.V,1,ihb,&ROff_1,&ROff_2,&RPs_1,&RPs_2,pstatus );
   if (*pstatus!=Success) return;
