@@ -30,7 +30,7 @@
   30-Apr-1999 PB V1.12 status set to Success before calling edf routines
   30-Apr-1999 PB V1.13 Tried to free static variable 'DetectorType' lead
                        to segmentation fault in Linux, is now mallocated. 
-  16-Jun-1999 PB V1.14 edfio_debug added
+  16-Jun-1999 PB V1.14 edfio_debug_set added
   26-Jun-1999 PB DevSeparator, DirSeparator, ExtSeparator defined in SaxsInput
   23-Jul-1999 PB V1.15 call to edf_write_data_2d_raw updated for edfio 
                  version 1.07, SaxsDataVersion = 2.0
@@ -54,11 +54,12 @@
   2007-04-23  PB V1.40 write_data_raw: eps defined as float
   2008-12-05  PB V1.41 open all input files as binary file to avoid
                        translation of \r\n to \n when compiled on windows.
+  2017-02-12  PB V1.42 update for iodbg
 
 
 
 ---*/
-# define Version  "V1.41 2008-12-05, Peter Boesecke"
+# define Version  "V1.42 2017-02-12, Peter Boesecke"
 
 # include <errno.h>
 # include <unistd.h>
@@ -140,7 +141,7 @@ int ImageNumberExists      ( int stream, int ImageNumber, int * pErrorValue )
 
 /*---------------------------------------------------------------------------
   write_data_raw
-  Writes raw data into the block "<keyword> = <ImNum>"
+  Writes raw data to the block "<keyword> = <ImNum>"
   A raw data header is created. 
   return value TRUE if no Error
   SaxsDataVersion 2.0
@@ -240,7 +241,7 @@ void write_data_raw ( int stream,
   register int i;
 
   if (G2S_debug) printf("write_data_raw BEGIN\n");
-  if (G2S_debug>1)  edfio_debug(1);
+  if (G2S_debug>1)  edfio_debug_set(IODBG_DEBUG1);
 
         *pErrorValue = RoutineSucceeded;
 
@@ -536,7 +537,7 @@ void CCD2Saxs(CmdBlk * pcb, FILE * InFstream, int In, int DataLength,
   int      HmScalerI0S, HmScalerI1S;
   int      HmScalerAnodeS, HmScalerTimeS;
   float    HmDeltaTime;
-  char     HmStartTime[InputLineLength];
+  char     HmStartTime[IO_size];
   char   * HmDldVersion;
 
   float    Center_1, Center_2, PixSize_1, PixSize_2;
@@ -711,8 +712,8 @@ void user_io( int argc, char *argv[], int * parg_no, CmdBlk *pcb,
               int * pOff_1, int * pOff_2, int * pstatus)
 
 {
-  char  progfn[InputLineLength];
-  char  InNamePre[InputLineLength];
+  char  progfn[IO_size];
+  char  InNamePre[IO_size];
   IO_line * pExtension;
 
   FILE * InFstream;
@@ -723,7 +724,7 @@ void user_io( int argc, char *argv[], int * parg_no, CmdBlk *pcb,
   if (G2S_debug) printf("user_io BEGIN\n");
 
  /* Determine program name without directory name */
-   (void) RightFR((char *) argv[0],DirSeparator,progfn,InputLineLength);
+   (void) RightFR((char *) argv[0],DirSeparator,progfn,IO_size);
 
   /* switch to prompt mode if input file name is missing and no 
      instruction has been given */
@@ -755,17 +756,17 @@ void user_io( int argc, char *argv[], int * parg_no, CmdBlk *pcb,
     } 
 
   /* Expand InName to the full file name */
-  if (!filename_full(InName,IO_len,InName)) {
+  if (!filename_full(InName,IO_size,InName)) {
      printf("% s: Input file name or path not allowed\n",progfn);
      *pstatus=FileNotFound; return;}
 
   /* Determine directory and prefix ouf histogramming memory name (InHmName) */
-  (void) LeftFL((char *) InName,DevSeparator,InHmName,InputLineLength);
+  (void) LeftFL((char *) InName,DevSeparator,InHmName,IO_size);
 
   /* Determine prefix of input file name (InName) */
-  (void) RightFR((char *) InName,DevSeparator,InNamePre,InputLineLength);
-  (void) RightFR((char *) InNamePre,DirSeparator,InNamePre,InputLineLength);
-  (void) LeftFL((char *) InNamePre,ExtSeparator,InNamePre,InputLineLength);
+  (void) RightFR((char *) InName,DevSeparator,InNamePre,IO_size);
+  (void) RightFR((char *) InNamePre,DirSeparator,InNamePre,IO_size);
+  (void) LeftFL((char *) InNamePre,ExtSeparator,InNamePre,IO_size);
 
  /*--- Read header information of input file --- */
 
@@ -844,7 +845,7 @@ void user_io( int argc, char *argv[], int * parg_no, CmdBlk *pcb,
      }
 
   /* Expand OutName to the full file name */
-  if (!filename_full(OutName,IO_len,OutName)) {
+  if (!filename_full(OutName,IO_size,OutName)) {
      printf("% s: Output file name or path not allowed\n",progfn);
      *pstatus = FileNameError; return; }
 
@@ -906,7 +907,7 @@ void user_io( int argc, char *argv[], int * parg_no, CmdBlk *pcb,
      }
 
   /* Expand InHmName to the full file name */
-  if (!filename_full(InHmName,IO_len,InHmName)) {
+  if (!filename_full(InHmName,IO_size,InHmName)) {
      printf("% s: Histogramming memory file or path not allowed\n", progfn);
      *pstatus = FileNameError; return; }
 
@@ -957,12 +958,12 @@ int MAIN (int argc, char *argv[])
   CmdBlk cb;  /* command block  */
 
   FILE  * InFstream, *InHmstream;
-  char  InName[InputLineLength];
-  char  InHmName[InputLineLength];
+  char  InName[IO_size];
+  char  InHmName[IO_size];
   int   InFirst, InLast, InInc;
   int   MinFrame, MaxFrame;
   int   OutStream;
-  char  OutName[InputLineLength];
+  char  OutName[IO_size];
   int   OutFirst; 
   int   ReadDummy;
   int   DataLength, Dim_1, Dim_2, Off_1, Off_2;
