@@ -18,7 +18,7 @@
 /*    You should have received a copy of the GNU General Public License    */
 /*    along with this program.  If not, see <http://www.gnu.org/licenses/>.*/
 /***************************************************************************/
-# define Version  "edfdump 1.15 2015-02-23, Peter Boesecke"
+# define Version  "edfdump 1.19 2018-02-08, Peter Boesecke"
 /*---------------------------------------------------------------------------
 NAME
   edfdump - print parameters of an edf file
@@ -39,8 +39,12 @@ DESCRIPTION
   -h, --help    print help
   -b <block>    block key, e.g. 1 (default: all blocks)
   -c <chain>    chain key, e.g. Image.Psd (default: all chains)
-  -e <keyword>  input of a header keyword to look for, by repeting this
-                option several keywords can be defined.
+  -e <keyword>  name of the header key to extract, by repeting this
+                option each key value is written separately. 
+  -E <string>   output string, by repeating this option each string 
+                is written to a separately. If the string contains 
+                keys between square brackets, they are replaced with 
+                the corresponding header values, if possible. 
   -s            print filename, chain key, block key and keywords in 
                 a single line separated by <tab>, if no keywords are 
                 given print file name, chain key and all block keys 
@@ -49,6 +53,8 @@ DESCRIPTION
                 omit filename, chain key, block key
   -i            ignore non-edf files
   -I            show names of edf files only 
+  -n            mark missing keys as "-"
+  -N            print a header label for each column, otherweise like -n
   -f            print file format
   -d <debug>    debug
 
@@ -67,7 +73,12 @@ HISTORY
   V1.11 2007-04-23 PB <string.h>, <ctype.h> included
   V1.12 2007-06-15 PB option -f (FileFormat) added
   V1.13 2008-05-27 PB file name added for option -f,
-                      -f included into help
+                      -f included in help
+  V1.16 2017-05-21 PB allow hex-numbers in -d <debug> 
+  V1.17 2017-05-22 PB option -n: print "-" as value of a missing key
+  V1.18 2018-02-05 PB option -N: print a header label for each column
+  V1.19 2018-02-08 PB allocate sizeof(char *), not sizeof(size_t),
+                      -c <ChainKey> corrected (Chainkey was not read)
 ---------------------------------------------------------------------------*/
 
 # include <errno.h>
@@ -83,15 +94,15 @@ HISTORY
 static const char * OptionStrings[] =
                      { "Invalid",
                        "-h",  "--help",  "-b",     "-c",     "-e",
-                       "-s",  "-S",      "-x",     "-i",     "-I",     
-                       "-p",  "-f", "-d", "--debug",
-                       (const char *) NULL };
+                       "-E",  "-s",      "-S",     "-x",     "-i",
+                       "-I",  "-p",      "-f",     "-n",     "-N",     
+                       "-d",  "--debug", (const char *) NULL };
 
 enum Option          { InValidOption,
                        Help1, Help2,     BlockKey, ChainKey, Keyword, 
-                       Short, VeryShort, ShortNoLF, Ignore, IgnoreShort, 
-                       Pipe, FileFormat, Debug1, Debug2,
-                       EndOption };
+                       String, Short, VeryShort, ShortNoLF,  Ignore, 
+                       IgnoreShort, Pipe, FileFormat, DoMark,   DoLabel,
+                       Debug1, Debug2, EndOption };
 
 /*---------------------------------------------------------------------------
 NAME
@@ -99,7 +110,7 @@ NAME
 
 SYNOPSIS
 
-  int opt2num( const char * string )
+  int opt2num( const char *string )
 
 DESCRIPTION
   Returns the option number according to OptionStrings
@@ -107,7 +118,7 @@ DESCRIPTION
 RETURN VALUE
   0: not found, otherwise option number
 ---------------------------------------------------------------------------*/
-int opt2num( const char * string )
+int opt2num( const char *string )
 {
   int NE=1,optno=0;
   while ( (NE && OptionStrings[optno]) )
@@ -126,9 +137,11 @@ void edfdumphelp( FILE * out, const char * progname, int verbose )
   if (!verbose) return;
   fprintf(out,"Data format version: %s\n",edf_dataformat_version());
   fprintf(out,"   -h : this help\n");
-  fprintf(out,"   -c <chainkey> -b <blockkey> -e <keyword>\n");
+  fprintf(out,"   -c <chainkey> -b <blockkey> -e <keyword> -E <string of replacement keys>\n");
   fprintf(out,"   -s : short format, -S : print keywords only\n");
   fprintf(out,"   -i : ignore non-edf files, -I : show edf file names\n");
+  fprintf(out,"   -n : do not ignore a missing key, print \"-\" instead\n");
+  fprintf(out,"   -N : write column labels\n");
   fprintf(out,"   -p : read file names from stdin (stop with <ctrl-d>)\n");
   fprintf(out,"   -f : print format (block start and end addresses)\n");
   fprintf(out,"\n");
@@ -144,40 +157,45 @@ NAME
 
 SYNOPSIS
 
-  int edfdump( char * fname[], int ignore, int format, 
-               char * chainkey[], char * blockkey[], char * keyword[] );
+  int edfdump( char *fname[], int ignore, int format, 
+               char *chainkey[], char *blockkey[], char *keyword[],
+               int domark, int dolabel );
 
 DESCRIPTION
   edfdump to stdout for all files in fname[], fname[] must stop with NULL
 
 ARGUMENTS
-  char * fname[]   : table of filenames, terminated with NULL
-  int ignore       :  1 : ignore non-edf files
-                      0 : normal
-  int format       : output format 
-  char * keyword[] : table of keywords, terminated with NULL
+  char *fname[]   : table of filenames, terminated with NULL
+  int ignore      :  1 : ignore non-edf files
+                     0 : normal
+  int format      : output format 
+  char *keyword[] : table of keywords, terminated with NULL
+  char *str[]     : table of replacement strings, terminated with NULL
 
 RETURN VALUE
   0, if OK, otherwise error
 ---------------------------------------------------------------------------*/
-int edfdump( char * fname[], int ignore, int format, 
-             char * chainkey[], char * blockkey[], char * keyword[] )
-{int status, ErrorValue;
- int stream;
- int return_status=0; 
+int edfdump( char *fname[], int ignore, int format, 
+             char *chainkey[], char *blockkey[], char *keyword[], char *str[],
+             int domark, int dolabel )
+{ int status, ErrorValue;
+  int stream;
+  int return_status=0; 
 
- for (;*fname!=(char*) NULL;fname++) {
-   stream = edf_open_data_file ( *fname, "read", &ErrorValue, &status );
-   if (status) {
+  for (;*fname!=(char*) NULL;fname++) {
+    stream = edf_open_data_file ( *fname, "read", &ErrorValue, &status );
+    if (status) {
       if (!ignore) {
         return_status=ErrorValue;
         fprintf(stderr,"%s: no edf file\r\n", *fname); 
       }
       // release allocated memory
       edf_free_all ( );
-   } else {
+    } else {
       if (format==1001) fprintf(stdout,"%s\r\n", *fname);
-        else edf_dump ( stdout, stream, format, chainkey, blockkey, keyword );
+        else {
+          edf_dump ( stdout, stream, format, chainkey, blockkey, keyword, str, domark, dolabel );
+        }
 
 //     edf_print_filetable ( stdout, 4, 0 ); //++++++++++++++++++++
 
@@ -187,15 +205,16 @@ int edfdump( char * fname[], int ignore, int format,
          fprintf(stderr,"%s\n",edf_report_data_error(ErrorValue));
          return(return_status); 
        }
-     }
-   }
+    }
+    dolabel=0; // write only a single label line
+  }
 
- return(return_status);
+  return(return_status);
 
 } // edfdump
 
 /* removes trailing line feed \n */
-char * trimlf ( char * s )
+char *trimlf ( char * s )
 { size_t slen;
   if ( s ) {
     slen = strlen( s );
@@ -230,10 +249,14 @@ main
 
 int MAIN (int argc, char *argv[])
 { char *nul = (char *) NULL;
-  char **ckeyv, **bkeyv, **keyv;
-  char **pargv, **pckeyv, **pbkeyv, **pkeyv;
-  int optno, ckeyc, bkeyc, keyc;
+  char **pargv;
+  char **ckeyv=NULL, **bkeyv=NULL, **keyv=NULL, **strv=NULL;
+  char **pckeyv=NULL, **pbkeyv=NULL, **pkeyv=NULL, **pstrv=NULL;
+  int optno;
+  int ckeyc=1, bkeyc=1, keyc=1, strc=1;
   int format=0, ignore=0, pipe=0;
+  int domark=0;
+  int dolabel=0;
   int status=-1;
 
   char *pstring[2];
@@ -243,68 +266,79 @@ int MAIN (int argc, char *argv[])
 
   // count options or help
   pargv = &argv[1];
-  ckeyc = 1;
-  bkeyc = 1;
-  keyc  = 1;
   for (;*pargv!=nul && (**pargv=='-');*pargv++) {
     switch ( opt2num( *pargv ) ) {
-       case   Help1  :
-       case   Help2  : edfdumphelp( stdout, argv[0], 1 ); 
-                       status=-2; return(status);
-       case ChainKey : if (pargv[1]!=nul) { ckeyc++; *(++pargv); } break;
-       case BlockKey : if (pargv[1]!=nul) { bkeyc++; *(++pargv); } break;
-       case Keyword  : if (pargv[1]!=nul) { keyc++; *(++pargv); } break; 
-       default       : break;
-      }
+      case   Help1  :
+      case   Help2  : edfdumphelp( stdout, argv[0], 1 ); 
+                      status=-2; return(status);
+      case ChainKey : if (pargv[1]!=nul) { ckeyc++; *(++pargv); } break;
+      case BlockKey : if (pargv[1]!=nul) { bkeyc++; *(++pargv); } break;
+      case Keyword  : if (pargv[1]!=nul) { keyc++; *(++pargv); } break; 
+      case String   : if (pargv[1]!=nul) { strc++; *(++pargv); } break; 
+      default       : break;
     }
+  }
 
   // allocate memory for chainkeys
-  if (!(ckeyv = (char **) malloc( sizeof(size_t) * ckeyc ))) {
-    perror("malloc( sizeof(size_t) * ckeyc )");
+  if (!(ckeyv = (char **) malloc( sizeof(char *) *ckeyc ))) {
+    perror("malloc( sizeof(char *) * ckeyc )");
     return(status);
-    }
+  }
   // allocate memory for blockkeys
-  if (!(bkeyv = (char **) malloc( sizeof(size_t) * bkeyc ))) {
-    perror("malloc( sizeof(size_t) * bkeyc )");
+  if (!(bkeyv = (char **) malloc( sizeof(char *) *bkeyc ))) {
+    perror("malloc( sizeof(char *) * bkeyc )");
     return(status);
-    }
+  }
   // allocate memory for keywords 
-  if (!(keyv = (char **) malloc( sizeof(size_t) * keyc ))) {
-    perror("malloc( sizeof(size_t) * keyc )");
+  if (!(keyv = (char **) malloc( sizeof(char *) *keyc ))) {
+    perror("malloc( sizeof(char *) * keyc )");
     return(status);
-    }
+  }
+  // allocate memory for strings
+  if (!(strv = (char **) malloc( sizeof(char *) *strc ))) {
+    perror("malloc( sizeof(char *) *strc )");
+    return(status);
+  }
 
   // get options
   pargv  = &argv[1];
   pckeyv = ckeyv;
   pbkeyv = bkeyv;
   pkeyv  = keyv;
+  pstrv  = strv;
   for (;*pargv!=nul && (**pargv=='-');*pargv++) {
     optno = opt2num( *pargv );
     switch ( optno ) {
-       case Help1     :
-       case Help2     : break;
-       case Debug1    :
-       case   Debug2 : if (pargv[1]!=nul) {
-           edfio_debug( num_str2long ( *(++pargv), NULL, NULL) );
-         } break;
-       case BlockKey  : if (pargv[1]!=nul) *pbkeyv++ = *(++pargv); break;
-       case Keyword   : if (pargv[1]!=nul) *pkeyv++  = *(++pargv); break;
-       case Short     : format=1; break; 
-       case VeryShort : format=2; break;
-       case ShortNoLF : format=6; break;
-       case Ignore    : ignore=1; break;
-       case IgnoreShort : ignore=1; format=1001; break;
-       case Pipe      : pipe=1; break;
-       case FileFormat: format=10001; break;
-       default        : break;
-      }
+      case Help1     :
+      case Help2     : break;
+      case Debug1    :
+      case   Debug2 : if (pargv[1]!=nul) {
+        double debug;
+        debug=num_str2double ( *(++pargv), NULL, NULL);
+        edfio_debug_set( (long) debug );
+      } break;
+      case ChainKey  : if (pargv[1]!=nul) *pckeyv++ = *(++pargv); break;
+      case BlockKey  : if (pargv[1]!=nul) *pbkeyv++ = *(++pargv); break;
+      case Keyword   : if (pargv[1]!=nul) *pkeyv++  = *(++pargv); break;
+      case String    : if (pargv[1]!=nul) *pstrv++  = *(++pargv); break; 
+      case Short     : format=1; break; 
+      case VeryShort : format=2; break;
+      case ShortNoLF : format=4; break;
+      case Ignore    : ignore=1; break;
+      case IgnoreShort : ignore=1; format=1001; break;
+      case Pipe      : pipe=1; break;
+      case FileFormat: format=3072; break;
+      case DoMark    : domark=1; break;
+      case DoLabel   : domark=1; dolabel=1; break;
+      default        : break;
     }
+  }
   *pckeyv = nul;
   *pbkeyv = nul;
   *pkeyv  = nul;
+  *pstrv  = nul;
 
-  if (*pargv) status=edfdump( pargv, ignore, format, ckeyv, bkeyv, keyv );
+  if (*pargv) status=edfdump( pargv, ignore, format, ckeyv, bkeyv, keyv, strv, domark, dolabel );
   else if (pipe) {
     // read from standard input
     pstring[1] = (char *) NULL;
@@ -318,7 +352,7 @@ int MAIN (int argc, char *argv[])
         // edfdump 
 
         if (!isempty (pstring[0])) {
-          status=edfdump( pstring, ignore, format, ckeyv, bkeyv, keyv );
+          status=edfdump( pstring, ignore, format, ckeyv, bkeyv, keyv, strv, domark, dolabel );
         } else fprintf( stdout,"\n");
       }
     }
@@ -330,8 +364,8 @@ int MAIN (int argc, char *argv[])
   free( ckeyv );
   free( bkeyv );
   free( keyv );
+  free( strv );
 
   return( status );
 
 } // main
-
