@@ -78,9 +78,17 @@
   2007-04-23 PB V4.60 compatibility to lcc
   2007-06-11 PB V4.61 ImageLoop parameters updated
   2012-01-31 PB V4.62 ImageLoop parameters updated
+  2017-07-19 PB V4.63 Usage updated
+                      rapid2saxs: updated for reading scalers
+                      and timing data from h5-files:
+                      writing delta_time as HMDeltaTime,
+                      writing Intensity0ShutCor as Intensity0,
+                      writing Intensity1ShutCor as Intensity1.
+  2017-10-20 PB V4.64 ReadHeaderFloat: parameter num added
+                      Header must be read explicitly
 
 ---*/
-# define Version  "rapid2saxs V4.62 2012-01-31, Peter Boesecke"
+# define Version  "rapid2saxs V4.64 2017-10-20, Peter Boesecke"
 
 # include <errno.h>
 # include <stdio.h>
@@ -89,7 +97,20 @@
 
 # include "SaxsPrograms.h"
 
-# define Usage "[options] <i1nam> <onam> <odum> <i2mem> <i3mem> <bibo>"
+# define STRLEN(s) (s == (char *) NULL ? 0 : strlen ( (char *) s))
+
+# define Usage "[options] <i1nam> <onam> <odum> <i2mem> <i3mem> <bibo>\n\
+         (<i2> : scalers, <i3> : timing)"
+
+# define EXAMPLES "\n\
+  Examples:\n\
+  - Conversion of h5 raw data files to edf \n\
+    rapid2saxs +pass \n\
+               -i2mem 1 -i2nam '[ScalersFilePath]/[ScalersFileName]'\n\
+               -i3mem 1 -i3nam '[ScalersFilePath]/[ScalersFileName]'\n\
+               <h5 raw data file> '[FBASE].edf'\n\
+  - Summing n=100 subsequent raw data frames, e.g. eiger data\n\
+    rapid2saxs +pass +add 100 <ehf raw data file> '[FBASE]_sum.edf.gz'"
 
 # define BlockNum 4       /* 3 input sequences + 1 output sequence */
 
@@ -97,7 +118,6 @@ static const char * _BOrderStrings[4] = { "UnSpecified",
                                           "LowByteFirst", "HighByteFirst",
                                           (const char *) NULL };
 
-
 /*---------------------------------------------------------------------------
 NAME
 
@@ -274,6 +294,10 @@ void rapid2saxs (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
   float I3Dummy, I3DDummy;
   float I3Value;
 
+  int scaler_image_number=1;
+  int timing_image_number=1;
+  char KeyValue[IO_size], TmpValue[IO_size];
+
   int i_1, i_2;
   float f_1[BlockNum], f_2[BlockNum];
 
@@ -290,12 +314,12 @@ void rapid2saxs (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
 
   long  frameno=1;
   long  scalerno;
-  char  scalerkey[IO_len];
+  char  scalerkey[IO_size];
   float value, scalervalue;
   int   scalerok;
-  char  timinginfo[IO_len];
-  char  buffer[IO_len];
-  float hmdeltatime, hmexposuretime;
+  char  timinginfo[IO_size];
+  char  buffer[IO_size];
+  float hmdeltatime, hmexposuretime, hmstartepoch;
   float intensity0, intensity1;
   float tmp;
 
@@ -389,23 +413,30 @@ void rapid2saxs (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
       if (*pstatus!=Success) return;
 
       // HMStartTime
-      if (starttime(buffer, IO_len, ihb[1].Title.V)) {
-        CopyLine(buffer,ihb[0].Time.V,IO_len,0);
+      if (starttime(buffer, IO_size, ihb[1].Title.V)) {
+        CopyLine(buffer,ihb[0].Time.V,IO_size,0);
         ihb[0].Time.I = TRUE;
       }
+    } else {
+      // Force reading the output header, because it was updated
+      ReadHeader ( pcb, 0, num, ihb, pstatus);
+      if (*pstatus!=Success) return;
     } // ImageIsNew
+
   } else return; // no further action, if image does not exist
   
-  /* write scalers into header */
-  if (ihb[2].I) {
+  /* write scalers to header */
+  if (ihb[2].I) scaler_image_number=2; // read scaler values from ihb[2]
+  else scaler_image_number=1; // from header of first input image
 
+  if ((scaler_image_number==2)&&(ihb[2].Dim[1]>0)) { // read scalers from ihb[2].Data
     I2Data   = ihb[2].Data;
     I2Dummy  = ihb[2].Dummy.V;
     I2DDummy = ihb[2].DDummy.V;
     I2Dim_1  = (int) ihb[2].Dim[1];
     I2Dim_2  = (int) ihb[2].Dim[2];
 
-    for (i_2=0;i_2<I2Dim_2;i_2++) { // read scalers from ihb[2]
+    for (i_2=0;i_2<I2Dim_2;i_2++) {
       scalerno = i_2+1;
       i_1 = frameno-1;
       if ((i_1>=0)&&(i_1<I2Dim_1)) {
@@ -414,7 +445,7 @@ void rapid2saxs (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
         // update already existing scaler value
         I2Value = I2Dummy;
         if (!ImageIsNew( 0, ihb )) 
-          if (ReadHeaderFloat ( pcb, 0, ihb, scalerkey, &I2Value, pstatus ))
+          if (ReadHeaderFloat ( pcb, 0, num, ihb, scalerkey, &I2Value, pstatus ))
             if (*pstatus!=Success) return;
 
         pI2Data = ABSPTR(I2Data,I2Dim_1,I2Dim_2,frameno-1,i_2);
@@ -425,20 +456,21 @@ void rapid2saxs (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
         if (*pstatus!=Success) return;
       }
     }
-  } else { // read scaler values from header until a value is missing
+  } else { // read scaler values from ihb[scaler_image_number] header until a value is missing
     scalerok=1;
+
     for (scalerno=1;scalerok;scalerno++) {
 
       sprintf(scalerkey,"%s%s%02u",KHScaler,KHC,scalerno);
 
       // read scaler value
-      if (ReadHeaderFloat ( pcb, 1, ihb, scalerkey, &scalervalue, pstatus)) {
+      if (ReadHeaderFloat ( pcb, scaler_image_number, num, ihb, scalerkey, &scalervalue, pstatus)) {
         if (*pstatus!=Success) return;
 
         // update output scaler value
         value = 0.0;
         if (!ImageIsNew( 0, ihb ))
-          if (ReadHeaderFloat ( pcb, 0, ihb, scalerkey, &value, pstatus ))
+          if (ReadHeaderFloat ( pcb, 0, num, ihb, scalerkey, &value, pstatus ))
             if (*pstatus!=Success) return;
 
         value += scalervalue;
@@ -449,9 +481,11 @@ void rapid2saxs (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
     }
   }
 
-  /* write timing into header */
-  /* only DeltaTime possible */
-  if (ihb[3].I) {
+  /* write timing to header */
+  if (ihb[3].I) timing_image_number=3; // read timing values from ihb[3]
+  else timing_image_number=1; // from header of first input image
+
+  if ((timing_image_number==3)&&(ihb[3].Dim[1]>0)) { // read timing from ihb[3].Data
     I3Data   = ihb[3].Data;
     I3Dummy  = ihb[3].Dummy.V;
     I3DDummy = ihb[3].DDummy.V;
@@ -464,7 +498,7 @@ void rapid2saxs (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
       if ((i_1>=0)&&(i_1<I3Dim_1)) {
  
         pI3Data = ABSPTR(I3Data,I3Dim_1,I3Dim_2,frameno-1,i_2);
-        if (strlen(timinginfo)<IO_len-30)
+        if (strlen(timinginfo)<IO_size-30)
           sprintf(timinginfo,"%s%15.7g ",timinginfo,*pI3Data);
       }
     }
@@ -485,7 +519,7 @@ void rapid2saxs (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
     if (tmp>=0.0) {
       hmexposuretime=0.0;
       if (!ImageIsNew( 0, ihb ))
-        if (ReadHeaderFloat ( pcb, 0, ihb,
+        if (ReadHeaderFloat ( pcb, 0, num, ihb,
                               KValueTime, &hmexposuretime, pstatus ))
           if (*pstatus!=Success) return;
       hmexposuretime+=tmp;
@@ -493,16 +527,16 @@ void rapid2saxs (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
       if (*pstatus!=Success) return;
     }
 
-  } else { // read ExposureTime from header 
+  } else { // read ExposureTime, DeltaTime from ihb[3].header 
     // ExposureTime
-    if (ReadHeaderFloat ( pcb, 1, ihb,
+    if (ReadHeaderFloat ( pcb, timing_image_number, num, ihb,
                           KValueTime, &tmp, pstatus )) {
       if (*pstatus!=Success) return;
 
       if (tmp>=0.0) {
         hmexposuretime=0.0;
         if (!ImageIsNew( 0, ihb ))
-          if (ReadHeaderFloat ( pcb, 0, ihb,
+          if (ReadHeaderFloat ( pcb, 0, num, ihb,
                                 KValueTime, &hmexposuretime, pstatus ))
             if (*pstatus!=Success) return;
         hmexposuretime+=tmp;
@@ -510,17 +544,99 @@ void rapid2saxs (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
         if (*pstatus!=Success) return;
       }
     }
+
+    // HMDeltaTime
+    if (ReadHeaderFloat ( pcb, timing_image_number, num, ihb,
+                          KHDeltaTime, &hmdeltatime, pstatus )) {
+      if (*pstatus!=Success) return;
+
+      if (hmdeltatime>=0.0) {
+        tmp=0.0;
+        if (!ImageIsNew( 0, ihb )) {
+          if (ReadHeaderFloat ( pcb, 0, num, ihb,
+                                KHDeltaTime, &tmp, pstatus ))
+            if (*pstatus!=Success) return;
+          if (tmp<hmdeltatime) hmdeltatime=tmp; // use the earliest time
+        }
+       
+        WriteHeaderFloat(pcb,0,ihb,KHDeltaTime, hmdeltatime, pstatus);
+        if (*pstatus!=Success) return;
+      }
+    } else if (ReadHeaderFloat ( pcb, timing_image_number, num, ihb,
+                          "delta_time", &hmdeltatime, pstatus )) {
+      if (*pstatus!=Success) return;
+
+      if (hmdeltatime>=0.0) {
+        tmp=0.0;
+        if (!ImageIsNew( 0, ihb )) {
+          if (ReadHeaderFloat ( pcb, 0, num, ihb, 
+                                KHDeltaTime, &tmp, pstatus ))
+            if (*pstatus!=Success) return;
+          if (tmp<hmdeltatime) hmdeltatime=tmp; // use the earliest time
+        }
+
+        WriteHeaderFloat(pcb,0,ihb,KHDeltaTime, hmdeltatime, pstatus);
+        if (*pstatus!=Success) return;
+      }
+    }
+
+    // HMStartTime (KHStartTime)
+    if (ReadHeaderLine ( pcb, timing_image_number, num, ihb, 
+                         KHStartTime, KeyValue, pstatus )) {
+      if (*pstatus!=Success) return;
+
+      if (STRLEN(KeyValue)>0) {
+        TmpValue[0]='\0';
+        if (!ImageIsNew( 0, ihb )) {
+          if (ReadHeaderLine ( pcb, 0, num, ihb, 
+                               KHStartTime, TmpValue, pstatus ))
+            if (*pstatus!=Success) return;
+          if ( strcmp(TmpValue,KeyValue)<0 ) { // use the earliest time
+            strcpy(KeyValue,TmpValue);
+          }
+        }
+
+        WriteHeaderLine(pcb,0,ihb,KHStartTime, KeyValue, pstatus);
+        if (*pstatus!=Success) return;
+      }
+    }
+
+    // HMStartEpoch
+    if (ReadHeaderFloat ( pcb, timing_image_number, num, ihb,
+                          KHStartEpoch, &hmstartepoch, pstatus )) {
+      if (*pstatus!=Success) return;
+
+      if (hmstartepoch>=0.0) {
+        tmp=0.0;
+        if (!ImageIsNew( 0, ihb )) {
+          if (ReadHeaderFloat ( pcb, 0, num, ihb,
+                                KHStartEpoch, &tmp, pstatus ))
+            if (*pstatus!=Success) return;
+          if (tmp<hmstartepoch) hmstartepoch=tmp; // use the earliest time
+        }
+
+        WriteHeaderFloat(pcb,0,ihb,KHStartEpoch, hmstartepoch, pstatus);
+        if (*pstatus!=Success) return;
+      }
+    }
   }
 
   // Intensity0
-  if (ReadHeaderFloat ( pcb, 1, ihb,
+  if (ReadHeaderFloat ( pcb, scaler_image_number, num, ihb,
                         KValueI0, &tmp, pstatus )) {
     if (*pstatus!=Success) return;
+    // Use Intensity0ShutCor, if it exists 
+    float tmp1;
+    if (ReadHeaderFloat ( pcb, scaler_image_number, num, ihb,
+                          "Intensity0ShutCor", &tmp1, pstatus )) {
+      if (*pstatus!=Success) return;
+      tmp=tmp1;
+    }
 
     if (tmp>=0.0) {
       intensity0=0.0;
       if (!ImageIsNew( 0, ihb ))
-        if (ReadHeaderFloat ( pcb, 0, ihb,
+        if (ReadHeaderFloat ( pcb, 0, num, ihb,
                               KValueI0, &intensity0, pstatus ))
           if (*pstatus!=Success) return;
       intensity0+=tmp;
@@ -530,14 +646,21 @@ void rapid2saxs (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
   }
 
   // Intensity1
-  if (ReadHeaderFloat ( pcb, 1, ihb,
+  if (ReadHeaderFloat ( pcb, scaler_image_number, num, ihb,
                         KValueI1, &tmp, pstatus )) {
     if (*pstatus!=Success) return;
+    // Use Intensity1ShutCor, if it exists
+    float tmp1;
+    if (ReadHeaderFloat ( pcb, scaler_image_number, num, ihb,
+                          "Intensity1ShutCor", &tmp1, pstatus )) {
+      if (*pstatus!=Success) return;
+      tmp=tmp1;
+    }
 
     if (tmp>=0.0) {
-      intensity1=0.0;
+      intensity0=0.0;
       if (!ImageIsNew( 0, ihb ))
-        if (ReadHeaderFloat ( pcb, 0, ihb,
+        if (ReadHeaderFloat ( pcb, 0, num, ihb,
                               KValueI1, &intensity1, pstatus ))
           if (*pstatus!=Success) return;
       intensity1+=tmp;
@@ -598,10 +721,10 @@ AUTHOR
 03-Mar-1998 PB Specification
   -------------------------------------------------------------------------*/
 char * outputname_default( char *buffer, size_t buflen, const char *pattern )
-{ char name[IO_len];
+{ char name[IO_size];
 
   // copy filename to name 
-  filename_name ( name, IO_len, pattern );
+  filename_name ( name, IO_size, pattern );
 
   // default only for bsl-type header names
   if ( (strlen(name)==10) && (buflen>=7) &&
@@ -628,16 +751,16 @@ Do all the keyboard io and return cb, and ib
 ---------------------------------------------------------------------------*/
 void user_io(CmdBlk * pcb, ImgBlk * ib, int * pstatus)
 {
-  char  progfn[InputLineLength];
+  char  progfn[IO_size];
   ImgHeadBlk ihb[BlockNum];
 
   float ROff_1, RPs_1, UOff_1, UPs_1;
   float ROff_2, RPs_2, UOff_2, UPs_2;
 
-  char outputname[IO_len];
+  char outputname[IO_size];
 
  /* Determine program name without directory */
-   (void) RightFR((char *) pcb->argv[0],DirSeparator,progfn,InputLineLength);
+   (void) RightFR((char *) pcb->argv[0],DirSeparator,progfn,IO_size);
 
   /* show usage if no arguments are given */
   if (pcb->argc<=1) printf("Usage: %s %s\n",progfn,Usage);
@@ -654,7 +777,7 @@ void user_io(CmdBlk * pcb, ImgBlk * ib, int * pstatus)
 
   /*--- 2nd input file name (scalers) */
   if (!ib[2].Name.I) {
-    CopyLine(ib[1].Name.V,ib[2].Name.V,IO_len,0);
+    CopyLine(ib[1].Name.V,ib[2].Name.V,IO_size,0);
     ib[2].Name.I = TRUE;
     if (!ib[2].First.I) { ib[2].First.V = 1; ib[2].First.I = TRUE; }
     if (!ib[2].Inc.I) { ib[2].Inc.V = 0; ib[2].Inc.I = TRUE; }
@@ -664,7 +787,7 @@ void user_io(CmdBlk * pcb, ImgBlk * ib, int * pstatus)
 
   /*--- 3rd input file name (timing) */
   if (!ib[3].Name.I) {
-    CopyLine(ib[1].Name.V,ib[3].Name.V,IO_len,0);
+    CopyLine(ib[1].Name.V,ib[3].Name.V,IO_size,0);
     ib[3].Name.I = TRUE;
     if (!ib[3].First.I) { ib[3].First.V = 1; ib[3].First.I = TRUE; }
     if (!ib[3].Inc.I) { ib[3].Inc.V = 0; ib[3].Inc.I = TRUE; }
@@ -674,7 +797,7 @@ void user_io(CmdBlk * pcb, ImgBlk * ib, int * pstatus)
 
   /*--- Argument  : output file name */
   argv_filename(pcb,"Output file name",ib,0,
-    outputname_default( outputname, IO_len, ib[1].Name.V ), pstatus);
+    outputname_default( outputname, IO_size, ib[1].Name.V ), pstatus);
   if (*pstatus!=Success) return; 
 
   /*--- OpenMode of input file */
@@ -765,7 +888,7 @@ int MAIN (int argc, char *argv[])
   int arg_no = 0;
 
  /* Init options, control block and image blocks */
-  InitOptions( Usage, Version, NULL, TRUE, &cb, ib, BlockNum );
+  InitOptions( Usage EXAMPLES, Version, NULL, TRUE, &cb, ib, BlockNum );
 
  /* defaults */
   ib[2].Memory.V = 2l;
