@@ -18,7 +18,7 @@
 /*    You should have received a copy of the GNU General Public License    */
 /*    along with this program.  If not, see <http://www.gnu.org/licenses/>.*/
 /***************************************************************************/
-# define Version  "V1.6 2013-12-09 Peter Boesecke"
+# define Version  "V1.7 2017-05-21 Peter Boesecke"
 /*---------------------------------------------------------------------------
 NAME
    roca - calibrate detector rotation angles
@@ -166,6 +166,7 @@ HISTORY
   V1.5 2011-07-21 Peter Boesecke new makefile, is_white etc. replaced with
                                  strlib functions
   V1.6 2013-12-09 Peter Boesecke clean up
+  V1.7 2017-05-21 Peter Boesecke ioalloc
 ---------------------------------------------------------------------------*/
 
 /****************************************************************************
@@ -502,11 +503,11 @@ int MAIN (int argc, char *argv[])
   // create table with start parameters for  "pix1","pix2","cen1", etc.
   if ( search_list( "start", &list ) ) goto main_error;
 
-  if ( !(ParStart=(double *) malloc (sizeof(double)*NParams)) ) goto main_error;
-  if ( !(ParMode=(int *) malloc (sizeof(int)*NParams)) ) goto main_error;
-  if ( !(ParOpt=(double *) malloc (sizeof(double)*NParams)) ) goto main_error;
-  if ( !(ParDev=(double *) malloc (sizeof(double)*NParams)) ) goto main_error;
-  if ( !(ParStat=(double *) malloc (sizeof(double)*NStats)) ) goto main_error;
+  if ( !(ParStart=(double *) MALLOC (sizeof(double)*NParams)) ) goto main_error;
+  if ( !(ParMode=(int *) MALLOC (sizeof(int)*NParams)) ) goto main_error;
+  if ( !(ParOpt=(double *) MALLOC (sizeof(double)*NParams)) ) goto main_error;
+  if ( !(ParDev=(double *) MALLOC (sizeof(double)*NParams)) ) goto main_error;
+  if ( !(ParStat=(double *) MALLOC (sizeof(double)*NStats)) ) goto main_error;
 
   cnt=0;
   for ( num=0; num<NParams; num++ ) {
@@ -540,10 +541,10 @@ int MAIN (int argc, char *argv[])
     goto main_error;
   }
 
-  if ( !(TwoTheta=(double *) malloc (sizeof(double)*ndata)) ) goto main_error;
-  if ( !(Coord1=(double *) malloc (sizeof(double)*ndata)) ) goto main_error;
-  if ( !(Coord2=(double *) malloc (sizeof(double)*ndata)) ) goto main_error;
-  if ( !(Weight=(double *) malloc (sizeof(double)*ndata)) ) goto main_error;
+  if ( !(TwoTheta=(double *) MALLOC (sizeof(double)*ndata)) ) goto main_error;
+  if ( !(Coord1=(double *) MALLOC (sizeof(double)*ndata)) ) goto main_error;
+  if ( !(Coord2=(double *) MALLOC (sizeof(double)*ndata)) ) goto main_error;
+  if ( !(Weight=(double *) MALLOC (sizeof(double)*ndata)) ) goto main_error;
 
   if ( first_element( list, &element ) ) goto main_error;
 
