@@ -57,7 +57,7 @@
   <odim_2>            : <number of angular intervals>
 
   The image coordinate of the center is (<i1cen_1>, <i1cen_2>). The input 
-  data array is radially cut into <odim_1> rings with the thicknesses <dr>.  
+  data array is radially cut in <odim_1> rings with the thicknesses <dr>.  
   The rings are defined in the following way:
   0 <= o_1 <= (odim_1-1) from <minimum radius> + o_1 * <radial interval> to
   <minimum radius> + (o_1+1) * <radial interval>
@@ -142,7 +142,7 @@
   2005-10-08  PB  V4.14 ImageIsNew added
   2005-10-08  PB  V4.15 angle_sum
   2005-10-13  PB  V4.16 Xy
-  2005-10-14  PB  V4.17 write DisplacedDim_... into displacement files
+  2005-10-14  PB  V4.17 write DisplacedDim_... to displacement files
   2005-10-28  PB  V4.18 ERROUT
   2007-04-23  PB  V4.19 void SetOutputDimension, 
                         bug in function concat correction (avoids bufferovfl)
@@ -160,9 +160,11 @@
   2012-01-31  PB  V4.30 ImageLoop parameters updated
   2012-06-27  PB  V4.31 lcc workaround
   2015-10-03  PB  V4.32 FileAlreadyOpen
+  2017-05-21  PB  V4.33 ioalloc
+  2017-10-19  PB  V4.34 OpenImageFile
 
 ---*/
-# define Version  "saxs_angle V4.32 2015-10-03, Peter Boesecke"
+# define Version  "saxs_angle V4.34 2017-10-19, Peter Boesecke"
 
 # include <errno.h>
 # include <stdio.h>
@@ -173,8 +175,7 @@
 # include "SaxsPrograms.h"
 # include "angle.h"
 # include "strlib.h"
-
-# define FREE( a) { if ( a) { free( a); a = NULL; } }
+# include "ioalloc.h"
 
 /* Defaults */
 # define Default_dim1  300
@@ -415,13 +416,13 @@ void saxs_angle_init( CmdBlk * pcb, long num [], ImgHeadBlk ihb[],
   SetOutputDimension( pcb, ihb, 1, pstatus );
   if (*pstatus!=Success) return;
 
-  R0 = option_float ( SR0, pcb, pcb->ib, num, 1, pstatus );
+  R0 = option_float ( SR0, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
-  Dr = option_float ( SDr, pcb, pcb->ib, num, 1, pstatus );
+  Dr = option_float ( SDr, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
-  A0 = option_float ( SA0, pcb, pcb->ib, num, 1, pstatus );
+  A0 = option_float ( SA0, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
-  Da = option_float ( SDa, pcb, pcb->ib, num, 1, pstatus );
+  Da = option_float ( SDa, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
 
   S2 = option_flag ( SS2, pcb, pstatus );
@@ -680,7 +681,7 @@ void saxs_angle_init( CmdBlk * pcb, long num [], ImgHeadBlk ihb[],
            pihb->BinSiz[2].V = 1.0; pihb->BinSiz[2].I = TRUE;
            }
          if (pihb->Offset[2].I>=0) {
-           // write R2OFFSET into two lines, lcc workaround
+           // write R2OFFSET to two lines, lcc workaround
            pihb->Offset[2].V = R2OFFSET(a0,\
              pihb->PixSiz[2].V);
            pihb->Offset[2].I = TRUE;
@@ -765,9 +766,9 @@ void saxs_angle (CmdBlk * pcb, long num [], ImgHeadBlk ihb[], int * pstatus)
   IO_long  S2, Csym, Vsum;
 
  /* xyfile parameters */
-  char xypath[IO_len], xyname[IO_len];
-  char xy_pattern[IO_len], xyfull[IO_len];
-  char xfile[IO_len], yfile[IO_len];
+  char xypath[IO_size], xyname[IO_size];
+  char xy_pattern[IO_size], xyfull[IO_size];
+  char xfile[IO_size], yfile[IO_size];
   int xstream, ystream;
   float *xdata, *ydata;
   float *pxdata, *pydata;
@@ -779,7 +780,7 @@ void saxs_angle (CmdBlk * pcb, long num [], ImgHeadBlk ihb[], int * pstatus)
   IO_long Xy;
   IO_line Xyname;
 
-  char key[IO_len];
+  char key[IO_size];
 
   float I0Off_1, I0Ps_1;
   float I0Off_2, I0Ps_2;
@@ -809,7 +810,7 @@ void saxs_angle (CmdBlk * pcb, long num [], ImgHeadBlk ihb[], int * pstatus)
             pcb->ImgBlkLen); *pstatus=Failed; return; }
 
  /* --- Get option values */
-  Asym = option_float ( SAsym, pcb, pcb->ib, num, 1, pstatus );
+  Asym = option_float ( SAsym, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
   S2   = option_flag ( SS2, pcb, pstatus );
   if (*pstatus!=Success) return;
@@ -820,7 +821,7 @@ void saxs_angle (CmdBlk * pcb, long num [], ImgHeadBlk ihb[], int * pstatus)
 
   Xy = option_flag( SXy, pcb, pstatus);
   if (*pstatus!=Success) return;
-  Xyname = option_line ( SXyname, pcb, pcb->ib, num, 1, pstatus );
+  Xyname = option_line ( SXyname, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
 
   /* input block */
@@ -948,11 +949,11 @@ void saxs_angle (CmdBlk * pcb, long num [], ImgHeadBlk ihb[], int * pstatus)
   xstream   = ystream   = -1;
   xdata     = ydata     = (float *) NULL;
   if (Xy.V) {
-    filename_parameter (xy_pattern,IO_len,Xyname.V,0);
-    filename_pattern (xy_pattern,IO_len,xy_pattern,pcb->ib[0].FileNumber);
-    filename_full ( xyfull, IO_len, xy_pattern );
-    filename_path ( xypath, IO_len, xyfull );
-    filename_name ( xyname, IO_len, xyfull );
+    filename_parameter (xy_pattern,IO_size,Xyname.V,0);
+    filename_pattern (xy_pattern,IO_size,xy_pattern,pcb->ib[0].FileNumber);
+    filename_full ( xyfull, IO_size, xy_pattern );
+    filename_path ( xypath, IO_size, xyfull );
+    filename_name ( xyname, IO_size, xyfull );
     xysize = edf_dim_product ( ihb[1].Dim ); // identical to input array
     /* open x-file */
     sprintf(xfile,"%s%s%s",xypath,"x",xyname);
@@ -964,7 +965,7 @@ void saxs_angle (CmdBlk * pcb, long num [], ImgHeadBlk ihb[], int * pstatus)
     SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
     if (*pstatus!=Success) return;
     if (xysize>0) {
-      if (!(xdata = (float *) malloc(sizeof(float)*xysize))) {
+      if (!(xdata = (float *) MALLOC(sizeof(float)*xysize))) {
         *pstatus = NotEnoughMemoryAvailable; return; }
       pxdata = xdata;
       for (idata=0;idata<xysize;idata++) *(pxdata++)=0.0;
@@ -979,7 +980,7 @@ void saxs_angle (CmdBlk * pcb, long num [], ImgHeadBlk ihb[], int * pstatus)
     SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
     if (*pstatus!=Success) {  FREE(xdata); return; }
     if (xysize>0) {
-      if (!(ydata = (float *) malloc(sizeof(float)*xysize))) {
+      if (!(ydata = (float *) MALLOC(sizeof(float)*xysize))) {
         FREE(xdata); *pstatus = NotEnoughMemoryAvailable; return; }
       pydata = ydata;
       for (idata=0;idata<xysize;idata++) *(pydata++)=0.0;
@@ -1112,59 +1113,59 @@ void saxs_angle (CmdBlk * pcb, long num [], ImgHeadBlk ihb[], int * pstatus)
 
    /* Write parameters of transformed image to x-displacement file */
     edf_write_header_float(xstream,xynum,1l,
-                strlib_concat(key,IO_len,KDisplaced,"Dim_1"),ihb[0].Dim[1],
+                strlib_concat(key,IO_size,KDisplaced,"Dim_1"),ihb[0].Dim[1],
                 &ErrorValue, pstatus );
     SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
     if (*pstatus!=Success) { FREE(xdata); FREE(ydata); return; }
     edf_write_header_float(xstream,xynum,1l,
-                strlib_concat(key,IO_len,KDisplaced,"Dim_2"),ihb[0].Dim[2],
+                strlib_concat(key,IO_size,KDisplaced,"Dim_2"),ihb[0].Dim[2],
                 &ErrorValue, pstatus );
     SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
     if (*pstatus!=Success) { FREE(xdata); FREE(ydata); return; }
     edf_write_header_float(xstream,xynum,1l,
-                strlib_concat(key,IO_len,KDisplaced,KOffset_1),ihb[0].Offset[1].V,
+                strlib_concat(key,IO_size,KDisplaced,KOffset_1),ihb[0].Offset[1].V,
                 &ErrorValue, pstatus );
     SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
     if (*pstatus!=Success) { FREE(xdata); FREE(ydata); return; }
     edf_write_header_float(xstream,xynum,1l,
-                strlib_concat(key,IO_len,KDisplaced,KOffset_2),ihb[0].Offset[2].V,
+                strlib_concat(key,IO_size,KDisplaced,KOffset_2),ihb[0].Offset[2].V,
                 &ErrorValue, pstatus );
     SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
     if (*pstatus!=Success) { FREE(xdata); FREE(ydata); return; }
     edf_write_header_float(xstream,xynum,1l,
-                 strlib_concat(key,IO_len,KDisplaced,KBinSiz_1),ihb[0].BinSiz[1].V,
+                 strlib_concat(key,IO_size,KDisplaced,KBinSiz_1),ihb[0].BinSiz[1].V,
                  &ErrorValue, pstatus );
     SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
     if (*pstatus!=Success) { FREE(xdata); FREE(ydata); return; }
     edf_write_header_float(xstream,xynum,1l,
-                 strlib_concat(key,IO_len,KDisplaced,KBinSiz_2),ihb[0].BinSiz[2].V,
+                 strlib_concat(key,IO_size,KDisplaced,KBinSiz_2),ihb[0].BinSiz[2].V,
                  &ErrorValue, pstatus );
     SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
     if (*pstatus!=Success) { FREE(xdata); FREE(ydata); return; }
     edf_write_header_float(xstream,xynum,1l,
-                 strlib_concat(key,IO_len,KDisplaced,KPixSiz_1),ihb[0].PixSiz[1].V,
+                 strlib_concat(key,IO_size,KDisplaced,KPixSiz_1),ihb[0].PixSiz[1].V,
                  &ErrorValue, pstatus );
     SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
     if (*pstatus!=Success) { FREE(xdata); FREE(ydata); return; }
     edf_write_header_float(xstream,xynum,1l,
-                 strlib_concat(key,IO_len,KDisplaced,KPixSiz_2),ihb[0].PixSiz[2].V,
+                 strlib_concat(key,IO_size,KDisplaced,KPixSiz_2),ihb[0].PixSiz[2].V,
                  &ErrorValue, pstatus );
     SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
     if (*pstatus!=Success) { FREE(xdata); FREE(ydata); return; }
     edf_write_header_float(xstream,xynum,1l,
-                 strlib_concat(key,IO_len,KDisplaced,KCenter_1),ihb[0].Center[1].V,
+                 strlib_concat(key,IO_size,KDisplaced,KCenter_1),ihb[0].Center[1].V,
                  &ErrorValue, pstatus );
     SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
     if (*pstatus!=Success) { FREE(xdata); FREE(ydata); return; }
     edf_write_header_float(xstream,xynum,1l,
-                 strlib_concat(key,IO_len,KDisplaced,KCenter_2),ihb[0].Center[2].V,
+                 strlib_concat(key,IO_size,KDisplaced,KCenter_2),ihb[0].Center[2].V,
                  &ErrorValue, pstatus );
     SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
     if (*pstatus!=Success) { FREE(xdata); FREE(ydata); return; }
 
    if ( (pcb->RSys.V == IO_Tangens) || (pcb->RSys.V == IO_Saxs) ) {
       edf_write_header_float(xstream,xynum,1l,
-        strlib_concat(key,IO_len,KDisplaced,KSampleDistance),ihb[0].SampleDistance.V,
+        strlib_concat(key,IO_size,KDisplaced,KSampleDistance),ihb[0].SampleDistance.V,
         &ErrorValue, pstatus );
       SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
       if (*pstatus!=Success) { FREE(xdata); FREE(ydata); return; }
@@ -1172,7 +1173,7 @@ void saxs_angle (CmdBlk * pcb, long num [], ImgHeadBlk ihb[], int * pstatus)
 
    if ( pcb->RSys.V == IO_Saxs ) {
       edf_write_header_float(xstream,xynum,1l,
-        strlib_concat(key,IO_len,KDisplaced,KWaveLength),ihb[0].WaveLength.V,
+        strlib_concat(key,IO_size,KDisplaced,KWaveLength),ihb[0].WaveLength.V,
         &ErrorValue, pstatus );
       SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
       if (*pstatus!=Success) { FREE(xdata); FREE(ydata); return; }
@@ -1251,59 +1252,59 @@ void saxs_angle (CmdBlk * pcb, long num [], ImgHeadBlk ihb[], int * pstatus)
 
    /* Write parameters of transformed image to y-displacement file */
     edf_write_header_float(ystream,xynum,1l,
-                strlib_concat(key,IO_len,KDisplaced,"Dim_1"),ihb[0].Dim[1],
+                strlib_concat(key,IO_size,KDisplaced,"Dim_1"),ihb[0].Dim[1],
                 &ErrorValue, pstatus );
     SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
     if (*pstatus!=Success) { FREE(ydata); return; }
     edf_write_header_float(ystream,xynum,1l,
-                strlib_concat(key,IO_len,KDisplaced,"Dim_2"),ihb[0].Dim[2],
+                strlib_concat(key,IO_size,KDisplaced,"Dim_2"),ihb[0].Dim[2],
                 &ErrorValue, pstatus );
     SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
     if (*pstatus!=Success) { FREE(ydata); return; }
     edf_write_header_float(ystream,xynum,1l,
-                strlib_concat(key,IO_len,KDisplaced,KOffset_1),ihb[0].Offset[1].V,
+                strlib_concat(key,IO_size,KDisplaced,KOffset_1),ihb[0].Offset[1].V,
                 &ErrorValue, pstatus );
     SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
     if (*pstatus!=Success) { FREE(ydata); return; }
     edf_write_header_float(ystream,xynum,1l,
-                strlib_concat(key,IO_len,KDisplaced,KOffset_2),ihb[0].Offset[2].V,
+                strlib_concat(key,IO_size,KDisplaced,KOffset_2),ihb[0].Offset[2].V,
                 &ErrorValue, pstatus );
     SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
     if (*pstatus!=Success) { FREE(ydata); return; }
     edf_write_header_float(ystream,xynum,1l,
-                strlib_concat(key,IO_len,KDisplaced,KBinSiz_1),ihb[0].BinSiz[1].V,
+                strlib_concat(key,IO_size,KDisplaced,KBinSiz_1),ihb[0].BinSiz[1].V,
                 &ErrorValue, pstatus );
     SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
     if (*pstatus!=Success) { FREE(ydata); return; }
     edf_write_header_float(ystream,xynum,1l,
-                strlib_concat(key,IO_len,KDisplaced,KBinSiz_2),ihb[0].BinSiz[2].V,
+                strlib_concat(key,IO_size,KDisplaced,KBinSiz_2),ihb[0].BinSiz[2].V,
                 &ErrorValue, pstatus );
     SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
     if (*pstatus!=Success) { FREE(ydata); return; }
     edf_write_header_float(ystream,xynum,1l,
-                 strlib_concat(key,IO_len,KDisplaced,KPixSiz_1),ihb[0].PixSiz[1].V,
+                 strlib_concat(key,IO_size,KDisplaced,KPixSiz_1),ihb[0].PixSiz[1].V,
                  &ErrorValue, pstatus );
     SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
     if (*pstatus!=Success) { FREE(ydata); return; }
     edf_write_header_float(ystream,xynum,1l,
-                 strlib_concat(key,IO_len,KDisplaced,KPixSiz_2),ihb[0].PixSiz[2].V,
+                 strlib_concat(key,IO_size,KDisplaced,KPixSiz_2),ihb[0].PixSiz[2].V,
                  &ErrorValue, pstatus );
     SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
     if (*pstatus!=Success) { FREE(ydata); return; }
     edf_write_header_float(ystream,xynum,1l,
-                 strlib_concat(key,IO_len,KDisplaced,KCenter_1),ihb[0].Center[1].V,
+                 strlib_concat(key,IO_size,KDisplaced,KCenter_1),ihb[0].Center[1].V,
                  &ErrorValue, pstatus );
     SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
     if (*pstatus!=Success) { FREE(ydata); return; }
     edf_write_header_float(ystream,xynum,1l,
-                 strlib_concat(key,IO_len,KDisplaced,KCenter_2),ihb[0].Center[2].V,
+                 strlib_concat(key,IO_size,KDisplaced,KCenter_2),ihb[0].Center[2].V,
                  &ErrorValue, pstatus );
     SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
     if (*pstatus!=Success) { FREE(ydata); return; }
 
     if ( (pcb->RSys.V == IO_Tangens) || (pcb->RSys.V == IO_Saxs) ) {
       edf_write_header_float(ystream,xynum,1l,
-        strlib_concat(key,IO_len,KDisplaced,KSampleDistance),ihb[0].SampleDistance.V,
+        strlib_concat(key,IO_size,KDisplaced,KSampleDistance),ihb[0].SampleDistance.V,
         &ErrorValue, pstatus );
       SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
       if (*pstatus!=Success) { FREE(ydata); return; }
@@ -1311,7 +1312,7 @@ void saxs_angle (CmdBlk * pcb, long num [], ImgHeadBlk ihb[], int * pstatus)
 
     if ( pcb->RSys.V == IO_Saxs ) {
       edf_write_header_float(ystream,xynum,1l,
-        strlib_concat(key,IO_len,KDisplaced,KWaveLength),ihb[0].WaveLength.V,
+        strlib_concat(key,IO_size,KDisplaced,KWaveLength),ihb[0].WaveLength.V,
         &ErrorValue, pstatus );
       SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
       if (*pstatus!=Success) { FREE(ydata); return; }
@@ -1342,13 +1343,13 @@ Ask for all file names in the following way:
 
   output dummy [<dummy in first image>] :
 
-The header block of the first image is read into ihb[1].
+The header block of the first image is read in ihb[1].
 ---------------------------------------------------------------------------*/
 void ArgvAngleFilenames ( CmdBlk * pcb, ImgBlk ib[], ImgHeadBlk ihb[],
                           int * pstatus)
 { int blkno;
   long int image_1st, image_lst;
-  char tmp_line[IO_len];
+  char tmp_line[IO_size];
   float Bin_1, Bin_2;
   long num[BlockNum];
   
@@ -1369,7 +1370,9 @@ void ArgvAngleFilenames ( CmdBlk * pcb, ImgBlk ib[], ImgHeadBlk ihb[],
     /* Exit in all cases after an abort from the first argument */
     if (*pstatus!=Success) return;
 
-    OpenImageFile(pcb,ib,blkno,ib[blkno].Name.V,ib[1].FFirst.V,
+    /* No key value replacement -> ihb=NULL */
+    OpenImageFile(pcb,ib,blkno,NULL,
+                  ib[blkno].Name.V,ib[1].FFirst.V,
                   IO_Old|IO_FileProtect,pstatus);
     if (*pstatus!=Success) return;
 
@@ -1435,7 +1438,7 @@ void ArgvAngleFilenames ( CmdBlk * pcb, ImgBlk ib[], ImgHeadBlk ihb[],
 
     /* Defaults for output file */
     sprintf(tmp_line,"%g",ihb[1].Dummy.V);
-    if (!ib[0].Dummy.I) CopyLine(tmp_line,ib[0].Dummy.V,IO_len,0);
+    if (!ib[0].Dummy.I) CopyLine(tmp_line,ib[0].Dummy.V,IO_size,0);
 
     /*--- Argument : output dummy */
     sprintf(tmp_line,"Output dummy, e.g. %s",ib[0].Dummy.V);
@@ -1455,10 +1458,10 @@ void user_io(CmdBlk * pcb, ImgBlk * ib, int * pstatus)
 {
   const char * InputRefSys  = "Input reference system is";
 
-  char  progfn[IO_len];
+  char  progfn[IO_size];
   ImgHeadBlk ihb[BlockNum];
 
-  char tmp_line[IO_len];
+  char tmp_line[IO_size];
 
   float ROff_1, RPs_1, UOff_1, UPs_1;
   float ROff_2, RPs_2, UOff_2, UPs_2;
@@ -1466,7 +1469,7 @@ void user_io(CmdBlk * pcb, ImgBlk * ib, int * pstatus)
   IO_flexp *pR0, *pDr, *pA0, *pDa;
 
  /* Determine program name without directory */
-   (void) RightFR((char *) pcb->argv[0],DirSeparator,progfn,InputLineLength);
+   (void) RightFR((char *) pcb->argv[0],DirSeparator,progfn,IO_size);
 
   /* show usage if no arguments are given */
   if (pcb->argc<=1) printf("Usage: %s %s\n",progfn,Usage);
