@@ -39,7 +39,7 @@
 
   The output dimensions are automatically calculated. The option
   -odim = <number of azimuths/arc> sets the number of azimuthal sections
-  (arc sections), -odim = 1 regroups the input image into a single line.
+  (arc sections), -odim = 1 regroups the input image to a single line.
 
   If the input and output projection types are different and if the reference
   system is IO_Saxs the output image is calculated taking into account the
@@ -104,9 +104,12 @@
   2014-02-07  PB  V4.37 saxs_arc: waxs_Init parameters updated
                         ang_range, arc_sum, ang_sum parameters updated.
   2015-10-03  PB  V4.38 FileAlreadyOpen
+  2017-04-19  PB  V4.39 avoiding option duplication: dbg => mdbg
+  2017-05-21  PB  V4.40 ioalloc
+  2017-10-19  PB  V4.41 OpenImageFile 
 
 ---*/
-# define Version  "saxs_arc V4.38 2014-02-07, Peter Boesecke"
+# define Version  "saxs_arc V4.41 2017-10-19, Peter Boesecke"
 
 # include <errno.h>
 # include <stdio.h>
@@ -118,8 +121,7 @@
 # include "arc.h"
 # include "waxs.h"
 # include "strlib.h"
-
-# define FREE( a) { if ( a) { free( a); a = NULL; } }
+# include "ioalloc.h"
 
 /* Defaults */
 # define Default_da deg2rad  // 1_deg
@@ -139,7 +141,7 @@
 
 # define SWaxs     "waxs"  /* flag */ // take detector rotations into acount
 # define SMask     "mask"  /* flag */ // mask input image before regrouping
-# define SDbg      "dbg"   /* flag */ // write mask file for debugging 
+# define SDbg      "mdbg"  /* flag */ // write mask file for debugging 
 
 # define Usage "[options] \n\
                 <i1nam> <onam> <i1fst> <i1lst> <i1inc> <odum> \n\
@@ -743,9 +745,9 @@ void saxs_arc_init( CmdBlk * pcb, long num [], ImgHeadBlk ihb[],
   if (pcb->TestBit) printf("saxs_arc_init START\n");
 
   // Get options
-  R0 = option_float ( SR0, pcb, pcb->ib, num, 1, pstatus );
+  R0 = option_float ( SR0, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) goto saxs_arc_init_end;
-  A0 = option_float ( SA0, pcb, pcb->ib, num, 1, pstatus );
+  A0 = option_float ( SA0, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) goto saxs_arc_init_end;
 
   S2 = option_flag ( SS2, pcb, pstatus );
@@ -841,9 +843,9 @@ void saxs_arc (CmdBlk * pcb, long num [], ImgHeadBlk ihb[], int * pstatus)
   float AngleMin, AngleMax;
 
  /* xyfile parameters */
-  char xypath[IO_len], xyname[IO_len];
-  char xy_pattern[IO_len], xyfull[IO_len];
-  char xfile[IO_len], yfile[IO_len];
+  char xypath[IO_size], xyname[IO_size];
+  char xy_pattern[IO_size], xyfull[IO_size];
+  char xfile[IO_size], yfile[IO_size];
   int xstream, ystream;
   float *xdata, *ydata;
   float *pxdata, *pydata;
@@ -858,7 +860,7 @@ void saxs_arc (CmdBlk * pcb, long num [], ImgHeadBlk ihb[], int * pstatus)
   IO_long Waxs;
   IO_long Mask;
 
-  char key[IO_len];
+  char key[IO_size];
 
   float I0Off_1, I0Ps_1;
   float I0Off_2, I0Ps_2;
@@ -888,10 +890,10 @@ void saxs_arc (CmdBlk * pcb, long num [], ImgHeadBlk ihb[], int * pstatus)
 //            pcb->ImgBlkLen); *pstatus=Failed; return; }
 
  /* --- Get other option values */
-  A0 = option_float ( SA0, pcb, pcb->ib, num, 1, pstatus );
+  A0 = option_float ( SA0, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
 
-  Asym = option_float ( SAsym, pcb, pcb->ib, num, 1, pstatus );
+  Asym = option_float ( SAsym, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
   S2   = option_flag ( SS2, pcb, pstatus );
   if (*pstatus!=Success) return;
@@ -905,7 +907,7 @@ void saxs_arc (CmdBlk * pcb, long num [], ImgHeadBlk ihb[], int * pstatus)
 
   Xy = option_flag( SXy, pcb, pstatus);
   if (*pstatus!=Success) return;
-  Xyname = option_line ( SXyname, pcb, pcb->ib, num, 1, pstatus );
+  Xyname = option_line ( SXyname, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
 
   Waxs = option_flag( SWaxs, pcb, pstatus);
@@ -1104,11 +1106,11 @@ void saxs_arc (CmdBlk * pcb, long num [], ImgHeadBlk ihb[], int * pstatus)
   xstream   = ystream   = -1;
   xdata     = ydata     = (float *) NULL;
   if (Xy.V) {
-    filename_parameter (xy_pattern,IO_len,Xyname.V,0);
-    filename_pattern (xy_pattern,IO_len,xy_pattern,pcb->ib[0].FileNumber);
-    filename_full ( xyfull, IO_len, xy_pattern );
-    filename_path ( xypath, IO_len, xyfull );
-    filename_name ( xyname, IO_len, xyfull );
+    filename_parameter (xy_pattern,IO_size,Xyname.V,0);
+    filename_pattern (xy_pattern,IO_size,xy_pattern,pcb->ib[0].FileNumber);
+    filename_full ( xyfull, IO_size, xy_pattern );
+    filename_path ( xypath, IO_size, xyfull );
+    filename_name ( xyname, IO_size, xyfull );
     xysize = edf_dim_product ( ihb[1].Dim ); // identical to input array
     /* open x-file */
     sprintf(xfile,"%s%s%s",xypath,"x",xyname);
@@ -1120,7 +1122,7 @@ void saxs_arc (CmdBlk * pcb, long num [], ImgHeadBlk ihb[], int * pstatus)
     SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
     if (*pstatus!=Success) return;
     if (xysize>0) {
-      if (!(xdata = (float *) malloc(sizeof(float)*xysize))) {
+      if (!(xdata = (float *) MALLOC(sizeof(float)*xysize))) {
         *pstatus = NotEnoughMemoryAvailable; return; }
       pxdata = xdata;
       for (idata=0;idata<xysize;idata++) *(pxdata++)=0.0;
@@ -1135,7 +1137,7 @@ void saxs_arc (CmdBlk * pcb, long num [], ImgHeadBlk ihb[], int * pstatus)
     SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
     if (*pstatus!=Success) {  FREE(xdata); return; }
     if (xysize>0) {
-      if (!(ydata = (float *) malloc(sizeof(float)*xysize))) {
+      if (!(ydata = (float *) MALLOC(sizeof(float)*xysize))) {
         FREE(xdata); *pstatus = NotEnoughMemoryAvailable; return; }
       pydata = ydata;
       for (idata=0;idata<xysize;idata++) *(pydata++)=0.0;
@@ -1288,64 +1290,64 @@ void saxs_arc (CmdBlk * pcb, long num [], ImgHeadBlk ihb[], int * pstatus)
 
    /* Write parameters of transformed image to x-displacement file */
     edf_write_header_float(xstream,xynum,1l,
-                strlib_concat(key,IO_len,KDisplaced,"Dim_1"),ihb[0].Dim[1],
+                strlib_concat(key,IO_size,KDisplaced,"Dim_1"),ihb[0].Dim[1],
                 &ErrorValue, pstatus );
     SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
     if (*pstatus!=Success) { FREE(xdata); FREE(ydata); return; }
     edf_write_header_float(xstream,xynum,1l,
-                strlib_concat(key,IO_len,KDisplaced,"Dim_2"),ihb[0].Dim[2],
+                strlib_concat(key,IO_size,KDisplaced,"Dim_2"),ihb[0].Dim[2],
                 &ErrorValue, pstatus );
     SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
     if (*pstatus!=Success) { FREE(xdata); FREE(ydata); return; }
     edf_write_header_float(xstream,xynum,1l,
-                strlib_concat(key,IO_len,KDisplaced,KOffset_1),ihb[0].Offset[1].V,
+                strlib_concat(key,IO_size,KDisplaced,KOffset_1),ihb[0].Offset[1].V,
                 &ErrorValue, pstatus );
     SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
     if (*pstatus!=Success) { FREE(xdata); FREE(ydata); return; }
     edf_write_header_float(xstream,xynum,1l,
-                strlib_concat(key,IO_len,KDisplaced,KOffset_2),ihb[0].Offset[2].V,
+                strlib_concat(key,IO_size,KDisplaced,KOffset_2),ihb[0].Offset[2].V,
                 &ErrorValue, pstatus );
     SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
     if (*pstatus!=Success) { FREE(xdata); FREE(ydata); return; }
     edf_write_header_float(xstream,xynum,1l,
-                 strlib_concat(key,IO_len,KDisplaced,KBinSiz_1),ihb[0].BinSiz[1].V,
+                 strlib_concat(key,IO_size,KDisplaced,KBinSiz_1),ihb[0].BinSiz[1].V,
                  &ErrorValue, pstatus );
     SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
     if (*pstatus!=Success) { FREE(xdata); FREE(ydata); return; }
     edf_write_header_float(xstream,xynum,1l,
-                 strlib_concat(key,IO_len,KDisplaced,KBinSiz_2),ihb[0].BinSiz[2].V,
+                 strlib_concat(key,IO_size,KDisplaced,KBinSiz_2),ihb[0].BinSiz[2].V,
                  &ErrorValue, pstatus );
     SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
     if (*pstatus!=Success) { FREE(xdata); FREE(ydata); return; }
     edf_write_header_float(xstream,xynum,1l,
-                 strlib_concat(key,IO_len,KDisplaced,KPixSiz_1),ihb[0].PixSiz[1].V,
+                 strlib_concat(key,IO_size,KDisplaced,KPixSiz_1),ihb[0].PixSiz[1].V,
                  &ErrorValue, pstatus );
     SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
     if (*pstatus!=Success) { FREE(xdata); FREE(ydata); return; }
     edf_write_header_float(xstream,xynum,1l,
-                 strlib_concat(key,IO_len,KDisplaced,KPixSiz_2),ihb[0].PixSiz[2].V,
+                 strlib_concat(key,IO_size,KDisplaced,KPixSiz_2),ihb[0].PixSiz[2].V,
                  &ErrorValue, pstatus );
     SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
     if (*pstatus!=Success) { FREE(xdata); FREE(ydata); return; }
     edf_write_header_float(xstream,xynum,1l,
-                 strlib_concat(key,IO_len,KDisplaced,KCenter_1),ihb[0].Center[1].V,
+                 strlib_concat(key,IO_size,KDisplaced,KCenter_1),ihb[0].Center[1].V,
                  &ErrorValue, pstatus );
     SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
     if (*pstatus!=Success) { FREE(xdata); FREE(ydata); return; }
     edf_write_header_float(xstream,xynum,1l,
-                 strlib_concat(key,IO_len,KDisplaced,KCenter_2),ihb[0].Center[2].V,
+                 strlib_concat(key,IO_size,KDisplaced,KCenter_2),ihb[0].Center[2].V,
                  &ErrorValue, pstatus );
     SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
     if (*pstatus!=Success) { FREE(xdata); FREE(ydata); return; }
 
     if ( pcb->RSys.V == IO_Saxs ) {
       edf_write_header_float(xstream,xynum,1l,
-        strlib_concat(key,IO_len,KDisplaced,KSampleDistance),ihb[0].SampleDistance.V,
+        strlib_concat(key,IO_size,KDisplaced,KSampleDistance),ihb[0].SampleDistance.V,
         &ErrorValue, pstatus );
       SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
       if (*pstatus!=Success) { FREE(xdata); FREE(ydata); return; }
       edf_write_header_float(xstream,xynum,1l,
-        strlib_concat(key,IO_len,KDisplaced,KWaveLength),ihb[0].WaveLength.V,
+        strlib_concat(key,IO_size,KDisplaced,KWaveLength),ihb[0].WaveLength.V,
         &ErrorValue, pstatus );
       SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
       if (*pstatus!=Success) { FREE(xdata); FREE(ydata); return; }
@@ -1353,7 +1355,7 @@ void saxs_arc (CmdBlk * pcb, long num [], ImgHeadBlk ihb[], int * pstatus)
 
     if ( I0Pro != IO_ProSaxs ) {
       edf_write_header_line(xstream,xynum,1l,
-        strlib_concat(key,IO_len,KDisplaced,KProjection),protostr(I0Pro),
+        strlib_concat(key,IO_size,KDisplaced,KProjection),protostr(I0Pro),
         &ErrorValue, pstatus );
       SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
       if (*pstatus!=Success) { FREE(xdata); FREE(ydata); return; }
@@ -1361,7 +1363,7 @@ void saxs_arc (CmdBlk * pcb, long num [], ImgHeadBlk ihb[], int * pstatus)
 
     if ( I0Axis_2 != IO_AxisTypeDistance ) {
       edf_write_header_line(xstream,xynum,1l,
-        strlib_concat(key,IO_len,KDisplaced,KAxisType_2),axistostr(I0Axis_2),
+        strlib_concat(key,IO_size,KDisplaced,KAxisType_2),axistostr(I0Axis_2),
         &ErrorValue, pstatus );
       SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
       if (*pstatus!=Success) { FREE(xdata); FREE(ydata); return; }
@@ -1440,71 +1442,71 @@ void saxs_arc (CmdBlk * pcb, long num [], ImgHeadBlk ihb[], int * pstatus)
 
    /* Write parameters of transformed image to y-displacement file */
     edf_write_header_float(ystream,xynum,1l,
-                strlib_concat(key,IO_len,KDisplaced,"Dim_1"),ihb[0].Dim[1],
+                strlib_concat(key,IO_size,KDisplaced,"Dim_1"),ihb[0].Dim[1],
                 &ErrorValue, pstatus );
     SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
     if (*pstatus!=Success) { FREE(ydata); return; }
     edf_write_header_float(ystream,xynum,1l,
-                strlib_concat(key,IO_len,KDisplaced,"Dim_2"),ihb[0].Dim[2],
+                strlib_concat(key,IO_size,KDisplaced,"Dim_2"),ihb[0].Dim[2],
                 &ErrorValue, pstatus );
     SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
     if (*pstatus!=Success) { FREE(ydata); return; }
     edf_write_header_float(ystream,xynum,1l,
-                strlib_concat(key,IO_len,KDisplaced,KOffset_1),ihb[0].Offset[1].V,
+                strlib_concat(key,IO_size,KDisplaced,KOffset_1),ihb[0].Offset[1].V,
                 &ErrorValue, pstatus );
     SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
     if (*pstatus!=Success) { FREE(ydata); return; }
     edf_write_header_float(ystream,xynum,1l,
-                strlib_concat(key,IO_len,KDisplaced,KOffset_2),ihb[0].Offset[2].V,
+                strlib_concat(key,IO_size,KDisplaced,KOffset_2),ihb[0].Offset[2].V,
                 &ErrorValue, pstatus );
     SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
     if (*pstatus!=Success) { FREE(ydata); return; }
     edf_write_header_float(ystream,xynum,1l,
-                strlib_concat(key,IO_len,KDisplaced,KBinSiz_1),ihb[0].BinSiz[1].V,
+                strlib_concat(key,IO_size,KDisplaced,KBinSiz_1),ihb[0].BinSiz[1].V,
                 &ErrorValue, pstatus );
     SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
     if (*pstatus!=Success) { FREE(ydata); return; }
     edf_write_header_float(ystream,xynum,1l,
-                strlib_concat(key,IO_len,KDisplaced,KBinSiz_2),ihb[0].BinSiz[2].V,
+                strlib_concat(key,IO_size,KDisplaced,KBinSiz_2),ihb[0].BinSiz[2].V,
                 &ErrorValue, pstatus );
     SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
     if (*pstatus!=Success) { FREE(ydata); return; }
     edf_write_header_float(ystream,xynum,1l,
-                 strlib_concat(key,IO_len,KDisplaced,KPixSiz_1),ihb[0].PixSiz[1].V,
+                 strlib_concat(key,IO_size,KDisplaced,KPixSiz_1),ihb[0].PixSiz[1].V,
                  &ErrorValue, pstatus );
     SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
     if (*pstatus!=Success) { FREE(ydata); return; }
     edf_write_header_float(ystream,xynum,1l,
-                 strlib_concat(key,IO_len,KDisplaced,KPixSiz_2),ihb[0].PixSiz[2].V,
+                 strlib_concat(key,IO_size,KDisplaced,KPixSiz_2),ihb[0].PixSiz[2].V,
                  &ErrorValue, pstatus );
     SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
     if (*pstatus!=Success) { FREE(ydata); return; }
     edf_write_header_float(ystream,xynum,1l,
-                 strlib_concat(key,IO_len,KDisplaced,KCenter_1),ihb[0].Center[1].V,
+                 strlib_concat(key,IO_size,KDisplaced,KCenter_1),ihb[0].Center[1].V,
                  &ErrorValue, pstatus );
     SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
     if (*pstatus!=Success) { FREE(ydata); return; }
     edf_write_header_float(ystream,xynum,1l,
-                 strlib_concat(key,IO_len,KDisplaced,KCenter_2),ihb[0].Center[2].V,
+                 strlib_concat(key,IO_size,KDisplaced,KCenter_2),ihb[0].Center[2].V,
                  &ErrorValue, pstatus );
     SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
     if (*pstatus!=Success) { FREE(ydata); return; }
 
     if ( pcb->RSys.V == IO_Saxs ) {
       edf_write_header_float(ystream,xynum,1l,
-        strlib_concat(key,IO_len,KDisplaced,KSampleDistance),ihb[0].SampleDistance.V,
+        strlib_concat(key,IO_size,KDisplaced,KSampleDistance),ihb[0].SampleDistance.V,
         &ErrorValue, pstatus );
       SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
       if (*pstatus!=Success) { FREE(ydata); return; }
       edf_write_header_float(ystream,xynum,1l,
-        strlib_concat(key,IO_len,KDisplaced,KWaveLength),ihb[0].WaveLength.V,
+        strlib_concat(key,IO_size,KDisplaced,KWaveLength),ihb[0].WaveLength.V,
         &ErrorValue, pstatus );
       SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
       if (*pstatus!=Success) { FREE(ydata); return; }
     }
     if ( I0Pro != IO_ProSaxs ) {
       edf_write_header_line(ystream,xynum,1l,
-        strlib_concat(key,IO_len,KDisplaced,KProjection),protostr(I0Pro),
+        strlib_concat(key,IO_size,KDisplaced,KProjection),protostr(I0Pro),
         &ErrorValue, pstatus );
       SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
       if (*pstatus!=Success) { FREE(ydata); return; }
@@ -1512,7 +1514,7 @@ void saxs_arc (CmdBlk * pcb, long num [], ImgHeadBlk ihb[], int * pstatus)
 
     if ( I0Axis_2 != IO_AxisTypeDistance ) {
       edf_write_header_line(ystream,xynum,1l,
-        strlib_concat(key,IO_len,KDisplaced,KAxisType_2),axistostr(I0Axis_2),
+        strlib_concat(key,IO_size,KDisplaced,KAxisType_2),axistostr(I0Axis_2),
         &ErrorValue, pstatus );
       SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
       if (*pstatus!=Success) { FREE(ydata); return; }
@@ -1543,13 +1545,13 @@ Ask for all file names in the following way:
 
   output dummy [<dummy in first image>] :
 
-The header block of the first image is read into ihb[1].
+The header block of the first image is read in ihb[1].
 ---------------------------------------------------------------------------*/
 void ArgvAngleFilenames ( CmdBlk * pcb, ImgBlk ib[], ImgHeadBlk ihb[],
                           int * pstatus)
 { int blkno;
   long int image_1st, image_lst;
-  char tmp_line[IO_len];
+  char tmp_line[IO_size];
   float Bin_1, Bin_2;
   long num[BlockNum];
   
@@ -1570,7 +1572,9 @@ void ArgvAngleFilenames ( CmdBlk * pcb, ImgBlk ib[], ImgHeadBlk ihb[],
     /* Exit in all cases after an abort from the first argument */
     if (*pstatus!=Success) return;
 
-    OpenImageFile(pcb,ib,blkno,ib[blkno].Name.V,ib[1].FFirst.V,
+    /* No key value replacement -> ihb=NULL */
+    OpenImageFile(pcb,ib,blkno,NULL,
+                  ib[blkno].Name.V,ib[1].FFirst.V,
                   IO_Old|IO_FileProtect,pstatus);
     if (*pstatus!=Success) return;
 
@@ -1636,7 +1640,7 @@ void ArgvAngleFilenames ( CmdBlk * pcb, ImgBlk ib[], ImgHeadBlk ihb[],
 
     /* Defaults for output file */
     sprintf(tmp_line,"%g",ihb[1].Dummy.V);
-    if (!ib[0].Dummy.I) CopyLine(tmp_line,ib[0].Dummy.V,IO_len,0);
+    if (!ib[0].Dummy.I) CopyLine(tmp_line,ib[0].Dummy.V,IO_size,0);
 
     /*--- Argument : output dummy */
     sprintf(tmp_line,"Output dummy, e.g. %s",ib[0].Dummy.V);
@@ -1656,10 +1660,10 @@ Do all the keyboard io and return cb, and ib
 void user_io(CmdBlk * pcb, ImgBlk * ib, int * pstatus)
 { 
 
-  char  progfn[IO_len];
+  char  progfn[IO_size];
   ImgHeadBlk ihb[BlockNum];
 
-  char tmp_line[IO_len];
+  char tmp_line[IO_size];
 
   float ROff_1, RPs_1, UOff_1, UPs_1;
   float ROff_2, RPs_2, UOff_2, UPs_2;
@@ -1672,7 +1676,7 @@ void user_io(CmdBlk * pcb, ImgBlk * ib, int * pstatus)
   IO_long Mask;
 
  /* Determine program name without directory */
-   (void) RightFR((char *) pcb->argv[0],DirSeparator,progfn,InputLineLength);
+   (void) RightFR((char *) pcb->argv[0],DirSeparator,progfn,IO_size);
 
   /* show usage if no arguments are given */
   if (pcb->argc<=1) printf("Usage: %s %s\n",progfn,Usage);
@@ -1846,7 +1850,7 @@ int MAIN (int argc, char *argv[])
       }
     } else {
       if (Mask.V) { // Use the path of the input file
-        CopyLine("[FPATH][MaskFileName]",ib[2].Name.V,IO_len,0);
+        CopyLine("[FPATH][MaskFileName]",ib[2].Name.V,IO_size,0);
         if (!ib[2].OpenMode.I) {
           ib[2].OpenMode.V = IO_Old | IO_FileProtect;
           ib[2].OpenMode.I = TRUE;
