@@ -67,7 +67,7 @@
   +-hd2               : enables/disables the i_2 coordinate printing
   +-abs               : absolut values of coordinates
   -scf <factor>       : multiplication of coordinates with float <factor>
-  +-hist              : writes/do not write history into header
+  +-hist              : writes/do not write history to header
   +-noi               : suppress/do not suppress image number extensions in 
                         output file
   +-waxs              : calculate exact scattering vector length from 
@@ -95,7 +95,7 @@
   2000-08-04  PB V3.5  %f->%g
   2000-08-15  PB V3.6  first column header changed
   2000-09-15  PB V3.7  option -hedl for head line string
-  2000-11-18  PB V3.8  WriteHistory, WriteHeader, Time keyword into header
+  2000-11-18  PB V3.8  WriteHistory, WriteHeader, Time keyword to header
                        new options: +/-hist (NHist), -len  (NHlen)
   2000-11-18  PB V3.9  default header line starts with #
   2001-03-02  PB V3.10 space before \r\n
@@ -133,7 +133,7 @@
                        SetSaxsErrorExternal added
   2005-08-04  PB V4.64 PrintImageHeaderBlock paramters updated
   2005-09-17  PB V4.65 VarDat
-  2006-07-26  PB V4.66 stddev written into headline of stddev file
+  2006-07-26  PB V4.66 stddev written to headline of stddev file
   2007-04-23  PB V4.67 for (cnt;...) -> for (;...)
   2007-06-11  PB V4.68 ImageLoop parameters updated
   2008-08-05  PB V4.69 SaxsAsciiArgvFilenames: Dummy, Dim_1 and Dim_2 
@@ -149,9 +149,11 @@
                        (because of if (num) it would have never be
                         executed)
   2016-02-17  PB V4.75 remove .gz suffix when creating output filename
+  2016-10-07  PB V4.66 filename_full expansion of Prefix option removed
+  2017-10-19  PB V4.67 OpenImageFile
 
 ---*/
-# define Version  "saxs_ascii V4.75 2016-02-17, Peter Boesecke"
+# define Version  "saxs_ascii V4.77 2017-10-19, Peter Boesecke"
 
 # include <errno.h>
 # include <stdio.h>
@@ -188,7 +190,7 @@
                 <i1nam> <onam> <i1fst> <i1lst> <i1inc>\n\
                 <odum> <odim1> <odim2>\n\
                 <header> <swap> <head_1> <head_2>\n\
-Purpose: Transformation into ASCII format\n\
+Purpose: Transformation to ASCII format\n\
 Options: +-head +-swap +-hd1 +-hd2 -hdmk <marker> -ext <extension> +-noi +-waxs"
 
 # define BlockNum 2       /* 1 input sequence + 1 output sequence */
@@ -217,14 +219,14 @@ Ask for all file names in the following way:
 
   output dummy [<dummy in first image>] :
 
-If blkno >= 1 the header block of the first image is read into ihb[1].
+If blkno >= 1 the header block of the first image is read in ihb[1].
 The routine must be used for all images at once.
 ---------------------------------------------------------------------------*/
 void SaxsAsciiArgvFilenames ( CmdBlk * pcb, ImgBlk ib[], ImgHeadBlk ihb[],
                      int block_1st, int block_lst, int * pstatus)
 { int blkno, first_block;
   long image_1st, image_lst;
-  char linebuffer[IO_len];
+  char linebuffer[IO_size];
   int nameblk;
   long num[BlockNum];
   IO_line * pPrefix;
@@ -249,7 +251,9 @@ void SaxsAsciiArgvFilenames ( CmdBlk * pcb, ImgBlk ib[], ImgHeadBlk ihb[],
     /* Exit in all cases after an abort from the first argument */
     if (*pstatus!=Success) return;
 
-    OpenImageFile( pcb,ib,blkno,ib[blkno].Name.V,ib[1].FFirst.V,
+    /* No key value replacement -> ihb=NULL */
+    OpenImageFile( pcb,ib,blkno,NULL,
+                   ib[blkno].Name.V,ib[1].FFirst.V,
                    IO_Old|IO_FileProtect,pstatus);
     if (*pstatus!=Success) return;
 
@@ -277,19 +281,19 @@ void SaxsAsciiArgvFilenames ( CmdBlk * pcb, ImgBlk ib[], ImgHeadBlk ihb[],
     if (!pPrefix->I) {
       /* Determine prefix from file name */
       /* Check for .gz extension (use pPrefix->V as temporary buffer) */
-      filename_extension ( pPrefix->V, IO_len, ib[nameblk].Name.V );
+      filename_extension ( pPrefix->V, IO_size, ib[nameblk].Name.V );
       if (!strcmp(pPrefix->V,".gz")) {
         // Remove .gz extension
         // remove .gz
-        filename_body ( pPrefix->V, IO_len, ib[nameblk].Name.V ); // remove .gz
+        filename_body ( pPrefix->V, IO_size, ib[nameblk].Name.V ); // remove .gz
         // remove extension
-        filename_body ( pPrefix->V, IO_len,  pPrefix->V );
+        filename_body ( pPrefix->V, IO_size,  pPrefix->V );
       } else {
         // remove extension
-        filename_body ( pPrefix->V, IO_len, ib[nameblk].Name.V );
+        filename_body ( pPrefix->V, IO_size, ib[nameblk].Name.V );
       }
       // remove path
-      filename_name ( pPrefix->V, IO_len, pPrefix->V );
+      filename_name ( pPrefix->V, IO_size, pPrefix->V );
       pPrefix->I = TRUE;
     } 
 
@@ -302,11 +306,8 @@ void SaxsAsciiArgvFilenames ( CmdBlk * pcb, ImgBlk ib[], ImgHeadBlk ihb[],
     extract_filenumbers( &ib[0], pPrefix, TRUE, pstatus );
     if (*pstatus!=Success) return;
 
-    /* Expand Prefix to the full file name */
-    if (!filename_full (pPrefix->V,IO_len,pPrefix->V)) {
-      fprintf(ERROUT,"Input file name or path not allowed\n");
-      *pstatus = FileNameError; return;
-    }
+    /* Prefix cannot be expanded with filename_full
+       before replacing all keyvalues */
 
     /*--- Take prefix as default for output filename */
     if (!ib[0].Name.I) {
@@ -399,7 +400,7 @@ NAME
   ascii_write_history
 
 DESCRIPTION
-  Writes the full history into the output file 'out'
+  Writes the full history to the output file 'out'
 
 RETURN VALUE
   Number of written lines
@@ -428,7 +429,7 @@ long ascii_write_history( FILE * out, CmdBlk * pcb, long num[], ImgHeadBlk ihb[]
   SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
   if (*pstatus!=Success) { edf_free_header( header_key ); return( cnt ); }
 
-  Hdmk = option_line ( SHdmk, pcb, pcb->ib, num, 1, pstatus );
+  Hdmk = option_line ( SHdmk, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) { edf_free_header( header_key ); return( cnt ); }
 
   edf_first_header_element ( header_key, &key, &value, &ErrorValue, pstatus );
@@ -469,13 +470,13 @@ long ascii_write_header( FILE *out, CmdBlk *pcb, long num[], ImgHeadBlk ihb[],
   IO_long cntmax, Hist;
   IO_line Hdmk;
 
-  Hdmk = option_line ( SHdmk, pcb, pcb->ib, num, 1, pstatus );
+  Hdmk = option_line ( SHdmk, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return( cnt );
 
   Hist = option_flag ( SHist, pcb, pstatus );
   if (*pstatus!=Success) return( cnt );
 
-  cntmax = option_long ( SHlen, pcb, pcb->ib, num, 1, pstatus ); // header lines
+  cntmax = option_long ( SHlen, pcb, ihb, num, 1, pstatus ); // header lines
   if (*pstatus!=Success) return( cnt );
 
   fprintf(out,"%s{\r\n",Hdmk.V); cnt++;
@@ -562,9 +563,9 @@ void saxs_ascii (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
   float Wmin_1, Wmax_1, W_1, DW_1;
   float Wmin_2, Wmax_2, W_2, DW_2;
 
-  char OutputFileName[InputLineLength];
+  char OutputFileName[IO_size];
   FILE *OutputFileId;
-  char OutputErrorFileName[InputLineLength];
+  char OutputErrorFileName[IO_size];
   FILE *OutputErrorFileId;
 
   IO_float Scf, *ptmp;
@@ -572,7 +573,7 @@ void saxs_ascii (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
   IO_long ihbPrint, Hist, i1Head, i2Head, Abs, Swap, NoImNum, Waxs;
 
   char *label=(char *) NULL, *nextlabel=(char *) NULL, *headl=(char *) NULL;
-  char fname_full[IO_len];
+  char fname_full[IO_size];
 
   /* boolean indicators */
   int SwapI, i1HeadI, i2HeadI, AbsI, WaxsI;
@@ -614,19 +615,19 @@ void saxs_ascii (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
                           &Wmin_1, &Wmax_1, &DW_1, &Wmin_2, &Wmax_2, &DW_2, 1);
 
  /* options */
-  Hdmk = option_line ( SHdmk, pcb, pcb->ib, num, 1, pstatus );
+  Hdmk = option_line ( SHdmk, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
 
-  Spacer = option_line ( SSpacer, pcb, pcb->ib, num, 1, pstatus );
+  Spacer = option_line ( SSpacer, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
 
-  HeadL = option_line ( SHeadL, pcb, pcb->ib, num, 1, pstatus );
+  HeadL = option_line ( SHeadL, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
 
-  Extension = option_line ( SExtension, pcb, pcb->ib, num, 1, pstatus );
+  Extension = option_line ( SExtension, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
 
-  Prefix = option_line ( SPrefix, pcb, pcb->ib, num, 1, pstatus );
+  Prefix = option_line ( SPrefix, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return; 
 
   ihbPrint = option_flag ( SihbPrint, pcb, pstatus );
@@ -653,7 +654,7 @@ void saxs_ascii (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
   Waxs = option_flag ( SWaxs, pcb, pstatus );
   if (*pstatus!=Success) return;
 
-  Scf = option_float ( SScf, pcb, pcb->ib, num, 1, pstatus );
+  Scf = option_float ( SScf, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return; 
 
  /* 1 input, 1 output */
@@ -714,9 +715,9 @@ void saxs_ascii (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
   /* construct the file name from Prefix.V and the image number */
   
   /* Generate filename of ASCII file */
-  filename_parameter (fname_full,IO_len,Prefix.V,0);
-  filename_pattern (fname_full,IO_len,fname_full,(pcb->ib)[0].FileNumber);
-  filename_full (fname_full,IO_len,fname_full);
+  filename_parameter (fname_full,IO_size,Prefix.V,0);
+  filename_pattern (fname_full,IO_size,fname_full,(pcb->ib)[0].FileNumber);
+  filename_full (fname_full,IO_size,fname_full);
 
   if ((NoImNum.V) && ImageIsFirst(pcb, 0, ihb ) && ImageIsLast(pcb, 0, ihb )) {
     /* suppress image number if not required */
@@ -739,7 +740,7 @@ void saxs_ascii (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
   if (E0Data) printf(", %s",OutputErrorFileName);
   printf("\n");
 
-  /* writing of the image header block into the ouput file */
+  /* writing of the image header block to the ouput file */
   if (ihbPrint.V) {
     /* Writing header */
     ascii_write_header( OutputFileId, pcb, num, ihb, pstatus );
@@ -966,18 +967,18 @@ Do all the keyboard io and return cb, and ib
 
 void user_io(CmdBlk * pcb, ImgBlk * ib, int * pstatus)
 {
-  char  progfn[InputLineLength];
+  char  progfn[IO_size];
   ImgHeadBlk ihb[BlockNum];
 
   float ROff_1, RPs_1, UOff_1, UPs_1;
   float ROff_2, RPs_2, UOff_2, UPs_2;
 
-  IO_line *pPrefix;
+//+++++++   IO_line *pPrefix;
   IO_long *pihbPrint, *pi1Head, *pi2Head, *pSwap;
 
  /* Determine program name without directory */
    (void) RightFR((char *) pcb->argv[0],DirSeparator,progfn,
-           InputLineLength);
+           IO_size);
 
   /* show usage if no arguments are given */
   if (pcb->argc<=1) printf("Usage: %s %s\n",progfn,Usage);
@@ -987,14 +988,17 @@ void user_io(CmdBlk * pcb, ImgBlk * ib, int * pstatus)
   printf("   %s %s\n",progfn,Version);
   printf("\n");
 
-  /* Expand Prefix to the full file name */
-  pPrefix = (IO_line*) option_parameter_search( SPrefix, pcb, pstatus );
-  if (*pstatus!=Success) return;
+  /* Prefix cannot be expanded with filename_full
+     before replacing all keyvalues */
 
-  if (!filename_full (pPrefix->V,IO_len,pPrefix->V)) {
-    fprintf(ERROUT,"% s. Input file name or path not allowed\n",progfn);
-    *pstatus = FileNameError; return;
-  }
+  //+++++++ /* Expand Prefix to the full file name */
+  //+++++++ pPrefix = (IO_line*) option_parameter_search( SPrefix, pcb, pstatus );
+  //+++++++ if (*pstatus!=Success) return;
+
+  //+++++++ if (!filename_full (pPrefix->V,IO_size,pPrefix->V)) {
+  //+++++++   fprintf(ERROUT,"% s. Input file name or path not allowed\n",progfn);
+  //+++++++   *pstatus = FileNameError; return;
+  //+++++++ }
  
   SaxsAsciiArgvFilenames ( pcb, ib, ihb, 0, BlockNum-1, pstatus);
   if (*pstatus!=Success) return;
