@@ -20,10 +20,10 @@
 /***************************************************************************/
 /*+++
 1 saxs_bsl
-  Transformation of an edf image into bsl format (daresbury otoko)
+  Transformation of an edf image to bsl format (daresbury otoko)
 
 2 PURPOSE
-  Transformation of an edf image into bsl format (daresbury otoko)
+  Transformation of an edf image to bsl format (daresbury otoko)
 
   Calculation:       I0 = I1
   Error propagation: V0 = V1 
@@ -51,7 +51,7 @@
 
 2 HISTORY
   28-Nov-1995 PB
-  21-May-1996 PB  option +bsl outputs into a single bsl binary file
+  21-May-1996 PB  option +bsl outputs to a single bsl binary file
   26-Jun-1999 PB  DevSeparator, DirSeparator, ExtSeparator defined in SaxsInput
   2000-08-04  PB  %f->%g
   2001-07-07  PB  IO_DontReadWrite
@@ -77,9 +77,10 @@
                         translation of \r\n to \n when compiled on windows.
   2012-01-31  PB  V4.62 ImageLoop parameters updated
   2014-03-05  PB  V4.63 Bsl output byte order added, option -bobo 1|2
+  2017-10-19  PB  V4.64 OpenImageFile
 
 ---*/
-# define Version  "saxs_bsl V4.63 2014-03-05, Peter Boesecke"
+# define Version  "saxs_bsl V4.64 2017-10-19, Peter Boesecke"
 
 # include <errno.h>
 # include <stdio.h>
@@ -113,18 +114,18 @@ in NNFlg     0: NN taken from second and third character
 ---------------------------------------------------------------------------*/
 void ToBslName ( char * sout, char * sin, int NNFlg )
 { register int i, k;
-  char extension[InputLineLength];
-  char name[InputLineLength];
+  char extension[IO_size];
+  char name[IO_size];
   char NN[3];
 
   /* conversion to capital letters */
-  CopyLine( sin, sout, InputLineLength, 1);
+  CopyLine( sin, sout, IO_size, 1);
   /* extraction of extension */
-  RightFL(sout,".",extension,InputLineLength);
+  RightFL(sout,".",extension,IO_size);
   for (i=strlen(extension);i<3;i++) extension[i] = 'B';
   extension[3] = (char) 0;
   /* extraction of number NN*/
-  LeftFL(sout,".",name,InputLineLength);
+  LeftFL(sout,".",name,IO_size);
   for (i=0;i<2;i++) NN[i]='0';NN[2]=(char) 0;
   for (k=1,i=strlen(name)-1;(k>=0) && (i>=0);k--,i--)
     if ( (i>0) && 
@@ -159,10 +160,10 @@ Replacement of nnn in the string BNNnnn.BSL
 ---------------------------------------------------------------------------*/
 void BslFileName ( char * sout, const char * sin, int nnn )
 { int ddd;
-  char Temp1[IO_len], Temp2[IO_len];
+  char Temp1[IO_size], Temp2[IO_size];
 
-  filename_parameter(Temp1,IO_len,sin,0);
-  filename_name(Temp2, IO_len,Temp1);
+  filename_parameter(Temp1,IO_size,sin,0);
+  filename_name(Temp2, IO_size,Temp1);
   CopyLine( Temp2, Temp2, 11, 1);
 
   nnn = nnn%1000; if (nnn<0) nnn = nnn+1000;
@@ -171,7 +172,7 @@ void BslFileName ( char * sout, const char * sin, int nnn )
   ddd = nnn%10; nnn = nnn / 10; Temp2[4] = '0'+ (char) ddd;
   ddd = nnn%10; nnn = nnn / 10; Temp2[3] = '0'+ (char) ddd;
 
-  sprintf(sout,"%s%s", filename_path(Temp1,IO_len,Temp1),Temp2);
+  sprintf(sout,"%s%s", filename_path(Temp1,IO_size,Temp1),Temp2);
 
 } /* BslFileName */
 
@@ -190,15 +191,15 @@ Ask for all file names in the following way:
 
   output dummy [<dummy in first image>] :
 
-If blkno >= 1 the header block of the first image is read into ihb[1].
+If blkno >= 1 the header block of the first image is read to ihb[1].
 The routine must be used for all images at once.
 ---------------------------------------------------------------------------*/
 void Saxs2BslFilenames ( CmdBlk * pcb, ImgBlk ib[], ImgHeadBlk ihb[],
                      int block_1st, int block_lst, int * pstatus)
 { int blkno, first_block;
   long image_1st, image_lst;
-  char linebuffer[IO_len];
-  char Path[IO_len], Name[IO_len];
+  char linebuffer[IO_size];
+  char Path[IO_size], Name[IO_size];
   long num[BlockNum];
 
   IO_line * pPrefix;
@@ -223,7 +224,9 @@ void Saxs2BslFilenames ( CmdBlk * pcb, ImgBlk ib[], ImgHeadBlk ihb[],
     /* Exit in all cases after an abort from the first argument */
     if (*pstatus!=Success) return;
 
-    OpenImageFile( pcb,ib,blkno,ib[blkno].Name.V,ib[1].FFirst.V,
+    /* No key value replacement -> ihb=NULL */
+    OpenImageFile( pcb,ib,blkno,NULL,
+                   ib[blkno].Name.V,ib[1].FFirst.V,
                    IO_Old|IO_FileProtect,pstatus);
     if (*pstatus!=Success) return;
 
@@ -253,7 +256,7 @@ void Saxs2BslFilenames ( CmdBlk * pcb, ImgBlk ib[], ImgHeadBlk ihb[],
 
     if (!pPrefix->I) {
       /* Determine prefix and extension from input file name */
-      filename_name(pPrefix->V,IO_len,ib[1].Name.V);
+      filename_name(pPrefix->V,IO_size,ib[1].Name.V);
       pPrefix->I = TRUE;
       }
 
@@ -262,9 +265,9 @@ void Saxs2BslFilenames ( CmdBlk * pcb, ImgBlk ib[], ImgHeadBlk ihb[],
     if (*pstatus!=Success) return;
 
     // filter bsl file name
-    filename_parameter ( Name,IO_len,pPrefix->V,0);
-    filename_path ( Path, IO_len-10, Name ); //bsl name<10 char
-    filename_name ( Name, IO_len, Name );
+    filename_parameter ( Name,IO_size,pPrefix->V,0);
+    filename_path ( Path, IO_size-10, Name ); //bsl name<10 char
+    filename_name ( Name, IO_size, Name );
     ToBslName(Name,Name,TRUE);
     if (strcmp(Path,"./"))
       sprintf(pPrefix->V,"%s%s",Path,Name);
@@ -278,9 +281,9 @@ void Saxs2BslFilenames ( CmdBlk * pcb, ImgBlk ib[], ImgHeadBlk ihb[],
     if (*pstatus!=Success) return;
 
     // filter bsl file name
-    filename_parameter ( Name,IO_len,pPrefix->V,0);
-    filename_path ( Path, IO_len-10, Name ); //bsl name<10 char
-    filename_name ( Name, IO_len, Name );
+    filename_parameter ( Name,IO_size,pPrefix->V,0);
+    filename_path ( Path, IO_size-10, Name ); //bsl name<10 char
+    filename_name ( Name, IO_size, Name );
     ToBslName(Name,Name,FALSE);
     if (strcmp(Path,"./"))
       sprintf(pPrefix->V,"%s%s",Path,Name);
@@ -394,15 +397,15 @@ void saxs_bsl (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
   float Wmin_1, Wmax_1, W_1, DW_1;
   float Wmin_2, Wmax_2, W_2, DW_2;
 
-  char OutputHeaderName[InputLineLength];
-  char OutputBinaryName[InputLineLength];
+  char OutputHeaderName[IO_size];
+  char OutputBinaryName[IO_size];
   FILE *OutputHeaderId;
   FILE *OutputBinaryId;
   int  nn;
   char CC;
   char NN[3];
   size_t t_length;
-  char fname_full[IO_len], Temp[IO_len];
+  char fname_full[IO_size], Temp[IO_size];
 
   IO_line Prefix;
   IO_long Bobo;
@@ -425,10 +428,10 @@ void saxs_bsl (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
      printf("%d images found, 1 input and 1 output image required\n",
              pcb->ImgBlkLen); *pstatus=Failed; return; }
 
-  Prefix = option_line ( SPrefix, pcb, pcb->ib, num, 1, pstatus );
+  Prefix = option_line ( SPrefix, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
 
-  Bobo = option_long ( SBobo, pcb, pcb->ib, num, 1, pstatus );
+  Bobo = option_long ( SBobo, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
 
   /* set byte order of all bsl input files */
@@ -519,17 +522,17 @@ void saxs_bsl (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
    else FrameCounter+=1l;
 
   // Generate BSL filename
-  filename_parameter (fname_full,IO_len,Prefix.V,0);
-  filename_pattern (fname_full,IO_len,fname_full,(pcb->ib)[0].FileNumber);
-  filename_full (fname_full,IO_len,fname_full);
+  filename_parameter (fname_full,IO_size,Prefix.V,0);
+  filename_pattern (fname_full,IO_size,fname_full,(pcb->ib)[0].FileNumber);
+  filename_full (fname_full,IO_size,fname_full);
  
   /* Reset frame counter */
   if  (ImageIsFirst( pcb, 0, ihb )) FrameCounter=1l;
  
   if (!Single.V) {
-    if (pcb->TestBit) printf("Output into multi bsl file\n");
+    if (pcb->TestBit) printf("Output to multi bsl file\n");
  
-    /* Output into multi bsl file (otoko) format */
+    /* Output to multi bsl file (otoko) format */
  
     BslFileName ( OutputHeaderName, fname_full, 0 );
     BslFileName ( OutputBinaryName, fname_full, ihb[0].ImNum );
@@ -555,7 +558,7 @@ void saxs_bsl (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
       else fprintf(OutputHeaderId,"%8d%8d%8d%8d%8d%8d%8d%8d%8d%8d\n",
                    I0Dim_1,I0Dim_2,1,0,0,0,0,0,0,0);
     fprintf(OutputHeaderId,"% s\n",
-            filename_name(Temp,IO_len,OutputBinaryName));
+            filename_name(Temp,IO_size,OutputBinaryName));
  
     fclose(OutputHeaderId);
  
@@ -581,9 +584,9 @@ void saxs_bsl (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
     fclose(OutputBinaryId);
  
   } else {         
-    if (pcb->TestBit) printf("Output into multi bsl file\n");
+    if (pcb->TestBit) printf("Output to multi bsl file\n");
  
-    /* Output into multi bsl file (otoko) format */
+    /* Output to multi bsl file (otoko) format */
  
     BslFileName ( OutputHeaderName, fname_full, 0 );
     BslFileName ( OutputBinaryName, fname_full, 1 );
@@ -601,7 +604,7 @@ void saxs_bsl (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
  
     fprintf(OutputHeaderId,"%8d%8d%8d%8d%8d%8d%8d%8d%8d%8d\n",
             I0Dim_1,I0Dim_2,FrameCounter,0,0,0,0,0,0,0);
-    fprintf(OutputHeaderId,"% s\n",filename_name(Temp,IO_len,OutputBinaryName));
+    fprintf(OutputHeaderId,"% s\n",filename_name(Temp,IO_size,OutputBinaryName));
  
     fclose(OutputHeaderId);
  
@@ -653,7 +656,7 @@ void saxs_bsl (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
   printf("\n");
 
   printf("Input file         : % s\n",
-          filename_full(fname_full,IO_len,pcb->ib[1].FileName));
+          filename_full(fname_full,IO_size,pcb->ib[1].FileName));
   printf("Input image number : % d\n",ihb[1].ImNum);
   printf("Output header file : % s\n",OutputHeaderName);
   printf("Output binary file : % s\n",OutputBinaryName);
@@ -666,7 +669,7 @@ Do all the keyboard io and return cb, and ib
 ---------------------------------------------------------------------------*/
 void user_io(CmdBlk * pcb, ImgBlk * ib, int * pstatus)
 {
-  char  progfn[InputLineLength];
+  char  progfn[IO_size];
   ImgHeadBlk ihb[BlockNum];
 
   float ROff_1, RPs_1, UOff_1, UPs_1;
@@ -675,7 +678,7 @@ void user_io(CmdBlk * pcb, ImgBlk * ib, int * pstatus)
   IO_line * pPrefix;
 
  /* Determine program name without directory */
-   (void) RightFR((char *) pcb->argv[0],DirSeparator,progfn,InputLineLength);
+   (void) RightFR((char *) pcb->argv[0],DirSeparator,progfn,IO_size);
 
   /* show usage if no arguments are given */
   if (pcb->argc<=1) printf("Usage: %s %s\n",progfn,Usage);
