@@ -48,7 +48,7 @@
   If AxisType_1 is Distance (option -i1type Distance) the default reference    
   system is Saxs (option -rsys Saxs). The projection type can either be 
   Saxs or Waxs.  The option +waxs (default) writes the exact scattering 
-  vector into the x-column. The option -waxs ignores the Projection Type 
+  vector to the x-column. The option -waxs ignores the Projection Type 
   (option -i1pro Saxs|Waxs). In the case of ProjectionType Saxs 
   (option -i1pro Saxs|Waxs) the x-column is calculated in small angle 
   approximation (sin x ~= x ~= tan x).
@@ -95,7 +95,7 @@
   +-gpl                   : enables/disables gnuplot compatibility (default: +gpl)
   -scf <factor>           : multiplication of scattering vector s (default: 1),
                             -scf 2_pi converts s*nm to q*nm
-  +-hist                  : writes/do not write history into header (default: +hist)
+  +-hist                  : writes/do not write history to header (default: +hist)
   +-waxs                  : calculate exact scattering vector length from 
                             saxs-coordinate (default: +waxs) 
 
@@ -129,9 +129,13 @@
                        gnuplot> replot 'sphere_arc.txt' i 4 with  errorlines
   2016-04-08  PB V4.79 +gpl adds always separator lines between scans, independent
                        of +-hd1.
-
+  2016-06-18  PB V4.80 Projection_1U is running with Ipolmin==0 (project.c V1.05). 
+                       Here, it does not need to be set any more.
+  2017-05-21  PB V4.81 ioalloc
+  2017-10-20  PB V4.82 ReadHeaderLine updated
+  2018-01-16  PB V4.83 option -dlbl (DatLabel) added
 ---*/
-# define Version  "saxs_curve V4.79 2016-04-08, Peter Boesecke"
+# define Version  "saxs_curve V4.83 2018-01-16, Peter Boesecke"
 
 # include <errno.h>
 # include <stdio.h>
@@ -140,8 +144,7 @@
 
 # include "SaxsPrograms.h"
 # include "strlib.h"
-
-# define FREE( a) { if ( a) { free( a); a = NULL; } }
+# include "ioalloc.h"
 
 /* Defaults */
 # define Usage "[options] \n\
@@ -154,6 +157,7 @@
 # define SHeadL     "hedl"
 # define SExtension "ext"
 # define SKey       "key"  /* header key to write */
+# define SDatLabel  "dlbl" /* Intensity column label */
 
 // flag
 # define SHist  "hist" // write history
@@ -179,7 +183,7 @@ NAME
   write_history
 
 DESCRIPTION
-  Writes the full history into the output file 'out'
+  Writes the full history to the output file 'out'
 
 RETURN VALUE
   Number of written lines
@@ -209,7 +213,7 @@ long write_history( FILE * out, CmdBlk * pcb, long num[], ImgHeadBlk ihb[],
   SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
   if (*pstatus!=Success) { edf_free_header( header_key ); return( cnt ); }
 
-  Hdmk = option_line ( SHdmk, pcb, pcb->ib, num, 1, pstatus );
+  Hdmk = option_line ( SHdmk, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) { edf_free_header( header_key ); return( cnt ); }
 
   edf_first_header_element ( header_key, &key, &value, &ErrorValue, pstatus );
@@ -277,21 +281,21 @@ long write_head( FILE * out, CmdBlk *pcb, long num[], ImgHeadBlk ihb[],
   IO_long cntmax, Hist;
   IO_line Hdmk, KeyList;
 
-  char Key[IO_len], KeyValue[IO_len];
+  char Key[IO_size], KeyValue[IO_size];
   int blkno=0, parno;
 
   const char *hdmk;
 
-  Hdmk = option_line ( SHdmk, pcb, pcb->ib, num, 1, pstatus );
+  Hdmk = option_line ( SHdmk, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return( cnt );
 
-  KeyList = option_line ( SKey, pcb, pcb->ib, num, 1, pstatus );
+  KeyList = option_line ( SKey, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return( cnt );
 
   Hist = option_flag ( SHist, pcb, pstatus );
   if (*pstatus!=Success) return( cnt );
 
-  cntmax = option_long ( SHlen, pcb, pcb->ib, num, 1, pstatus ); // header lines
+  cntmax = option_long ( SHlen, pcb, ihb, num, 1, pstatus ); // header lines
   if (*pstatus!=Success) return( cnt );
 
   if (spec) hdmk="# "; else hdmk=Hdmk.V;
@@ -319,9 +323,9 @@ long write_head( FILE * out, CmdBlk *pcb, long num[], ImgHeadBlk ihb[],
 
   if (strlen(KeyList.V)) {
     parno = 1;
-    while (strlib_param ( Key, IO_len, KeyList.V, ',', parno++ )) {
+    while (strlib_param ( Key, IO_size, KeyList.V, ',', parno++ )) {
       if (strlen(Key)) {
-        ReadHeaderLine          ( pcb, 1, ihb, Key, KeyValue, pstatus );
+        ReadHeaderLine          ( pcb, 1, num, ihb, Key, KeyValue, pstatus );
         if (*pstatus==Success)fprintf(out,"%s%s = %s \r\n",hdmk,Key,KeyValue);
       }
     }
@@ -351,11 +355,11 @@ DESCRIPTION
 HISTORY
   2007-11-12 PB from saxs_ascii
 ----------------------------------------------------------------------------*/
-void write_labels ( FILE * out, float scf, char *hdmk,
-                   const char * spacer, const char * headline, int spec,
-                   const char * firstlabel,
-                   float *data, float *vardat, int dim_1, int dim_2,
-                   float off_2, float ps_2, int * pstatus)
+void write_labels ( FILE *out, float scf, char *hdmk,
+                    const char *spacer, const char *headline, int spec,
+                    const char *firstlabel, const char *datlbl,
+                    float *data, float *vardat, int dim_1, int dim_2,
+                    float off_2, float ps_2, int * pstatus)
 {   int i_2;
     char buffer[1024];
     char *label=(char *) NULL, *nextlabel=(char *) NULL;
@@ -364,7 +368,7 @@ void write_labels ( FILE * out, float scf, char *hdmk,
     if (!hdmk) hdmk="#";
 
     if (headline) {
-      headl = malloc(strlen(headline)+1);
+      headl = MALLOC(strlen(headline)+1);
       nextlabel = strcpy(headl,headline);
     }
 
@@ -408,10 +412,22 @@ void write_labels ( FILE * out, float scf, char *hdmk,
       // write Intensity label
       if (nextlabel) label = strlib_tok((char *) NULL, delim);
       else label=(char *) NULL;
-      if ((!label) || (!strncmp(label,WildCard,strlen(WildCard)))) {
-        if (dim_2==1) sprintf(buffer,"I");
-        else sprintf(buffer,"I_%d", i_2+1);
+
+      if ((i_2==0)&&(!strlib_is_empty( datlbl ))) {
+        // use always datlbl for first data column if it is not empty
+        if (dim_2==1) {
+          sprintf(buffer,"%s",datlbl);
+        } else {
+          sprintf(buffer,"%s_%d",datlbl,i_2+1);
+        }
         label = &buffer[0];
+      } else {
+        if ((!label) || (!strncmp(label,WildCard,strlen(WildCard)))) {
+          if (dim_2==1) {
+            sprintf(buffer,"I");
+          } else sprintf(buffer,"I_%d", i_2+1);
+          label = &buffer[0];
+        }
       }
       fprintf(out,"%s%s",spacer, label); 
 
@@ -435,7 +451,7 @@ void write_labels ( FILE * out, float scf, char *hdmk,
 
     fprintf(out," \r\n");
 
-    if (headl) free (headl);
+    if (headl) FREE (headl);
 
     *pstatus = Success;
 
@@ -527,7 +543,7 @@ void write_saxs_s ( CmdBlk * pcb, long num[], ImgHeadBlk ihb[],
                     float dummy, float ddummy, int * pstatus)
 { FILE *OutputFileId;
 
-  IO_line Hdmk, Spacer, HeadL;
+  IO_line Hdmk, Spacer, HeadL, DatLabel;
   IO_float Scf;
   IO_long All, Waxs, Spec, i1Head, Head, GnuPlot;
 
@@ -538,11 +554,13 @@ void write_saxs_s ( CmdBlk * pcb, long num[], ImgHeadBlk ihb[],
 
   Head = option_flag ( SHead, pcb, pstatus );
   if (*pstatus!=Success) return;
-  Hdmk = option_line ( SHdmk, pcb, pcb->ib, num, 1, pstatus );
+  Hdmk = option_line ( SHdmk, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
-  Spacer = option_line ( SSpacer, pcb, pcb->ib, num, 1, pstatus );
+  Spacer = option_line ( SSpacer, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
-  HeadL = option_line ( SHeadL, pcb, pcb->ib, num, 1, pstatus );
+  HeadL = option_line ( SHeadL, pcb, ihb, num, 1, pstatus );
+  if (*pstatus!=Success) return;
+  DatLabel = option_line ( SDatLabel, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
   All = option_flag ( SAll, pcb, pstatus );
   if (*pstatus!=Success) return;
@@ -550,7 +568,7 @@ void write_saxs_s ( CmdBlk * pcb, long num[], ImgHeadBlk ihb[],
   if (*pstatus!=Success) return;
   Spec = option_flag ( SSpec, pcb, pstatus );
   if (*pstatus!=Success) return;
-  Scf = option_float ( SScf, pcb, pcb->ib, num, 1, pstatus );
+  Scf = option_float ( SScf, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
   i1Head = option_flag ( Si1Head, pcb, pstatus );
   if (*pstatus!=Success) return;
@@ -614,8 +632,11 @@ void write_saxs_s ( CmdBlk * pcb, long num[], ImgHeadBlk ihb[],
       else if (fabs(Scf.V-twopi)<0.00253) sprintf(buffer,"q*nm");
       else sprintf(buffer,"(s*%g)*nm",Scf.V);
 
+      char datlbl[1024];
+      linekeyvalue( datlbl, 1024, DatLabel.V, 1, pcb, ihb, num, 0, pstatus );
+
       write_labels ( OutputFileId, Scf.V, Hdmk.V, Spacer.V, HeadL.V, SpecI,
-                     &buffer[0], data, vardat, dim_1, dim_2,
+                     &buffer[0], datlbl, data, vardat, dim_1, dim_2,
                      off_2, ps_2, pstatus);
       if (*pstatus) return;
     }
@@ -677,11 +698,11 @@ void write_saxs_a ( CmdBlk * pcb, long num[], ImgHeadBlk ihb[],
 
   Head = option_flag ( SHead, pcb, pstatus );
   if (*pstatus!=Success) return;
-  Hdmk = option_line ( SHdmk, pcb, pcb->ib, num, 1, pstatus );
+  Hdmk = option_line ( SHdmk, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
-  Spacer = option_line ( SSpacer, pcb, pcb->ib, num, 1, pstatus );
+  Spacer = option_line ( SSpacer, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
-  HeadL = option_line ( SHeadL, pcb, pcb->ib, num, 1, pstatus );
+  HeadL = option_line ( SHeadL, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
   All = option_flag ( SAll, pcb, pstatus );
   if (*pstatus!=Success) return;
@@ -689,7 +710,7 @@ void write_saxs_a ( CmdBlk * pcb, long num[], ImgHeadBlk ihb[],
   if (*pstatus!=Success) return;
   Spec = option_flag ( SSpec, pcb, pstatus );
   if (*pstatus!=Success) return;
-  Scf = option_float ( SScf, pcb, pcb->ib, num, 1, pstatus );
+  Scf = option_float ( SScf, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
   i1Head = option_flag ( Si1Head, pcb, pstatus );
   if (*pstatus!=Success) return;
@@ -755,7 +776,7 @@ void write_saxs_a ( CmdBlk * pcb, long num[], ImgHeadBlk ihb[],
       }
 
       write_labels ( OutputFileId, Scf.V, Hdmk.V, Spacer.V, HeadL.V, SpecI,
-                     &buffer[0], data, vardat, dim_1, dim_2,
+                     &buffer[0], "I", data, vardat, dim_1, dim_2,
                      off_2, ps_2, pstatus);
       if (*pstatus) return;
     }
@@ -836,7 +857,7 @@ void saxs_w (CmdBlk * pcb, long num[], ImgHeadBlk ihb[],
   long fullcircles=0, circles;
 
   /* output file */
-  char fname_full[IO_len];
+  char fname_full[IO_size];
 
   IO_line Extension;
 
@@ -845,8 +866,6 @@ void saxs_w (CmdBlk * pcb, long num[], ImgHeadBlk ihb[],
   float W1_2, W3_2;
   IO_float Alf1, Alf2;
 
-  float ipolmin_cur;
-
   char buffer[1024];
 
   *pstatus = Success;
@@ -894,11 +913,11 @@ void saxs_w (CmdBlk * pcb, long num[], ImgHeadBlk ihb[],
   if (*pstatus!=Success) goto saxs_w_error;
 
  /* Get options */
-  Extension = option_line ( SExtension, pcb, pcb->ib, num, 1, pstatus );
+  Extension = option_line ( SExtension, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) goto saxs_w_error;
-  Alf1 = option_float ( "alf1", pcb, pcb->ib, num, 1, pstatus );
+  Alf1 = option_float ( "alf1", pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) goto saxs_w_error;
-  Alf2 = option_float ( "alf2", pcb, pcb->ib, num, 1, pstatus );
+  Alf2 = option_float ( "alf2", pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) goto saxs_w_error;
 
   // Get input image ranges (using Off and Ps)
@@ -914,14 +933,6 @@ void saxs_w (CmdBlk * pcb, long num[], ImgHeadBlk ihb[],
                      Off_1, Ps_1, Off_2, Ps_2,
                      &W1_2, &W3_2 );
 
-  // keep original Ipolmin in ipolmin_cur
-  ipolmin_cur = Ipolmin(-1);
-  Ipolmin(0); // do not use Ipolmin for Projection_1U
-
-  if (pcb->TestBit) {
-    printf("Ipolmin has been set from %g to 0\n",ipolmin_cur);
-  }
-
   // Order Alf1 and Alf2
   if (Alf1.I) alf1 = Alf1.V;
   else alf1 = W1_2;
@@ -1057,14 +1068,14 @@ void saxs_w (CmdBlk * pcb, long num[], ImgHeadBlk ihb[],
 
       // I0Weight, E0Weight
       if (!I0Weight) {
-        I0Weight=(float*) malloc(sizeof(float)*I0Dim_1);
+        I0Weight=(float*) MALLOC(sizeof(float)*I0Dim_1);
         if (!I0Weight) {
           *pstatus = NotEnoughMemoryAvailable;
           goto saxs_w_error;
         }
       }
       if ((E0Data)&&(!E0Weight)) {
-        E0Weight=(float*) malloc(sizeof(float)*I0Dim_1);
+        E0Weight=(float*) MALLOC(sizeof(float)*I0Dim_1);
       if (!E0Weight) {
           *pstatus = NotEnoughMemoryAvailable;
           goto saxs_w_error;
@@ -1099,14 +1110,14 @@ void saxs_w (CmdBlk * pcb, long num[], ImgHeadBlk ihb[],
     if (fullcircles>0) {
       // I0Weight, E0Weight
       if (!I0Weight) {
-        I0Weight=(float*) malloc(sizeof(float)*I0Dim_1);
+        I0Weight=(float*) MALLOC(sizeof(float)*I0Dim_1);
         if (!I0Weight) {
           *pstatus = NotEnoughMemoryAvailable;
           goto saxs_w_error;
         }
       }
       if ((E0Data)&&(!E0Weight)) {
-        E0Weight=(float*) malloc(sizeof(float)*I0Dim_1);
+        E0Weight=(float*) MALLOC(sizeof(float)*I0Dim_1);
         if (!E0Weight) {
           *pstatus = NotEnoughMemoryAvailable;
           goto saxs_w_error;
@@ -1193,24 +1204,17 @@ void saxs_w (CmdBlk * pcb, long num[], ImgHeadBlk ihb[],
 
   } // for i_2
 
-  if (pcb->TestBit) {
-    printf("Resetting Ipolmin to %g\n",ipolmin_cur);
-  }
-
-  // resetting Ipolmin to original value
-  Ipolmin(ipolmin_cur);
-
   /* Generate filename of ASCII file */
 
   /* Check for .gz extension (use fname_full as temporary buffer) */
-  filename_extension ( fname_full, IO_len, pcb->ib[0].FileName );
+  filename_extension ( fname_full, IO_size, pcb->ib[0].FileName );
   if (!strcmp(fname_full,".gz")) {
     // Remove .gz extension
-    filename_body ( fname_full, IO_len,  pcb->ib[0].FileName ); // remove .gz
-    filename_body ( fname_full, IO_len,  fname_full ); // remove ext.
+    filename_body ( fname_full, IO_size,  pcb->ib[0].FileName ); // remove .gz
+    filename_body ( fname_full, IO_size,  fname_full ); // remove ext.
   } else {
     /* Generate filename of ASCII file */
-    filename_body ( fname_full, IO_len,  pcb->ib[0].FileName ); // remove ext.
+    filename_body ( fname_full, IO_size,  pcb->ib[0].FileName ); // remove ext.
   }
 
   if ( filename_compare ( fname_full, DUMMYFILENAME ) )
@@ -1250,7 +1254,7 @@ NAME
  saxs_curve
 PURPOSE
   Writes intensity and standard error against scattering vector or angle
-  of a row by row file into a text file.
+  of a row by row file to a text file.
 ---------------------------------------------------------------------------*/
 void saxs_curve (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
 {
@@ -1344,14 +1348,14 @@ Do all the keyboard io and return cb, and ib
 ---------------------------------------------------------------------------*/
 void user_io(CmdBlk * pcb, ImgBlk * ib, int * pstatus)
 {
-  char  progfn[InputLineLength];
+  char  progfn[IO_size];
   ImgHeadBlk ihb[BlockNum];
 
   float ROff_1, RPs_1, UOff_1, UPs_1;
   float ROff_2, RPs_2, UOff_2, UPs_2;
 
  /* Determine program name without directory */
-   (void) RightFR((char *) pcb->argv[0],DirSeparator,progfn,InputLineLength);
+   (void) RightFR((char *) pcb->argv[0],DirSeparator,progfn,IO_size);
 
   /* show usage if no arguments are given */
   if (pcb->argc<=1) printf("Usage: %s %s\n",progfn,Usage);
@@ -1421,6 +1425,7 @@ int MAIN (int argc, char *argv[])
 
   option_define ( IO_tpflag, SHead, "TRUE", &cb ); /* default on */
   option_define ( IO_tpline, SKey,"Intensity0,Intensity1,SampleThickness", &cb);
+  option_define ( IO_tpline, SDatLabel,"", &cb); /* empty, not used */
   option_define ( IO_tpflag, SHist, "TRUE", &cb );
   option_define ( IO_tpline, SHdmk, "#", &cb ); /* default is # */
   option_define ( IO_tpline, SHeadL, "#= I_[Title,,1]", &cb); /*#L s+Title++*/
