@@ -45,7 +45,7 @@
   If AxisType_1 is Distance (option -i1type Distance) the default reference    
   system is Saxs (option -rsys Saxs). The projection type can either be 
   Saxs or Waxs.  The option +waxs (default) writes the exact scattering 
-  vector into the x-column. The option -waxs ignores the Projection Type 
+  vector to the x-column. The option -waxs ignores the Projection Type 
   (option -i1pro Saxs|Waxs). In the case of ProjectionType Saxs 
   (option -i1pro Saxs|Waxs) the x-column is calculated in small angle 
   approximation (sin x ~= x ~= tan x).
@@ -86,7 +86,7 @@
   +-gpl                   : enables/disables gnuplot compatibility (default: +gpl)
   -scf <factor>       : multiplication of scattering vector s (default: 1),
                         -scf 2_pi converts s*nm to q*nm
-  +-hist              : writes/do not write history into header (default: +hist)
+  +-hist              : writes/do not write history to header (default: +hist)
   +-waxs              : calculate exact scattering vector length from 
                         saxs-coordinate (default: +waxs) 
 
@@ -132,9 +132,10 @@
                        ...
                        gnuplot> replot 'sphere_arc.txt' i 4 with  errorlines
   2016-05-19  PB V4.77 forgotten definition of option gpl added
+  2017-10-20  PB V4.78 ReadHeaderLine updated
 
 ---*/
-# define Version  "saxs_curves V4.77 2016-05-19, Peter Boesecke"
+# define Version  "saxs_curves V4.78 2017-10-20, Peter Boesecke"
 
 # include <errno.h>
 # include <stdio.h>
@@ -180,7 +181,7 @@ NAME
   write_history
 
 DESCRIPTION
-  Writes the full history into the output file 'out'
+  Writes the full history to the output file 'out'
 
 RETURN VALUE
   Number of written lines
@@ -210,7 +211,7 @@ long write_history( FILE * out, CmdBlk * pcb, long num[], ImgHeadBlk ihb[],
   SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
   if (*pstatus!=Success) { edf_free_header( header_key ); return( cnt ); }
 
-  Hdmk = option_line ( SHdmk, pcb, pcb->ib, num, 1, pstatus );
+  Hdmk = option_line ( SHdmk, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) { edf_free_header( header_key ); return( cnt ); }
 
   edf_first_header_element ( header_key, &key, &value, &ErrorValue, pstatus );
@@ -278,21 +279,21 @@ long write_head( FILE * out, CmdBlk *pcb, long num[], ImgHeadBlk ihb[],
   IO_long cntmax, Hist;
   IO_line Hdmk, KeyList;
 
-  char Key[IO_len], KeyValue[IO_len];
+  char Key[IO_size], KeyValue[IO_size];
   int blkno=0, parno;
 
   const char *hdmk;
 
-  Hdmk = option_line ( SHdmk, pcb, pcb->ib, num, 1, pstatus );
+  Hdmk = option_line ( SHdmk, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return( cnt );
 
-  KeyList = option_line ( SKey, pcb, pcb->ib, num, 1, pstatus );
+  KeyList = option_line ( SKey, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return( cnt );
 
   Hist = option_flag ( SHist, pcb, pstatus );
   if (*pstatus!=Success) return( cnt );
 
-  cntmax = option_long ( SHlen, pcb, pcb->ib, num, 1, pstatus ); // header lines
+  cntmax = option_long ( SHlen, pcb, ihb, num, 1, pstatus ); // header lines
   if (*pstatus!=Success) return( cnt );
 
   if (spec) hdmk="# "; else hdmk=Hdmk.V;
@@ -320,9 +321,9 @@ long write_head( FILE * out, CmdBlk *pcb, long num[], ImgHeadBlk ihb[],
 
   if (strlen(KeyList.V)) {
     parno = 1;
-    while (strlib_param ( Key, IO_len, KeyList.V, ',', parno++ )) {
+    while (strlib_param ( Key, IO_size, KeyList.V, ',', parno++ )) {
       if (strlen(Key)) {
-        ReadHeaderLine          ( pcb, 1, ihb, Key, KeyValue, pstatus );
+        ReadHeaderLine          ( pcb, 1, num, ihb, Key, KeyValue, pstatus );
         if (*pstatus==Success)fprintf(out,"%s%s = %s \r\n",hdmk,Key,KeyValue);
       }
     }
@@ -550,7 +551,7 @@ void saxs_s (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
   float Wmin_2, Wmax_2, DW_2;
 
   /* output file */
-  char fname_full[IO_len];
+  char fname_full[IO_size];
   FILE *OutputFileId;
 
   IO_line Hdmk, Spacer, HeadL, Extension;
@@ -590,13 +591,13 @@ void saxs_s (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
  /* options */
   Head = option_flag ( SHead, pcb, pstatus );
   if (*pstatus!=Success) return;
-  Hdmk = option_line ( SHdmk, pcb, pcb->ib, num, 1, pstatus );
+  Hdmk = option_line ( SHdmk, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
-  Extension = option_line ( SExtension, pcb, pcb->ib, num, 1, pstatus );
+  Extension = option_line ( SExtension, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
-  Spacer = option_line ( SSpacer, pcb, pcb->ib, num, 1, pstatus );
+  Spacer = option_line ( SSpacer, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
-  HeadL = option_line ( SHeadL, pcb, pcb->ib, num, 1, pstatus );
+  HeadL = option_line ( SHeadL, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
   All = option_flag ( SAll, pcb, pstatus );
   if (*pstatus!=Success) return;
@@ -604,7 +605,7 @@ void saxs_s (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
   if (*pstatus!=Success) return;
   Spec = option_flag ( SSpec, pcb, pstatus );
   if (*pstatus!=Success) return;
-  Scf = option_float ( SScf, pcb, pcb->ib, num, 1, pstatus );
+  Scf = option_float ( SScf, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
   i1Head = option_flag ( Si1Head, pcb, pstatus );
   if (*pstatus!=Success) return;
@@ -676,14 +677,14 @@ void saxs_s (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
   } /* for i_2 ... */
 
   /* Check for .gz extension (use fname_full as temporary buffer) */
-  filename_extension ( fname_full, IO_len, pcb->ib[0].FileName );
+  filename_extension ( fname_full, IO_size, pcb->ib[0].FileName );
   if (!strcmp(fname_full,".gz")) {
     // Remove .gz extension
-    filename_body ( fname_full, IO_len,  pcb->ib[0].FileName ); // remove .gz
-    filename_body ( fname_full, IO_len,  fname_full ); // remove ext.
+    filename_body ( fname_full, IO_size,  pcb->ib[0].FileName ); // remove .gz
+    filename_body ( fname_full, IO_size,  fname_full ); // remove ext.
   } else {
     /* Generate filename of ASCII file */
-    filename_body ( fname_full, IO_len,  pcb->ib[0].FileName ); // remove ext. 
+    filename_body ( fname_full, IO_size,  pcb->ib[0].FileName ); // remove ext. 
   }
 
   if ( filename_compare ( fname_full, DUMMYFILENAME ) )
@@ -862,7 +863,7 @@ void saxs_a (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
   float Wmin_2, Wmax_2, DW_2;
 
   /* output file */
-  char fname_full[IO_len];
+  char fname_full[IO_size];
   FILE *OutputFileId;
 
   IO_line Hdmk, Spacer, HeadL, Extension;
@@ -901,19 +902,19 @@ void saxs_a (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
  /* options */
   Head = option_flag ( SHead, pcb, pstatus );
   if (*pstatus!=Success) return;
-  Hdmk = option_line ( SHdmk, pcb, pcb->ib, num, 1, pstatus );
+  Hdmk = option_line ( SHdmk, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
-  Extension = option_line ( SExtension, pcb, pcb->ib, num, 1, pstatus );
+  Extension = option_line ( SExtension, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
-  Spacer = option_line ( SSpacer, pcb, pcb->ib, num, 1, pstatus );
+  Spacer = option_line ( SSpacer, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
-  HeadL = option_line ( SHeadL, pcb, pcb->ib, num, 1, pstatus );
+  HeadL = option_line ( SHeadL, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
   All = option_flag ( SAll, pcb, pstatus );
   if (*pstatus!=Success) return;
   Spec = option_flag ( SSpec, pcb, pstatus );
   if (*pstatus!=Success) return;
-  Scf = option_float ( SScf, pcb, pcb->ib, num, 1, pstatus );
+  Scf = option_float ( SScf, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
   i1Head = option_flag ( Si1Head, pcb, pstatus );
   if (*pstatus!=Success) return;
@@ -982,14 +983,14 @@ void saxs_a (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
   /* Generate filename of ASCII file */
 
   /* Check for .gz extension (use fname_full as temporary buffer) */
-  filename_extension ( fname_full, IO_len, pcb->ib[0].FileName );
+  filename_extension ( fname_full, IO_size, pcb->ib[0].FileName );
   if (!strcmp(fname_full,".gz")) {
     // Remove .gz extension
-    filename_body ( fname_full, IO_len,  pcb->ib[0].FileName ); // remove .gz
-    filename_body ( fname_full, IO_len,  fname_full ); // remove ext.
+    filename_body ( fname_full, IO_size,  pcb->ib[0].FileName ); // remove .gz
+    filename_body ( fname_full, IO_size,  fname_full ); // remove ext.
   } else {
     /* Generate filename of ASCII file */
-    filename_body ( fname_full, IO_len,  pcb->ib[0].FileName ); // remove ext.
+    filename_body ( fname_full, IO_size,  pcb->ib[0].FileName ); // remove ext.
   }
 
   if ( filename_compare ( fname_full, DUMMYFILENAME ) )
@@ -1136,7 +1137,7 @@ NAME
  saxs_curves
 PURPOSE
   Writes intensity and standard error against scattering vector or angle
-  of a row by row file into a text file.
+  of a row by row file to a text file.
 ---------------------------------------------------------------------------*/
 void saxs_curves (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
 {
@@ -1164,14 +1165,14 @@ Do all the keyboard io and return cb, and ib
 ---------------------------------------------------------------------------*/
 void user_io(CmdBlk * pcb, ImgBlk * ib, int * pstatus)
 {
-  char  progfn[InputLineLength];
+  char  progfn[IO_size];
   ImgHeadBlk ihb[BlockNum];
 
   float ROff_1, RPs_1, UOff_1, UPs_1;
   float ROff_2, RPs_2, UOff_2, UPs_2;
 
  /* Determine program name without directory */
-   (void) RightFR((char *) pcb->argv[0],DirSeparator,progfn,InputLineLength);
+   (void) RightFR((char *) pcb->argv[0],DirSeparator,progfn,IO_size);
 
   /* show usage if no arguments are given */
   if (pcb->argc<=1) printf("Usage: %s %s\n",progfn,Usage);
