@@ -187,10 +187,11 @@
   2015-10-04  PB V4.66 OpenImageFile parameter blkno changed 
                        from 3 to INflat,
                        Default_ipfn3 => Default_ipfnflat
- 
+  2017-10-20  PB V4.67 No key value replacement in user_io
+                       ReadHeader... arguments updated 
 ---*/
  
-# define Version  "saxs_normn V4.66 2015-10-04, Peter Boesecke"
+# define Version  "saxs_normn V4.67 2017-10-20, Peter Boesecke"
 
 # include <errno.h>
 # include <stdio.h>
@@ -298,7 +299,7 @@ Options:  -trm  <transmission>, -bcf <binning correction factor>,\n\
  * Global Initialised Variables                                            *
  ***************************************************************************/
 
-PRIVATE char ScalerKeyNameBuffer[InputLineLength];
+PRIVATE char ScalerKeyNameBuffer[IO_size];
 
 /*---------------------------------------------------------------------------
 ScalerKey
@@ -319,7 +320,7 @@ Do all the keyboard io and return cb, and ib
 
 void user_io( CmdBlk * pcb, ImgBlk * ib, int * pstatus )
 {
-  char  progfn[InputLineLength];
+  char  progfn[IO_size];
 
   long  image_1st;
   long  image_lst;
@@ -327,13 +328,13 @@ void user_io( CmdBlk * pcb, ImgBlk * ib, int * pstatus )
   int   next_block;
   int   INflat; /* image block number of flat field file */
 
-  char  MonitorName[InputLineLength];
-  char  TimeName[InputLineLength];
-  char  AnodeName[InputLineLength];
+  char  MonitorName[IO_size];
+  char  TimeName[IO_size];
+  char  AnodeName[IO_size];
 
-  char  MonitorNameS[InputLineLength];
-  char  TimeNameS[InputLineLength];
-  char  AnodeNameS[InputLineLength];
+  char  MonitorNameS[IO_size];
+  char  TimeNameS[IO_size];
+  char  AnodeNameS[IO_size];
 
   ImgHeadBlk ihb[BlockNum];
   long num[BlockNum];
@@ -347,7 +348,7 @@ void user_io( CmdBlk * pcb, ImgBlk * ib, int * pstatus )
   IO_long MonSca, TimSca, AnoSca, MonScaS, TimScaS, AnoScaS;
 
  /* Determine program name without directory */
-   (void) RightFR((char *) pcb->argv[0],DirSeparator,progfn, InputLineLength);
+   (void) RightFR((char *) pcb->argv[0],DirSeparator,progfn, IO_size);
 
   /* show usage if no arguments are given */
   if (pcb->argc<=1) printf("Usage: %s %s\n",progfn,pcb->MainHelp);
@@ -384,7 +385,9 @@ void user_io( CmdBlk * pcb, ImgBlk * ib, int * pstatus )
   /* Exit in all cases after an abort from the first argument */
   if (*pstatus!=Success) return;
 
-  OpenImageFile( pcb,ib,1,ib[1].Name.V,ib[1].FFirst.V,
+  /* No key value replacement in user_io -> ihb=NULL */
+  OpenImageFile( pcb,ib,1,NULL,
+                 ib[1].Name.V,ib[1].FFirst.V,
                  IO_Old|IO_FileProtect,pstatus);
   if (*pstatus!=Success) return;
 
@@ -398,7 +401,9 @@ void user_io( CmdBlk * pcb, ImgBlk * ib, int * pstatus )
   argv_filename( pcb, "Scaler file name", ib, 2, ib[1].Name.V, pstatus);
   if (*pstatus!=Success) return;
 
-  OpenImageFile( pcb,ib,2,ib[2].Name.V,ib[2].FFirst.V,
+  /* No key value replacement in user_io -> ihb=NULL */
+  OpenImageFile( pcb,ib,2, NULL,
+                 ib[2].Name.V,ib[2].FFirst.V,
                  IO_Old|IO_FileProtect,pstatus);
   if (*pstatus!=Success) return;
 
@@ -414,7 +419,9 @@ void user_io( CmdBlk * pcb, ImgBlk * ib, int * pstatus )
     argv_filename(pcb,"Flatfield file name",ib,INflat,Default_ipfnflat, pstatus);
     if (*pstatus!=Success) return;
 
-    OpenImageFile( pcb,ib,INflat,ib[INflat].Name.V,ib[INflat].FFirst.V,
+    /* No key value replacement in user_io -> ihb=NULL */
+    OpenImageFile( pcb,ib,INflat,NULL,
+                   ib[INflat].Name.V,ib[INflat].FFirst.V,
                    IO_Old|IO_FileProtect,pstatus);
     if (*pstatus!=Success) return;
   } /* Flatfield file */
@@ -540,7 +547,7 @@ void user_io( CmdBlk * pcb, ImgBlk * ib, int * pstatus )
     pMonSca = (IO_lvexp*) option_parameter_search( SMonSca, pcb, pstatus );
     if (*pstatus!=Success) return;
     if (!pMonSca->I) {
-      if (ReadHeaderLine( pcb,2,ihb,KHScalerI0,pMonSca->V,pstatus)) {
+      if (ReadHeaderLine( pcb,2,num,ihb,KHScalerI0,pMonSca->V,pstatus)) {
         if (*pstatus!=Success) return;
       }
     }
@@ -549,9 +556,9 @@ void user_io( CmdBlk * pcb, ImgBlk * ib, int * pstatus )
     if (*pstatus!=Success) return;
    
     // Get monitor scaler name as info
-    MonSca = option_long ( SMonSca, pcb, pcb->ib, num, 2, pstatus );
+    MonSca = option_long ( SMonSca, pcb, ihb, num, 2, pstatus );
     if (*pstatus!=Success) return;
-    if (ReadHeaderLine( pcb,2,ihb,ScalerKey(KHN,MonSca.V),\
+    if (ReadHeaderLine( pcb,2,num,ihb,ScalerKey(KHN,MonSca.V),\
          MonitorName,pstatus)) {
       if (*pstatus!=Success) return;
     } else MonitorName[0] = (char) 0;
@@ -565,7 +572,7 @@ void user_io( CmdBlk * pcb, ImgBlk * ib, int * pstatus )
     pTimSca = (IO_lvexp*) option_parameter_search( STimSca, pcb, pstatus );
     if (*pstatus!=Success) return;
     if (!pTimSca->I) {
-      if (ReadHeaderLine( pcb,2,ihb,KHScalerTime,pTimSca->V,pstatus)) {
+      if (ReadHeaderLine( pcb,2,num,ihb,KHScalerTime,pTimSca->V,pstatus)) {
         if (*pstatus!=Success) return;
       }
     }
@@ -574,9 +581,9 @@ void user_io( CmdBlk * pcb, ImgBlk * ib, int * pstatus )
     if (*pstatus!=Success) return;
  
     // Get time scaler name as info
-    TimSca = option_long ( STimSca, pcb, pcb->ib, num, 2, pstatus );
+    TimSca = option_long ( STimSca, pcb, ihb, num, 2, pstatus );
     if (*pstatus!=Success) return;
-    if (ReadHeaderLine( pcb,2,ihb,ScalerKey(KHN,TimSca.V),\
+    if (ReadHeaderLine( pcb,2,num,ihb,ScalerKey(KHN,TimSca.V),\
          TimeName,pstatus)) {
       if (*pstatus!=Success) return;
     } else TimeName[0] = (char) 0;
@@ -589,7 +596,7 @@ void user_io( CmdBlk * pcb, ImgBlk * ib, int * pstatus )
     pAnoSca = (IO_lvexp*) option_parameter_search( SAnoSca, pcb, pstatus );
     if (*pstatus!=Success) return;
     if (!pAnoSca->I) {
-      if (ReadHeaderLine( pcb,2,ihb,KHScalerAnode,pAnoSca->V,pstatus)) {
+      if (ReadHeaderLine( pcb,2,num,ihb,KHScalerAnode,pAnoSca->V,pstatus)) {
         if (*pstatus!=Success) return;
       }
     }
@@ -599,9 +606,9 @@ void user_io( CmdBlk * pcb, ImgBlk * ib, int * pstatus )
     if (*pstatus!=Success) return;
  
     // Get anode scaler name as info
-    AnoSca = option_long ( SAnoSca, pcb, pcb->ib, num, 2, pstatus );
+    AnoSca = option_long ( SAnoSca, pcb, ihb, num, 2, pstatus );
     if (*pstatus!=Success) return;
-    if (ReadHeaderLine( pcb,2,ihb,ScalerKey(KHN,AnoSca.V),\
+    if (ReadHeaderLine( pcb,2,num,ihb,ScalerKey(KHN,AnoSca.V),\
          AnodeName,pstatus)) {
       if (*pstatus!=Success) return;
     } else AnodeName[0] = (char) 0;
@@ -617,7 +624,7 @@ void user_io( CmdBlk * pcb, ImgBlk * ib, int * pstatus )
     pMonScaS = (IO_lvexp*) option_parameter_search( SMonScaS, pcb, pstatus );
     if (*pstatus!=Success) return;
     if (!pMonScaS->I)
-      if (ReadHeaderLine( pcb,2,ihb,KHScalerI0S,pMonScaS->V,pstatus)) {
+      if (ReadHeaderLine( pcb,2,num,ihb,KHScalerI0S,pMonScaS->V,pstatus)) {
         if (*pstatus!=Success) return;
       } 
 
@@ -627,9 +634,9 @@ void user_io( CmdBlk * pcb, ImgBlk * ib, int * pstatus )
     if (*pstatus!=Success) return;
 
     // Get number of 2nd I0 monitor scaler name as info
-    MonScaS = option_long ( SMonScaS, pcb, pcb->ib, num, 2, pstatus );
+    MonScaS = option_long ( SMonScaS, pcb, ihb, num, 2, pstatus );
     if (*pstatus!=Success) return;
-    if (ReadHeaderLine( pcb,2,ihb,ScalerKey(KHN,MonScaS.V),\
+    if (ReadHeaderLine( pcb,2,num,ihb,ScalerKey(KHN,MonScaS.V),\
          MonitorNameS,pstatus)) {
       if (*pstatus!=Success) return;
     } else MonitorNameS[0] = (char) 0;
@@ -642,7 +649,7 @@ void user_io( CmdBlk * pcb, ImgBlk * ib, int * pstatus )
     pTimScaS = (IO_lvexp*) option_parameter_search( STimScaS, pcb, pstatus );
     if (*pstatus!=Success) return;
     if (!pTimScaS->I)
-      if (ReadHeaderLine( pcb,2,ihb,KHScalerTimeS,pTimScaS->V,pstatus)) {
+      if (ReadHeaderLine( pcb,2,num,ihb,KHScalerTimeS,pTimScaS->V,pstatus)) {
         if (*pstatus!=Success) return;
       }
  
@@ -652,9 +659,9 @@ void user_io( CmdBlk * pcb, ImgBlk * ib, int * pstatus )
     if (*pstatus!=Success) return;
  
     // Get number of 2nd time scaler name as info
-    TimScaS = option_long ( STimScaS, pcb, pcb->ib, num, 2, pstatus );
+    TimScaS = option_long ( STimScaS, pcb, ihb, num, 2, pstatus );
     if (*pstatus!=Success) return;
-    if (ReadHeaderLine( pcb,2,ihb,ScalerKey(KHN,TimScaS.V),\
+    if (ReadHeaderLine( pcb,2,num,ihb,ScalerKey(KHN,TimScaS.V),\
          TimeNameS,pstatus)) {
       if (*pstatus!=Success) return;
     } else TimeNameS[0] = (char) 0;
@@ -667,7 +674,7 @@ void user_io( CmdBlk * pcb, ImgBlk * ib, int * pstatus )
     pAnoScaS = (IO_lvexp*) option_parameter_search( SAnoScaS, pcb, pstatus );
     if (*pstatus!=Success) return;
     if (!pAnoScaS->I) {
-      if (ReadHeaderLine( pcb,2,ihb,KHScalerAnodeS,pAnoScaS->V,pstatus)) {
+      if (ReadHeaderLine( pcb,2,num,ihb,KHScalerAnodeS,pAnoScaS->V,pstatus)) {
         if (*pstatus!=Success) return;
       }
     }
@@ -677,9 +684,9 @@ void user_io( CmdBlk * pcb, ImgBlk * ib, int * pstatus )
     if (*pstatus!=Success) return;
  
     // Get 2nd anode scaler name as info
-    AnoScaS = option_long ( SAnoScaS, pcb, pcb->ib, num, 2, pstatus );
+    AnoScaS = option_long ( SAnoScaS, pcb, ihb, num, 2, pstatus );
     if (*pstatus!=Success) return;
-    if (ReadHeaderLine( pcb,2,ihb,ScalerKey(KHN,AnoScaS.V),\
+    if (ReadHeaderLine( pcb,2,num,ihb,ScalerKey(KHN,AnoScaS.V),\
          AnodeNameS,pstatus)) {
       if (*pstatus!=Success) return;
     } else AnodeNameS[0] = (char) 0;
@@ -835,10 +842,10 @@ void saxs_normn (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
   float re2 = RELECTRON*RELECTRON;
   IO_long tmp;
 
-  char  monname[InputLineLength];
-  char  timname[InputLineLength];
-  char  anoname[InputLineLength];
-  char  trmname[InputLineLength];
+  char  monname[IO_size];
+  char  timname[IO_size];
+  char  anoname[IO_size];
+  char  trmname[IO_size];
 
   int   monI=FALSE, timI=FALSE, anoI=FALSE, trmI=FALSE;
 
@@ -911,81 +918,81 @@ void saxs_normn (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
              pcb->ImgBlkLen, NumberOfImages-1); *pstatus=Failed; return; }
 
  /* Get special long int options */
-  MonSca = option_long ( SMonSca, pcb, pcb->ib, num, 1, pstatus );
+  MonSca = option_long ( SMonSca, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
-  TimSca = option_long ( STimSca, pcb, pcb->ib, num, 1, pstatus );
+  TimSca = option_long ( STimSca, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
-  AnoSca = option_long ( SAnoSca, pcb, pcb->ib, num, 1, pstatus );
+  AnoSca = option_long ( SAnoSca, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
-  TrmSca = option_long ( STrmSca, pcb, pcb->ib, num, 1, pstatus );
+  TrmSca = option_long ( STrmSca, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
-  MonScaS = option_long ( SMonScaS, pcb, pcb->ib, num, 1, pstatus );
+  MonScaS = option_long ( SMonScaS, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
-  TimScaS = option_long ( STimScaS, pcb, pcb->ib, num, 1, pstatus );
+  TimScaS = option_long ( STimScaS, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
-  AnoScaS = option_long ( SAnoScaS, pcb, pcb->ib, num, 1, pstatus );
+  AnoScaS = option_long ( SAnoScaS, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
-  TrmScaS = option_long ( STrmScaS, pcb, pcb->ib, num, 1, pstatus );
+  TrmScaS = option_long ( STrmScaS, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
-  ScaLen = option_long ( SScaLen, pcb, pcb->ib, num, 1, pstatus );
+  ScaLen = option_long ( SScaLen, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
 
  /* Get special float options */
-  Trm = option_float ( STrm, pcb, pcb->ib, num, 1, pstatus );
+  Trm = option_float ( STrm, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
-  ScaDepth = option_float ( SScaDepth, pcb, pcb->ib, num, 1, pstatus );
+  ScaDepth = option_float ( SScaDepth, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
-  MonCount = option_float ( SMonCount, pcb, pcb->ib, num, 1, pstatus );
+  MonCount = option_float ( SMonCount, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
-  MonZero = option_float ( SMonZero, pcb, pcb->ib, num, 1, pstatus );
+  MonZero = option_float ( SMonZero, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
-  MonFac = option_float ( SMonFac, pcb, pcb->ib, num, 1, pstatus );
+  MonFac = option_float ( SMonFac, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
-  MonVal = option_float ( SMonVal, pcb, pcb->ib, num, 1, pstatus );
+  MonVal = option_float ( SMonVal, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
-  TimCount = option_float ( STimCount, pcb, pcb->ib, num, 1, pstatus );
+  TimCount = option_float ( STimCount, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
-  TimFac = option_float ( STimFac, pcb, pcb->ib, num, 1, pstatus );
+  TimFac = option_float ( STimFac, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
-  TimVal = option_float ( STimVal, pcb, pcb->ib, num, 1, pstatus );
+  TimVal = option_float ( STimVal, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
-  AnoCount = option_float ( SAnoCount, pcb, pcb->ib, num, 1, pstatus );
+  AnoCount = option_float ( SAnoCount, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
-  AnoZero = option_float ( SAnoZero, pcb, pcb->ib, num, 1, pstatus );
+  AnoZero = option_float ( SAnoZero, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
-  AnoFac = option_float ( SAnoFac, pcb, pcb->ib, num, 1, pstatus );
+  AnoFac = option_float ( SAnoFac, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
-  AnoVal = option_float ( SAnoVal, pcb, pcb->ib, num, 1, pstatus );
+  AnoVal = option_float ( SAnoVal, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
-  TrmCount = option_float ( STrmCount, pcb, pcb->ib, num, 1, pstatus );
+  TrmCount = option_float ( STrmCount, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
-  TrmZero = option_float ( STrmZero, pcb, pcb->ib, num, 1, pstatus );
+  TrmZero = option_float ( STrmZero, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
-  TrmFac = option_float ( STrmFac, pcb, pcb->ib, num, 1, pstatus );
+  TrmFac = option_float ( STrmFac, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
-  TrmVal = option_float ( STrmVal, pcb, pcb->ib, num, 1, pstatus );
+  TrmVal = option_float ( STrmVal, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
-  MonCountS = option_float ( SMonCountS, pcb, pcb->ib, num, 1, pstatus );
+  MonCountS = option_float ( SMonCountS, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
-  MonFacS = option_float ( SMonFacS, pcb, pcb->ib, num, 1, pstatus );
+  MonFacS = option_float ( SMonFacS, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
-  TimCountS = option_float ( STimCountS, pcb, pcb->ib, num, 1, pstatus );
+  TimCountS = option_float ( STimCountS, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
-  TimFacS = option_float ( STimFacS, pcb, pcb->ib, num, 1, pstatus );
+  TimFacS = option_float ( STimFacS, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
-  AnoCountS = option_float ( SAnoCountS, pcb, pcb->ib, num, 1, pstatus );
+  AnoCountS = option_float ( SAnoCountS, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
-  AnoFacS = option_float ( SAnoFacS, pcb, pcb->ib, num, 1, pstatus );
+  AnoFacS = option_float ( SAnoFacS, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
-  TrmCountS = option_float ( STrmCountS, pcb, pcb->ib, num, 1, pstatus );
+  TrmCountS = option_float ( STrmCountS, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
-  TrmFacS = option_float ( STrmFacS, pcb, pcb->ib, num, 1, pstatus );
+  TrmFacS = option_float ( STrmFacS, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
-  I1sum = option_float ( SI1sum, pcb, pcb->ib, num, 1, pstatus );
+  I1sum = option_float ( SI1sum, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
-  BinCorFac = option_float ( SBinCorFac, pcb, pcb->ib, num, 1, pstatus );
+  BinCorFac = option_float ( SBinCorFac, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
-  RelDis = option_float ( SRelDis, pcb, pcb->ib, num, 1, pstatus );
+  RelDis = option_float ( SRelDis, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
 
   /* Check, if rejeciton correction is requested */
@@ -1006,7 +1013,7 @@ void saxs_normn (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
   if (ScalOn.I) 
      scaleron=(int)ScalOn.V; 
    else /* test, if KHScalerI0 can be read */
-     scaleron=ReadHeaderLong(pcb,2,ihb,KHScalerI0,&tmp.V,pstatus);
+     scaleron=ReadHeaderLong(pcb,2,num,ihb,KHScalerI0,&tmp.V,pstatus);
 
   GetReferenceParameters( pcb, ihb, 0, imax-1,
                           Off_1, Off_2, Ps_1,  Ps_2, pstatus );
@@ -1052,7 +1059,7 @@ void saxs_normn (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
 
   /* if RelDis is set, calculate SampleDistance from detectorposition+RelDis */
   if (RelDis.I) {
-      ReadHeaderFloat(pcb,2,ihb,KDetectorPosition,&detectorposition,pstatus);
+      ReadHeaderFloat(pcb,2,num,ihb,KDetectorPosition,&detectorposition,pstatus);
       if (*pstatus!=Success) return;
       ihb[0].SampleDistance.V = detectorposition + RelDis.V;
       ihb[1].SampleDistance.V = ihb[0].SampleDistance.V;
@@ -1152,7 +1159,7 @@ void saxs_normn (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
 
     /* --- maxcount */
     if (!ScaDepth.I) 
-      if (ReadHeaderFloat(pcb,2,ihb,KHScalerDepth,&ScaDepth.V,
+      if (ReadHeaderFloat(pcb,2,num,ihb,KHScalerDepth,&ScaDepth.V,
           pstatus)) {
         if (*pstatus!=Success) return;
         } 
@@ -1167,7 +1174,7 @@ void saxs_normn (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
 
     /* --- scalerlen */
     if (!ScaLen.I) {
-      if (ReadHeaderLong(pcb,2,ihb,KHScalerLen, &ScaLen.V,
+      if (ReadHeaderLong(pcb,2,num,ihb,KHScalerLen, &ScaLen.V,
           pstatus)) { if (*pstatus!=Success) return;
         } 
       }
@@ -1180,7 +1187,7 @@ void saxs_normn (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
     monI = FALSE;
     /*     monscaler1 */
     if (!MonSca.I) {
-      if (!ReadHeaderLong(pcb,2,ihb,KHScalerI0,&MonSca.V,
+      if (!ReadHeaderLong(pcb,2,num,ihb,KHScalerI0,&MonSca.V,
           pstatus)) { *pstatus = KeywordMissing;
         fprintf(ERROUT,"\nMissing keyword for monscaler1 %s\n",KHScalerI0); } 
       if (*pstatus!=Success) return;
@@ -1191,7 +1198,7 @@ void saxs_normn (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
     if ((1<=monscaler) && (monscaler<=scalerlen)) {
       /*     moncount1 */
       if (!MonCount.I) {
-        if (!ReadHeaderFloat(pcb,2,ihb,ScalerKey(KHC,monscaler),&moncount,
+        if (!ReadHeaderFloat(pcb,2,num,ihb,ScalerKey(KHC,monscaler),&moncount,
             pstatus)) { *pstatus = KeywordMissing;
           fprintf(ERROUT,"\nMissing keyword for moncount1 %s\n",
             ScalerKey(KHC,monscaler));}
@@ -1200,7 +1207,7 @@ void saxs_normn (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
 
       /*      monfac1 */
       if (!MonFac.I) {
-        if (!ReadHeaderFloat(pcb,2,ihb,ScalerKey(KHF,monscaler),&monfac,
+        if (!ReadHeaderFloat(pcb,2,num,ihb,ScalerKey(KHF,monscaler),&monfac,
             pstatus)) { *pstatus = KeywordMissing;
           fprintf(ERROUT,"\nMissing keyword for monfac1 %s\n",
             ScalerKey(KHF,monscaler)); }
@@ -1209,7 +1216,7 @@ void saxs_normn (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
   
       /*     monzero1 */
       if (!MonZero.I) {
-        if (!ReadHeaderFloat(pcb,2,ihb,ScalerKey(KHZ,monscaler),&monzero,
+        if (!ReadHeaderFloat(pcb,2,num,ihb,ScalerKey(KHZ,monscaler),&monzero,
             pstatus)) { *pstatus = KeywordMissing;
           fprintf(ERROUT,"\nMissing keyword for monzero1 %s\n",
             ScalerKey(KHZ,monscaler)); }
@@ -1222,7 +1229,7 @@ void saxs_normn (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
 
     /*     monscaler2 */
     if (!MonScaS.I) {
-      if (ReadHeaderLong(pcb,2,ihb,KHScalerI0S, &MonScaS.V,
+      if (ReadHeaderLong(pcb,2,num,ihb,KHScalerI0S, &MonScaS.V,
           pstatus)) { if (*pstatus!=Success) return; 
         } 
       }
@@ -1231,7 +1238,7 @@ void saxs_normn (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
     if ((1<=monscaler2) && (monscaler2<=scalerlen)) {
       /*     moncount2 */
       if (!MonCountS.I) {
-        if (!ReadHeaderFloat(pcb,2,ihb,ScalerKey(KHC,monscaler2),&moncount2,
+        if (!ReadHeaderFloat(pcb,2,num,ihb,ScalerKey(KHC,monscaler2),&moncount2,
             pstatus)) { *pstatus = KeywordMissing;
          fprintf(ERROUT,"\nMissing keyword for moncount2 %s\n",
            ScalerKey(KHC,monscaler2));}
@@ -1240,7 +1247,7 @@ void saxs_normn (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
         
       /*      monfac2 */
       if (!MonFacS.I) {
-        if (!ReadHeaderFloat(pcb,2,ihb,ScalerKey(KHF,monscaler2),&monfac2,
+        if (!ReadHeaderFloat(pcb,2,num,ihb,ScalerKey(KHF,monscaler2),&monfac2,
             pstatus)) { *pstatus = KeywordMissing;
           fprintf(ERROUT,"\nMissing keyword for monfac2 %s\n",
             ScalerKey(KHF,monscaler2)); }
@@ -1249,7 +1256,7 @@ void saxs_normn (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
  
       /*     monzero2  (no separate option) */
       if (!MonZero.I) {
-        if (!ReadHeaderFloat(pcb,2,ihb,ScalerKey(KHZ,monscaler2),&monzero2,
+        if (!ReadHeaderFloat(pcb,2,num,ihb,ScalerKey(KHZ,monscaler2),&monzero2,
             pstatus)) { *pstatus = KeywordMissing;
           fprintf(ERROUT,"\nMissing keyword for monzero2 %s\n",
             ScalerKey(KHZ,monscaler2));}
@@ -1268,7 +1275,7 @@ void saxs_normn (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
       } /* if (monscaler2 ... */
 
     /*     monname */
-    if (ReadHeaderLine(pcb,2,ihb,ScalerKey(KHN,monscaler),monname, 
+    if (ReadHeaderLine(pcb,2,num,ihb,ScalerKey(KHN,monscaler),monname, 
         pstatus)) { if (*pstatus!=Success) return; 
       } else monname[0] = (char) 0;
 
@@ -1285,7 +1292,7 @@ void saxs_normn (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
     timI = FALSE;
     /*     timscaler1 */
     if (!TimSca.I) {
-      if (!ReadHeaderLong(pcb,2,ihb,KHScalerTime, &TimSca.V,
+      if (!ReadHeaderLong(pcb,2,num,ihb,KHScalerTime, &TimSca.V,
           pstatus)) { *pstatus = KeywordMissing;
         fprintf(ERROUT,"\nMissing keyword for timscaler1 %s\n",KHScalerTime); }
       if (*pstatus!=Success) return;
@@ -1296,7 +1303,7 @@ void saxs_normn (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
     if ((1<=timscaler) && (timscaler<=scalerlen)) {
       /*     timcount1 */
       if (!TimCount.I) {
-        if (!ReadHeaderFloat(pcb,2,ihb,ScalerKey(KHC,timscaler),&timcount, 
+        if (!ReadHeaderFloat(pcb,2,num,ihb,ScalerKey(KHC,timscaler),&timcount, 
             pstatus)) { *pstatus = KeywordMissing;
           fprintf(ERROUT,"\nMissing keyword for timcount1 %s\n",
             ScalerKey(KHC,timscaler));}
@@ -1305,7 +1312,7 @@ void saxs_normn (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
   
       /*      timfac1 */
       if (!TimFac.I) {
-        if (!ReadHeaderFloat(pcb,2,ihb,ScalerKey(KHF,timscaler), &timfac,
+        if (!ReadHeaderFloat(pcb,2,num,ihb,ScalerKey(KHF,timscaler), &timfac,
             pstatus)) { *pstatus = KeywordMissing;
           fprintf(ERROUT,"\nMissing keyword for timfac1 %s\n",
             ScalerKey(KHF,timscaler)); }
@@ -1318,7 +1325,7 @@ void saxs_normn (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
 
     /*     timscaler2 */
     if (!TimScaS.I) {
-      if (ReadHeaderLong(pcb,2,ihb,KHScalerTimeS,&TimScaS.V,
+      if (ReadHeaderLong(pcb,2,num,ihb,KHScalerTimeS,&TimScaS.V,
           pstatus)) { if (*pstatus!=Success) return;
         }
       }
@@ -1327,7 +1334,7 @@ void saxs_normn (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
     if ((1<=timscaler2) && (timscaler2<=scalerlen)) {
       /*     timcount2 */
       if (!TimCountS.I) {
-        if (!ReadHeaderFloat(pcb,2,ihb,ScalerKey(KHC,timscaler2),&timcount2, 
+        if (!ReadHeaderFloat(pcb,2,num,ihb,ScalerKey(KHC,timscaler2),&timcount2, 
             pstatus)) { *pstatus = KeywordMissing;
          fprintf(ERROUT,"\nMissing keyword for timcount2 %s\n",
            ScalerKey(KHC,timscaler2));}
@@ -1336,7 +1343,7 @@ void saxs_normn (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
   
       /*      timfac2 */
       if (!TimFacS.I) {
-        if (!ReadHeaderFloat(pcb,2,ihb,ScalerKey(KHF,timscaler2), &timfac2, 
+        if (!ReadHeaderFloat(pcb,2,num,ihb,ScalerKey(KHF,timscaler2), &timfac2, 
             pstatus)) { *pstatus = KeywordMissing;
           fprintf(ERROUT,"\nMissing keyword for timfac2 %s\n",
             ScalerKey(KHF,timscaler2));}
@@ -1354,7 +1361,7 @@ void saxs_normn (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
       } /* if (1<=timscaler2 ... */
   
     /*     timname */
-    if (ReadHeaderLine(pcb,2,ihb,ScalerKey(KHN,timscaler),timname, 
+    if (ReadHeaderLine(pcb,2,num,ihb,ScalerKey(KHN,timscaler),timname, 
         pstatus)) { if (*pstatus!=Success) return;
       } else timname[0] = (char) 0;
   
@@ -1370,7 +1377,7 @@ void saxs_normn (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
     anoI = FALSE;
     /*     anoscaler1 */
      if (!AnoSca.I) {
-      if (!ReadHeaderLong(pcb,2,ihb,KHScalerAnode, &AnoSca.V,
+      if (!ReadHeaderLong(pcb,2,num,ihb,KHScalerAnode, &AnoSca.V,
           pstatus)) { *pstatus = KeywordMissing;
         fprintf(ERROUT,"\nMissing keyword for anoscaler1 %s\n",KHScalerAnode); }
       if (*pstatus!=Success) return;
@@ -1380,7 +1387,7 @@ void saxs_normn (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
     if ((1<=anoscaler) && (anoscaler<=scalerlen)) {
       /*     anocount1 */
       if (!AnoCount.I) {
-        if (!ReadHeaderFloat(pcb,2,ihb,ScalerKey(KHC,anoscaler),&anocount,
+        if (!ReadHeaderFloat(pcb,2,num,ihb,ScalerKey(KHC,anoscaler),&anocount,
             pstatus)) { *pstatus = KeywordMissing;
           fprintf(ERROUT,"\nMissing keyword for anocount1 %s\n",
             ScalerKey(KHC,anoscaler));}
@@ -1389,7 +1396,7 @@ void saxs_normn (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
 
       /*      anofac1 */
       if (!AnoFac.I) {
-        if (!ReadHeaderFloat(pcb,2,ihb,ScalerKey(KHF,anoscaler), &anofac,
+        if (!ReadHeaderFloat(pcb,2,num,ihb,ScalerKey(KHF,anoscaler), &anofac,
             pstatus)) { *pstatus = KeywordMissing;
           fprintf(ERROUT,"\nMissing keyword for anofac1 %s\n",
             ScalerKey(KHF,anoscaler)); }
@@ -1398,7 +1405,7 @@ void saxs_normn (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
   
       /*     anozero1 */
       if (!AnoZero.I) {
-        if (!ReadHeaderFloat(pcb,2,ihb,ScalerKey(KHZ,anoscaler),&anozero,
+        if (!ReadHeaderFloat(pcb,2,num,ihb,ScalerKey(KHZ,anoscaler),&anozero,
             pstatus)) { *pstatus = KeywordMissing;
           fprintf(ERROUT,"\nMissing keyword for anozero1 %s\n",
             ScalerKey(KHZ,anoscaler)); }
@@ -1412,7 +1419,7 @@ void saxs_normn (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
 
     /*     anoscaler2 */
    if (!AnoScaS.I) {
-      if (ReadHeaderLong(pcb,2,ihb,KHScalerAnodeS, &AnoScaS.V,
+      if (ReadHeaderLong(pcb,2,num,ihb,KHScalerAnodeS, &AnoScaS.V,
           pstatus)) { if (*pstatus!=Success) return;
         }
       }
@@ -1421,7 +1428,7 @@ void saxs_normn (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
     if ((1<=anoscaler2) && (anoscaler2<=scalerlen)) {
       /*     anocount2 */
       if (!AnoCountS.I) {
-        if (!ReadHeaderFloat(pcb,2,ihb,ScalerKey(KHC,anoscaler2), &anocount2, 
+        if (!ReadHeaderFloat(pcb,2,num,ihb,ScalerKey(KHC,anoscaler2), &anocount2, 
             pstatus)) { *pstatus = KeywordMissing;
          fprintf(ERROUT,"\nMissing keyword for anocount2 %s\n",
            ScalerKey(KHC,anoscaler2));}
@@ -1430,7 +1437,7 @@ void saxs_normn (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
 
       /*      anofac2 */
       if (!AnoFacS.I) {
-        if (!ReadHeaderFloat(pcb,2,ihb,ScalerKey(KHF,anoscaler2),&anofac2,
+        if (!ReadHeaderFloat(pcb,2,num,ihb,ScalerKey(KHF,anoscaler2),&anofac2,
             pstatus)) { *pstatus = KeywordMissing;
           fprintf(ERROUT,"\nMissing keyword for anofac2 %s\n",
             ScalerKey(KHF,anoscaler2)); }
@@ -1439,7 +1446,7 @@ void saxs_normn (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
 
       /*     anozero2  (no separate option) */
       if (!AnoZero.I) {
-        if (!ReadHeaderFloat(pcb,2,ihb,ScalerKey(KHZ,anoscaler2),&anozero2,
+        if (!ReadHeaderFloat(pcb,2,num,ihb,ScalerKey(KHZ,anoscaler2),&anozero2,
             pstatus)) { *pstatus = KeywordMissing;
           fprintf(ERROUT,"\nMissing keyword for anozero2 %s\n",
             ScalerKey(KHZ,anoscaler2));
@@ -1460,7 +1467,7 @@ void saxs_normn (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
       } /* if (anoscaler2 ... */
 
     /*     anoname */
-    if (ReadHeaderLine(pcb,2,ihb,ScalerKey(KHN,anoscaler), anoname, 
+    if (ReadHeaderLine(pcb,2,num,ihb,ScalerKey(KHN,anoscaler), anoname, 
         pstatus)) { if (*pstatus!=Success) return;
       } else anoname[0] = (char) 0;
 
@@ -1477,7 +1484,7 @@ void saxs_normn (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
     trmI = FALSE;
     /*     trmscaler1 */
     if (!TrmSca.I) {
-      if (!ReadHeaderLong(pcb,2,ihb,KHScalerI1,&TrmSca.V,
+      if (!ReadHeaderLong(pcb,2,num,ihb,KHScalerI1,&TrmSca.V,
           pstatus)) { *pstatus = KeywordMissing;
         fprintf(ERROUT,"\nMissing keyword for trmscaler1 %s\n",KHScalerI1); }
       if (*pstatus!=Success) return;
@@ -1487,7 +1494,7 @@ void saxs_normn (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
     if ((1<=trmscaler) && (trmscaler<=scalerlen)) {
       /*     trmcount1 */
       if (!TrmCount.I) {
-        if (!ReadHeaderFloat(pcb,2,ihb,ScalerKey(KHC,trmscaler),&trmcount,
+        if (!ReadHeaderFloat(pcb,2,num,ihb,ScalerKey(KHC,trmscaler),&trmcount,
             pstatus)) { *pstatus = KeywordMissing;
           fprintf(ERROUT,"\nMissing keyword for trmcount1 %s\n",
             ScalerKey(KHC,trmscaler));}
@@ -1496,7 +1503,7 @@ void saxs_normn (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
 
       /*      trmfac1 */
       if (!TrmFac.I) {
-        if (!ReadHeaderFloat(pcb,2,ihb,ScalerKey(KHF,trmscaler),&trmfac,
+        if (!ReadHeaderFloat(pcb,2,num,ihb,ScalerKey(KHF,trmscaler),&trmfac,
             pstatus)) { *pstatus = KeywordMissing;
           fprintf(ERROUT,"\nMissing keyword for trmfac1 %s\n",
             ScalerKey(KHF,trmscaler)); }
@@ -1505,7 +1512,7 @@ void saxs_normn (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
 
       /*     trmzero1 */
       if (!TrmZero.I) {
-        if (!ReadHeaderFloat(pcb,2,ihb,ScalerKey(KHZ,trmscaler),&trmzero,
+        if (!ReadHeaderFloat(pcb,2,num,ihb,ScalerKey(KHZ,trmscaler),&trmzero,
             pstatus)) { *pstatus = KeywordMissing;
           fprintf(ERROUT,"\nMissing keyword for trmzero1 %s\n",
             ScalerKey(KHZ,trmscaler));}
@@ -1518,7 +1525,7 @@ void saxs_normn (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
 
     /*     trmscaler2 */
     if (!TrmScaS.I) {
-      if (ReadHeaderLong(pcb,2,ihb,KHScalerI1S, &TrmScaS.V,
+      if (ReadHeaderLong(pcb,2,num,ihb,KHScalerI1S, &TrmScaS.V,
           pstatus)) { if (*pstatus!=Success) return;
         }
       }
@@ -1527,7 +1534,7 @@ void saxs_normn (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
     if ((1<=trmscaler2) && (trmscaler2<=scalerlen)) {
       /*     trmcount2 */
       if (!TrmCountS.I) {
-        if (!ReadHeaderFloat(pcb,2,ihb,ScalerKey(KHC,trmscaler2),&trmcount2,
+        if (!ReadHeaderFloat(pcb,2,num,ihb,ScalerKey(KHC,trmscaler2),&trmcount2,
             pstatus)) { *pstatus = KeywordMissing;
             fprintf(ERROUT,"\nMissing keyword for trmcount2 %s\n",
             ScalerKey(KHC,trmscaler2));}
@@ -1536,7 +1543,7 @@ void saxs_normn (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
 
       /*      trmfac2 */
       if (!TrmFacS.I) {
-        if (!ReadHeaderFloat(pcb,2,ihb,ScalerKey(KHF,trmscaler2),&trmfac2,
+        if (!ReadHeaderFloat(pcb,2,num,ihb,ScalerKey(KHF,trmscaler2),&trmfac2,
             pstatus)) { *pstatus = KeywordMissing;
           fprintf(ERROUT,"\nMissing keyword for trmfac2 %s\n",
             ScalerKey(KHF,trmscaler2));}
@@ -1545,7 +1552,7 @@ void saxs_normn (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
 
       /*     trmzero2  (no separate option) */
       if (!TrmZero.I) {
-        if (!ReadHeaderFloat(pcb,2,ihb,ScalerKey(KHZ,trmscaler2),&trmzero2,
+        if (!ReadHeaderFloat(pcb,2,num,ihb,ScalerKey(KHZ,trmscaler2),&trmzero2,
             pstatus)) { *pstatus = KeywordMissing;
          fprintf(ERROUT,"\nMissing keyword for trmzero2 %s\n",
            ScalerKey(KHZ,trmscaler2));}
@@ -1565,7 +1572,7 @@ void saxs_normn (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
       } /* if (trmscaler2 ... */
 
       /*     trmname */
-      if (ReadHeaderLine(pcb,2,ihb,ScalerKey(KHN,trmscaler),trmname,
+      if (ReadHeaderLine(pcb,2,num,ihb,ScalerKey(KHN,trmscaler),trmname,
           pstatus)) { if (*pstatus!=Success) return;
          } else trmname[0] = (char) 0;
 
@@ -1595,7 +1602,7 @@ void saxs_normn (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
     if (monitornormalization) {
       if (MonVal.I) monval = MonVal.V;
         else {
-        if (!ReadHeaderFloat(pcb,2,ihb,KValueI0,&monval, pstatus)) { 
+        if (!ReadHeaderFloat(pcb,2,num,ihb,KValueI0,&monval, pstatus)) { 
           *pstatus = KeywordMissing;
           fprintf(ERROUT,"\nMissing keyword for monnal %s\n",KValueI0); }
         if (*pstatus!=Success) return;
@@ -1608,7 +1615,7 @@ void saxs_normn (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
     if (monitornormalization) {
       if (TrmVal.I) trmval = TrmVal.V;
         else {
-        if (!ReadHeaderFloat(pcb,2,ihb,KValueI1,&trmval, pstatus)) { 
+        if (!ReadHeaderFloat(pcb,2,num,ihb,KValueI1,&trmval, pstatus)) { 
           *pstatus = KeywordMissing;
           fprintf(ERROUT,"\nMissing keyword for trmval %s\n",KValueI1); }
         if (*pstatus!=Success) return;
@@ -1621,7 +1628,7 @@ void saxs_normn (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
     if (monitornormalization) {
       if (TimVal.I) timval = TimVal.V;
         else {
-        if (!ReadHeaderFloat(pcb,2,ihb,KValueTime,&timval, pstatus)) {
+        if (!ReadHeaderFloat(pcb,2,num,ihb,KValueTime,&timval, pstatus)) {
           *pstatus = KeywordMissing;
           fprintf(ERROUT,"\nMissing keyword for timval %s\n",KValueTime); }
         if (*pstatus!=Success) return;
@@ -1634,7 +1641,7 @@ void saxs_normn (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
     if (rejectioncorrection) {
       if (AnoVal.I) anoval = AnoVal.V;
         else {       
-        if (!ReadHeaderFloat(pcb,2,ihb,KValueAnode,&anoval, pstatus)) { 
+        if (!ReadHeaderFloat(pcb,2,num,ihb,KValueAnode,&anoval, pstatus)) { 
           *pstatus = KeywordMissing; 
           fprintf(ERROUT,"\nMissing keyword for anoval %s\n",KValueAnode); } 
         if (*pstatus!=Success) return;
