@@ -194,13 +194,13 @@ void saxs_row (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
      printf("%d images found, 1 input and 1 output image required\n",
              pcb->ImgBlkLen); *pstatus=Failed; return; }
 
-  Col1 = option_float ( SCol1, pcb, pcb->ib, num, 1, pstatus );
+  Col1 = option_float ( SCol1, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
-  Col2 = option_float ( SCol2, pcb, pcb->ib, num, 1, pstatus );
+  Col2 = option_float ( SCol2, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
-  Row1 = option_float ( SRow1, pcb, pcb->ib, num, 1, pstatus );
+  Row1 = option_float ( SRow1, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
-  Row2 = option_float ( SRow2, pcb, pcb->ib, num, 1, pstatus );
+  Row2 = option_float ( SRow2, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
   FlgInt = option_flag ( SFlgInt, pcb, pstatus );
   if (*pstatus!=Success) return; 
@@ -366,18 +366,18 @@ Do all the keyboard io and return cb, and ib
 
 void user_io(CmdBlk * pcb, ImgBlk * ib, int * pstatus)
 {
-  char  progfn[InputLineLength];
+  char  progfn[IO_size];
   ImgHeadBlk ihb[BlockNum];
 
   float ROff_1, RPs_1, UOff_1, UPs_1;
   float ROff_2, RPs_2, UOff_2, UPs_2;
 
-  char prompt_line[IO_len];
+  char prompt_line[IO_size];
  
   IO_flexp *pRow1, *pRow2;
 
  /* Determine program name without directory */
-   (void) RightFR((char *) pcb->argv[0],DirSeparator,progfn,InputLineLength);
+   (void) RightFR((char *) pcb->argv[0],DirSeparator,progfn,IO_size);
 
   /* show usage if no arguments are given */
   if (pcb->argc<=1) printf("Usage: %s %s\n",progfn,Usage);
@@ -387,7 +387,7 @@ void user_io(CmdBlk * pcb, ImgBlk * ib, int * pstatus)
   printf("   %s %s\n",progfn,Version);
   printf("\n");
 
-  /* --- output into first image */
+  /* --- output to first image */
   if (!ib[0].First.I) { ib[0].First.V = 1; ib[0].First.I = TRUE; }
   /* --- only 1 output image */
   if (!ib[0].Inc.I) { ib[0].Inc.V = 0; ib[0].Inc.I = TRUE; }
