@@ -73,9 +73,12 @@
   2011-05-14  PB  V4.11 split.h -> strlib.h
   2012-01-31  PB  V4.12 ImageLoop parameters updated
   2016-02-17  PB  V4.13 remove .gz suffix when creating output filename
+  2016-10-07  PB  V4.14 filename_full expansion of Prefix option removed
+  2017-10-20  PB  V4.15 No key value replacement in user_io,
+                        ReadHeaderFloat arguments updated
 
 ---*/
-# define Version  "saxs_scal V4.13 2016-02-17, Peter Boesecke"
+# define Version  "saxs_scal V4.15 2017-10-20, Peter Boesecke"
 
 # include <errno.h>
 # include <stdio.h>
@@ -97,7 +100,7 @@
 
 # define Usage "[options] \n\
                 <i1nam> <onam> <i1fst> <i1lst> <i1inc> <key>{,<key>}\n\
-Purpose: Extract keyword values from image headers and write them into a file\n\
+Purpose: Extract keyword values from image headers and write them to a file\n\
 Options: -key, -pref, -ext, -fnam"
 
 # define BlockNum 2       /* 1 input sequence + 1 output sequence */
@@ -117,7 +120,7 @@ Ask for all file names in the following way:
 
   output dummy [<dummy in first image>] :
 
-If blkno >= 1 the header block of the first image is read into ihb[1].
+If blkno >= 1 the header block of the first image is read in ihb[1].
 The routine must be used for all images at once.
 ---------------------------------------------------------------------------*/
 void SaxsScalArgvFilenames ( CmdBlk * pcb, ImgBlk ib[], ImgHeadBlk ihb[],
@@ -150,7 +153,9 @@ void SaxsScalArgvFilenames ( CmdBlk * pcb, ImgBlk ib[], ImgHeadBlk ihb[],
     /* Exit in all cases after an abort from the first argument */
     if (*pstatus!=Success) return;
 
-    OpenImageFile( pcb,ib,blkno,ib[blkno].Name.V,ib[1].FFirst.V,
+    /* No key value replacement in user_io -> ihb=NULL */
+    OpenImageFile( pcb,ib,blkno,NULL,
+                   ib[blkno].Name.V,ib[1].FFirst.V,
                    IO_Old|IO_FileProtect,pstatus);
     if (*pstatus!=Success) return;
 
@@ -169,7 +174,7 @@ void SaxsScalArgvFilenames ( CmdBlk * pcb, ImgBlk ib[], ImgHeadBlk ihb[],
       ib[0].OpenMode.V = IO_ImageProtect | IO_DontReadData | IO_DontReadWrite |\
                          IO_DontOpen;
       ib[0].OpenMode.I = TRUE;
-      }
+    }
 
     /*--- Argument  : text file prefix */
     if (ib[0].Name.I) nameblk = 0; else nameblk = 1;
@@ -179,17 +184,17 @@ void SaxsScalArgvFilenames ( CmdBlk * pcb, ImgBlk ib[], ImgHeadBlk ihb[],
     if (!pPrefix->I) {
       /* Determine prefix from input file name */
       /* Check for .gz extension (use pPrefix->V as temporary buffer) */
-      filename_extension ( pPrefix->V, IO_len, ib[nameblk].Name.V );
+      filename_extension ( pPrefix->V, IO_size, ib[nameblk].Name.V );
       if (!strcmp(pPrefix->V,".gz")) {
         // Remove .gz extension
-        filename_body ( pPrefix->V, IO_len, ib[nameblk].Name.V ); // remove .gz
-        filename_body ( pPrefix->V, IO_len,  pPrefix->V ); // remove ext.
+        filename_body ( pPrefix->V, IO_size, ib[nameblk].Name.V ); // remove .gz
+        filename_body ( pPrefix->V, IO_size,  pPrefix->V ); // remove ext.
       } else {
-        filename_body ( pPrefix->V, IO_len, ib[nameblk].Name.V ); // remove ext.
+        filename_body ( pPrefix->V, IO_size, ib[nameblk].Name.V ); // remove ext.
       }
 
       // remove path
-      filename_name ( pPrefix->V, IO_len, pPrefix->V );
+      filename_name ( pPrefix->V, IO_size, pPrefix->V );
       pPrefix->I = TRUE;
     }
 
@@ -202,12 +207,15 @@ void SaxsScalArgvFilenames ( CmdBlk * pcb, ImgBlk ib[], ImgHeadBlk ihb[],
     extract_filenumbers( &ib[0], pPrefix, TRUE, pstatus );
     if (*pstatus!=Success) return;
  
-    /* Expand Prefix to the full file name */
-    if (!filename_full (pPrefix->V,IO_len,pPrefix->V)) {
-      fprintf(ERROUT,"Input file name or path not allowed\n");
-      *pstatus = FileNotFound;
-      return;
-    }
+    /* Prefix cannot be expanded with filename_full
+       before replacing all keyvalues */
+
+    //+++++++++ /* Expand Prefix to the full file name */
+    //+++++++++ if (!filename_full (pPrefix->V,IO_size,pPrefix->V)) {
+    //+++++++++   fprintf(ERROUT,"Input file name or path not allowed\n");
+    //+++++++++   *pstatus = FileNotFound;
+    //+++++++++   return;
+    //+++++++++ }
  
     /*--- Take prefix as default for output filename */
     if (!ib[0].Name.I) {
@@ -276,12 +284,12 @@ void saxs_scal (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
   int j;
   int ErrorValue;
 
-  char fname_full[IO_len];
+  char fname_full[IO_size];
 
   FILE *OutputFileId;
 
   IO_line KeyList, FName, Extension, Prefix;
-  char Key[IO_len], KeyValue[IO_len];
+  char Key[IO_size], KeyValue[IO_size];
   int parno;
 
   *pstatus = Success;
@@ -298,26 +306,26 @@ void saxs_scal (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
   }
 
   /* --- Get option values */
-  KeyList = option_line ( SKey, pcb, pcb->ib, num, 1, pstatus );
+  KeyList = option_line ( SKey, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
-  Extension = option_line ( SExtension, pcb, pcb->ib, num, 1, pstatus );
+  Extension = option_line ( SExtension, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
-  Prefix = option_line ( SPrefix, pcb, pcb->ib, num, 1, pstatus );
+  Prefix = option_line ( SPrefix, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
-  FName = option_line ( SFName, pcb, pcb->ib, num, 1, pstatus );
+  FName = option_line ( SFName, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
 
   /*--- Get output filename */
   if (!FName.I) {
     // first key is used for filename
-    strlib_param ( Key, IO_len, KeyList.V, ',', 1 );
+    strlib_param ( Key, IO_size, KeyList.V, ',', 1 );
     sprintf(FName.V,"%s_%s%s",Prefix.V,Key,Extension.V);
   }
 
   /* Generate filename of ASCII file */
-  filename_parameter (fname_full,IO_len,FName.V,0);
-  filename_pattern (fname_full,IO_len,fname_full,(pcb->ib)[0].FileNumber);
-  filename_full (fname_full,IO_len,fname_full);
+  filename_parameter (fname_full,IO_size,FName.V,0);
+  filename_pattern (fname_full,IO_size,fname_full,(pcb->ib)[0].FileNumber);
+  filename_full (fname_full,IO_size,fname_full);
 
   /*--- Open output file
     If this is the first image of the sequence, an existing file 
@@ -347,7 +355,7 @@ void saxs_scal (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
     /* --- Write key names */
     if (KeyList.I) {
       parno = 1;
-      while (strlib_param ( Key, IO_len, KeyList.V, ',', parno++ )) {
+      while (strlib_param ( Key, IO_size, KeyList.V, ',', parno++ )) {
         if (strlen(Key)) {
           fprintf(OutputFileId,"\t%s",Key);
         }
@@ -363,10 +371,10 @@ void saxs_scal (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
   /*--- Read and write keyword values */
   if (KeyList.I) {
     parno = 1;
-    while (strlib_param ( Key, IO_len, KeyList.V, ',', parno++ )) {
+    while (strlib_param ( Key, IO_size, KeyList.V, ',', parno++ )) {
       if (strlen(Key)) {
         if (pcb->TestBit) printf("Reading keyword %s\n",Key);
-        ReadHeaderLine          ( pcb, 1, ihb, Key, KeyValue, pstatus );
+        ReadHeaderLine          ( pcb, 1, num, ihb, Key, KeyValue, pstatus );
         if (*pstatus) fprintf(OutputFileId,"\t%s","(not found)");
         else fprintf(OutputFileId,"\t%s",KeyValue);
       }
@@ -393,7 +401,7 @@ Do all the keyboard io and return cb, and ib
 
 void user_io(CmdBlk * pcb, ImgBlk * ib, int * pstatus)
 {
-  char  progfn[InputLineLength];
+  char  progfn[IO_size];
   ImgHeadBlk ihb[BlockNum];
 
   float ROff_1, RPs_1, UOff_1, UPs_1;
@@ -403,7 +411,7 @@ void user_io(CmdBlk * pcb, ImgBlk * ib, int * pstatus)
 
  /* Determine program name without directory */
    (void) RightFR((char *) pcb->argv[0],DirSeparator,progfn,
-           InputLineLength);
+           IO_size);
 
   /* show usage if no arguments are given */
   if (pcb->argc<=1) printf("Usage: %s %s\n",progfn,Usage);
