@@ -21,7 +21,7 @@
 /*+++
 1 saxs_stat
   Calculate some statistics on the image input file (esrf format) and
-  write them into the output text file. The statistics calculated are:
+  write them to the output text file. The statistics calculated are:
                 inum    Image number
                 dim1    Dimension 1
                 dim2    Dimension 2
@@ -34,6 +34,7 @@
                 ndum    Number of dummies
                 max     Maximum Value
                 min     Minimum Value
+                amax    Maximum absolute value
                 amin    Minimum absolute value
                 dum     Dummy value
                 sum     Sum of data values
@@ -67,7 +68,7 @@
   -sel <column name>         : select the column that should be
                                written to the output file
   -key <key>                 : selects the key(s) which values are written
-                               into a separate column of the output file.
+                               to a separate column of the output file.
                                Several keys must be separated with commas.
 
   Specific options:
@@ -101,7 +102,7 @@
   2001-07-09  PB  V3.19 (pcb->ib)[0].FileNumber
   2001-07-09  PB  V4.00 new loop, no repetition
   2001-09-16  PB  V4.01 min abs value
-  2001-10-27  PB  V4.02 ImageIsFirst, %%%.edf -> into single txt file possible
+  2001-10-27  PB  V4.02 ImageIsFirst, %%%.edf -> to single txt file possible
   2002-06-01  PB  V4.50 ave
   2003-02-03  PB  V4.51 num[BlockNum]
   2003-02-09  PB  V4.52 line args for Dummy, DDummy, Dim
@@ -121,9 +122,14 @@
   2011-05-14  PB  V4.63 split.h -> strlib.h
   2012-01-31  PB  V4.64 ImageLoop parameters updated
   2016-02-17  PB  V4.65 remove .gz suffix when creating output filename
+  2016-10-02  PB  V4.66 NUsage corrected
+  2016-10-07  PB  V4.67 filename_full expansion of Prefix option removed
+  2017-10-20  PB  V4.68 No key value replacement in user_io,
+                        ReadHeaderLine arguments updated
+  2018-02-07  PB  V4.69 MaxAbsZ
 
 ---*/
-# define Version  "saxs_stat V4.65 2016-02-17, Peter Boesecke"
+# define Version  "saxs_stat V4.69 2018-02-07, Peter Boesecke"
 
 # include <errno.h>
 # include <stdio.h>
@@ -145,15 +151,20 @@
 # define SHist             "hist" /* write history */
 # define SSpec             "spec" /* write spec file */
 
-# define Default_extension ".txt"
-# define Default_prefix    "input"
 # define Default_select    SelectAll 
 
 # define NUsage "[options] \n\
-                <i1nam> <i2nam> <o/p prefix> <i1fst> <i1lst> <i1inc> \n\
-                <selection>\n\
-Purpose: Statistics\n\
-Options: "
+                <i1nam> <output prefix> <i1fst> <i1lst> <i1inc> \n\
+                <selection key>\n\n\
+                specific options: -sel   <selection key (default all)> \n\
+                                  -key   <header key>{,<header key>}> \n\
+                                  -pref  <output prefix> \n\
+                                  -ext   <output extension (default .txt)> \n\
+                                  -fnam  <output file name> \n\
+                                  -spc   <spacer (default \"  \\t\")> \n\
+                                  +-spec write spec header (default)\n\
+                                  +-show show results\n\n\
+                Calculate statistical parameters of an image.\n\n"
 
 /* keywords for the colunm selection*/
 # define SelectAll "all"
@@ -169,6 +180,7 @@ Options: "
 # define KeynDum "ndum"
 # define KeyMax  "max"
 # define KeyMin  "min"
+# define KeyaMax "amax"
 # define KeyaMin "amin"
 # define KeyDum  "dum"
 # define KeySum  "sum"
@@ -183,7 +195,7 @@ Options: "
 # define KeyGD2  "gd2"
 
 /* suit of the help */
-# define h00 "\tkeywords for the selection of the statistic :\n"
+# define h00 "\t\tselection keys:\n"
 # define h01 "\t\t"SelectAll"\tfor all\n"
 # define h02 "\t\t"KeyiNum"\tImage number\n"
 # define h03 "\t\t"KeyDim1"\tDimension 1\n"
@@ -197,22 +209,23 @@ Options: "
 # define h11 "\t\t"KeynDum"\tNumber of dummies\n"
 # define h12 "\t\t"KeyMax"\tMaximum Value\n"
 # define h13 "\t\t"KeyMin"\tMinimum Value\n"
-# define h14 "\t\t"KeyaMin"\tMinimum absolute value\n"
-# define h15 "\t\t"KeyDum"\tDummy value\n"
-# define h16 "\t\t"KeySum"\tSum of data values\n"
-# define h17 "\t\t"KeysSum"\tSum of squared data values\n"
-# define h18 "\t\t"KeyMean"\tMean value = sum / nval\n"
-# define h19 "\t\t"KeysMean"\tMean square value = ssum / nval\n"
-# define h20 "\t\t"KeyRMS"\tRoot mean square value = sqrt( smean )\n"
-# define h21 \
+# define h14 "\t\t"KeyaMax"\tMaximum absolute value\n"
+# define h15 "\t\t"KeyaMin"\tMinimum absolute value\n"
+# define h16 "\t\t"KeyDum"\tDummy value\n"
+# define h17 "\t\t"KeySum"\tSum of data values\n"
+# define h18 "\t\t"KeysSum"\tSum of squared data values\n"
+# define h19 "\t\t"KeyMean"\tMean value = sum / nval\n"
+# define h20 "\t\t"KeysMean"\tMean square value = ssum / nval\n"
+# define h21 "\t\t"KeyRMS"\tRoot mean square value = sqrt( smean )\n"
+# define h22 \
   "\t\t"KeyDevi"\tStandard deviation value = sqrt( smean - mean*mean )\n"
-# define h22 "\t\t"KeyGC1"\tCenter of mass 1\n"
-# define h23 "\t\t"KeyGC2"\tCenter of mass 2\n"
-# define h24 "\t\t"KeyGD1"\tStandard deviation 1 of center of mass\n"
-# define h25 "\t\t"KeyGD2"\tStandard deviation 2 of center of mass\n"
+# define h23 "\t\t"KeyGC1"\tCenter of mass 1\n"
+# define h24 "\t\t"KeyGC2"\tCenter of mass 2\n"
+# define h25 "\t\t"KeyGD1"\tStandard deviation 1 of center of mass\n"
+# define h26 "\t\t"KeyGD2"\tStandard deviation 2 of center of mass\n"
 # define Usage NUsage \
-h00 h01 h02 h03 h04 h05 h06 h07 h08 h09 h10 h11 h12 \
-h13 h14 h15 h16 h17 h18 h19 h20 h21 h22 h23 h24 h25
+h00 h01 h02 h03 h04 h05 h06 h07 h08 h09 h10 h11 h12 h13 \
+h14 h15 h16 h17 h18 h19 h20 h21 h22 h23 h24 h25 h26
 
 # define BlockNum 2       /* 1 input sequence + 1 output sequence */
 
@@ -233,7 +246,7 @@ Ask for all file names in the following way:
 
   output dummy [<dummy in first image>] :
 
-If blkno >= 1 the header block of the first image is read into ihb[1].
+If blkno >= 1 the header block of the first image is read in ihb[1].
 The routine must be used for all images at once.
 ---------------------------------------------------------------------------*/
 void SaxsScalArgvFilenames ( CmdBlk * pcb, ImgBlk ib[], ImgHeadBlk ihb[],
@@ -265,7 +278,9 @@ void SaxsScalArgvFilenames ( CmdBlk * pcb, ImgBlk ib[], ImgHeadBlk ihb[],
     /* Exit in all cases after an abort from the first argument */
     if (*pstatus!=Success) return;
 
-    OpenImageFile( pcb,ib,blkno,ib[blkno].Name.V,ib[1].FFirst.V,
+    /* No key value replacement in user_io -> ihb=>NULL */
+    OpenImageFile( pcb,ib,blkno,NULL,
+                   ib[blkno].Name.V,ib[1].FFirst.V,
                    IO_Old|IO_FileProtect,pstatus);
     if (*pstatus!=Success) return;
 
@@ -293,16 +308,16 @@ void SaxsScalArgvFilenames ( CmdBlk * pcb, ImgBlk ib[], ImgHeadBlk ihb[],
     if (!pPrefix->I) {
       /* Determine prefix from input file name */
       /* Check for .gz extension (use pPrefix->V as temporary buffer) */
-      filename_extension ( pPrefix->V, IO_len, ib[nameblk].Name.V );
+      filename_extension ( pPrefix->V, IO_size, ib[nameblk].Name.V );
       if (!strcmp(pPrefix->V,".gz")) {
         // Remove .gz extension
-        filename_body ( pPrefix->V, IO_len, ib[nameblk].Name.V ); // remove .gz
-        filename_body ( pPrefix->V, IO_len, pPrefix->V ); // remove ext.
+        filename_body ( pPrefix->V, IO_size, ib[nameblk].Name.V ); // remove .gz
+        filename_body ( pPrefix->V, IO_size, pPrefix->V ); // remove ext.
       } else {
         /* Generate filename of ASCII file */
-        filename_body ( pPrefix->V, IO_len, ib[nameblk].Name.V ); // remove ext.
+        filename_body ( pPrefix->V, IO_size, ib[nameblk].Name.V ); // remove ext.
       }
-      filename_name ( pPrefix->V, IO_len, pPrefix->V );
+      filename_name ( pPrefix->V, IO_size, pPrefix->V );
       pPrefix->I = TRUE;
     }
 
@@ -310,18 +325,14 @@ void SaxsScalArgvFilenames ( CmdBlk * pcb, ImgBlk ib[], ImgHeadBlk ihb[],
     argv_line( pcb,"Output file prefix",pPrefix,pPrefix->V,
                pstatus);
     if (*pstatus!=Success) return;
- 
+
     /* extract output filenumbers from ASCII file name */
     extract_filenumbers( &ib[0], pPrefix, TRUE, pstatus );
     if (*pstatus!=Success) return;
  
-    /* Expand Prefix to the full file name */
-    if (!filename_full (pPrefix->V,IO_len,pPrefix->V)) {
-      fprintf(ERROUT,"Input file name or path not allowed\n");
-      *pstatus = FileNotFound;
-      return;
-    }
- 
+    /* Prefix cannot be expanded with filename_full 
+       before replacing all keyvalues */
+       
     /*--- Take prefix as default for output filename */
     if (!ib[0].Name.I) {
       (void) sprintf(ib[0].Name.V,"%s",pPrefix->V);
@@ -380,7 +391,7 @@ NAME
   WriteHistory
 
 DESCRIPTION
-  Writes the full history into the output file 'out'
+  Writes the full history to the output file 'out'
 
 RETURN VALUE
   Number of written lines
@@ -499,7 +510,7 @@ void saxs_stat (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
   float Emin_2, Emax_2;
 
   long NbDumZ, NbGoodZ, NbPixZ;
-  double  MaxZ, MinZ, MinAbsZ;
+  double  MaxZ, MinZ, MaxAbsZ, MinAbsZ;
   double MeanZ = 0.0, MeanZ2 = 0.0, AbsMeanZ = 0.0, AbsMeanZ2 = 0.0;
   double SZ, SZ2;
   double SW1Z, SW2Z, SSW1Z, SSW2Z;
@@ -508,9 +519,9 @@ void saxs_stat (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
   int   ZI=FALSE;
   int   GI=FALSE;
 
-  char Sel[InputLineLength];
+  char Sel[IO_size];
 
-  char Key[IO_len], KeyValue[IO_len];
+  char Key[IO_size], KeyValue[IO_size];
 
   int parno, cols;
 
@@ -521,7 +532,7 @@ void saxs_stat (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
 
   /* output file */
   FILE *OutputFileId;
-  char fname_full[IO_len];
+  char fname_full[IO_size];
 
   *pstatus = Success;
 
@@ -551,17 +562,17 @@ void saxs_stat (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
   if (*pstatus!=Success) return;
 
   /* --- Get option values */
-  Select = option_line ( SSelect, pcb, pcb->ib, num, 1, pstatus );
+  Select = option_line ( SSelect, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
-  KeyList = option_line ( SKey, pcb, pcb->ib, num, 1, pstatus );
+  KeyList = option_line ( SKey, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
-  Extension = option_line ( SExtension, pcb, pcb->ib, num, 1, pstatus );
+  Extension = option_line ( SExtension, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
-  Prefix = option_line ( SPrefix, pcb, pcb->ib, num, 1, pstatus );
+  Prefix = option_line ( SPrefix, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
-  FName = option_line ( SFName, pcb, pcb->ib, num, 1, pstatus );
+  FName = option_line ( SFName, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
-  Spacer = option_line ( SSpacer, pcb, pcb->ib, num, 1, pstatus );
+  Spacer = option_line ( SSpacer, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
   Show = option_flag ( SShow, pcb, pstatus );
   if (*pstatus!=Success) return;
@@ -606,7 +617,7 @@ void saxs_stat (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
 
     NbGoodZ = 0;
 
-  /* copy input image into output image */
+  /* copy input image to output image */
   W_2 = Wmin_2; for (i=0;i<imax;i++) f_2[i]=f1_2[i];
   for (i_2=Imin_2;i_2<=Imax_2;i_2++) {
  
@@ -666,13 +677,13 @@ void saxs_stat (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
 
   if (NbGoodZ>0) {
    if (AbsMeanZ>eps) { 
-    /* normalize array with AbsMeanZ */
-    MaxZ = SIGN(MeanZ); MinZ = SIGN(MeanZ); MinAbsZ = 1.0;
-    } else {
-    /* no normalization of data with AbsMeanZ */
-    MaxZ = MeanZ; MinZ = MeanZ; MinAbsZ = AbsMeanZ;
-    AbsMeanZ = 1.0;
-    }
+     /* normalize array with AbsMeanZ */
+     MaxZ = SIGN(MeanZ); MinZ = SIGN(MeanZ); MaxAbsZ = 1.0; MinAbsZ = 1.0;
+   } else {
+     /* no normalization of data with AbsMeanZ */
+     MaxZ = MeanZ; MinZ = MeanZ; MaxAbsZ = AbsMeanZ; MinAbsZ = AbsMeanZ;
+     AbsMeanZ = 1.0;
+   }
 
     Wmin_1 = WORLD(INDEXSTART,Off_1[0],Ps_1[0]);   
     Wmin_2 = WORLD(INDEXSTART,Off_2[0],Ps_2[0]);  
@@ -691,6 +702,7 @@ void saxs_stat (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
             if (Value>MaxZ) MaxZ = Value;
             if (Value<MinZ) MinZ = Value;
             if (fabs(Value)<MinAbsZ) MinAbsZ = (double) fabs(Value);
+            if (fabs(Value)>MaxAbsZ) MaxAbsZ = (double) fabs(Value);
                         
             SW1Z+=W_1*Value;
             SW2Z+=W_2*Value;
@@ -706,8 +718,9 @@ void saxs_stat (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
       printf(" AbsMeanZ         = %g\n",AbsMeanZ);
       printf(" MaxZ/AbsMeanZ    = %g\n",MaxZ);
       printf(" MinZ/AbsMeanZ    = %g\n",MinZ);
+      printf(" MaxAbsZ/AbsMeanZ = %g\n",MaxAbsZ);
       printf(" MinAbsZ/AbsMeanZ = %g\n",MinAbsZ);
-      }
+    }
    
     SW1Z    *= AbsMeanZ;
     SW2Z    *= AbsMeanZ;
@@ -715,6 +728,7 @@ void saxs_stat (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
     SSW2Z   *= AbsMeanZ;
     MaxZ    *= AbsMeanZ;
     MinZ    *= AbsMeanZ;
+    MaxAbsZ *= AbsMeanZ;
     MinAbsZ *= AbsMeanZ;
     
     /* Calculate center of mass values */
@@ -766,6 +780,9 @@ void saxs_stat (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
     if ((strcmp(Sel,KeyMin)==0)  || (strcmp(Sel,SelectAll)==0))
        if (ZI) printf("  minimum                    = %g\n", MinZ);
           else printf("  minimum                    = undefined\n");
+    if ((strcmp(Sel,KeyaMax)==0) || (strcmp(Sel,SelectAll)==0))
+       if (ZI) printf("  max abs value              = %g\n", MaxAbsZ);
+          else printf("  max abs value              = undefined\n");
     if ((strcmp(Sel,KeyaMin)==0) || (strcmp(Sel,SelectAll)==0))
        if (ZI) printf("  min abs value              = %g\n", MinAbsZ);
           else printf("  min abs value              = undefined\n");
@@ -805,9 +822,9 @@ void saxs_stat (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
 
     if (KeyList.I) {
       parno = 1;
-      while (strlib_param ( Key, IO_len, KeyList.V, ',', parno++ )) {
+      while (strlib_param ( Key, IO_size, KeyList.V, ',', parno++ )) {
         if (strlen(Key)) {
-          ReadHeaderLine          ( pcb, 1, ihb, Key, KeyValue, pstatus );
+          ReadHeaderLine          ( pcb, 1, num, ihb, Key, KeyValue, pstatus );
           if (*pstatus) printf("%24s     = %s\n",Key,"(not found)");
           else printf("%24s     = %s\n",Key,KeyValue);
         }
@@ -820,12 +837,12 @@ void saxs_stat (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
   /*--- Get output filename */
   if (!FName.I) {
     sprintf(FName.V,"%s_%s%s",Prefix.V,Select.V,Extension.V);
-    }
+  }
 
   /* Generate filename of ASCII file */
-  filename_parameter (fname_full,IO_len,FName.V,0);
-  filename_pattern (fname_full,IO_len,fname_full,(pcb->ib)[0].FileNumber);
-  filename_full (fname_full,IO_len,fname_full);
+  filename_parameter (fname_full,IO_size,FName.V,0);
+  filename_pattern (fname_full,IO_size,fname_full,(pcb->ib)[0].FileNumber);
+  filename_full (fname_full,IO_size,fname_full);
 
   /*--- Open output file
     If this is the first image of the sequence, an existing file
@@ -867,7 +884,7 @@ void saxs_stat (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
     cols=0;
     if (KeyList.I) {
       parno = 1;
-      while (strlib_param ( Key, IO_len, KeyList.V, ',', parno++ )) {
+      while (strlib_param ( Key, IO_size, KeyList.V, ',', parno++ )) {
         if (strlen(Key)) cols++;
       }
     }
@@ -891,24 +908,25 @@ void saxs_stat (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
       fprintf(OutputFileId,"%s%s",Spacer.V,KeynDum); // 10
       fprintf(OutputFileId,"%s%s",Spacer.V,KeyMax); // 11
       fprintf(OutputFileId,"%s%s",Spacer.V,KeyMin); // 12
-      fprintf(OutputFileId,"%s%s",Spacer.V,KeyaMin); // 13
-      fprintf(OutputFileId,"%s%s",Spacer.V,KeyDum); // 14
-      fprintf(OutputFileId,"%s%s",Spacer.V,KeySum); // 15
-      fprintf(OutputFileId,"%s%s",Spacer.V,KeysSum); // 16
-      fprintf(OutputFileId,"%s%s",Spacer.V,KeyMean); // 17
-      fprintf(OutputFileId,"%s%s",Spacer.V,KeysMean); // 18
-      fprintf(OutputFileId,"%s%s",Spacer.V,KeyRMS); // 19
-      fprintf(OutputFileId,"%s%s",Spacer.V,KeyDevi); // 20
-      fprintf(OutputFileId,"%s%s",Spacer.V,KeyGC1); // 21
-      fprintf(OutputFileId,"%s%s",Spacer.V,KeyGC2); // 22
-      fprintf(OutputFileId,"%s%s",Spacer.V,KeyGD1); // 23
-      fprintf(OutputFileId,"%s%s",Spacer.V,KeyGD2); // 24
+      fprintf(OutputFileId,"%s%s",Spacer.V,KeyaMax); // 13
+      fprintf(OutputFileId,"%s%s",Spacer.V,KeyaMin); // 14
+      fprintf(OutputFileId,"%s%s",Spacer.V,KeyDum); // 15
+      fprintf(OutputFileId,"%s%s",Spacer.V,KeySum); // 16
+      fprintf(OutputFileId,"%s%s",Spacer.V,KeysSum); // 17
+      fprintf(OutputFileId,"%s%s",Spacer.V,KeyMean); // 18
+      fprintf(OutputFileId,"%s%s",Spacer.V,KeysMean); // 19
+      fprintf(OutputFileId,"%s%s",Spacer.V,KeyRMS); // 20
+      fprintf(OutputFileId,"%s%s",Spacer.V,KeyDevi); // 21
+      fprintf(OutputFileId,"%s%s",Spacer.V,KeyGC1); // 22
+      fprintf(OutputFileId,"%s%s",Spacer.V,KeyGC2); // 23
+      fprintf(OutputFileId,"%s%s",Spacer.V,KeyGD1); // 24
+      fprintf(OutputFileId,"%s%s",Spacer.V,KeyGD2); // 25
 /*++++++++++++
 %s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t\
-%s\t%s\t%s\t%s\t%s\t %s\t%s\t%s\t\
+%s\t%s\t%s\t%s\t%s\t%s\t\%s\t%s\t%s\t\
 %s\t%s\t%s\t%s\t%s\t%s\t%s\t%s",
         KeyiNum,KeyDim1,KeyDim2,KeyMin1,KeyMax1,KeyMin2,KeyMax2,KeynTot,
-        KeynVal,KeynDum,KeyMax,KeyMin,KeyaMin,KeyDum,KeySum,KeysSum,
+        KeynVal,KeynDum,KeyMax,KeyMin,KeyaMax,KeyaMin,KeyDum,KeySum,KeysSum,
         KeyMean,KeysMean,KeyRMS,KeyDevi,KeyGC1,KeyGC2,KeyGD1,KeyGD2);
 +++++++++++ */
     } else { 
@@ -922,7 +940,7 @@ void saxs_stat (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
 
     if (KeyList.I) {
       parno = 1;
-      while (strlib_param ( Key, IO_len, KeyList.V, ',', parno++ )) {
+      while (strlib_param ( Key, IO_size, KeyList.V, ',', parno++ )) {
         if (strlen(Key)) fprintf(OutputFileId,"%s%s",Spacer.V, Key);
       }
     }
@@ -959,6 +977,9 @@ void saxs_stat (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
   if ((strcmp(Sel,KeyMin)==0)  || (strcmp(Sel,SelectAll)==0))
     if (ZI) fprintf(OutputFileId, "%s%g", Spacer.V, MinZ);
        else fprintf(OutputFileId, "%s*", Spacer.V);
+  if ((strcmp(Sel,KeyaMax)==0) || (strcmp(Sel,SelectAll)==0))
+    if (ZI) fprintf(OutputFileId, "%s%g", Spacer.V,MaxAbsZ);
+       else fprintf(OutputFileId, "%s*", Spacer.V);
   if ((strcmp(Sel,KeyaMin)==0) || (strcmp(Sel,SelectAll)==0))
     if (ZI) fprintf(OutputFileId, "%s%g", Spacer.V,MinAbsZ);
        else fprintf(OutputFileId, "%s*", Spacer.V);
@@ -997,9 +1018,9 @@ void saxs_stat (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
 
   if (KeyList.I) {
     parno = 1;
-    while (strlib_param ( Key, IO_len, KeyList.V, ',', parno++ )) {
+    while (strlib_param ( Key, IO_size, KeyList.V, ',', parno++ )) {
       if (strlen(Key)) {
-        ReadHeaderLine          ( pcb, 1, ihb, Key, KeyValue, pstatus );
+        ReadHeaderLine          ( pcb, 1, num, ihb, Key, KeyValue, pstatus );
         if (*pstatus) fprintf(OutputFileId,"%s%s", Spacer.V,"*");
         else fprintf(OutputFileId,"%s%s", Spacer.V,KeyValue);
       }
@@ -1025,7 +1046,7 @@ Do all the keyboard io and return cb, and ib
 
 void user_io(CmdBlk * pcb, ImgBlk * ib, int * pstatus)
 {
-  char  progfn[InputLineLength];
+  char  progfn[IO_size];
   ImgHeadBlk ihb[BlockNum];
 
   float ROff_1, RPs_1, UOff_1, UPs_1;
@@ -1035,7 +1056,7 @@ void user_io(CmdBlk * pcb, ImgBlk * ib, int * pstatus)
 
  /* Determine program name without directory */
    (void) RightFR((char *) pcb->argv[0],DirSeparator,progfn,
-           InputLineLength);
+           IO_size);
 
   /* show usage if no arguments are given */
   if (pcb->argc<=1) printf("Usage: %s %s\n",progfn,Usage);
@@ -1100,8 +1121,8 @@ int MAIN (int argc, char *argv[])
 
   option_define ( IO_tpline, SSelect,Default_select, &cb );
   option_define ( IO_tpline, SKey,"", &cb );
-  option_define ( IO_tpline, SPrefix,Default_prefix, &cb );
-  option_define ( IO_tpline, SExtension, Default_extension, &cb );
+  option_define ( IO_tpline, SPrefix,"input", &cb );
+  option_define ( IO_tpline, SExtension, ".txt", &cb );
   option_define ( IO_tpline, SFName,"output.txt", &cb );
   option_define ( IO_tpline, SSpacer, "  \t", &cb ); /* default spc,spc,tab */
   option_define ( IO_tpflag, SShow,"FALSE", &cb );
