@@ -20,10 +20,10 @@
 /***************************************************************************/
 /*+++
 1 saxs_tiff
-  Transformation of an edf image sequence into 1byte/2byte/4byte tiff format
+  Transformation of an edf image sequence to 1byte/2byte/4byte tiff format
 
 2 PURPOSE
-  Transformation of an edf image sequence into 1byte/2byte/4byte tiff format
+  Transformation of an edf image sequence to 1byte/2byte/4byte tiff format
   (default 2 byte tiff)
 
   Arguments:
@@ -79,9 +79,10 @@
   2013-01-18  PB V4.61 Dummy and output dimensions are skipped
   2013-01-25  PB V4.62 Saxs2TiffFilenames according to ArgvBlocks
   2016-02-17  PB V4.63 remove .gz suffix when creating output filename
+  2017-10-19  PB V4.64 No key value replacement in user_io
 
 ---*/
-# define Version  "saxs_tiff 4.63 2016-02-16, Peter Boesecke"
+# define Version  "saxs_tiff 4.64 2017-10-19, Peter Boesecke"
 
 # include <errno.h>
 # include <stdio.h>
@@ -129,14 +130,14 @@
 /*---------------------------------------------------------------------------
 NAME
 
-   tiffnewstr --- allocate memory and copy a character string into it
+   tiffnewstr --- allocate memory and copy a character string to it
 
 SYNOPSIS
 
    char * tiffnewstr( const char * string );
 
 DESCRIPTION
-  Allocates strlen(string)+1 bytes of memory and copies string into it.
+  Allocates strlen(string)+1 bytes of memory and copies string to it.
   In case of success the pointer to the allocated memory is returned. The
   null pointer is returned in case of an error.
   If string is the NULL pointer the NULL pointer is returned.
@@ -210,12 +211,12 @@ RETURN VALUE
   char * prefix  (NULL, if filename too long)
 ---------------------------------------------------------------------------*/
 char * getprefix( const char * filename )
-{ char buffer[InputLineLength];
+{ char buffer[IO_size];
   size_t str_len;
   char *ps;
 
   str_len = strlen(filename);
-  if (str_len>=InputLineLength) return( (char *) NULL );
+  if (str_len>=IO_size) return( (char *) NULL );
 
   if (ps = strrchr( filename, (int) '.' )) {
     memcpy( buffer, filename, (ps-filename) );
@@ -247,12 +248,12 @@ RETURN VALUE
   char * extension (NULL, if filename too long)
 ---------------------------------------------------------------------------*/
 char * getextension( const char * filename )
-{ char buffer[InputLineLength];
+{ char buffer[IO_size];
   size_t str_len;
   char *ps;
 
   str_len = strlen(filename);
-  if (str_len>=InputLineLength) return( (char *) NULL );
+  if (str_len>=IO_size) return( (char *) NULL );
 
   if (ps = strrchr( filename, (int) '.' )) {
     memcpy( buffer, ps, (str_len-(ps-filename)) );
@@ -286,8 +287,8 @@ void Saxs2TiffFilenames ( CmdBlk * pcb, ImgBlk ib[], ImgHeadBlk ihb[],
                      int block_1st, int block_lst, int * pstatus)
 { int blkno, first_input, last_input;
   long image_1st, image_lst;
-  char linebuffer[IO_len];
-  char prefix[IO_len];
+  char linebuffer[IO_size];
+  char prefix[IO_size];
   long num[BlockNum];
   IO_line *pPrefix;
 
@@ -312,7 +313,9 @@ void Saxs2TiffFilenames ( CmdBlk * pcb, ImgBlk ib[], ImgHeadBlk ihb[],
     /* Exit in all cases after an abort from the first argument */
     if (*pstatus!=Success) return;
 
-    OpenImageFile( pcb,ib,blkno,ib[blkno].Name.V,ib[1].FFirst.V,
+    /* No key value replacement in user_io -> ihb=NULL */
+    OpenImageFile( pcb,ib,blkno,NULL,
+                   ib[blkno].Name.V,ib[1].FFirst.V,
                    IO_Old|IO_FileProtect,pstatus);
     if (*pstatus!=Success) return;
 
@@ -341,20 +344,20 @@ void Saxs2TiffFilenames ( CmdBlk * pcb, ImgBlk ib[], ImgHeadBlk ihb[],
     if (*pstatus!=Success) return;
 
     if (!pPrefix->I) {
-      char Suffix[IO_len];
+      char Suffix[IO_size];
 
       /* Determine prefix from file names */
       if (ib[0].Name.I) {
-       filename_body(prefix,IO_len-5,ib[0].Name.V);
+       filename_body(prefix,IO_size-5,ib[0].Name.V);
       } else {
-       filename_body(prefix,IO_len-5,filename_name(prefix,IO_len,ib[1].Name.V));
+       filename_body(prefix,IO_size-5,filename_name(prefix,IO_size,ib[1].Name.V));
       }
 
       /* Check for .gz extension (use Suffix as temporary buffer) */
-      filename_extension ( Suffix, IO_len, prefix );
+      filename_extension ( Suffix, IO_size, prefix );
       if (!strcmp(Suffix,".gz")) {
         // Remove .gz extension
-        filename_body ( prefix, IO_len, prefix ); // remove .gz
+        filename_body ( prefix, IO_size, prefix ); // remove .gz
       } 
  
       (void) sprintf(pPrefix->V,"%s%s", prefix, ".tif");
@@ -492,10 +495,10 @@ void saxs_tiff (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
   IO_float Min, Max;
 
   char *prefix, *extension;
-  char OutputBinaryName[InputLineLength];
+  char OutputBinaryName[IO_size];
   FILE *OutputBinaryId;
   size_t t_length;
-  char fname_full[IO_len], Temp[IO_len];
+  char fname_full[IO_size], Temp[IO_size];
 
   short bps;
   int   orientation;
@@ -504,7 +507,7 @@ void saxs_tiff (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
   long  xdim, ydim;
   short compression=1;
   char  *title, *description=(char *) NULL;
-  char  comment[InputLineLength];
+  char  comment[IO_size];
   float xdpcm, ydpcm;
 
   void *tiffdata;
@@ -537,15 +540,15 @@ void saxs_tiff (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
  /* --- Get option values */
   Auto = option_flag ( SAuto, pcb, pstatus );
   if (*pstatus!=Success) return;
-  Prefix = option_line ( SPrefix, pcb, pcb->ib, num, 1, pstatus );
+  Prefix = option_line ( SPrefix, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
-  Bps = option_long ( SBps, pcb, pcb->ib, num, 1, pstatus );
+  Bps = option_long ( SBps, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
-  Rori = option_long ( SRori, pcb, pcb->ib, num, 1, pstatus );
+  Rori = option_long ( SRori, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
-  Min = option_float ( SMin, pcb, pcb->ib, num, 1, pstatus );
+  Min = option_float ( SMin, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
-  Max = option_float ( SMax, pcb, pcb->ib, num, 1, pstatus );
+  Max = option_float ( SMax, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
 
  /* 1 input, 1 output */
@@ -629,17 +632,17 @@ void saxs_tiff (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
     W_2+=DW_2; for (i=0;i<imax;i++) { f_2[i]+=Df_2[i]; }
   } /* for i_2 ... */
 
-  if (pcb->TestBit) printf("Output into multiple tiff files\n");
+  if (pcb->TestBit) printf("Output to multiple tiff files\n");
 
   // Generate tiff filename
-  filename_parameter (fname_full,IO_len,Prefix.V,0);
-  filename_pattern (fname_full,IO_len,fname_full,(pcb->ib)[0].FileNumber);
-  filename_full (fname_full,IO_len,fname_full);
+  filename_parameter (fname_full,IO_size,Prefix.V,0);
+  filename_pattern (fname_full,IO_size,fname_full,(pcb->ib)[0].FileNumber);
+  filename_full (fname_full,IO_size,fname_full);
 
-  /* Output into multiple tiff files */
+  /* Output to multiple tiff files */
    if ( (pcb->ib[0].Inc.V==0) || (pcb->ib[0].First.V==pcb->ib[0].Last.V) ) {
     // fixed output file name
-    strncpy(OutputBinaryName,fname_full,InputLineLength);
+    strncpy(OutputBinaryName,fname_full,IO_size);
     } else {
     // numbered output file name
     prefix = getprefix(fname_full);
@@ -684,7 +687,7 @@ Do all the keyboard io and return cb, and ib
 
 void user_io(CmdBlk * pcb, ImgBlk * ib, int * pstatus)
 {
-  char buffer[IO_len];
+  char buffer[IO_size];
   ImgHeadBlk ihb[BlockNum];
   float tmp;
 
@@ -698,7 +701,7 @@ void user_io(CmdBlk * pcb, ImgBlk * ib, int * pstatus)
 
   IO_long Bps;
 
-  filename_name ( buffer, IO_len,  pcb->argv[0] );
+  filename_name ( buffer, IO_size,  pcb->argv[0] );
 
   /* show usage if no arguments are given */
   if (pcb->argc<=1) printf("Usage: %s %s\n", buffer, Usage);
@@ -749,7 +752,7 @@ void user_io(CmdBlk * pcb, ImgBlk * ib, int * pstatus)
     if (!(pMax->I)) {
       sprintf(buffer,"pow(2,%s)-1.0",pBps->V);
       tmp=floatexpr( buffer, pstatus );
-      if (*pstatus!=Success) CopyLine(buffer,pMax->V,IO_len,0);
+      if (*pstatus!=Success) CopyLine(buffer,pMax->V,IO_size,0);
       else sprintf(pMax->V,"%g",tmp); 
     }
     argv_flexp( pcb,"Maximum value", pMax, pMax->V, pstatus);
