@@ -28,7 +28,7 @@
   scattering) copies the input to the output image without projection.
 
   The input pattern is first back-projected to the Ewald-sphere and then
-  reprojected into the system of the symmetry axis by rotating it around 
+  reprojected to the system of the symmetry axis by rotating it around 
   the axis of rotational symmetry. In case of -styp 0 the two projections are
   inverse to each other and the input image will only be copied to the 
   output image. 
@@ -70,7 +70,7 @@
   <sro2>            : <ccw rotation around direction 2> (default 0_deg)
   <sro3>            : <ccw rotation around direction 3> (default 0_deg)
 
-  The input pattern is transformed into the coordinate system of the symmetry
+  The input pattern is transformed to the coordinate system of the symmetry
   axis, i.e. the intensity along axis 1 is projected to axis s_1 and the 
   intensity along axis 2 is projected to axis s_2.
 
@@ -92,10 +92,11 @@
   2012-01-31  PB V4.10 ImageLoop parameters updated
   2014-02-07  PB V4.11 saxs_uniax: waxs_Init and waxs_SymInit updated
   2015-10-03  PB V4.12 FileAlreadyOpen
+  2017-05-21  PB V4.13 ioalloc
 
 ---*/
 
-# define Version  "saxs_uniax V4.12 2015-10-03, Peter Boesecke"
+# define Version  "saxs_uniax V4.13 2017-05-21, Peter Boesecke"
 
 # include <errno.h>
 # include <stdio.h>
@@ -106,8 +107,7 @@
 # include "SaxsPrograms.h"
 # include "waxs.h"
 # include "strlib.h"
-
-# define FREE( a) if ( a) free( a)
+# include "ioalloc.h"
 
 # define Usage "[options] \n\
                 <i1nam> <onam> <i1fst> <i1lst> <i1inc>\n\
@@ -170,9 +170,9 @@ void saxs_uniax (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
   /*++++++++++ int Inverse, Transform; apparently not used */
 
  // xyfile parameters
-  char xypath[IO_len], xyname[IO_len];
-  char xy_pattern[IO_len], xyfull[IO_len];
-  char xfile[IO_len], yfile[IO_len];
+  char xypath[IO_size], xyname[IO_size];
+  char xy_pattern[IO_size], xyfull[IO_size];
+  char xfile[IO_size], yfile[IO_size];
   int xstream, ystream;
   float *xdata, *ydata;
   float *pxdata, *pydata;
@@ -188,7 +188,7 @@ void saxs_uniax (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
   IO_long  SymType, SymDo;
   IO_float SymRot1, SymRot2, SymRot3;
 
-  char key[IO_len];
+  char key[IO_size];
 
   float I0Off_1, I0Ps_1;
   float I0Off_2, I0Ps_2;
@@ -221,7 +221,7 @@ void saxs_uniax (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
   if (*pstatus!=Success) return;
   Xypix = option_flag( SXypix, pcb, pstatus);
   if (*pstatus!=Success) return;
-  Xyname = option_line ( SXyname, pcb, pcb->ib, num, 1, pstatus );
+  Xyname = option_line ( SXyname, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
 
  /* 1 input, 1 output */
@@ -250,11 +250,11 @@ void saxs_uniax (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
   xstream   = ystream   = -1;
   xdata     = ydata     = (float *) NULL;
   if (Xy.V) {
-    filename_parameter (xy_pattern,IO_len,Xyname.V,0);
-    filename_pattern (xy_pattern,IO_len,xy_pattern,pcb->ib[0].FileNumber);
-    filename_full ( xyfull, IO_len, xy_pattern );
-    filename_path ( xypath, IO_len, xyfull );
-    filename_name ( xyname, IO_len, xyfull );
+    filename_parameter (xy_pattern,IO_size,Xyname.V,0);
+    filename_pattern (xy_pattern,IO_size,xy_pattern,pcb->ib[0].FileNumber);
+    filename_full ( xyfull, IO_size, xy_pattern );
+    filename_path ( xypath, IO_size, xyfull );
+    filename_name ( xyname, IO_size, xyfull );
     xysize = edf_dim_product ( ihb[1].Dim ); // identical to input array
     // open x-file
     sprintf(xfile,"%s%s%s",xypath,"x",xyname);
@@ -266,7 +266,7 @@ void saxs_uniax (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
     SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
     if (*pstatus!=Success) return;
     if (xysize>0) {
-      if (!(xdata = (float *) malloc(sizeof(float)*xysize))) {
+      if (!(xdata = (float *) MALLOC(sizeof(float)*xysize))) {
         *pstatus = NotEnoughMemoryAvailable; return; }
       pxdata = xdata;
       for (idata=0;idata<xysize;idata++) *(pxdata++)=0.0;
@@ -281,7 +281,7 @@ void saxs_uniax (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
     SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
     if (*pstatus!=Success) {  FREE(xdata); return; }
     if (xysize>0) {
-      if (!(ydata = (float *) malloc(sizeof(float)*xysize))) {
+      if (!(ydata = (float *) MALLOC(sizeof(float)*xysize))) {
         FREE(xdata); *pstatus = NotEnoughMemoryAvailable; return; }
       pydata = ydata;
       for (idata=0;idata<xysize;idata++) *(pydata++)=0.0;
@@ -317,13 +317,13 @@ void saxs_uniax (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
 
   SymDo   = option_flag( SSymDo, pcb, pstatus);
   if (*pstatus!=Success) return;
-  SymType = option_long  ( SSymType, pcb, pcb->ib, num, 1, pstatus );
+  SymType = option_long  ( SSymType, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
-  SymRot1 = option_float ( SSymRot1, pcb, pcb->ib, num, 1, pstatus );
+  SymRot1 = option_float ( SSymRot1, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
-  SymRot2 = option_float ( SSymRot2, pcb, pcb->ib, num, 1, pstatus );
+  SymRot2 = option_float ( SSymRot2, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
-  SymRot3 = option_float ( SSymRot3, pcb, pcb->ib, num, 1, pstatus );
+  SymRot3 = option_float ( SSymRot3, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
 
   /* Symmetry operations are used for waxs projection coordinates. Because 
@@ -526,57 +526,57 @@ void saxs_uniax (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
 
     /* Write parameters of transformed image to x-displacement file */
     edf_write_header_float(xstream,xynum,1l,
-                strlib_concat(key,IO_len,KDisplaced,KOffset_1),ihb[0].Offset[1].V,
+                strlib_concat(key,IO_size,KDisplaced,KOffset_1),ihb[0].Offset[1].V,
                 &ErrorValue, pstatus );
     SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
     if (*pstatus!=Success) { FREE(xdata); FREE(ydata); return; }
     edf_write_header_float(xstream,xynum,1l,
-                strlib_concat(key,IO_len,KDisplaced,KOffset_2),ihb[0].Offset[2].V,
+                strlib_concat(key,IO_size,KDisplaced,KOffset_2),ihb[0].Offset[2].V,
                 &ErrorValue, pstatus );
     SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
     if (*pstatus!=Success) { FREE(xdata); FREE(ydata); return; }
     edf_write_header_float(xstream,xynum,1l,
-                 strlib_concat(key,IO_len,KDisplaced,KBinSiz_1),ihb[0].BinSiz[1].V,
+                 strlib_concat(key,IO_size,KDisplaced,KBinSiz_1),ihb[0].BinSiz[1].V,
                  &ErrorValue, pstatus );
     SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
     if (*pstatus!=Success) { FREE(xdata); FREE(ydata); return; }
     edf_write_header_float(xstream,xynum,1l,
-                 strlib_concat(key,IO_len,KDisplaced,KBinSiz_2),ihb[0].BinSiz[2].V,
+                 strlib_concat(key,IO_size,KDisplaced,KBinSiz_2),ihb[0].BinSiz[2].V,
                  &ErrorValue, pstatus );
     SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
     if (*pstatus!=Success) { FREE(xdata); FREE(ydata); return; }
     edf_write_header_float(xstream,xynum,1l,
-                 strlib_concat(key,IO_len,KDisplaced,KPixSiz_1),ihb[0].PixSiz[1].V,
+                 strlib_concat(key,IO_size,KDisplaced,KPixSiz_1),ihb[0].PixSiz[1].V,
                  &ErrorValue, pstatus );
     SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
     if (*pstatus!=Success) { FREE(xdata); FREE(ydata); return; }
     edf_write_header_float(xstream,xynum,1l,
-                 strlib_concat(key,IO_len,KDisplaced,KPixSiz_2),ihb[0].PixSiz[2].V,
+                 strlib_concat(key,IO_size,KDisplaced,KPixSiz_2),ihb[0].PixSiz[2].V,
                  &ErrorValue, pstatus );
     SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
     if (*pstatus!=Success) { FREE(xdata); FREE(ydata); return; }
     edf_write_header_float(xstream,xynum,1l,
-                 strlib_concat(key,IO_len,KDisplaced,KCenter_1),ihb[0].Center[1].V,
+                 strlib_concat(key,IO_size,KDisplaced,KCenter_1),ihb[0].Center[1].V,
                  &ErrorValue, pstatus );
     SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
     if (*pstatus!=Success) { FREE(xdata); FREE(ydata); return; }
     edf_write_header_float(xstream,xynum,1l,
-                 strlib_concat(key,IO_len,KDisplaced,KCenter_2),ihb[0].Center[2].V,
+                 strlib_concat(key,IO_size,KDisplaced,KCenter_2),ihb[0].Center[2].V,
                  &ErrorValue, pstatus );
     SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
     if (*pstatus!=Success) { FREE(xdata); FREE(ydata); return; }
     edf_write_header_float(xstream,xynum,1l,
-      strlib_concat(key,IO_len,KDisplaced,KSampleDistance),ihb[0].SampleDistance.V,
+      strlib_concat(key,IO_size,KDisplaced,KSampleDistance),ihb[0].SampleDistance.V,
       &ErrorValue, pstatus );
     SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
     if (*pstatus!=Success) { FREE(xdata); FREE(ydata); return; }
     edf_write_header_float(xstream,xynum,1l,
-      strlib_concat(key,IO_len,KDisplaced,KWaveLength),ihb[0].WaveLength.V,
+      strlib_concat(key,IO_size,KDisplaced,KWaveLength),ihb[0].WaveLength.V,
       &ErrorValue, pstatus );
     SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
     if (*pstatus!=Success) { FREE(xdata); FREE(ydata); return; }
     edf_write_header_line(xstream,xynum,1l,
-           strlib_concat(key,IO_len,KDisplaced,KProjection),protostr(I0Projection),
+           strlib_concat(key,IO_size,KDisplaced,KProjection),protostr(I0Projection),
            &ErrorValue, pstatus );
     SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
     if (*pstatus!=Success) { FREE(xdata); FREE(ydata); return; }
@@ -658,57 +658,57 @@ void saxs_uniax (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
 
     /* Write parameters of transformed image to y-displacement file */
     edf_write_header_float(ystream,xynum,1l,
-                strlib_concat(key,IO_len,KDisplaced,KOffset_1),ihb[0].Offset[1].V,
+                strlib_concat(key,IO_size,KDisplaced,KOffset_1),ihb[0].Offset[1].V,
                 &ErrorValue, pstatus );
     SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
     if (*pstatus!=Success) { FREE(ydata); return; }
     edf_write_header_float(ystream,xynum,1l,
-                strlib_concat(key,IO_len,KDisplaced,KOffset_2),ihb[0].Offset[2].V,
+                strlib_concat(key,IO_size,KDisplaced,KOffset_2),ihb[0].Offset[2].V,
                 &ErrorValue, pstatus );
     SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
     if (*pstatus!=Success) { FREE(ydata); return; }
     edf_write_header_float(ystream,xynum,1l,
-                strlib_concat(key,IO_len,KDisplaced,KBinSiz_1),ihb[0].BinSiz[1].V,
+                strlib_concat(key,IO_size,KDisplaced,KBinSiz_1),ihb[0].BinSiz[1].V,
                 &ErrorValue, pstatus );
     SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
     if (*pstatus!=Success) { FREE(ydata); return; }
     edf_write_header_float(ystream,xynum,1l,
-                strlib_concat(key,IO_len,KDisplaced,KBinSiz_2),ihb[0].BinSiz[2].V,
+                strlib_concat(key,IO_size,KDisplaced,KBinSiz_2),ihb[0].BinSiz[2].V,
                 &ErrorValue, pstatus );
     SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
     if (*pstatus!=Success) { FREE(ydata); return; }
     edf_write_header_float(ystream,xynum,1l,
-                 strlib_concat(key,IO_len,KDisplaced,KPixSiz_1),ihb[0].PixSiz[1].V,
+                 strlib_concat(key,IO_size,KDisplaced,KPixSiz_1),ihb[0].PixSiz[1].V,
                  &ErrorValue, pstatus );
     SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
     if (*pstatus!=Success) { FREE(xdata); FREE(ydata); return; }
     edf_write_header_float(ystream,xynum,1l,
-                 strlib_concat(key,IO_len,KDisplaced,KPixSiz_2),ihb[0].PixSiz[2].V,
+                 strlib_concat(key,IO_size,KDisplaced,KPixSiz_2),ihb[0].PixSiz[2].V,
                  &ErrorValue, pstatus );
     SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
     if (*pstatus!=Success) { FREE(xdata); FREE(ydata); return; }
     edf_write_header_float(ystream,xynum,1l,
-                 strlib_concat(key,IO_len,KDisplaced,KCenter_1),ihb[0].Center[1].V,
+                 strlib_concat(key,IO_size,KDisplaced,KCenter_1),ihb[0].Center[1].V,
                  &ErrorValue, pstatus );
     SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
     if (*pstatus!=Success) { FREE(xdata); FREE(ydata); return; }
     edf_write_header_float(ystream,xynum,1l,
-                 strlib_concat(key,IO_len,KDisplaced,KCenter_2),ihb[0].Center[2].V,
+                 strlib_concat(key,IO_size,KDisplaced,KCenter_2),ihb[0].Center[2].V,
                  &ErrorValue, pstatus );
     SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
     if (*pstatus!=Success) { FREE(xdata); FREE(ydata); return; }
     edf_write_header_float(ystream,xynum,1l,
-      strlib_concat(key,IO_len,KDisplaced,KSampleDistance),ihb[0].SampleDistance.V,
+      strlib_concat(key,IO_size,KDisplaced,KSampleDistance),ihb[0].SampleDistance.V,
       &ErrorValue, pstatus );
     SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
     if (*pstatus!=Success) { FREE(xdata); FREE(ydata); return; }
     edf_write_header_float(ystream,xynum,1l,
-      strlib_concat(key,IO_len,KDisplaced,KWaveLength),ihb[0].WaveLength.V,
+      strlib_concat(key,IO_size,KDisplaced,KWaveLength),ihb[0].WaveLength.V,
       &ErrorValue, pstatus );
     SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
     if (*pstatus!=Success) { FREE(xdata); FREE(ydata); return; }
     edf_write_header_line(ystream,xynum,1l,
-           strlib_concat(key,IO_len,KDisplaced,KProjection),protostr(I0Projection),
+           strlib_concat(key,IO_size,KDisplaced,KProjection),protostr(I0Projection),
            &ErrorValue, pstatus );
     SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
     if (*pstatus!=Success) { FREE(ydata); return; }
@@ -732,7 +732,7 @@ Do all the keyboard io and return cb, and ib
 
 void user_io(CmdBlk * pcb, ImgBlk * ib, int * pstatus)
 {
-  char  progfn[IO_len];
+  char  progfn[IO_size];
   ImgHeadBlk ihb[BlockNum];
 
   float ROff_1, RPs_1, UOff_1, UPs_1;
@@ -743,7 +743,7 @@ void user_io(CmdBlk * pcb, ImgBlk * ib, int * pstatus)
   IO_flexp *pSymRot1, *pSymRot2, *pSymRot3; // rotation of symmetry axis
 
   /*--- Determine program name without directory ---*/
-  filename_name ( progfn, IO_len,  pcb->argv[0] );
+  filename_name ( progfn, IO_size,  pcb->argv[0] );
 
   /*--- Show usage if no arguments are given ---*/
   if (pcb->argc<=1) printf("Usage: %s %s\n",progfn,Usage);
