@@ -144,9 +144,10 @@
   2012-01-31  PB V4.27 ImageLoop parameters updated
   2014-02-07  PB V4.28 saxs_waxs: waxs_Init and waxs_SymInit updated
   2015-10-03  PB V4.29 FileAlreadyOpen
+  2017-05-21  PB V4.30 ioalloc
 
 ---*/
-# define Version  "saxs_waxs V4.29 2015-10-03, Peter Boesecke"
+# define Version  "saxs_waxs V4.30 2017-05-21, Peter Boesecke"
 
 # include <errno.h>
 # include <stdio.h>
@@ -157,8 +158,7 @@
 # include "SaxsPrograms.h"
 # include "waxs.h"
 # include "strlib.h"
-
-# define FREE( a) { if ( a) { free( a); a = NULL; } }
+# include "ioalloc.h"
 
 # define Usage "[options] \n\
                 <i1nam> <onam> <i1fst> <i1lst> <i1inc> \n\
@@ -368,9 +368,9 @@ void saxs_waxs (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
   int transform;
 
  /* xyfile parameters */
-  char xypath[IO_len], xyname[IO_len];
-  char xy_pattern[IO_len], xyfull[IO_len];
-  char xfile[IO_len], yfile[IO_len];
+  char xypath[IO_size], xyname[IO_size];
+  char xy_pattern[IO_size], xyfull[IO_size];
+  char xfile[IO_size], yfile[IO_size];
   int xstream, ystream;
   float *xdata, *ydata;
   float *pxdata, *pydata;
@@ -385,7 +385,7 @@ void saxs_waxs (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
   IO_long  SymType;
   IO_float SymRot1, SymRot2, SymRot3;
 
-  char key[IO_len];
+  char key[IO_size];
 
   float I0Off_1, I0Ps_1;
   float I0Off_2, I0Ps_2;
@@ -417,7 +417,7 @@ void saxs_waxs (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
   if (*pstatus!=Success) return;
   Xypix = option_flag( SXypix, pcb, pstatus);
   if (*pstatus!=Success) return;
-  Xyname = option_line ( SXyname, pcb, pcb->ib, num, 1, pstatus );
+  Xyname = option_line ( SXyname, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
   Pdef = option_flag( SPdef, pcb, pstatus);
   if (*pstatus!=Success) return;
@@ -452,11 +452,11 @@ void saxs_waxs (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
   xstream   = ystream   = -1;
   xdata     = ydata     = (float *) NULL;
   if (Xy.V) {
-    filename_parameter (xy_pattern,IO_len,Xyname.V,0);
-    filename_pattern (xy_pattern,IO_len,xy_pattern,pcb->ib[0].FileNumber);
-    filename_full ( xyfull, IO_len, xy_pattern );
-    filename_path ( xypath, IO_len, xyfull );
-    filename_name ( xyname, IO_len, xyfull );
+    filename_parameter (xy_pattern,IO_size,Xyname.V,0);
+    filename_pattern (xy_pattern,IO_size,xy_pattern,pcb->ib[0].FileNumber);
+    filename_full ( xyfull, IO_size, xy_pattern );
+    filename_path ( xypath, IO_size, xyfull );
+    filename_name ( xyname, IO_size, xyfull );
     xysize = edf_dim_product ( ihb[1].Dim ); // identical to input array
     /* open x-file */
     sprintf(xfile,"%s%s%s",xypath,"x",xyname);
@@ -468,7 +468,7 @@ void saxs_waxs (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
     SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
     if (*pstatus!=Success) return;
     if (xysize>0) {
-      if (!(xdata = (float *) malloc(sizeof(float)*xysize))) {
+      if (!(xdata = (float *) MALLOC(sizeof(float)*xysize))) {
         *pstatus = NotEnoughMemoryAvailable; return; }
       pxdata = xdata;
       for (idata=0;idata<xysize;idata++) *(pxdata++)=0.0;
@@ -483,7 +483,7 @@ void saxs_waxs (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
     SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
     if (*pstatus!=Success) {  FREE(xdata); return; }
     if (xysize>0) {
-      if (!(ydata = (float *) malloc(sizeof(float)*xysize))) {
+      if (!(ydata = (float *) MALLOC(sizeof(float)*xysize))) {
         FREE(xdata); *pstatus = NotEnoughMemoryAvailable; return; }
       pydata = ydata;
       for (idata=0;idata<xysize;idata++) *(pydata++)=0.0;
@@ -524,13 +524,13 @@ void saxs_waxs (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
               (double) ihb[0].DetRot[2].V,
               (double) ihb[0].DetRot[3].V );
 
-  SymType  = option_long  (  SSymType, pcb, pcb->ib, num, 1, pstatus );
+  SymType  = option_long  (  SSymType, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
-  SymRot1 = option_float ( SSymRot1, pcb, pcb->ib, num, 1, pstatus );
+  SymRot1 = option_float ( SSymRot1, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
-  SymRot2 = option_float ( SSymRot2, pcb, pcb->ib, num, 1, pstatus );
+  SymRot2 = option_float ( SSymRot2, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
-  SymRot3 = option_float ( SSymRot3, pcb, pcb->ib, num, 1, pstatus );
+  SymRot3 = option_float ( SSymRot3, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
 
   /* symmetry operation are only used for waxs projection coordinates and
@@ -771,57 +771,57 @@ void saxs_waxs (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
 
     /* Write parameters of transformed image to x-displacement file */
     edf_write_header_float(xstream,xynum,1l,
-                strlib_concat(key,IO_len,KDisplaced,KOffset_1),ihb[0].Offset[1].V,
+                strlib_concat(key,IO_size,KDisplaced,KOffset_1),ihb[0].Offset[1].V,
                 &ErrorValue, pstatus );
     SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
     if (*pstatus!=Success) { FREE(xdata); FREE(ydata); return; }
     edf_write_header_float(xstream,xynum,1l,
-                strlib_concat(key,IO_len,KDisplaced,KOffset_2),ihb[0].Offset[2].V,
+                strlib_concat(key,IO_size,KDisplaced,KOffset_2),ihb[0].Offset[2].V,
                 &ErrorValue, pstatus );
     SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
     if (*pstatus!=Success) { FREE(xdata); FREE(ydata); return; }
     edf_write_header_float(xstream,xynum,1l,
-                 strlib_concat(key,IO_len,KDisplaced,KBinSiz_1),ihb[0].BinSiz[1].V,
+                 strlib_concat(key,IO_size,KDisplaced,KBinSiz_1),ihb[0].BinSiz[1].V,
                  &ErrorValue, pstatus );
     SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
     if (*pstatus!=Success) { FREE(xdata); FREE(ydata); return; }
     edf_write_header_float(xstream,xynum,1l,
-                 strlib_concat(key,IO_len,KDisplaced,KBinSiz_2),ihb[0].BinSiz[2].V,
+                 strlib_concat(key,IO_size,KDisplaced,KBinSiz_2),ihb[0].BinSiz[2].V,
                  &ErrorValue, pstatus );
     SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
     if (*pstatus!=Success) { FREE(xdata); FREE(ydata); return; }
     edf_write_header_float(xstream,xynum,1l,
-                 strlib_concat(key,IO_len,KDisplaced,KPixSiz_1),ihb[0].PixSiz[1].V,
+                 strlib_concat(key,IO_size,KDisplaced,KPixSiz_1),ihb[0].PixSiz[1].V,
                  &ErrorValue, pstatus );
     SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
     if (*pstatus!=Success) { FREE(xdata); FREE(ydata); return; }
     edf_write_header_float(xstream,xynum,1l,
-                 strlib_concat(key,IO_len,KDisplaced,KPixSiz_2),ihb[0].PixSiz[2].V,
+                 strlib_concat(key,IO_size,KDisplaced,KPixSiz_2),ihb[0].PixSiz[2].V,
                  &ErrorValue, pstatus );
     SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
     if (*pstatus!=Success) { FREE(xdata); FREE(ydata); return; }
     edf_write_header_float(xstream,xynum,1l,
-                 strlib_concat(key,IO_len,KDisplaced,KCenter_1),ihb[0].Center[1].V,
+                 strlib_concat(key,IO_size,KDisplaced,KCenter_1),ihb[0].Center[1].V,
                  &ErrorValue, pstatus );
     SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
     if (*pstatus!=Success) { FREE(xdata); FREE(ydata); return; }
     edf_write_header_float(xstream,xynum,1l,
-                 strlib_concat(key,IO_len,KDisplaced,KCenter_2),ihb[0].Center[2].V,
+                 strlib_concat(key,IO_size,KDisplaced,KCenter_2),ihb[0].Center[2].V,
                  &ErrorValue, pstatus );
     SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
     if (*pstatus!=Success) { FREE(xdata); FREE(ydata); return; }
     edf_write_header_float(xstream,xynum,1l,
-      strlib_concat(key,IO_len,KDisplaced,KSampleDistance),ihb[0].SampleDistance.V,
+      strlib_concat(key,IO_size,KDisplaced,KSampleDistance),ihb[0].SampleDistance.V,
       &ErrorValue, pstatus );
     SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
     if (*pstatus!=Success) { FREE(xdata); FREE(ydata); return; }
     edf_write_header_float(xstream,xynum,1l,
-      strlib_concat(key,IO_len,KDisplaced,KWaveLength),ihb[0].WaveLength.V,
+      strlib_concat(key,IO_size,KDisplaced,KWaveLength),ihb[0].WaveLength.V,
       &ErrorValue, pstatus );
     SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
     if (*pstatus!=Success) { FREE(xdata); FREE(ydata); return; }
     edf_write_header_line(xstream,xynum,1l,
-           strlib_concat(key,IO_len,KDisplaced,KProjection),protostr(I0Projection),
+           strlib_concat(key,IO_size,KDisplaced,KProjection),protostr(I0Projection),
            &ErrorValue, pstatus );
     SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
     if (*pstatus!=Success) { FREE(xdata); FREE(ydata); return; }
@@ -903,57 +903,57 @@ void saxs_waxs (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
 
     /* Write parameters of transformed image to y-displacement file */
     edf_write_header_float(ystream,xynum,1l,
-                strlib_concat(key,IO_len,KDisplaced,KOffset_1),ihb[0].Offset[1].V,
+                strlib_concat(key,IO_size,KDisplaced,KOffset_1),ihb[0].Offset[1].V,
                 &ErrorValue, pstatus );
     SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
     if (*pstatus!=Success) { FREE(ydata); return; }
     edf_write_header_float(ystream,xynum,1l,
-                strlib_concat(key,IO_len,KDisplaced,KOffset_2),ihb[0].Offset[2].V,
+                strlib_concat(key,IO_size,KDisplaced,KOffset_2),ihb[0].Offset[2].V,
                 &ErrorValue, pstatus );
     SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
     if (*pstatus!=Success) { FREE(ydata); return; }
     edf_write_header_float(ystream,xynum,1l,
-                strlib_concat(key,IO_len,KDisplaced,KBinSiz_1),ihb[0].BinSiz[1].V,
+                strlib_concat(key,IO_size,KDisplaced,KBinSiz_1),ihb[0].BinSiz[1].V,
                 &ErrorValue, pstatus );
     SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
     if (*pstatus!=Success) { FREE(ydata); return; }
     edf_write_header_float(ystream,xynum,1l,
-                strlib_concat(key,IO_len,KDisplaced,KBinSiz_2),ihb[0].BinSiz[2].V,
+                strlib_concat(key,IO_size,KDisplaced,KBinSiz_2),ihb[0].BinSiz[2].V,
                 &ErrorValue, pstatus );
     SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
     if (*pstatus!=Success) { FREE(ydata); return; }
     edf_write_header_float(ystream,xynum,1l,
-                 strlib_concat(key,IO_len,KDisplaced,KPixSiz_1),ihb[0].PixSiz[1].V,
+                 strlib_concat(key,IO_size,KDisplaced,KPixSiz_1),ihb[0].PixSiz[1].V,
                  &ErrorValue, pstatus );
     SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
     if (*pstatus!=Success) { FREE(ydata); return; }
     edf_write_header_float(ystream,xynum,1l,
-                 strlib_concat(key,IO_len,KDisplaced,KPixSiz_2),ihb[0].PixSiz[2].V,
+                 strlib_concat(key,IO_size,KDisplaced,KPixSiz_2),ihb[0].PixSiz[2].V,
                  &ErrorValue, pstatus );
     SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
     if (*pstatus!=Success) { FREE(ydata); return; }
     edf_write_header_float(ystream,xynum,1l,
-                 strlib_concat(key,IO_len,KDisplaced,KCenter_1),ihb[0].Center[1].V,
+                 strlib_concat(key,IO_size,KDisplaced,KCenter_1),ihb[0].Center[1].V,
                  &ErrorValue, pstatus );
     SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
     if (*pstatus!=Success) { FREE(ydata); return; }
     edf_write_header_float(ystream,xynum,1l,
-                 strlib_concat(key,IO_len,KDisplaced,KCenter_2),ihb[0].Center[2].V,
+                 strlib_concat(key,IO_size,KDisplaced,KCenter_2),ihb[0].Center[2].V,
                  &ErrorValue, pstatus );
     SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
     if (*pstatus!=Success) { FREE(ydata); return; }
     edf_write_header_float(ystream,xynum,1l,
-      strlib_concat(key,IO_len,KDisplaced,KSampleDistance),ihb[0].SampleDistance.V,
+      strlib_concat(key,IO_size,KDisplaced,KSampleDistance),ihb[0].SampleDistance.V,
       &ErrorValue, pstatus );
     SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
     if (*pstatus!=Success) { FREE(ydata); return; }
     edf_write_header_float(ystream,xynum,1l,
-      strlib_concat(key,IO_len,KDisplaced,KWaveLength),ihb[0].WaveLength.V,
+      strlib_concat(key,IO_size,KDisplaced,KWaveLength),ihb[0].WaveLength.V,
       &ErrorValue, pstatus );
     SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
     if (*pstatus!=Success) { FREE(ydata); return; }
     edf_write_header_line(ystream,xynum,1l,
-           strlib_concat(key,IO_len,KDisplaced,KProjection),protostr(I0Projection),
+           strlib_concat(key,IO_size,KDisplaced,KProjection),protostr(I0Projection),
            &ErrorValue, pstatus );
     SetSaxsErrorExternal( ErrorValue, ReportSaxsImageError );
     if (*pstatus!=Success) { FREE(ydata); return; }
@@ -977,7 +977,7 @@ Do all the keyboard io and return cb, and ib
 
 void user_io(CmdBlk * pcb, ImgBlk * ib, int * pstatus)
 {
-  char  progfn[IO_len];
+  char  progfn[IO_size];
   ImgHeadBlk ihb[BlockNum];
 
   float ROff_1, RPs_1, UOff_1, UPs_1;
@@ -989,7 +989,7 @@ void user_io(CmdBlk * pcb, ImgBlk * ib, int * pstatus)
   static const double deg2rad   = SAXS_PI/180.0; 
 
   /*--- Determine program name without directory ---*/
-  filename_name ( progfn, IO_len,  pcb->argv[0] );
+  filename_name ( progfn, IO_size,  pcb->argv[0] );
 
   /*--- Show usage if no arguments are given ---*/
   if (pcb->argc<=1) printf("Usage: %s %s\n",progfn,Usage);
@@ -1072,7 +1072,7 @@ ATTENTION: You are using the option -rot1 <rot1[deg]>, -rot2 <rot2[deg]> or\n\
   /*--- Argument  : rotation around axis 1 */
   if (pRot1->I) { 
     ib[1].DetRot[1].I = pRot1->I;
-    num_double2str( ib[1].DetRot[1].V, IO_len, pRot1->V*deg2rad, 
+    num_double2str( ib[1].DetRot[1].V, IO_size, pRot1->V*deg2rad, 
       "deg", EdfNDigitsFloat, &errval);
     if (errval) { *pstatus=Failed; return; }
   }
@@ -1083,7 +1083,7 @@ ATTENTION: You are using the option -rot1 <rot1[deg]>, -rot2 <rot2[deg]> or\n\
   /*--- Argument  : rotation around axis 2 */
   if (pRot2->I) {
     ib[1].DetRot[2].I = pRot2->I;
-    num_double2str( ib[1].DetRot[2].V, IO_len, pRot2->V*deg2rad, 
+    num_double2str( ib[1].DetRot[2].V, IO_size, pRot2->V*deg2rad, 
       "deg", EdfNDigitsFloat, &errval);
     if (errval) { *pstatus=Failed; return; }
   }
@@ -1094,7 +1094,7 @@ ATTENTION: You are using the option -rot1 <rot1[deg]>, -rot2 <rot2[deg]> or\n\
   /*--- Argument  : rotation around axis 3 */
   if (pRot3->I) {
     ib[1].DetRot[3].I = pRot3->I;
-    num_double2str( ib[1].DetRot[3].V, IO_len, pRot3->V*deg2rad, 
+    num_double2str( ib[1].DetRot[3].V, IO_size, pRot3->V*deg2rad, 
       "deg", EdfNDigitsFloat, &errval);
     if (errval) { *pstatus=Failed; return; }
   }
