@@ -37,7 +37,7 @@
   the sample transmission are required. 
   To simulate a detector pattern the not normalized image (-norm) should 
   additionally be multiplied by the detector sensitivity (flat field). 
-  The result can be fed into a poissonian noise generator (saxs_poisson) to 
+  The result can be fed to a poissonian noise generator (saxs_poisson) to 
   generate a detector pattern. 
 
   METHOD
@@ -1077,12 +1077,12 @@ void sphere2saxs (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
   float dummy = -1.0;
   float ddummy = DDSET(dummy);
   float SampleDistance, WaveLength;
-  char  line1[InputLineLength],
-        line2[InputLineLength],
-        filename[InputLineLength],
-        comment[InputLineLength],
-        liml[InputLineLength],
-        limc[InputLineLength];
+  char  line1[IO_size],
+        line2[IO_size],
+        filename[IO_size],
+        comment[IO_size],
+        liml[IO_size],
+        limc[IO_size];
 
   IO_long  Norm;
   IO_line  Rad;
@@ -1108,25 +1108,25 @@ void sphere2saxs (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
  /* --- Get option values */
   Norm = option_flag( SNorm, pcb, pstatus);
   if (*pstatus!=Success) return;
-  Rad = option_line ( SRad, pcb, pcb->ib, num, 1, pstatus );
+  Rad = option_line ( SRad, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
-  I0 = option_float ( SI0, pcb, pcb->ib, num, 1, pstatus );
+  I0 = option_float ( SI0, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
-  Trm = option_float ( STrm, pcb, pcb->ib, num, 1, pstatus );
+  Trm = option_float ( STrm, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
-  Tim = option_float ( STim, pcb, pcb->ib, num, 1, pstatus );
+  Tim = option_float ( STim, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
-  Dre = option_float ( SDre, pcb, pcb->ib, num, 1, pstatus );
+  Dre = option_float ( SDre, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
-  Vfr = option_float ( SVfr, pcb, pcb->ib, num, 1, pstatus );
+  Vfr = option_float ( SVfr, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
-  D = option_float ( SD, pcb, pcb->ib, num, 1, pstatus );
+  D = option_float ( SD, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
-  Pol = option_float ( SPol, pcb, pcb->ib, num, 1, pstatus );
+  Pol = option_float ( SPol, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
-  PChi = option_float ( SPChi, pcb, pcb->ib, num, 1, pstatus );
+  PChi = option_float ( SPChi, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
-  PPsi = option_float ( SPPsi, pcb, pcb->ib, num, 1, pstatus );
+  PPsi = option_float ( SPPsi, pcb, ihb, num, 1, pstatus );
   if (*pstatus!=Success) return;
   DoPol = option_flag( SDoPol, pcb, pstatus);
   if (*pstatus!=Success) return;
@@ -1177,7 +1177,7 @@ void sphere2saxs (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
     *pstatus=NotEnoughMemoryAvailable; return;
   }
 
-  LeftFL(arg,",",line1,InputLineLength);
+  LeftFL(arg,",",line1,IO_size);
   distribution_type = string2distributiontype( line1 );
 
   switch (distribution_type) {
@@ -1197,12 +1197,12 @@ void sphere2saxs (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
       break;
 
     case Schulz: Schultz: Shultz : // "Schulz", R, Z
-      RightFL(arg,",",line1,InputLineLength);
-      LeftFL(line1,",",line2,InputLineLength);
+      RightFL(arg,",",line1,IO_size);
+      LeftFL(line1,",",line2,IO_size);
       arg1  = floatexpr( line2, pstatus ); if (*pstatus!=Success) return;
 
-      RightFL(line1,",",line1,InputLineLength);
-      LeftFL(line1,",",line2,InputLineLength);
+      RightFL(line1,",",line1,IO_size);
+      LeftFL(line1,",",line2,IO_size);
       arg2  = floatexpr( line2, pstatus ); if (*pstatus!=Success) return;
 
       if (!ihb[0].Title.I) {
@@ -1220,64 +1220,64 @@ void sphere2saxs (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
     case File : 
       // File,filename,skipl,skipc,swap,colx,coly,readl,commentx,liml,limc
       // filename
-      RightFL(arg,",",line1,InputLineLength);
-      LeftFL(line1,",",filename,InputLineLength);
+      RightFL(arg,",",line1,IO_size);
+      LeftFL(line1,",",filename,IO_size);
       EditLine (Trim, filename);
       if (strlen(filename)==0) sprintf(filename,"xy.txt");
 
       // skiplines
-      RightFL(line1,",",line1,InputLineLength);
-      LeftFL(line1,",",line2,InputLineLength);
+      RightFL(line1,",",line1,IO_size);
+      LeftFL(line1,",",line2,IO_size);
       EditLine (Trim, line2);
       if (strlen(line2)>0)
         skiplines = longexpr( line2, pstatus ); if (*pstatus!=Success) return;
 
       // skipchars
-      RightFL(line1,",",line1,InputLineLength);
-      LeftFL(line1,",",line2,InputLineLength);
+      RightFL(line1,",",line1,IO_size);
+      LeftFL(line1,",",line2,IO_size);
       EditLine (Trim, line2);
       if (strlen(line2)>0)
         skipchars = longexpr( line2, pstatus ); if (*pstatus!=Success) return;
 
       // swap
-      RightFL(line1,",",line1,InputLineLength);
-      LeftFL(line1,",",line2,InputLineLength);
+      RightFL(line1,",",line1,IO_size);
+      LeftFL(line1,",",line2,IO_size);
       EditLine (Trim, line2);
       if (strlen(line2)>0)
         swap = (int) longexpr( line2, pstatus ); if (*pstatus!=Success) return;
 
       // colx 
-      RightFL(line1,",",line1,InputLineLength);
-      LeftFL(line1,",",line2,InputLineLength);
+      RightFL(line1,",",line1,IO_size);
+      LeftFL(line1,",",line2,IO_size);
       EditLine (Trim, line2);
       if (strlen(line2)>0)
         colx = longexpr( line2, pstatus ); if (*pstatus!=Success) return;
 
       // coly
-      RightFL(line1,",",line1,InputLineLength);
-      LeftFL(line1,",",line2,InputLineLength);
+      RightFL(line1,",",line1,IO_size);
+      LeftFL(line1,",",line2,IO_size);
       EditLine (Trim, line2);
       if (strlen(line2)>0)
         coly = longexpr( line2, pstatus ); if (*pstatus!=Success) return;
 
       // readlines 
-      RightFL(line1,",",line1,InputLineLength);
-      LeftFL(line1,",",line2,InputLineLength);
+      RightFL(line1,",",line1,IO_size);
+      LeftFL(line1,",",line2,IO_size);
       EditLine (Trim, line2);
       if (strlen(line2)>0)
         readlines = longexpr( line2, pstatus ); if (*pstatus!=Success) return;
 
       // comment 
-      RightFL(line1,",",line1,InputLineLength);
-      LeftFL(line1,",",comment,InputLineLength);
+      RightFL(line1,",",line1,IO_size);
+      LeftFL(line1,",",comment,IO_size);
 
       // liml 
-      RightFL(line1,",",line1,InputLineLength);
-      LeftFL(line1,",",liml,InputLineLength);
+      RightFL(line1,",",line1,IO_size);
+      LeftFL(line1,",",liml,IO_size);
 
       // limc
-      RightFL(line1,",",line1,InputLineLength);
-      LeftFL(line1,",",limc,InputLineLength);
+      RightFL(line1,",",line1,IO_size);
+      LeftFL(line1,",",limc,IO_size);
 
       if (!ihb[0].Title.I) {
         sprintf(ihb[0].Title.V, "from : %s",filename );
@@ -1300,7 +1300,7 @@ void sphere2saxs (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
 
     case Function :
       // Function,expression
-      RightFL(arg,",",line1,InputLineLength);
+      RightFL(arg,",",line1,IO_size);
       EditLine (Trim, line1);
       if (strlen(line1)==0) sprintf(line1,"%g",dummy);
 
@@ -1389,7 +1389,7 @@ void sphere2saxs (CmdBlk * pcb, long num[], ImgHeadBlk ihb[], int * pstatus)
                    ihb[0].Center[1].V, ihb[0].Center[2].V,
                    ihb[0].SampleDistance.V, i0*trm,
                    pcb->TestBit );
-    // write intensity and time into header
+    // write intensity and time to header
     WriteHeaderFloat(pcb,0,ihb,KValueI0,i0, pstatus); 
     if (*pstatus!=Success) return; 
     WriteHeaderFloat(pcb,0,ihb,KValueI1,i0*trm, pstatus);
@@ -1406,15 +1406,15 @@ Do all the keyboard io and return cb, and ib
 ---------------------------------------------------------------------------*/
 void user_io(CmdBlk * pcb, ImgBlk * ib, int * pstatus)
 {
-  char  progfn[IO_len];
-  char  tmp[InputLineLength];
+  char  progfn[IO_size];
+  char  tmp[IO_size];
   ImgHeadBlk ihb[BlockNum];
 
   float ROff_1, RPs_1, UOff_1, UPs_1;
   float ROff_2, RPs_2, UOff_2, UPs_2;
 
   int distribution_type;
-  char line1[InputLineLength];
+  char line1[IO_size];
 
   long num[BlockNum];
   long tmp_long;
@@ -1427,7 +1427,7 @@ void user_io(CmdBlk * pcb, ImgBlk * ib, int * pstatus)
   IO_flexp *pPol, *pPChi, *pPPsi;
 
   /*--- Determine program name without directory ---*/
-  filename_name ( progfn, IO_len,  pcb->argv[0] );
+  filename_name ( progfn, IO_size,  pcb->argv[0] );
 
   /* show usage if no arguments are given */
   if (pcb->argc<=1) printf("Usage: %s %s\n",progfn,Usage);
@@ -1504,7 +1504,7 @@ void user_io(CmdBlk * pcb, ImgBlk * ib, int * pstatus)
     pRad,pRad->V,pstatus);
   if (*pstatus!=Success) return;
 
-  LeftFL( pRad->V,",",line1,InputLineLength);
+  LeftFL( pRad->V,",",line1,IO_size);
   distribution_type = string2distributiontype( line1 );
 
   pDre = (IO_flexp*) option_parameter_search( SDre, pcb, pstatus );
